Type: Package
Package: SimplexRegression
Title: Simplex Regression Models with Parametric or Fixed Mean Link
        Functions
Version: 0.1.5
Authors@R: c(
    person("Maria Eduarda da Cruz", "Justino", , "eueduardacruz@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9501-2642")),
    person("Francisco", "Cribari-Neto", , "francisco.cribari@ufpe.br", role = c("ctb", "ths"),
           comment = c(ORCID = "0000-0002-5909-6698"))
  )
Author: Maria Eduarda da Cruz Justino [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9501-2642>),
  Francisco Cribari-Neto [ctb, ths] (ORCID:
    <https://orcid.org/0000-0002-5909-6698>)
Maintainer: Maria Eduarda da Cruz Justino <eueduardacruz@gmail.com>
Description: Fits and analyzes simplex regression models with either
    fixed or parametric mean link functions. Implements the simplex
    probability density function, cumulative distribution function,
    quantile function, random number generation, and variance evaluation.
    Offers several fixed and parametric link functions for the mean
    submodel, tools for residual analysis and diagnostic plotting,
    hypothesis testing procedures, and influence measures such as Cook's
    distance and leverage (hat values). Includes the Scout Score (SS)
    criterion for model selection, enabling comprehensive inference and
    diagnostic analysis within the simplex regression framework.
    For more details see Barndorff-Nielsen and Jorgensen (1991)
    <doi:10.1016/0047-259X(91)90008-P> and Justino and Cribari-Neto (2026) 
    <doi:10.1016/j.apm.2025.116713>.
License: MIT + file LICENSE
URL: https://github.com/dudajustino/SimplexRegression
BugReports: https://github.com/dudajustino/SimplexRegression/issues
Depends: R (>= 3.5)
Imports: Formula, lmtest, Matrix, moments, parallel, pracma, sandwich
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0), tseries
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
LazyDataCompression: xz
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-19 18:01:57 UTC; euedu
Repository: CRAN
Date/Publication: 2026-07-19 18:30:02 UTC
Built: R 4.5.2; ; 2026-07-19 21:22:48 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,simplexregression)
S3method(BIC,simplexregression)
S3method(HQIC,simplexregression)
S3method(bread,simplexregression)
S3method(coef,simplexregression)
S3method(coeftest,simplexregression)
S3method(cooks.distance,simplexregression)
S3method(deviance,simplexregression)
S3method(df.residual,simplexregression)
S3method(estfun,simplexregression)
S3method(fitted,simplexregression)
S3method(formula,simplexregression)
S3method(gleverage,simplexregression)
S3method(hatvalues,simplexregression)
S3method(logLik,simplexregression)
S3method(lrtest,simplexregression)
S3method(model.frame,simplexregression)
S3method(model.matrix,simplexregression)
S3method(nobs,simplexregression)
S3method(plot,simplexregression)
S3method(predict,simplexregression)
S3method(print,simplexregression)
S3method(print,summary.simplexregression)
S3method(residuals,simplexregression)
S3method(simulate,simplexregression)
S3method(summary,simplexregression)
S3method(terms,simplexregression)
S3method(update,simplexregression)
S3method(vcov,simplexregression)
export(HQIC)
export(dev.unit.simplex)
export(diag.distances)
export(diag.im)
export(dispersion_link)
export(dispersion_link_deriv1)
export(dispersion_link_inv)
export(dispersion_link_inv_deriv1)
export(dsimplex)
export(fixed_mean_link)
export(fixed_mean_link_deriv1)
export(fixed_mean_link_deriv2)
export(fixed_mean_link_inv)
export(fixed_mean_link_inv_deriv1)
export(gleverage)
export(halfnormal.plot)
export(local.influence)
export(parametric_mean_link)
export(parametric_mean_link_deriv1)
export(parametric_mean_link_deriv2)
export(parametric_mean_link_inv)
export(parametric_mean_link_inv_deriv1)
export(penalized.ic)
export(penalized.ss)
export(press)
export(psimplex)
export(qsimplex)
export(r2)
export(resettest)
export(rsimplex)
export(scoretest)
export(simplexreg)
export(simplexreg.control)
export(simplexreg.fit)
export(variance.simplex)
importFrom(Formula,as.Formula)
importFrom(Matrix,nearPD)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(lmtest,coeftest)
importFrom(lmtest,lrtest)
importFrom(moments,skewness)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(pracma,quadgk)
importFrom(sandwich,bread)
importFrom(sandwich,estfun)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cooks.distance)
importFrom(stats,cor)
importFrom(stats,dcauchy)
importFrom(stats,delete.response)
importFrom(stats,deviance)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,formula)
importFrom(stats,hatvalues)
importFrom(stats,integrate)
importFrom(stats,lm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pgamma)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,qcauchy)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,rchisq)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(utils,modifyList)

## ----include = FALSE------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  fig.align  = "center",
  message    = FALSE,
  warning    = FALSE
)
old_opts <- options(width = 70, prompt = "R> ", continue = "+  ", digits = 5)

## ----setup----------------------------------------------------------
library(SimplexRegression)
data(RelativeHumidity, package = "SimplexRegression")
head(RelativeHumidity, 5)

## ----data-prep------------------------------------------------------
RH       <- RelativeHumidity
RH$hs    <- sin(2 * pi * seq_len(nrow(RH)) / 12)
RH$hc    <- cos(2 * pi * seq_len(nrow(RH)) / 12)
RH$dummy <- as.integer(as.integer(format(RH$date, "%m")) %in% 10:12)

## ----summary-rh-----------------------------------------------------
summary(RH$rh)
cat(sprintf(
  "Std. dev.: %.4f  |  Skewness: %.4f\n",
  sd(RH$rh),
  mean(((RH$rh - mean(RH$rh)) / sd(RH$rh))^3)
))

## ----formula--------------------------------------------------------
formula <- rh ~ ins2 + tmax + ws + hs + hc + dummy + I(dummy * ws) | pre2

## ----models-plogit--------------------------------------------------
fit_p1 <- simplexreg(formula, data = RH, link.mu = "plogit1")
fit_p2 <- simplexreg(formula, data = RH, link.mu = "plogit2")

## ----penalized-ss-param---------------------------------------------
penalized.ss(fit_p1, fit_p2, kappa = 0.1)

## ----penalized-ic---------------------------------------------------
penalized.ic(fit_p1, fit_p2, kappa = 0.1)

## ----models-fixed---------------------------------------------------
fit_loglog  <- simplexreg(formula, data = RH, link.mu = "loglog")
fit_logit   <- simplexreg(formula, data = RH, link.mu = "logit")
fit_probit  <- simplexreg(formula, data = RH, link.mu = "probit")
fit_cauchit <- simplexreg(formula, data = RH, link.mu = "cauchit")
fit_cloglog <- simplexreg(formula, data = RH, link.mu = "cloglog")

## ----penalized-ss-all-----------------------------------------------
penalized.ss(
  fit_loglog, fit_logit, fit_probit,
  fit_cauchit, fit_cloglog, fit_p1,
  kappa = 0
)

## ----summary-fit----------------------------------------------------
summary(fit_loglog)

## ----aic-bic--------------------------------------------------------
AIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
BIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
HQIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)

## ----coef-vcov------------------------------------------------------
coef(fit_loglog)                          # full coefficient vector
coef(fit_loglog, model = "mean")          # mean submodel only
coef(fit_loglog, model = "dispersion")    # dispersion submodel only
round(vcov(fit_loglog, model = "mean"), 6)  # vcov of mean submodel

## ----loglik---------------------------------------------------------
logLik(fit_loglog)

## ----lrtest---------------------------------------------------------
fit_loglog_null <- update(fit_loglog, . ~ . | 1)
lmtest::lrtest(fit_loglog, fit_loglog_null)

## ----scoretest------------------------------------------------------
fit_logit_h0 <- simplexreg(formula, data = RH, link.mu = "logit")
scoretest(fit_logit_h0, link.mu = "plogit1")
scoretest(fit_logit_h0, link.mu = "plogit2")

## ----resettest------------------------------------------------------
resettest(fit_loglog)                      # both submodels augmented
resettest(fit_loglog, dispersion = FALSE)  # mean submodel only

## ----fitted-resid---------------------------------------------------
head(fitted(fit_loglog))
head(residuals(fit_loglog, type = "quantile"))  # approx. N(0,1)
head(residuals(fit_loglog, type = "pearson"))
head(residuals(fit_loglog, type = "weighted"))  # for halfnormal.plot

## ----press----------------------------------------------------------
press(fit_loglog)                            # single model
press(fit_loglog, fit_logit, fit_probit)     # comparing models

## ----predict--------------------------------------------------------
head(predict(fit_loglog, type = "response"))        # fitted means
head(predict(fit_loglog, type = "link")$mean)       # mean linear predictor
head(predict(fit_loglog, type = "link")$dispersion) # dispersion predictor
head(predict(fit_loglog, type = "dispersion"))      # fitted sigma^2

# Out-of-sample prediction
new_obs <- RH[1:3, ]
predict(fit_loglog, newdata = new_obs, type = "response")

## ----simulate-------------------------------------------------------
set.seed(2026)
sims <- simulate(fit_loglog, nsim = 3)
head(sims)

## ----influence------------------------------------------------------
hii  <- hatvalues(fit_loglog)
cook <- cooks.distance(fit_loglog, type = "pearson")
cat(sprintf("Leverages  — max: %.4f  mean: %.4f\n", max(hii),  mean(hii)))
cat(sprintf("Cook's D   — max: %.4f\n", max(cook)))

## ----gleverage------------------------------------------------------
gl <- gleverage(fit_loglog)
cat(sprintf("Generalized leverage — max: %.4f  mean: %.4f\n",
            max(gl), mean(gl)))

## ----plots-1-7, fig.height = 8, fig.cap = "Diagnostic plots (1–7) for the fitted simplex regression model with log-log link."----
oldpar <- par(mfrow = c(3, 3))
plot(fit_loglog, which = 1:5, ask = FALSE)
plot(fit_loglog, which = 6, threshold = 0.15, label.pos = c(4,2,3))
plot(fit_loglog, which = 7, threshold = 0.08, label.pos = c(3,3,3,4))
par(oldpar)

## ----local-influence-cw, fig.height = 4.5, fig.cap = "Total local influence $C_i$ under case-weight and response perturbation for all parameters."----
oldpar <- par(mfrow = c(1, 2))
local.influence(
  fit_loglog,
  scheme    = "case.weight",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.5,
  label.pos = c(3, 2, 4)
)
local.influence(
  fit_loglog,
  scheme    = "response",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.4,
  label.pos = 2
)
par(oldpar)

## ----halfnormal, fig.height = 5, fig.cap = "Half-normal plot of absolute weighted residuals with 95% simulated envelope (100 replications)."----
halfnormal.plot(fit_loglog, nsim = 19, type = "weighted", seed = 2026)

## ----timeseries, fig.height = 4, fig.cap = "Observed (solid black) and fitted (dashed red) monthly relative humidity in Brasília, January 2000 to December 2025."----
plot(RH$date, RH$rh,
     type = "l", col = "black", lwd = 1.2,
     xlab = "Date", ylab = "Relative humidity",
     main = "Observed vs Fitted RH — Brasília (2000–2025)")
lines(RH$date, fitted(fit_loglog), col = "red", lwd = 1.5, lty = 2)
legend("bottomleft",
       legend = c("Observed", "Fitted"),
       col    = c("black", "red"),
       lty    = c(1, 2), lwd = c(1.2, 1.5),
       bty    = "n", cex = 0.85)

## ----restore-options, include = FALSE-----------------------------------------
options(old_opts)

## ----session------------------------------------------------------------------
sessionInfo()

################################################################################
#               SIMPLEX REGRESSION - LOCAL AND GLOBAL INFLUENCE                #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2025-11-08                                                             #
# Description: Influence measures: Local influence via case-weight and response#
#                               perturbation scheme, and generalized leverage  #
################################################################################

# ==============================================================================
# 1. LOCAL INFLUENCE
# ==============================================================================

#' @title Local Influence for Simplex Regression Models
#' @description Computes local influence measures under case-weight and
#' response perturbation schemes for simplex regression models with
#' parametric or fixed mean link function.
#'
#' @param model An object of class \code{simplexregression}.
#' @param scheme Character string specifying the perturbation scheme:
#' \code{"case.weight"} or \code{"response"}.
#' @param parameter Character string indicating the parameter block:
#' \code{"theta"} (default, all parameters), \code{"beta"} (mean submodel),
#' or \code{"gamma"} (dispersion submodel).
#' @param type Character string specifying the influence measure to compute:
#' \code{"Ci"} (default, total local influence / normal curvature) or
#' \code{"dmax"} (maximum influence direction).
#' @param plot Logical; if \code{TRUE}, produces an index plot of the selected
#' measure. Default is \code{FALSE}.
#' @param threshold Numeric threshold for identifying influential observations in.
#' If \code{NULL} (default), no observations are highlighted.
#' @param label.pos Position(s) for outlier labels in plot. Can be a single value
#' (applied to all labels) or a vector. Values: 1 = below, 2 = left, 3 = above,
#' 4 = right.
#' @param plot.type Character string controlling the plot style when
#' \code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} (vertical
#' lines). Passed to the \code{type} argument of \code{plot()}.
#' @param ... Additional graphical parameters passed to \code{plot()}.
#'
#' @return If \code{plot = FALSE} (default), a list containing:
#' \itemize{
#'   \item \code{dmax.beta}: Maximum influence direction for mean parameters;
#'   \item \code{dmax.gamma}: Maximum influence direction for dispersion parameters;
#'   \item \code{dmax.theta}: Maximum influence direction for all parameters;
#'   \item \code{Ci.beta}: Total local influence for mean parameters;
#'   \item \code{Ci.gamma}: Total local influence for dispersion parameters;
#'   \item \code{Ci.theta}: Total local influence for all parameters.
#' }
#' If \code{plot = TRUE}, the same list is returned invisibly.
#'
#' @details
#' Measures local influence based on the curvature of the log-likelihood surface
#' under small perturbations. Two perturbation schemes are implemented:
#' \itemize{
#'   \item \strong{Case-weight}: Perturbs observation weights;
#'   \item \strong{Response perturbation}: Perturbs response values.
#' }
#'
#' The index plot of \code{dmax} can be used to detect observations that are
#' jointly influential for parameters. The index plot of the normal curvature
#' \code{Ci} can be used to detect observations that are individually influential
#' for parameters.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Local influence under case-weight perturbation — return results
#' infl_cw <- local.influence(fit, scheme = "case.weight")
#'
#' # Plot Ci for beta directly from the function call
#' local.influence(fit, scheme = "case.weight",
#'                 parameter = "beta", type = "Ci", plot = TRUE)
#'
#' # Plot dmax for all parameters under response perturbation
#' local.influence(fit, scheme = "response",
#'                 parameter = "theta", type = "dmax", plot = TRUE)
#'
#' @references
#' Espinheira, P. L., Silva, A. O. (2020). Residual and influence analysis to a
#' general class of simplex regression. \emph{TEST}, \bold{29}, 523–-552.
#' \doi{10.1007/s11749-019-00665-3}
#'
#' @seealso \code{\link{hatvalues.simplexregression}},
#' \code{\link{cooks.distance.simplexregression}},
#' \code{\link{gleverage.simplexregression}}.
#'
#' @importFrom stats plogis quantile
#' @export
local.influence <- function(model, scheme = c("case.weight", "response"),
                            parameter = c("theta", "beta", "gamma"),
                            type      = c("Ci", "dmax"),
                            plot      = FALSE, threshold = NULL,
                            label.pos = 3, plot.type = NULL, ...) {

  if (!inherits(model, "simplexregression")) {
    stop("'model' must be an object of class 'simplexregression'")
  }

  scheme <- match.arg(scheme)
  parameter <- match.arg(parameter)
  type      <- match.arg(type)

  parametric <- is_parametric(model)

  n <- model$nobs
  y <- model$y
  mu <- model$mu.fv
  sigma2 <- model$sigma2.fv
  link_mu <- model$mu.link
  link_sigma2 <- model$sigma2.link
  eta1 <- model$mu.lp
  eta2 <- model$sigma2.lp
  X <- model$mu.x
  Z <- model$sigma2.x

  p <- ncol(X)
  q <- ncol(Z)
  r <- if (parametric) p + q + 1 else p + q

  diff <- y - mu
  yoneminy <- y * (1 - y)
  muonemu <- mu * (1 - mu)
  dev <- (diff / muonemu)^2 / yoneminy

  Ui <- (dev/muonemu) + (1/(muonemu^3))
  UI <- diag(Ui, ncol=n, nrow = n)
  Vi <- 1/(2*sigma2^2)
  VI <- diag(Vi, ncol=n, nrow = n)
  ai <- -1/(2*sigma2) + dev*Vi
  AI <- diag(ai, ncol=n, nrow = n)
  Mi <- 2 * ai / sigma2
  YDAG <- diag(diff, ncol=n, nrow = n)
  Sy <- diag(sqrt(variance.simplex(mu, sigma2)), n, n)

  sigma2i <- 1/sigma2
  SIG <- diag(sigma2i, ncol=n, nrow = n)
  Dlink.sigma2 <- dispersion_link_inv_deriv1(eta2,  link_sigma2)
  HI <- diag(Dlink.sigma2, ncol=n, nrow = n)

  deriv2_linksigma2 <- switch(link_sigma2,
                              log = -1 / sigma2^2,
                              sqrt = -1 / (4 * sigma2 * sqrt(sigma2)),
                              identity = 0)

  sdag <- ai * deriv2_linksigma2

  deriv2_dev <- - (2 / ((1 - y) * muonemu^3)) *
    (10*y - (3*(y^2 + mu^4) / (y*muonemu)) + (2*(1 + 6*mu^2 - 4*mu) / (1-mu)))
  dev2 <- deriv2_dev / 2

  b_1 <- ( 2 / (y * (1-mu)^3) + (1 - 3*mu) / (mu^2 * (1-mu)^3) + (diff * Ui)) / yoneminy
  B_1 <- diag(b_1, ncol=n, nrow = n)
  b_2 <- (dev + (2*diff / (y * mu * (1-mu)^2))) / yoneminy
  B_2 <- diag(b_2, ncol=n, nrow = n)

  B <- function(Delta,I,M) {
    t(Delta)%*%(I-M)%*%Delta
  }

  if(parametric){
    lambda <- model$lambda.fv
    Dlink.mu <- parametric_mean_link_inv_deriv1(eta1, lambda, link_mu)

    if(link_mu == "plogit2") {
      exp_aval_frac <- plogis(eta1)^(1/lambda)
      log_aval <- log(plogis(eta1))
      rho <- as.vector(- exp_aval_frac * log_aval / (lambda^2))
      deriv2_linkmu <- as.vector((lambda * mu^lambda * (1+lambda) - lambda) /
                                   (mu^2 * (1-mu^lambda)^2))
      VARTHETA <- as.vector(- (1 / (lambda^3 * (1 + exp(eta1)))) * exp_aval_frac *
                              (lambda + log_aval))
      VARSIG <- as.vector(1/lambda^4 * exp_aval_frac * log_aval * (log_aval + 2*lambda))
    } else {
      log_aval <- log(1+exp(eta1))
      rho <- as.vector((-1/(lambda^2)) * ((1 + exp(eta1)) ^ (-1/lambda)) * log_aval)
      deriv2_linkmu <- as.vector(lambda * (1 - (1+lambda) * (1-mu)^lambda) /
                                   ((1-mu)^2 * (1 - (1-mu)^lambda)^2))
      VARTHETA <- as.vector(exp(eta1) * (1 + exp(eta1))^(-1 - 1/lambda) *
                              (log_aval - lambda) / (lambda^3))
      VARSIG <- as.vector((1 + exp(eta1))^(-1/lambda) * log_aval *
                            (2*lambda - log_aval) / lambda^4)
    }

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L2 <- crossprod(X, sigma2i * (dev2 * Dlink.mu * rho - diff * Ui * VARTHETA))
    L3 <- crossprod(Z, sigma2i^2 * diff * Ui * Dlink.sigma2 * rho)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1, L2),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z), L3),
      cbind(t(L2), t(L3), sum(sigma2i * (dev2 * rho^2 - diff * Ui * VARSIG)))
    )

    Lbetabeta <- L[1:p,1:p]
    Lbetalambda <- L[1:p, r]
    Llambdabeta <- t(Lbetalambda)
    Llambdalambda <- L[r, r]
    Ldeltadelta <- L[(p+1):(p+q),(p+1):(p+q)]
    Ldeltalambda <- L[(p+1):(p+q),r]
    Llambdadelta <- t(Ldeltalambda)

    # For influence on BETA
    sub_deltalambda <- rbind(
      cbind(Ldeltadelta, Ldeltalambda),
      cbind(Llambdadelta, Llambdalambda)
    )

    sub_deltalambda_inv <- solve(sub_deltalambda)
    B1 <- matrix(0, nrow = r, ncol = r)
    B1[(p+1):(r), (p+1):(r)] <- sub_deltalambda_inv

    # For influence on DELTA
    sub_betalambda <- rbind(
      cbind(Lbetabeta, Lbetalambda),
      cbind(Llambdabeta, Llambdalambda)
    )

    sub_betalambda_inv <- solve(sub_betalambda)
    B2 <- matrix(0, nrow = r, ncol = r)
    B2[c(1:p, r), c(1:p, r)] <- sub_betalambda_inv

    # For influence on THETA
    B3 <- matrix(0, nrow = r, ncol = r)

  } else {
    Dlink.mu <- fixed_mean_link_inv_deriv1(eta1, link_mu)
    deriv2_linkmu <- fixed_mean_link_deriv2(mu, link_mu)

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z)))

    Lbetabeta <- L[1:p,1:p]
    Ldeltadelta <- L[(p+1):r,(p+1):r]

    # For influence on BETA
    sub_delta_inv <- solve(Ldeltadelta)
    B1 <- matrix(0, nrow = r, ncol = r)
    B1[(p+1):r, (p+1):r] <- sub_delta_inv

    # For influence on DELTA
    sub_beta_inv <- solve(Lbetabeta)
    B2 <- matrix(0, nrow = r, ncol = r)
    B2[1:p, 1:p] <- sub_beta_inv

    # For influence on THETA
    B3 <- matrix(0, nrow = r, ncol = r)
  }

  TI <- diag(Dlink.mu, ncol=n, nrow = n)

  if(scheme=="case.weight")
  {
    # Case Weight Perturbation Scheme
    Deltamu <- t(X) %*% SIG %*% TI %*% UI %*% YDAG
    Deltasigma2 <- t(Z) %*% HI %*% AI
    if(parametric){
      Deltarho <- t(rho) %*% SIG %*% UI %*% YDAG
      Delta <- rbind(Deltamu, Deltasigma2, Deltarho)
    } else {
      Delta <- rbind(Deltamu, Deltasigma2)
    }
  } else {
    # Response Perturbation Scheme
    Deltamu <- t(X) %*% SIG %*% TI %*% B_1 %*% Sy
    Deltasigma2 <- t(Z) %*% VI %*% HI %*% B_2 %*% Sy
    if(parametric){
      Deltarho <- t(rho) %*% SIG %*% B_1 %*% Sy
      Delta <- rbind(Deltamu, Deltasigma2, Deltarho)
    } else{
      Delta <- rbind(Deltamu, Deltasigma2)
    }
  }

  L_inv <- solve(L)

  compute_block <- function(Bmat) {
    BX <- B(Delta, L_inv, Bmat)
    eig <- eigen(BX, symmetric = TRUE)
    list(
      dmax = abs(eig$vec[, 1]),
      Ci   = 2 * abs(diag(BX))
    )
  }

  res_theta <- compute_block(B3)
  res_beta  <- compute_block(B1)
  res_gamma <- compute_block(B2)

  result <- list(
    dmax.beta  = res_beta$dmax,
    dmax.gamma = res_gamma$dmax,
    dmax.theta = res_theta$dmax,
    Ci.beta    = res_beta$Ci,
    Ci.gamma   = res_gamma$Ci,
    Ci.theta   = res_theta$Ci
  )

  if (plot) {
    op <- par(no.readonly = TRUE)
    on.exit(par(op), add = TRUE)

    measure_name <- paste0(type, ".", parameter)
    values <- result[[measure_name]]

    ylab_expr <- if (type == "Ci") expression(C[i]) else expression(d[max])

    par(mar = c(3, 3, 2, 3), oma = c(0.5, 0.5, 0.5, 0.5), mgp = c(1.92, 0.6, 0))

    pt <- if (is.null(plot.type)) "h" else plot.type

    # Use string ylab to avoid expression serialization issues in do.call
    plot_args <- modifyList(
      list(type     = pt,
           xlab     = "Observation index",
           ylab     = ylab_expr,
           cex.lab  = 1.2,
           cex      = 1,
           cex.axis = 0.8,
           ylim     = c(0, max(values, na.rm = TRUE) * 1.05)),
      list(...)
    )

    do.call(graphics::plot, c(list(values), plot_args))

    if (!is.null(threshold)) {
      outliers <- which(values > threshold)
      if (length(outliers) > 0)
        text(outliers, values[outliers], labels = outliers,
             pos = label.pos, cex = 0.8, col = "red")
    }

    invisible(result)
  } else {
    result
  }
}


# ==============================================================================
# 2. GENERALIZED LEVERAGE
# ==============================================================================

#' @title Generalized Leverage Values for Simplex Regression Models
#' @description Compute the generalized leverage values for simplex regression
#' models with parametric or fixed mean link function.
#'
#' @param model An object of class \code{simplexregression}.
#'
#' @return A numeric vector of generalized leverage values.
#'
#' @details
#' \code{gleverage} computing generalized leverage values as suggested by Wei, Hu,
#' and Fung (1998). Generalized leverage extends the concept of hat values to account for both
#' mean and dispersion parameters. High leverage values indicate observations
#' that have potentially large influence on parameter estimates.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Compute generalized leverage
#' glev <- gleverage(fit)
#'
#' # Plot leverage values
#' plot(glev, type = "h", ylab = "Generalized leverage",
#'      xlab = "Observation index")
#' abline(h = 2 * mean(glev), lty = 2, col = "red")
#'
#' @seealso \code{\link{hatvalues.simplexregression}},
#' \code{\link{cooks.distance.simplexregression}},
#' \code{\link{local.influence}}.
#'
#' @references
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' Wei, B. C., Hu, Y. Q., and Fung, W. K. (1998).
#' Generalized Leverage and Its Applications.
#' \emph{Scandinavian Journal of Statistics}, \bold{25}, 25–37.
#'
#' @importFrom stats plogis
#' @export
gleverage <- function(model) {
  UseMethod("gleverage")
}

#' @rdname gleverage
#' @method gleverage simplexregression
#' @export
gleverage.simplexregression <- function(model){

  parametric <- is_parametric(model)

  n <- model$nobs
  y <- model$y
  mu <- model$mu.fv
  sigma2 <- model$sigma2.fv
  link_mu <- model$mu.link
  link_sigma2 <- model$sigma2.link
  eta1 <- model$mu.lp
  eta2 <- model$sigma2.lp
  X <- model$mu.x
  Z <- model$sigma2.x

  p <- ncol(X)
  q <- ncol(Z)
  r <- if (parametric) p + q + 1 else p + q

  diff <- y - mu
  yoneminy <- y * (1 - y)
  muonemu <- mu * (1 - mu)
  dev <- (diff / muonemu)^2 / yoneminy

  Ui <- (dev/muonemu) + (1/(muonemu^3))
  Vi <- 1/(2*sigma2^2)
  ai <- -1/(2*sigma2) + dev*Vi
  Mi <- 2 * ai / sigma2

  ci <- Ui + (diff / (yoneminy * muonemu)) * (dev + 2*diff / (y * mu * (1 - mu)^2))
  cidag <- (1 / yoneminy) * (dev + 2*diff / (y * mu * (1 - mu)^2))

  sigma2i <- 1/sigma2
  Dlink.sigma2 <- dispersion_link_inv_deriv1(eta2,  link_sigma2)

  deriv2_linksigma2 <- switch(link_sigma2,
                              log = -1 / sigma2^2,
                              sqrt = -1 / (4 * sigma2 * sqrt(sigma2)),
                              identity = 0)

  sdag <- ai * deriv2_linksigma2

  deriv2_dev <- - (2 / ((1 - y) * muonemu^3)) *
    (10*y - (3*(y^2 + mu^4) / (y*muonemu)) + (2*(1 + 6*mu^2 - 4*mu) / (1-mu)))
  dev2 <- deriv2_dev / 2

  if(parametric){
    lambda <- model$lambda.fv
    Dlink.mu <- parametric_mean_link_inv_deriv1(eta1, lambda, link_mu)

    if(link_mu == "plogit2") {
      exp_aval_frac <- plogis(eta1)^(1/lambda)
      log_aval <- log(plogis(eta1))
      rho <- as.vector(- exp_aval_frac * log_aval / (lambda^2))
      deriv2_linkmu <- as.vector((lambda * mu^lambda * (1+lambda) - lambda) /
                                   (mu^2 * (1-mu^lambda)^2))
      VARTHETA <- as.vector(- (1 / (lambda^3 * (1 + exp(eta1)))) * exp_aval_frac *
                              (lambda + log_aval))
      VARSIG <- as.vector(1/lambda^4 * exp_aval_frac * log_aval * (log_aval + 2*lambda))
    } else {
      log_aval <- log(1+exp(eta1))
      rho <- as.vector((-1/(lambda^2)) * ((1 + exp(eta1)) ^ (-1/lambda)) * log_aval)
      deriv2_linkmu <- as.vector(lambda * (1 - (1+lambda) * (1-mu)^lambda) /
                                   ((1-mu)^2 * (1 - (1-mu)^lambda)^2))
      VARTHETA <- as.vector(exp(eta1) * (1 + exp(eta1))^(-1 - 1/lambda) *
                              (log_aval - lambda) / (lambda^3))
      VARSIG <- as.vector((1 + exp(eta1))^(-1/lambda) * log_aval *
                            (2*lambda - log_aval) / lambda^4)
    }

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L2 <- crossprod(X, sigma2i * (dev2 * Dlink.mu * rho - diff * Ui * VARTHETA))
    L3 <- crossprod(Z, sigma2i^2 * diff * Ui * Dlink.sigma2 * rho)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1, L2),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z), L3),
      cbind(t(L2), t(L3), sum(sigma2i * (dev2 * rho^2 - diff * Ui * VARSIG)))
    )

    D <- cbind(Dlink.mu * X, matrix(0, nrow = n, ncol = q), rho)
    Lty <- t(cbind(sigma2i * Dlink.mu * ci * X, Vi * Dlink.sigma2 * cidag * Z,
                   sigma2i * ci * rho))

  } else {
    Dlink.mu <- fixed_mean_link_inv_deriv1(eta1, link_mu)
    deriv2_linkmu <- fixed_mean_link_deriv2(mu, link_mu)

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z))
    )

    D <- cbind(Dlink.mu * X, matrix(0, nrow = n, ncol = q))
    Lty <- t(cbind(sigma2i * Dlink.mu * ci * X, Vi * Dlink.sigma2 * cidag * Z))
  }

  L_inv <- solve(L)
  rowSums((D %*% L_inv) * t(Lty))
}

# ==============================================================================
# 3. s_{j,i} MEASURES
# ==============================================================================

#' @title Sample Influence Measures for Simplex Regression Models
#' @description Computes leave-one-out sample influence measures \eqn{s_{3,i}}
#' and \eqn{s_{5,i}} for simplex regression models, based on the
#' information-matrix-based criteria measures proposed by Cribari-Neto,
#' Vasconcellos and Santana e Silva (2025).
#'
#' @param model An object of class \code{simplexregression}.
#' @param data The data frame used to fit \code{model}.
#' @param type Character vector specifying measure(s): \code{"s3"}, \code{"s5"},
#'   or both (default).
#' @param interval Character string specifying the outlier detection threshold:
#'   \code{"I1"} (default, moderate) or \code{"I2"} (strict).
#' @param parameter Character string indicating the parameter block:
#'   \code{"theta"} (default, all parameters), \code{"beta"} (mean submodel),
#'   or \code{"gamma"} (dispersion submodel).
#' @param plot Logical; if \code{TRUE}, produces index plots of \eqn{s_{3,i}}
#'   and \eqn{s_{5,i}} with threshold lines and flagged-observation labels.
#'   Default is \code{FALSE}.
#' @param verbose Logical; if \code{TRUE} (default), prints progress during
#'   leave-one-out refitting. Ignored (set to \code{FALSE}) when
#'   \code{ncores > 1}, since output from parallel workers does not reach
#'   the main console.
#' @param ncores Positive integer specifying the number of CPU cores to use
#'   for the leave-one-out loop. Default is \code{1} (sequential). Values
#'   greater than \code{1} activate parallel computation via
#'   \code{\link[parallel]{parLapply}}. If \code{ncores} exceeds
#'   \code{parallel::detectCores() - 1}, it is silently clamped to that
#'   value to avoid overloading the system. A safe explicit choice is
#'   \code{parallel::detectCores() - 1}.
#' @param label.pos Position(s) for outlier labels in plot. Can be a single
#'   value (applied to all labels) or a vector. Values: 1 = below, 2 = left,
#'   3 = above, 4 = right.
#' @param plot.type Character string controlling the plot style when
#'   \code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} for
#'   \eqn{n \le 150} and \code{"p"} for \eqn{n > 150} (automatic). Passed
#'   to the \code{type} argument of \code{plot()}.
#' @param ... Additional graphical parameters passed to \code{plot()}.
#'
#' @return If \code{plot = FALSE} (default), a list containing only the
#' requested measures:
#' \describe{
#'   \item{\code{s3_i}}{(if requested) Numeric vector of \eqn{s_{3,i}} values.}
#'   \item{\code{s5_i}}{(if requested) Numeric vector of \eqn{s_{5,i}} values.}
#'   \item{\code{outliers_s3}}{(if requested) Data frame of flagged observations
#'     for \eqn{s_{3,i}}.}
#'   \item{\code{outliers_s5}}{(if requested) Data frame of flagged observations
#'     for \eqn{s_{5,i}}.}
#'   \item{\code{limits_s3}}{(if requested) Named vector with lower and upper
#'     thresholds for \eqn{s_{3,i}}.}
#'   \item{\code{limits_s5}}{(if requested) Named vector with lower and upper
#'     thresholds for \eqn{s_{5,i}}.}
#'   \item{\code{interval}}{Interval type used.}
#'   \item{\code{parameter}}{Parameter block used.}
#'   \item{\code{n}}{Number of observations.}
#' }
#' If \code{plot = TRUE}, the same list is returned invisibly.
#'
#' @details
#' For each observation \eqn{i}, the model is refit on the dataset with
#' observation \eqn{i} removed. Let \eqn{\hat\theta} and
#' \eqn{\hat\theta_{(i)}} denote the full and leave-one-out MLEs. Following
#' Cribari-Neto, Vasconcellos and Santana e Silva (2025), let \eqn{A_n}
#' denote the sample average of the second-order log-likelihood derivatives
#' (\eqn{n^{-1}\sum_i \partial^2\ell_i/\partial\theta\partial\theta'}) and
#' \eqn{B_n} denote the sample average of the outer products of the
#' individual score vectors
#' (\eqn{n^{-1}\sum_i (\partial\ell_i/\partial\theta)(\partial\ell_i/\partial\theta)'}),
#' the two matrices that comprise the information matrix equality. Let
#' \eqn{A^{(i)}_{n-1}} and \eqn{B^{(i)}_{n-1}} denote these same quantities
#' recomputed with observation \eqn{i} excluded (i.e. based on \eqn{n-1}
#' observations).
#'
#' \strong{Measures computed:}
#' \deqn{s_{3,i} = m_{3,(i)} / m_3}
#' \deqn{s_{5,i} = D_i^{\mathrm{mod}} - D_i^{\mathrm{gen}}}
#'
#' Here \eqn{m_3 = \|\mathrm{vech}(P_n^{-1} B_n P_n^{-\top} - I)\|_2}, where
#' \eqn{P_n} is obtained from the Cholesky decomposition of \eqn{-A_n}
#' (i.e. \eqn{-A_n = P_n P_n^\top}); \eqn{m_{3,(i)}} is the same quantity
#' recomputed with observation \eqn{i} excluded from the sample.
#'
#' \deqn{D_i^{\mathrm{gen}} = (n-1) \,
#'   (\hat\theta - \hat\theta_{(i)})^\top (-A^{(i)}_{n-1})
#'   (\hat\theta - \hat\theta_{(i)})}
#' \deqn{D_i^{\mathrm{mod}} = (n-1) \,
#'   (\hat\theta - \hat\theta_{(i)})^\top
#'   0.5(-A^{(i)}_{n-1} + B^{(i)}_{n-1})
#'   (\hat\theta - \hat\theta_{(i)})}
#'
#' If \eqn{-A^{(i)}_{n-1}} is not positive definite, \code{nearPD} is used to
#' find the nearest positive-definite matrix and a message is printed.
#'
#' \strong{Threshold intervals} use two asymmetric IQR spreads, as proposed
#' by Cribari-Neto, Vasconcellos and Santana e Silva (2025). Let
#' \eqn{Q(p)} denote the empirical \eqn{p}-th quantile of the relevant
#' measure (\eqn{s_{3,i}} or \eqn{s_{5,i}}, computed separately for each):
#' \eqn{IQR_1 = Q(0.50) - Q(0.125)} (left) and
#' \eqn{IQR_2 = Q(0.875) - Q(0.50)} (right).
#' Limits are \eqn{v - z \cdot IQR_1} (lower) and \eqn{v + z \cdot IQR_2}
#' (upper), with reference value \eqn{v = 1} for \eqn{s_3} and \eqn{v = 0}
#' for \eqn{s_5}:
#' \itemize{
#'   \item \code{I1} (moderate): \eqn{z = 2.5} for \eqn{s_3};
#'     \eqn{z = 4.0} for \eqn{s_5}.
#'   \item \code{I2} (strict): \eqn{z = 5.0} for \eqn{s_3};
#'     \eqn{z = 8.0} for \eqn{s_5}.
#' }
#'
#' \strong{Parallel computation:} when \code{ncores > 1}, the
#' leave-one-out loop is distributed across workers using
#' \code{\link[parallel]{parLapply}} from the \pkg{parallel} package
#' (included in base R). Each worker receives the necessary objects and
#' loads the \pkg{simplexregression} package. The random-number stream is
#' initialized with \code{\link[parallel]{clusterSetRNGStream}} to ensure
#' reproducibility across runs. Progress messages (\code{verbose}) are
#' suppressed in parallel mode because worker output does not reach the
#' main console.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' \donttest{
#' # Sequential (default)
#' im <- diag.im(fit, data = ReadingSkills, type = "s3", interval = "I1",
#'               parameter = "theta")
#'
#' # Produce index plots directly
#' diag.im(fit, data = ReadingSkills, type = "s3", interval = "I1",
#'         parameter = "theta", plot = TRUE)
#' }
#'
#' @references
#' Cribari-Neto, F.; Vasconcellos, K. L. P.; Santana e Silva, J. J. (2025).
#' New strategies for detecting atypical observations based on the information
#' matrix equality. \emph{Journal of Applied Statistics}, \bold{52},
#' 2873--2893. \doi{10.1080/02664763.2025.2487914}
#'
#' @seealso \code{\link{diag.distances}}, \code{\link{local.influence}},
#' \code{\link{gleverage}}.
#'
#' @importFrom stats quantile
#' @importFrom Matrix nearPD
#' @importFrom parallel makeCluster stopCluster clusterSetRNGStream
#'   clusterExport clusterEvalQ parLapply
#' @export
diag.im <- function(model, data, type = c("s3", "s5"), interval = c("I1", "I2"),
                    parameter = c("theta", "beta", "gamma"),
                    plot = FALSE, verbose = TRUE, ncores = 1,
                    label.pos = 3, plot.type = NULL, ...) {

  # --------------------------------------------------------------------
  # Input validation
  # --------------------------------------------------------------------
  if (!inherits(model, "simplexregression"))
    stop("'model' must be an object of class 'simplexregression'")

  type      <- match.arg(type, several.ok = TRUE)
  interval  <- match.arg(interval)
  parameter <- match.arg(parameter)

  if (!all(ncores == floor(ncores)) || ncores < 1)
    stop("'ncores' must be a positive integer >= 1")

  max_cores <- max(1L, parallel::detectCores() - 1L)
  if (ncores > max_cores) {
    warning(sprintf(
      "'ncores' (%d) exceeds detectCores() - 1 (%d); clamped to %d.",
      ncores, max_cores, max_cores))
    ncores <- max_cores
  }

  parametric <- is_parametric(model)

  dispersion_is_fixed <- ncol(model$sigma2.x) == 1L &&
    length(unique(round(model$sigma2.fv, 10))) == 1L

  if (parameter == "gamma" && dispersion_is_fixed)
    stop(
      "parameter = \"gamma\" is not applicable when the dispersion is fixed ",
      "(intercept-only dispersion submodel with a single estimated value). ",
      "Use parameter = \"beta\" or parameter = \"theta\" instead.",
      call. = FALSE
    )

  # --------------------------------------------------------------------
  # Z multipliers for thresholds
  # --------------------------------------------------------------------
  z_vals <- list(
    I1 = list(s3 = 2.5, s5 = 4.0),
    I2 = list(s3 = 5.0, s5 = 8.0)
  )

  n        <- model$nobs
  p        <- ncol(model$mu.x)
  q        <- ncol(model$sigma2.x)
  full_sj  <- compute_m3(model, parameter)
  m3_full  <- full_sj$v_m3

  # --------------------------------------------------------------------
  # Auxiliary: single LOO iteration
  # Defined here so it closes over the constants above when run
  # sequentially, and exported explicitly when run in parallel.
  # --------------------------------------------------------------------
  loo_one <- function(i) {

    data_i <- data[-i, , drop = FALSE]

    fit_i <- tryCatch(
      simplexreg(
        formula     = model$formula,
        link.mu     = model$mu.link,
        link.sigma2 = model$sigma2.link,
        data        = data_i
      ),
      error = function(e) {
        warning(sprintf("Refit failed for observation %d: %s",
                        i, conditionMessage(e)))
        NULL
      }
    )

    if (is.null(fit_i))
      return(list(s3 = NA_real_, s5 = NA_real_))

    # --- s3: ratio of m3 measures ---
    sj_i   <- compute_m3(fit_i, parameter = parameter)
    s3_val <- sj_i$v_m3 / m3_full

    # --- coefficient difference ---
    if (parametric) {
      coef_full <- unlist(model$coefficients)
      coef_loo  <- unlist(fit_i$coefficients)
    } else {
      coef_full <- unlist(model$coefficients[c("mean", "dispersion")])
      coef_loo  <- unlist(fit_i$coefficients[c("mean", "dispersion")])
    }

    theta_d <- switch(parameter,
                      beta  = coef_full[seq_len(p)]          - coef_loo[seq_len(p)],
                      gamma = coef_full[(p + 1):(p + q)]     - coef_loo[(p + 1):(p + q)],
                      coef_full                       - coef_loo
    )

    n_loo <- fit_i$nobs
    An_i <- sj_i$An_neg * n_loo
    Bn_i <- sj_i$Bn     * n_loo

    cook_gen <- as.numeric(t(theta_d) %*% An_i               %*% theta_d)
    cook_mod <- as.numeric(t(theta_d) %*% (0.5*(An_i + Bn_i)) %*% theta_d)

    list(s3 = s3_val, s5 = cook_mod - cook_gen)
  }

  # --------------------------------------------------------------------
  # Execute: sequential or parallel
  # --------------------------------------------------------------------
  if (ncores == 1L) {

    if (verbose) message("Running leave-one-out refits (sequential)...")

    raw <- lapply(seq_len(n), function(i) {
      if (verbose && (i == 1L || i %% 10L == 0L))
        message(sprintf("  Leave-one-out refit: observation %d / %d", i, n))
      loo_one(i)
    })

  } else {

    if (verbose)
      message(sprintf(
        "Running leave-one-out refits in parallel (%d cores)...", ncores))

    cl <- parallel::makeCluster(ncores)
    on.exit(parallel::stopCluster(cl), add = TRUE)

    parallel::clusterSetRNGStream(cl, iseed = 123)

    parallel::clusterExport(
      cl,
      varlist = c(
        "loo_one", "m3_full",
        "model", "data", "parameter", "parametric", "p", "q",
        "simplexreg", "compute_m3"
      ),
      envir = environment()
    )

    parallel::clusterEvalQ(cl, library(SimplexRegression))

    raw <- parallel::parLapply(cl, seq_len(n), function(i) loo_one(i))
  }

  # --------------------------------------------------------------------
  # Unpack results
  # --------------------------------------------------------------------
  s3_i <- vapply(raw, `[[`, numeric(1L), "s3")
  s5_i <- vapply(raw, `[[`, numeric(1L), "s5")

  # --------------------------------------------------------------------
  # Robust threshold computation
  # --------------------------------------------------------------------
  iqr_limits <- function(x, ref, z) {
    q <- unname(quantile(x, c(0.125, 0.5, 0.875), na.rm = TRUE))
    iqr_left  <- q[2L] - q[1L]
    iqr_right <- q[3L] - q[2L]
    c(lower = ref - z * iqr_left,
      upper = ref + z * iqr_right)
  }

  limits <- list(
    s3 = if ("s3" %in% type)
      iqr_limits(s3_i, ref = 1, z = z_vals[[interval]]$s3) else NULL,
    s5 = if ("s5" %in% type)
      iqr_limits(s5_i, ref = 0, z = z_vals[[interval]]$s5) else NULL
  )

  # Flag outliers
  outliers <- list()
  if ("s3" %in% type) {
    idx <- which(s3_i < limits$s3["lower"] | s3_i > limits$s3["upper"])
    outliers$s3 <- if (length(idx))
      data.frame(Obs = idx, s3_i = s3_i[idx])
    else
      data.frame(Obs = integer(0L), s3_i = numeric(0L))
  }
  if ("s5" %in% type) {
    idx <- which(s5_i < limits$s5["lower"] | s5_i > limits$s5["upper"])
    outliers$s5 <- if (length(idx))
      data.frame(Obs = idx, s5_i = s5_i[idx])
    else
      data.frame(Obs = integer(0L), s5_i = numeric(0L))
  }

  # --------------------------------------------------------------------
  # Assemble return value
  # --------------------------------------------------------------------
  result <- list(interval = interval, parameter = parameter, n = n)

  if ("s3" %in% type) {
    result$s3_i        <- s3_i
    result$outliers_s3 <- outliers$s3
    result$limits_s3   <- limits$s3
  }
  if ("s5" %in% type) {
    result$s5_i        <- s5_i
    result$outliers_s5 <- outliers$s5
    result$limits_s5   <- limits$s5
  }

  # --------------------------------------------------------------------
  # Plot
  # --------------------------------------------------------------------

  if (plot) {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par), add = TRUE)

    n_plots <- length(type)
    if (n_plots == 2L) par(mfrow = c(1L, 2L))

    pt <- if (is.null(plot.type)) (if (n > 150L) "p" else "h") else plot.type

    make_ylim <- function(vals, lims) {
      rng <- diff(range(vals, lims, na.rm = TRUE))
      pad <- rng * 0.08
      c(min(vals, lims, na.rm = TRUE) - pad * 0.4,
        max(vals, lims, na.rm = TRUE) + pad)
    }

    label_obs <- function(vals, lims, lpos) {
      idx <- which(vals < lims[1L] | vals > lims[2L])
      for (k in seq_along(idx)) {
        ii  <- idx[k]
        pos <- lpos[((k - 1L) %% length(lpos)) + 1L]
        text(ii, vals[ii], labels = ii, pos = pos,
             cex = 0.8, col = "red", offset = 0.3)
      }
    }

    make_plot <- function(vals, lims, ylab_expr) {
      par(mar = c(3, 3, 2, 3), oma = c(0.5, 0.5, 0.5, 0.5),
          mgp = c(2, 0.6, 0))
      plot_args <- modifyList(
        list(type = pt, pch = 1, xlab = "Observation index",
             ylab = ylab_expr, cex.lab = 1.2, cex = 1, cex.axis = 0.8,
             ylim = make_ylim(vals, lims)),
        list(...)
      )
      do.call(graphics::plot, c(list(vals), plot_args))
      graphics::abline(h = lims, lty = 2, col = "gray60")
      label_obs(vals, lims, label.pos)
    }

    if ("s3" %in% type)
      make_plot(result$s3_i, result$limits_s3, expression(s[3 * i]))
    if ("s5" %in% type)
      make_plot(result$s5_i, result$limits_s5, expression(s[5 * i]))

    invisible(result)

  } else {
    result
  }
}

#' @keywords internal
compute_m3 <- function(model, parameter = c("theta", "beta", "gamma")) {

  parametric <- is_parametric(model)

  n <- model$nobs
  y <- model$y
  mu <- model$mu.fv
  sigma2 <- model$sigma2.fv

  X <- model$mu.x
  Z <- model$sigma2.x

  p <- ncol(X)
  q <- ncol(Z)

  eta1 <- model$mu.lp
  eta2 <- model$sigma2.lp

  link_mu <- model$mu.link
  link_sigma2 <- model$sigma2.link

  diff <- y - mu
  yoneminy <- y * (1 - y)
  muonemu <- mu * (1 - mu)
  dev <- (diff / muonemu)^2 / yoneminy

  Ui <- (dev/muonemu) + (1/(muonemu^3))
  Vi <- 1/(2*sigma2^2)
  ai <- -1/(2*sigma2) + dev*Vi
  Mi <- 2 * ai / sigma2

  sigma2i <- 1/sigma2
  Dlink.sigma2 <- dispersion_link_inv_deriv1(eta2,  link_sigma2)

  deriv2_linksigma2 <- switch(link_sigma2,
                              log = -1 / sigma2^2,
                              sqrt = -1 / (4 * sigma2 * sqrt(sigma2)),
                              identity = 0)

  sdag <- ai * deriv2_linksigma2

  deriv2_dev <- - (2 / ((1 - y) * muonemu^3)) *
    (10*y - (3*(y^2 + mu^4) / (y*muonemu)) + (2*(1 + 6*mu^2 - 4*mu) / (1-mu)))
  dev2 <- deriv2_dev / 2

  # Parametric vs fixed link mean
  if(parametric){
    lambda <- model$lambda.fv
    Dlink.mu <- parametric_mean_link_inv_deriv1(eta1, lambda, link_mu)

    if(link_mu == "plogit2") {
      exp_aval_frac <- plogis(eta1)^(1/lambda)
      log_aval <- log(plogis(eta1))
      rho <- as.vector(- exp_aval_frac * log_aval / (lambda^2))
      deriv2_linkmu <- as.vector((lambda * mu^lambda * (1+lambda) - lambda) /
                                   (mu^2 * (1-mu^lambda)^2))
      VARTHETA <- as.vector(- (1 / (lambda^3 * (1 + exp(eta1)))) * exp_aval_frac *
                              (lambda + log_aval))
      VARSIG <- as.vector(1/lambda^4 * exp_aval_frac * log_aval * (log_aval + 2*lambda))
    } else {
      log_aval <- log(1+exp(eta1))
      rho <- as.vector((-1/(lambda^2)) * ((1 + exp(eta1)) ^ (-1/lambda)) * log_aval)
      deriv2_linkmu <- as.vector(lambda * (1 - (1+lambda) * (1-mu)^lambda) /
                                   ((1-mu)^2 * (1 - (1-mu)^lambda)^2))
      VARTHETA <- as.vector(exp(eta1) * (1 + exp(eta1))^(-1 - 1/lambda) *
                              (log_aval - lambda) / (lambda^3))
      VARSIG <- as.vector((1 + exp(eta1))^(-1/lambda) * log_aval *
                            (2*lambda - log_aval) / lambda^4)
    }

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L2 <- crossprod(X, sigma2i * (dev2 * Dlink.mu * rho - diff * Ui * VARTHETA))
    L3 <- crossprod(Z, sigma2i^2 * diff * Ui * Dlink.sigma2 * rho)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1, L2),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z), L3),
      cbind(t(L2), t(L3), sum(sigma2i * (dev2 * rho^2 - diff * Ui * VARSIG)))
    )

    M <- cbind(
      sigma2i * diff * Ui * Dlink.mu * X,
      ai * Dlink.sigma2 * Z,
      sigma2i * Ui * diff * rho
    )

  } else {
    Dlink.mu <- fixed_mean_link_inv_deriv1(eta1, link_mu)
    deriv2_linkmu <- fixed_mean_link_deriv2(mu, link_mu)

    L1 <- crossprod(X, sigma2i^2 * diff * Ui * Dlink.mu * Dlink.sigma2 * Z)
    L <- rbind(
      cbind(crossprod(X, sigma2i * (dev2 + diff * Ui * Dlink.mu * deriv2_linkmu) *
                        Dlink.mu^2 * X), L1),
      cbind(t(L1), crossprod(Z, (Vi + Mi + sdag * Dlink.sigma2) * Dlink.sigma2^2 * Z)))

    M <- cbind(
      sigma2i * diff * Ui * Dlink.mu * X,
      ai * Dlink.sigma2 * Z
    )
  }

  Bn <- crossprod(M) / n
  An <- - L/n
  An_neg <- -An

  L_complete <- L
  Bn_complete <- Bn * n

  # Select sub-block according to 'parameter'
  if(parameter == "theta"){
    Bn <- Bn
    An <- An
    An_neg <- An_neg
    r <- if (parametric) p + q + 1 else p + q
  } else if (parameter == "beta") {
    Bn <- Bn[1:p, 1:p]
    An <- An[1:p, 1:p]
    An_neg <- An_neg[1:p, 1:p]
    r <- p
  } else {
    Bn <- Bn[(p+1):(p+q), (p+1):(p+q)]
    An <- An[(p+1):(p+q), (p+1):(p+q)]
    An_neg <- An_neg[(p+1):(p+q), (p+1):(p+q)]
    r <- q
  }

  eigenvalues <- eigen(An_neg, only.values = TRUE)$values

  if (min(eigenvalues) <= 1e-10) {
    message("The matrix -A_n is not positive definite. Using nearPD approximation.")
    An_neg = as.matrix(Matrix::nearPD(An_neg)$mat)
  }

  Pn <- t(chol(An_neg))
  Pn_inv <- solve(Pn)

  C3n <- Pn_inv %*% Bn %*% t(Pn_inv) - diag(1, nrow = r, ncol = r)
  v_m3 <- norm(C3n[lower.tri(C3n, diag = TRUE)], type = "2")

  return(list(v_m3 = v_m3, An_neg =  An_neg, Bn =  Bn))
}

# ==============================================================================
# 4. HELLINGER AND WASSERSTEIN DISTANCES
# ==============================================================================

#' @title Distance-Based Influence Diagnostics for Simplex Regression
#' @description Computes leave-one-out influence measures based on
#' distributional distances (Wasserstein with \eqn{p_W = 1} and \eqn{p_W = 2} or
#' Hellinger) for simplex regression models, as proposed by Justino and
#' Cribari-Neto (2026).
#'
#' @param model An object of class \code{simplexregression}.
#' @param data The data frame used to fit \code{model}.
#' @param type Character string or integer specifying the distance measure:
#'   \code{"W1"} (default, Wasserstein with \eqn{p_W = 1}),
#'   \code{"W2"} (Wasserstein with \eqn{p_W = 2}), \code{"H"} (Hellinger).
#' @param plot Logical; if \code{FALSE} (default), returns the numeric vector
#'   of distances. If \code{TRUE}, produces an index plot with the ad hoc
#'   threshold and flagged-observation labels.
#' @param verbose Logical; if \code{TRUE} (default), prints progress during
#'   leave-one-out refitting. Ignored when \code{ncores > 1}, since output
#'   from parallel workers does not reach the main console.
#' @param ncores Positive integer specifying the number of CPU cores to use
#'   for the leave-one-out loop. Default is \code{1} (sequential). Values
#'   greater than \code{1} activate parallel computation via
#'   \code{\link[parallel]{parLapply}}. If \code{ncores} exceeds
#'   \code{parallel::detectCores() - 1}, it is clamped to that value with a
#'   warning to avoid overloading the system. A safe explicit choice is
#'   \code{parallel::detectCores() - 1}.
#' @param label.pos Position(s) for outlier labels in the plot. Can be a
#'   single value (applied to all labels) or a vector. Values: 1 = below,
#'   2 = left, 3 = above, 4 = right.
#' @param plot.type Character string controlling the plot style when
#'   \code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} for
#'   \eqn{n \le 150} and \code{"p"} for \eqn{n > 150} (automatic). Passed
#'   to the \code{type} argument of \code{plot()}.
#' @param ... Additional graphical parameters passed to \code{plot()}.
#'
#' @return If \code{plot = FALSE}, a list containing:
#' \describe{
#'   \item{\code{distances}}{Numeric vector of length \eqn{n} with the
#'     leave-one-out distances.}
#'   \item{\code{threshold}}{Named numeric scalar with the ad hoc upper
#'     threshold.}
#'   \item{\code{outliers}}{Data frame of flagged observations (index and
#'     distance value). An empty data frame if no observations are flagged.}
#'   \item{\code{type}}{Distance type used (full label).}
#'   \item{\code{n}}{Number of observations.}
#' }
#' If \code{plot = TRUE}, the same list is returned invisibly.
#'
#' @details
#' For each observation \eqn{i}, the model is refit on the dataset with
#' observation \eqn{i} removed. The chosen distance between the full-model
#' and leave-one-out fitted distributions is then computed pointwise across
#' all \eqn{n} observations and summed to produce the scalar influence
#' measure \eqn{d_i}.
#'
#' \strong{Ad hoc threshold} uses an asymmetric IQR spread adjusted for
#' skewness, as proposed by Justino and Cribari-Neto (2026). Let \eqn{Q(p)} denote the
#' empirical \eqn{p}-th quantile of the distances \eqn{d_i}, the threshold is
#' \deqn{\text{threshold} = Q(0.75) + (1 + a)(Q(0.75) - Q(0.25))}
#' where \eqn{a} is the sample skewness of the distances. Observations with
#' \eqn{d_i} above this threshold are flagged as potentially influential.
#'
#' \strong{Warning:} for \eqn{n > 500}, the numerical integration used
#' internally by the distance functions may be slow. Consider using a subset
#' or a faster integration method.
#'
#' \strong{Parallel computation:} when \code{ncores > 1}, the
#' leave-one-out loop is distributed across workers using
#' \code{\link[parallel]{parLapply}} from the \pkg{parallel} package
#' (included in base R). Each worker receives the necessary objects and
#' loads the \pkg{simplexregression} package. The random-number stream is
#' initialized with \code{\link[parallel]{clusterSetRNGStream}} to ensure
#' reproducibility across runs. Progress messages (\code{verbose}) are
#' suppressed in parallel mode because worker output does not reach the
#' main console.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' \donttest{
#' # Sequential (default) — Wasserstein W1
#' dd <- diag.distances(fit, data = ReadingSkills, type = "W1")
#'
#' # Index plot with flagged observations
#' diag.distances(fit, data = ReadingSkills, type = "W1", plot = TRUE)
#'
#' # Hellinger distance
#' diag.distances(fit, data = ReadingSkills, type = "H", plot = TRUE)
#' }
#'
#'
#' @references
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Influence diagnostics in beta regression via Hellinger and Wasserstein
#' distances. \emph{Statistical Papers}, \bold{67}(3), 50.
#' \doi{10.1007/s00362-026-01823-0}
#'
#' @seealso \code{\link{diag.im}}, \code{\link{local.influence}},
#' \code{\link{gleverage}}.
#'
#' @importFrom pracma quadgk
#' @importFrom parallel makeCluster stopCluster clusterSetRNGStream
#'   clusterExport clusterEvalQ parLapply detectCores
#' @export
diag.distances <- function(model, data, type = c("W1", "W2", "H"),
                           plot = FALSE, verbose = TRUE, ncores = 1,
                           label.pos = 3, plot.type = NULL, ...) {

  # --------------------------------------------------------------------
  # Input validation
  # --------------------------------------------------------------------
  if (!inherits(model, "simplexregression"))
    stop("'model' must be an object of class 'simplexregression'")

  type <- match.arg(type)

  if (!all(ncores == floor(ncores)) || ncores < 1)
    stop("'ncores' must be a positive integer >= 1")

  max_cores <- max(1L, parallel::detectCores() - 1L)
  if (ncores > max_cores) {
    warning(sprintf(
      "'ncores' (%d) exceeds detectCores() - 1 (%d); clamped to %d.",
      ncores, max_cores, max_cores))
    ncores <- max_cores
  }

  n <- model$nobs

  if (n > 500)
    warning(sprintf(paste0(
      "n = %d > 500: numerical integration used by the distance ",
      "functions may be slow for large samples."), n),
      call. = FALSE)

  # --------------------------------------------------------------------
  # Full-model predictions (computed once, shared across all iterations)
  # --------------------------------------------------------------------
  mu_full     <- predict.simplexregression(model, newdata = data,
                                           type = "response")
  sigma2_full <- predict.simplexregression(model, newdata = data,
                                           type = "dispersion")

  # --------------------------------------------------------------------
  # Distance function selector
  # --------------------------------------------------------------------
  dist_fun <- switch(type,
                     W1 = function(mf, sf, mr, sr)
                       wasserstein_simplex(mf, sf, mr, sr,
                                           p_W = 1)$sum,
                     W2 = function(mf, sf, mr, sr)
                       wasserstein_simplex(mf, sf, mr, sr, p_W = 2)$sum,
                     H  = function(mf, sf, mr, sr)
                       hellinger_simplex(mf, sf, mr, sr)$sum
  )

  # --------------------------------------------------------------------
  # Auxiliary: single LOO iteration
  # --------------------------------------------------------------------
  loo_one <- function(i) {

    fit_i <- tryCatch(
      simplexreg(
        formula     = model$formula,
        link.mu     = model$mu.link,
        link.sigma2 = model$sigma2.link,
        data        = data[-i, , drop = FALSE]
      ),
      error = function(e) {
        warning(sprintf("Refit failed for observation %d: %s",
                        i, conditionMessage(e)), call. = FALSE)
        NULL
      }
    )

    if (is.null(fit_i)) return(NA_real_)

    mu_red     <- predict.simplexregression(fit_i, newdata = data,
                                            type = "response")
    sigma2_red <- predict.simplexregression(fit_i, newdata = data,
                                            type = "dispersion")

    dist_fun(mu_full, sigma2_full, mu_red, sigma2_red)
  }

  # --------------------------------------------------------------------
  # Execute: sequential or parallel
  # --------------------------------------------------------------------
  if (ncores == 1L) {

    if (verbose) message("Running leave-one-out refits (sequential)...")

    distances <- vapply(seq_len(n), function(i) {
      if (verbose && (i == 1L || i %% 10L == 0L))
        message(sprintf("  Leave-one-out refit: observation %d / %d", i, n))
      loo_one(i)
    }, numeric(1L))

  } else {

    if (verbose)
      message(sprintf(
        "Running leave-one-out refits in parallel (%d cores)...", ncores))

    cl <- parallel::makeCluster(ncores)
    on.exit(parallel::stopCluster(cl), add = TRUE)

    parallel::clusterSetRNGStream(cl, iseed = 123)

    parallel::clusterExport(
      cl,
      varlist = c(
        "loo_one", "mu_full", "sigma2_full", "dist_fun",
        "model", "data",
        "simplexreg", "predict.simplexregression",
        "wasserstein_simplex", "hellinger_simplex"
      ),
      envir = environment()
    )

    parallel::clusterEvalQ(cl, library(SimplexRegression))

    distances <- unlist(
      parallel::parLapply(cl, seq_len(n), function(i) loo_one(i))
    )
  }

  na_idx <- which(is.na(distances))
  if (length(na_idx) > 0L) {
    warning(sprintf(
      paste0("%d observation(s) produced NA distances due to numerical ",
             "integration failure (obs: %s). They are excluded from the ",
             "threshold computation but kept as NA in the output."),
      length(na_idx),
      paste(na_idx, collapse = ", ")
    ), call. = FALSE)
  }

  # --------------------------------------------------------------------
  # Ad hoc threshold (asymmetric IQR adjusted for skewness)
  # --------------------------------------------------------------------
  ad_hoc_threshold <- function(x) {
    q       <- quantile(x, c(0.25, 0.75), na.rm = TRUE)
    iqr_right <- q[2L] - q[1L]
    unname(q[2L] + (1 + moments::skewness(x)) * iqr_right)
  }

  thr <- ad_hoc_threshold(distances)

  idx      <- which(distances > thr)
  outliers <- if (length(idx))
    data.frame(Obs = idx, distance = distances[idx])
  else
    data.frame(Obs = integer(0L), distance = numeric(0L))

  # --------------------------------------------------------------------
  # Assemble result
  # --------------------------------------------------------------------
  type_label <- unname(c(W1 = "Wasserstein-1",
                         W2 = "Wasserstein-2",
                         H = "Hellinger")[type])

  result <- list(
    distances = distances,
    threshold = c(upper = thr),
    outliers  = outliers,
    type      = type_label,
    n         = n
  )

  # --------------------------------------------------------------------
  # Plot
  # --------------------------------------------------------------------

  if (plot) {
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par), add = TRUE)

    ylab_expr <- switch(type,
                        W1 = "Wasserstein (p_W = 1) distance",
                        W2 = "Wasserstein (p_W = 2) distance",
                        H  = "Hellinger distance"
    )

    pt <- if (is.null(plot.type)) (if (n > 150L) "p" else "h") else plot.type

    y_max <- max(distances, na.rm = TRUE) *
      if (nrow(outliers) > 0L) 1.12 else 1.05

    par(mar = c(3, 3, 2, 3), oma = c(0.5, 0.5, 0.5, 0.5),
        mgp = c(2, 0.6, 0))

    plot_args <- modifyList(
      list(type = pt, pch = 1, xlab = "Observation index",
           ylab = ylab_expr, cex = 1, cex.lab = 1.2, cex.axis = 0.8,
           ylim = c(0, y_max)),
      list(...)
    )
    do.call(graphics::plot, c(list(distances), plot_args))
    graphics::abline(h = thr, lty = 2, col = "gray60", lwd = 1.5)

    if (nrow(outliers) > 0L) {
      for (k in seq_len(nrow(outliers))) {
        ii  <- outliers$Obs[k]
        pos <- label.pos[((k - 1L) %% length(label.pos)) + 1L]
        text(ii, distances[ii], labels = ii, pos = pos,
             cex = 0.8, col = "red", offset = 0.3)
      }
    }

    invisible(result)

  } else {
    result
  }
}

#' @keywords internal
hellinger_simplex <- function(mu1, sig1, mu2, sig2) {

  rho_simplex <- function(mu1, sig1, mu2, sig2) {
    integrand <- function(y) {
      sqrt(dsimplex(y, mu1, sig1) * dsimplex(y, mu2, sig2))
    }
    pracma::quadgk(integrand, 0, 1, tol = 1e-10)
  }

  n <- length(mu1)

  rho <- sapply(seq_len(n), function(i) {
    rho_simplex(mu1[i], sig1[i], mu2[i], sig2[i])
  })

  distances <- sqrt(1 - rho)

  return(list(distances = distances, sum = sum(distances)))
}

#' @keywords internal
wasserstein_simplex_p_W1 <- function(mu1, sig1, mu2, sig2, lower = 0, upper = 1) {
  integrand <- function(y) {
    abs(psimplex(y, mu1, sig1) - psimplex(y, mu2, sig2))
  }

  pracma::quadgk(integrand, lower, upper, tol = 1e-10)
}

#' @keywords internal
wasserstein_simplex_general <- function(mu1, sig1, mu2, sig2, p_W = 2) {
  integrand <- function(q) {
    abs(qsimplex(q, mu1, sig1) - qsimplex(q, mu2, sig2))^p_W
  }

  integral_result <- pracma::quadgk(integrand, 0, 1, tol = 1e-10)
  return(integral_result^(1/p_W))
}

#' @keywords internal
wasserstein_simplex <- function(mu1, sig1, mu2, sig2, p_W = 1) {
  if (p_W < 1) stop("p_W must be >= 1 to be a valid distance")

  n <- length(mu1)

  if (p_W == 1) {
    distances <- vapply(seq_len(n), function(i) {
      wasserstein_simplex_p_W1(mu1[i], sig1[i], mu2[i], sig2[i])
    }, numeric(1L))
  } else {
    distances <- vapply(seq_len(n), function(i) {
      wasserstein_simplex_general(mu1[i], sig1[i], mu2[i], sig2[i], p_W = p_W)
    }, numeric(1L))
  }

  list(distances = distances, sum = sum(distances), p_W = p_W)
}

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_influence.R
# Tests for: local.influence, gleverage, diag.im, diag.distances
# ==============================================================================

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 50
x1 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- fixed_mean_link_inv(1 - 1.5*x1, "loglog")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, z1 = z1)

fit_loglog <- simplexreg(y ~ x1 | z1, data = data, link.mu = "loglog")
fit_logit  <- simplexreg(y ~ x1 | z1, data = data, link.mu = "logit")

# ------------------------------------------------------------------------------
# diag.im fixtures: computed once, reused across all diag.im tests
# Each unique (type x parameter x interval) combination = one LOO run
# ------------------------------------------------------------------------------
im_s3        <- diag.im(fit_loglog, data = data, type = "s3",
                        parameter = "theta", interval = "I1", verbose = FALSE)
im_s5        <- diag.im(fit_loglog, data = data, type = "s5",
                        parameter = "theta", interval = "I1", verbose = FALSE)
im_both      <- diag.im(fit_loglog, data = data, type = c("s3", "s5"),
                        parameter = "theta", interval = "I1", verbose = FALSE)
im_s3_beta   <- diag.im(fit_loglog, data = data, type = "s3",
                        parameter = "beta",  interval = "I1", verbose = FALSE)
im_s3_gamma  <- diag.im(fit_loglog, data = data, type = "s3",
                        parameter = "gamma", interval = "I1", verbose = FALSE)

# ==============================================================================
# 1. local.influence
# ==============================================================================

test_that("local.influence requires an object of class simplexregression", {
  expect_error(local.influence(list()),   "class 'simplexregression'")
  expect_error(local.influence("model"),  "class 'simplexregression'")
})

test_that("local.influence returns a list with all named components", {
  result <- local.influence(fit_loglog, scheme = "case.weight")
  expect_type(result, "list")
  expect_named(result,
               c("dmax.beta", "dmax.gamma", "dmax.theta",
                 "Ci.beta",   "Ci.gamma",   "Ci.theta"),
               ignore.order = TRUE)
})

test_that("local.influence all components have length n", {
  result <- local.influence(fit_loglog, scheme = "case.weight")
  for (nm in names(result)) {
    expect_length(result[[nm]], n)
  }
})

test_that("local.influence all components are finite numeric vectors", {
  result <- local.influence(fit_loglog, scheme = "case.weight")
  for (nm in names(result)) {
    expect_true(is.numeric(result[[nm]]),
                label = paste(nm, "is numeric"))
    expect_true(all(is.finite(result[[nm]])),
                label = paste(nm, "all finite"))
  }
})

test_that("local.influence Ci components are non-negative", {
  result <- local.influence(fit_loglog, scheme = "case.weight")
  expect_true(all(result$Ci.beta  >= 0))
  expect_true(all(result$Ci.gamma >= 0))
  expect_true(all(result$Ci.theta >= 0))
})

test_that("local.influence dmax components are non-negative", {
  result <- local.influence(fit_loglog, scheme = "case.weight")
  expect_true(all(result$dmax.beta  >= 0))
  expect_true(all(result$dmax.gamma >= 0))
  expect_true(all(result$dmax.theta >= 0))
})

test_that("local.influence works with response perturbation scheme", {
  result <- local.influence(fit_loglog, scheme = "response")
  expect_type(result, "list")
  expect_true(all(is.finite(result$Ci.theta)))
})

test_that("local.influence case.weight and response give different results", {
  cw <- local.influence(fit_loglog, scheme = "case.weight")
  rp <- local.influence(fit_loglog, scheme = "response")
  expect_false(identical(cw$Ci.theta, rp$Ci.theta))
})

test_that("local.influence plot = TRUE returns result invisibly", {
  pdf(NULL)
  result <- withVisible(
    local.influence(fit_loglog, scheme = "case.weight",
                    parameter = "beta", type = "Ci", plot = TRUE)
  )
  dev.off()
  expect_false(result$visible)
  expect_type(result$value, "list")
})

test_that("local.influence plot = TRUE runs without error for all parameter/type combos", {
  params <- c("theta", "beta", "gamma")
  types  <- c("Ci", "dmax")
  for (par in params) {
    for (tp in types) {
      expect_no_error({
        pdf(NULL)
        local.influence(fit_loglog, scheme = "case.weight",
                        parameter = par, type = tp, plot = TRUE)
        dev.off()
      })
    }
  }
})

test_that("local.influence plot with threshold runs without error", {
  expect_no_error({
    pdf(NULL)
    local.influence(fit_logit, scheme = "case.weight",
                    parameter = "beta", type = "Ci",
                    plot = TRUE, threshold = 0.5)
    dev.off()
  })
})

test_that("local.influence both schemes give consistent structure", {
  cw <- local.influence(fit_logit, scheme = "case.weight")
  rp <- local.influence(fit_logit, scheme = "response")
  expect_equal(names(cw), names(rp))
})

# ==============================================================================
# 2. gleverage
# ==============================================================================

test_that("gleverage.simplexregression returns numeric vector of length n", {
  glev <- gleverage(fit_loglog)
  expect_true(is.numeric(glev))
  expect_length(glev, n)
})

test_that("gleverage returns finite values", {
  glev <- gleverage(fit_loglog)
  expect_true(all(is.finite(glev)))
})

test_that("gleverage values are non-negative", {
  glev <- gleverage(fit_loglog)
  expect_true(all(glev >= 0))
})

test_that("gleverage gives same result on repeated calls (deterministic)", {
  expect_equal(gleverage(fit_loglog), gleverage(fit_loglog))
})

# ==============================================================================
# 3. diag.im
# Fixtures im_s3, im_s5, im_both, im_s3_I2, im_s3_beta, im_s3_gamma
# are pre-computed above — no LOO runs happen inside these tests.
# ==============================================================================

test_that("diag.im requires an object of class simplexregression", {
  expect_error(diag.im(list(),  data = data), "class 'simplexregression'")
  expect_error(diag.im("model", data = data), "class 'simplexregression'")
})

test_that("diag.im rejects invalid ncores", {
  expect_error(diag.im(fit_logit, data = data, ncores = 0),   "positive integer")
  expect_error(diag.im(fit_logit, data = data, ncores = -1),  "positive integer")
  expect_error(diag.im(fit_logit, data = data, ncores = 1.5), "positive integer")
})

test_that("diag.im parameter = 'gamma' errors for intercept-only dispersion", {
  fit_nodispers <- simplexreg(y ~ x1, data = data, link.mu = "logit")
  expect_error(
    diag.im(fit_nodispers, data = data, type = "s3", parameter = "gamma",
            verbose = FALSE),
    "gamma"
  )
})

test_that("diag.im returns list with s3 components when type = 's3'", {
  expect_type(im_s3, "list")
  expect_true("s3_i"        %in% names(im_s3))
  expect_true("limits_s3"   %in% names(im_s3))
  expect_true("outliers_s3" %in% names(im_s3))
  expect_false("s5_i" %in% names(im_s3))
})

test_that("diag.im returns list with s5 components when type = 's5'", {
  expect_type(im_s5, "list")
  expect_true("s5_i"        %in% names(im_s5))
  expect_true("limits_s5"   %in% names(im_s5))
  expect_true("outliers_s5" %in% names(im_s5))
  expect_false("s3_i" %in% names(im_s5))
})

test_that("diag.im returns both s3 and s5 when both requested", {
  expect_true("s3_i" %in% names(im_both))
  expect_true("s5_i" %in% names(im_both))
})

test_that("diag.im s3_i has length n and is finite numeric", {
  expect_length(im_s3$s3_i, n)
  expect_true(is.numeric(im_s3$s3_i))
  expect_true(all(is.finite(im_s3$s3_i)))
})

test_that("diag.im s5_i has length n and is finite numeric", {
  expect_length(im_s5$s5_i, n)
  expect_true(is.numeric(im_s5$s5_i))
  expect_true(all(is.finite(im_s5$s5_i)))
})

test_that("diag.im s3_i values are positive", {
  expect_true(all(im_s3$s3_i > 0))
})

test_that("diag.im limits_s3 has lower and upper named elements", {
  expect_named(im_s3$limits_s3, c("lower", "upper"), ignore.order = TRUE)
  expect_true(im_s3$limits_s3["lower"] < im_s3$limits_s3["upper"])
})

test_that("diag.im outliers_s3 is a data.frame", {
  expect_s3_class(im_s3$outliers_s3, "data.frame")
})

test_that("diag.im outliers_s3 obs indices are within 1:n", {
  if (nrow(im_s3$outliers_s3) > 0)
    expect_true(all(im_s3$outliers_s3$Obs %in% seq_len(n)))
})

test_that("diag.im stores interval, parameter, and n in result", {
  expect_equal(im_s3$interval,  "I1")
  expect_equal(im_s3$parameter, "theta")
  expect_equal(im_s3$n,         n)
})

test_that("diag.im parameter = 'beta' returns finite s3_i", {
  expect_true(all(is.finite(im_s3_beta$s3_i)))
})

test_that("diag.im parameter = 'gamma' returns finite s3_i", {
  expect_true(all(is.finite(im_s3_gamma$s3_i)))
})

test_that("diag.im plot = TRUE on pre-computed result returns invisibly and plots without error", {
  pdf(NULL)
  # Re-run with plot = TRUE reusing same config as im_both fixture
  result <- withVisible(
    diag.im(fit_logit, data = data, type = c("s3", "s5"),
            parameter = "theta", verbose = FALSE, plot = TRUE)
  )
  dev.off()
  expect_false(result$visible)
  expect_type(result$value, "list")
})

# ==============================================================================
# 4. diag.distances
# H is the fastest type — used as main fixture for all structural tests.
# W1 and W2 get minimal smoke tests (type label only) to avoid LOO + quadgk
# overhead from slower numerical integration.
# ==============================================================================

dd_H <- diag.distances(fit_logit, data = data, type = "H", verbose = FALSE)
dd_W1 <- diag.distances(fit_logit, data = data, type = "W1", verbose = FALSE)

test_that("diag.distances requires an object of class simplexregression", {
  expect_error(diag.distances(list(),  data = data), "class 'simplexregression'")
  expect_error(diag.distances("model", data = data), "class 'simplexregression'")
})

test_that("diag.distances rejects invalid ncores", {
  expect_error(diag.distances(fit_logit, data = data, ncores = 0),   "positive integer")
  expect_error(diag.distances(fit_logit, data = data, ncores = -1),  "positive integer")
  expect_error(diag.distances(fit_logit, data = data, ncores = 1.5), "positive integer")
})

test_that("diag.distances rejects invalid type string", {
  expect_error(diag.distances(fit_logit, data = data, type = "X", verbose = FALSE))
})

test_that("diag.distances returns a list with required components", {
  expect_type(dd_H, "list")
  expect_named(dd_H, c("distances", "threshold", "outliers", "type", "n"),
               ignore.order = TRUE)
})

test_that("diag.distances distances vector has length n", {
  expect_length(dd_H$distances, n)
})

test_that("diag.distances distances are non-negative and finite", {
  expect_true(all(dd_H$distances >= 0))
  expect_true(all(is.finite(dd_H$distances)))
})

test_that("diag.distances threshold is a positive named scalar", {
  expect_length(dd_H$threshold, 1L)
  expect_named(dd_H$threshold, "upper")
  expect_true(dd_H$threshold > 0)
})

test_that("diag.distances outliers is a data.frame", {
  expect_s3_class(dd_H$outliers, "data.frame")
})

test_that("diag.distances flagged observations exceed threshold", {
  if (nrow(dd_H$outliers) > 0) {
    expect_true(all(dd_H$outliers$Obs %in% seq_len(n)))
    expect_true(all(dd_H$outliers$distance > dd_H$threshold["upper"]))
  }
})

test_that("diag.distances n field equals number of observations", {
  expect_equal(dd_H$n, n)
})

test_that("diag.distances type label is correct for H", {
  expect_equal(dd_H$type, "Hellinger")
})

test_that("diag.distances plot = TRUE returns list invisibly", {
  pdf(NULL)
  result <- withVisible(
    diag.distances(fit_logit, data = data, type = "H",
                   verbose = FALSE, plot = TRUE)
  )
  dev.off()
  expect_false(result$visible)
  expect_type(result$value, "list")
})

test_that("diag.distances W1 returns correct type label", {
  expect_equal(dd_W1$type, "Wasserstein-1")
})

test_that("diag.distances W1 distances are non-negative and finite", {
  expect_true(all(dd_W1$distances >= 0))
  expect_true(all(is.finite(dd_W1$distances)))
})

test_that("diag.distances H and W1 give different distances", {
  expect_false(identical(dd_H$distances, dd_W1$distances))
})

################################################################################
#               SIMPLEX REGRESSION - PRESS AND P2                              #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: PRESS-Based P2 Statistics for Simplex Regression                #
################################################################################

#' @title PRESS-Based \eqn{P2} Statistics for Simplex Regression
#' @description Computes the PRESS (Predicted Residual Error Sum of Squares)
#' statistic and the associated \eqn{P^2} and adjusted \eqn{P^2} measures for simplex
#' regression models with parametric or fixed mean link function, as proposed by
#' Espinheira and Silva (2026).
#'
#' @param ... One or more objects of class \code{simplexregression}.
#' @param type Character string specifying the type of residual to use.
#' Options are \code{"standardized"} (default) or \code{"biasvariance"}
#' (see \code{\link{residuals.simplexregression}}).
#'
#' @details
#' The PRESS statistic for the simplex regression model is given by:
#' \deqn{\text{PRESS} = \sum_{i=1}^{n} \left(\frac{r_i}{1 - h_{ii}}\right)^2,}
#' where \eqn{r_i} denotes the residual for observation \eqn{i} and
#' \eqn{h_{ii}} is the \eqn{i}-th diagonal element of the hat matrix.
#'
#' The \eqn{P^2} statistic is a cross-validation analog of \eqn{R^2}, defined
#' for the simplex regression mode as:
#' \deqn{P^2 = 1 - \frac{\text{PRESS}}{\left(\frac{n}{n-r}\right)^2 \text{SST}},}
#' where \eqn{\text{SST} = \sum_{i=1}^{n}(\hat{y}_i - \bar{\hat{y}})^2} and
#' \eqn{r} is the number of estimated parameters.
#'
#' The adjusted \eqn{P^2} is given by:
#' \deqn{P^2_c = 1 - (1 - P^2)\frac{n-1}{n-r}.}
#'
#' The \code{type} argument controls which residuals \eqn{r_i} are used in
#' the PRESS computation.
#'
#' @return When a single model is provided, a named numeric vector with components
#' \code{P2}, \code{P2_c}, and \code{PRESS}. When multiple models are provided,
#' a data frame with one row per model and columns \code{P2}, \code{P2_c}, and
#' \code{PRESS}.
#'
#' @references
#' Espinheira, P. L., Silva, A. O. (2026). Prediction in the nonlinear simplex model.
#' \emph{International Journal of Data Science and Analytics}, \bold{22}, 161.
#' \doi{10.1007/s41060-026-01114-9}
#'
#' Espinheira, P. L., Silva, A. O. (2020). Residual and influence analysis to a
#' general class of simplex regression. \emph{TEST}, \bold{29}, 523–-552.
#' \doi{10.1007/s11749-019-00665-3}
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills, link.mu = "loglog")
#' # Single model
#' press(fit)
#'
#' # Comparing multiple models
#' press(fit, fit1)
#'
#' # Using bias-variance residuals
#' press(fit, fit1, type = "biasvariance")
#'
#' @seealso \code{\link{simplexreg}}, \code{\link{residuals.simplexregression}}.
#' @export
press <- function(..., type = c("standardized", "biasvariance")) {

  models <- list(...)
  M <- length(models)

  # Verify all objects are simplexregression models
  if (!all(sapply(models, function(x) inherits(x, "simplexregression")))) {
    stop("All arguments must be objects of class 'simplexregression'.")
  }

  type <- match.arg(type)

  # Get model names (supports named arguments)
  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- vapply(as.list(substitute(list(...)))[-1L], deparse, character(1))
  }

  p2_values <- p2_c_values <- press_values <- numeric(M)

  for (i in seq_len(M)) {
    model <- models[[i]]
    parametric <- !is.na(model$coefficients$lambda)

    res    <- residuals(model, type)
    n      <- model$nobs
    p      <- ncol(model$mu.x)
    q      <- ncol(model$sigma2.x)
    r      <- if (parametric) p + q + 1 else p + q

    eta1   <- model$mu.lp
    y      <- as.vector(model$y)
    mu     <- as.vector(model$mu.fv)
    sigma2 <- as.vector(model$sigma2.fv)

    diff     <- y - mu
    muonemu  <- mu * (1 - mu)
    yoneminy <- y * (1 - y)
    dev      <- (diff / muonemu)^2 / yoneminy

    if (parametric) {
      Dlink <- parametric_mean_link_inv_deriv1(eta1, model$lambda.fv, model$mu.link)
    } else {
      Dlink <- fixed_mean_link_inv_deriv1(eta1, model$mu.link)
    }

    Wi    <- (3 * sigma2 / muonemu) + (1 / (muonemu^3))
    Ui    <- (dev / muonemu) + (1 / (muonemu^3))
    ychap <- sqrt((1 / sigma2) * Wi * Dlink^2) * (eta1 + (Ui * diff) / (Dlink * Wi))

    hii <- hatvalues(model)
    press_values[i] <- sum((res / (1 - hii))^2)

    ychap_mean      <- mean(ychap)
    sst             <- sum((ychap - ychap_mean)^2)
    n_factor        <- (n / (n - r))^2
    p2_values[i]    <- 1 - (press_values[i] / (n_factor * sst))
    p2_c_values[i]  <- 1 - (1 - p2_values[i]) * ((n - 1) / (n - r))
  }

  if (M == 1) {
    return(c(P2 = p2_values, P2_c = p2_c_values, PRESS = press_values))
  } else {
    return(data.frame(P2 = p2_values,
                      P2_c =  p2_c_values,
                      PRESS = press_values,
                      row.names = model_names))
  }
}

################################################################################
#                 SIMPLEX REGRESSION - VARIANCE FUNCTION                       #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Computes the variance function for the simplex distribution     #
################################################################################

#' @title Variance Function of the Simplex Distribution
#' @description Computes the variance of the simplex distribution as a function
#' of the mean parameter \eqn{\mu} and dispersion parameter \eqn{\sigma^2}.
#'
#' @param mu Numeric scalar or vector of mean parameters (\eqn{0 < \mu < 1}).
#' If a vector, must be the same length as \code{sigma2} or recyclable.
#' @param sigma2 Numeric scalar or vector of dispersion parameters
#' (\eqn{\sigma^2 > 0}).
#' If a vector, must be the same length as \code{mu} or recyclable.
#'
#' @details
#' The variance function for the simplex distribution is given by:
#' \deqn{Var(Y) = \mu(1-\mu) - \frac{1}{\sqrt{2\sigma^2}} \exp(a) \Gamma(0.5, a),}
#' where \eqn{a = \frac{1}{2\sigma^2[\mu(1-\mu)]^2}} and \eqn{\Gamma(\cdot,\cdot)}
#' is the upper incomplete gamma function.
#'
#' For large values of \eqn{a (> 700)}, an asymptotic approximation is used
#' to avoid numerical overflow:
#' \deqn{Var(Y) \approx \mu(1-\mu) - \frac{1}{\sqrt{2\sigma^2}} \sqrt{\frac{1}{a}}.}
#'
#' @return A numeric scalar or vector of variance values.
#'
#' @examples
#' # Single value
#' variance.simplex(mu = 0.5, sigma2 = 0.1)
#'
#' # Vector of values
#' mu_vec <- c(0.3, 0.5, 0.7)
#' sigma2_vec <- c(0.1, 0.15, 0.2)
#' variance.simplex(mu = mu_vec, sigma2 = sigma2_vec)
#'
#' @references
#' Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
#' Chapman and Hall, London.
#'
#' Song, P. X.-K. and Tan, M. (2000).
#' Marginal models for longitudinal continuous proportional data.
#' \emph{Biometrics}, \bold{56}(2), 496--502.
#' \doi{10.1111/j.0006-341X.2000.00496.x}
#'
#' @seealso \code{\link{dev.unit.simplex}}, \code{\link{dsimplex}}.
#' @importFrom stats pgamma
#'
#' @export
variance.simplex <- function(mu, sigma2) {
  # Input validation
  stopifnot(
    "parameter 'mu' must always be in (0, 1)" = all(mu > 0 & mu < 1),
    "parameter 'sigma2' must always be positive" = all(sigma2 > 0)
  )

  term1 <- mu * (1 - mu)
  a <- 1/(2*sigma2*term1^2)

  # Compute adjustment term with numerical stability
  # For large a (> 700), use asymptotic approximation to avoid overflow
  term2 <- ifelse(a <= 700,
                  1/sqrt(2*sigma2) * exp(a) * sqrt(pi) * pgamma(a, 0.5, lower.tail = FALSE),
                  1/sqrt(2*sigma2) * sqrt(1 / a))

  tol <- 1e-10
  simplex_variance <- ifelse(term2 < term1 - tol,
                            term1 - term2,
                            term1 - .Machine$double.eps)
  return(simplex_variance)
}

library(testthat)
library(SimplexRegression)

test_that("variance.simplex validates inputs correctly", {
  # mu must be in (0, 1)
  expect_error(variance.simplex(mu = 0,    sigma2 = 0.5), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(variance.simplex(mu = 1,    sigma2 = 0.5), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(variance.simplex(mu = -0.5, sigma2 = 0.5), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(variance.simplex(mu = 1.5,  sigma2 = 0.5), "parameter 'mu' must always be in \\(0, 1\\)")

  # sigma2 must be positive
  expect_error(variance.simplex(mu = 0.5, sigma2 = 0),    "parameter 'sigma2' must always be positive")
  expect_error(variance.simplex(mu = 0.5, sigma2 = -0.1), "parameter 'sigma2' must always be positive")

  # NA values
  expect_error(variance.simplex(mu = NA,  sigma2 = 0.5), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(variance.simplex(mu = 0.5, sigma2 = NA),  "parameter 'sigma2' must always be positive")
})

test_that("variance.simplex accepts valid inputs without error", {
  expect_no_error(variance.simplex(mu = 0.5, sigma2 = 0.5))
  expect_no_error(variance.simplex(mu = 0.1, sigma2 = 0.1))
  expect_no_error(variance.simplex(mu = 0.9, sigma2 = 1.0))
})

test_that("variance.simplex returns correct type and dimensions", {
  # scalar input
  result <- variance.simplex(mu = 0.5, sigma2 = 0.1)
  expect_type(result, "double")
  expect_length(result, 1)

  # vector input
  result <- variance.simplex(mu = c(0.3, 0.5, 0.7), sigma2 = c(0.1, 0.15, 0.2))
  expect_type(result, "double")
  expect_length(result, 3)

  # recycling
  result <- variance.simplex(mu = c(0.3, 0.5, 0.7), sigma2 = 0.1)
  expect_length(result, 3)
})

test_that("variance.simplex returns values within theoretical bounds", {
  mu <- 0.5
  bernoulli_var <- mu * (1 - mu)  # theoretical upper bound

  result <- variance.simplex(mu = mu, sigma2 = 0.5)
  expect_true(result > 0)
  expect_true(result < bernoulli_var)

  # all vector values must respect the bounds
  mu_vec <- c(0.2, 0.5, 0.8)
  result_vec <- variance.simplex(mu = mu_vec, sigma2 = 0.3)
  expect_true(all(result_vec > 0))
  expect_true(all(result_vec < mu_vec * (1 - mu_vec)))
})

test_that("variance.simplex is symmetric around mu = 0.5", {
  expect_equal(
    variance.simplex(mu = 0.3, sigma2 = 0.5),
    variance.simplex(mu = 0.7, sigma2 = 0.5),
    tolerance = 1e-10
  )
  expect_equal(
    variance.simplex(mu = 0.2, sigma2 = 0.3),
    variance.simplex(mu = 0.8, sigma2 = 0.3),
    tolerance = 1e-10
  )
})

test_that("variance.simplex is monotonically increasing in sigma2", {
  sigma2_vals <- c(0.1, 0.5, 1, 10)
  vars <- sapply(sigma2_vals, function(s) variance.simplex(mu = 0.5, sigma2 = s))

  for (i in seq_len(length(vars) - 1)) {
    expect_true(vars[i] < vars[i + 1])
  }
})

test_that("variance.simplex converges to mu*(1-mu) as sigma2 grows large", {
  mu <- 0.5
  bernoulli_var <- mu * (1 - mu)

  # Variance should increase toward bernoulli_var as sigma2 grows
  result_small  <- variance.simplex(mu = mu, sigma2 = 1)
  result_medium <- variance.simplex(mu = mu, sigma2 = 100)
  result_large  <- variance.simplex(mu = mu, sigma2 = 1e6)

  expect_true(result_small < result_medium)
  expect_true(result_medium < result_large)
  expect_true(result_large < bernoulli_var)
  expect_true(abs(result_large - bernoulli_var) < 0.01)
})

test_that("variance.simplex handles extreme values of mu and sigma2", {
  expect_true(is.finite(variance.simplex(mu = 0.001, sigma2 = 0.5)))
  expect_true(is.finite(variance.simplex(mu = 0.999, sigma2 = 0.5)))
  expect_true(is.finite(variance.simplex(mu = 0.5,   sigma2 = 1e-6)))
  expect_true(is.finite(variance.simplex(mu = 0.5,   sigma2 = 1e6)))
  expect_true(variance.simplex(mu = 0.5, sigma2 = 1e-6) > 0)
})

test_that("variance.simplex uses asymptotic approximation when a > 700", {
  sigma2_small <- 1e-8
  mu <- 0.5
  # a = 1 / (2 * sigma2 * (mu*(1-mu))^2) must be > 700
  a <- 1 / (2 * sigma2_small * (mu * (1 - mu))^2)
  expect_true(a > 700)

  result <- variance.simplex(mu = mu, sigma2 = sigma2_small)
  expect_true(is.finite(result))
  expect_true(result > 0)
})

## ----include = FALSE------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  fig.align  = "center",
  message    = FALSE,
  warning    = FALSE
)
old_opts <- options(width = 70, prompt = "R> ", continue = "+  ", digits = 5)

## ----setup----------------------------------------------------------
library(SimplexRegression)
data(RelativeHumidity, package = "SimplexRegression")
head(RelativeHumidity, 5)

## ----data-prep------------------------------------------------------
RH       <- RelativeHumidity
RH$hs    <- sin(2 * pi * seq_len(nrow(RH)) / 12)
RH$hc    <- cos(2 * pi * seq_len(nrow(RH)) / 12)
RH$dummy <- as.integer(as.integer(format(RH$date, "%m")) %in% 10:12)

## ----summary-rh-----------------------------------------------------
summary(RH$rh)
cat(sprintf(
  "Std. dev.: %.4f  |  Skewness: %.4f\n",
  sd(RH$rh),
  mean(((RH$rh - mean(RH$rh)) / sd(RH$rh))^3)
))

## ----formula--------------------------------------------------------
formula <- rh ~ ins2 + tmax + ws + hs + hc + dummy + I(dummy * ws) | pre2

## ----models-plogit--------------------------------------------------
fit_p1 <- simplexreg(formula, data = RH, link.mu = "plogit1")
fit_p2 <- simplexreg(formula, data = RH, link.mu = "plogit2")

## ----penalized-ss-param---------------------------------------------
ss.penalized(fit_p1, fit_p2, kappa = 0.1)

## ----penalized-ic---------------------------------------------------
ic.penalized(fit_p1, fit_p2, kappa = 0.1)

## ----models-fixed---------------------------------------------------
fit_loglog  <- simplexreg(formula, data = RH, link.mu = "loglog")
fit_logit   <- simplexreg(formula, data = RH, link.mu = "logit")
fit_probit  <- simplexreg(formula, data = RH, link.mu = "probit")
fit_cauchit <- simplexreg(formula, data = RH, link.mu = "cauchit")
fit_cloglog <- simplexreg(formula, data = RH, link.mu = "cloglog")

## ----penalized-ss-all-----------------------------------------------
ss.penalized(
  fit_loglog, fit_logit, fit_probit,
  fit_cauchit, fit_cloglog, fit_p1,
  kappa = 0
)

## ----summary-fit----------------------------------------------------
summary(fit_loglog)

## ----aic-bic--------------------------------------------------------
AIC(fit_loglog); AIC(fit_logit); AIC(fit_probit); AIC(fit_cauchit); AIC(fit_cloglog); AIC(fit_p1)
BIC(fit_loglog); BIC(fit_logit); BIC(fit_probit); BIC(fit_cauchit); BIC(fit_cloglog); BIC(fit_p1)

## ----coef-vcov------------------------------------------------------
coef(fit_loglog)                          # full coefficient vector
coef(fit_loglog, model = "mean")          # mean submodel only
coef(fit_loglog, model = "dispersion")    # dispersion submodel only
round(vcov(fit_loglog, model = "mean"), 6)  # vcov of mean submodel

## ----loglik---------------------------------------------------------
logLik(fit_loglog)

## ----lrtest---------------------------------------------------------
fit_loglog_null <- update(fit_loglog, . ~ . | 1)
lmtest::lrtest(fit_loglog, fit_loglog_null)

## ----scoretest------------------------------------------------------
fit_logit_h0 <- simplexreg(formula, data = RH, link.mu = "logit")
scoretest(fit_logit_h0, link.mu = "plogit1")
scoretest(fit_logit_h0, link.mu = "plogit2")

## ----resettest------------------------------------------------------
resettest(fit_loglog)                      # both submodels augmented
resettest(fit_loglog, dispersion = FALSE)  # mean submodel only

## ----fitted-resid---------------------------------------------------
head(fitted(fit_loglog))
head(residuals(fit_loglog, type = "quantile"))  # approx. N(0,1)
head(residuals(fit_loglog, type = "pearson"))
head(residuals(fit_loglog, type = "weighted"))  # for halfnormal.plot

## ----press----------------------------------------------------------
press(fit_loglog)                            # single model
press(fit_loglog, fit_logit, fit_probit)     # comparing models

## ----predict--------------------------------------------------------
head(predict(fit_loglog, type = "response"))        # fitted means
head(predict(fit_loglog, type = "link")$mean)       # mean linear predictor
head(predict(fit_loglog, type = "link")$dispersion) # dispersion predictor
head(predict(fit_loglog, type = "dispersion"))      # fitted sigma^2

# Out-of-sample prediction
new_obs <- RH[1:3, ]
predict(fit_loglog, newdata = new_obs, type = "response")

## ----simulate-------------------------------------------------------
set.seed(2026)
sims <- simulate(fit_loglog, nsim = 3)
head(sims)

## ----influence------------------------------------------------------
hii  <- hatvalues(fit_loglog)
cook <- cooks.distance(fit_loglog, type = "pearson")
cat(sprintf("Leverages  — max: %.4f  mean: %.4f\n", max(hii),  mean(hii)))
cat(sprintf("Cook's D   — max: %.4f\n", max(cook)))

## ----gleverage------------------------------------------------------
gl <- gleverage(fit_loglog)
cat(sprintf("Generalized leverage — max: %.4f  mean: %.4f\n",
            max(gl), mean(gl)))

## ----plots-1-7, fig.height = 8, fig.cap = "Diagnostic plots (1–7) for the fitted simplex regression model with log-log link."----
oldpar <- par(mfrow = c(3, 3))
plot(fit_loglog, which = 1:5, ask = FALSE)
plot(fit_loglog, which = 6, threshold = 0.15, label.pos = c(4,2,3))
plot(fit_loglog, which = 7, threshold = 0.08, label.pos = c(3,3,3,4))
par(oldpar)

## ----local-influence-cw, fig.height = 4.5, fig.cap = "Total local influence $C_i$ under case-weight and response perturbation for all parameters."----
oldpar <- par(mfrow = c(1, 2))
local.influence(
  fit_loglog,
  scheme    = "case.weight",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.5,
  label.pos = c(3, 2, 4)
)
local.influence(
  fit_loglog,
  scheme    = "response",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.4,
  label.pos = 2
)
par(oldpar)

## ----halfnormal, fig.height = 5, fig.cap = "Half-normal plot of absolute weighted residuals with 95% simulated envelope (100 replications)."----
halfnormal.plot(fit_loglog, nsim = 19, type = "weighted", seed = 2026)

## ----timeseries, fig.height = 4, fig.cap = "Observed (solid black) and fitted (dashed red) monthly relative humidity in Brasília, January 2000 to December 2025."----
plot(RH$date, RH$rh,
     type = "l", col = "black", lwd = 1.2,
     xlab = "Date", ylab = "Relative humidity",
     main = "Observed vs Fitted RH — Brasília (2000–2025)")
lines(RH$date, fitted(fit_loglog), col = "red", lwd = 1.5, lty = 2)
legend("bottomleft",
       legend = c("Observed", "Fitted"),
       col    = c("black", "red"),
       lty    = c(1, 2), lwd = c(1.2, 1.5),
       bty    = "n", cex = 0.85)

## ----restore-options, include = FALSE-----------------------------------------
options(old_opts)

## ----session------------------------------------------------------------------
sessionInfo()

################################################################################
#               SIMPLEX REGRESSION - PENALIZED INFORMATION CRITERIA            #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Penalized information criterion for simplex regression model    #
#              selection                                                       #
################################################################################

#' @title Penalized Information Criteria for Simplex Regression Model Selection
#' @description Implements the Akaike, Schwarz, and Hannan–Quinn information
#' criteria with a penalty term for selecting among competing simplex regression
#' models with parametric mean link functions.
#'
#' @param ... One or more objects of class \code{"simplexregression"} fitted with a
#' parametric mean link functions (\code{"plogit1"} or \code{"plogit2"}).
#' @param kappa A numeric value controlling the additional penalty for the link
#' mean parameter, \eqn{\kappa \geq 0}. Default is \code{0.1}.
#' @param verbose Logical. If \code{TRUE} (default), prints the criteria values.
#' If \code{FALSE}, returns results silently.
#' @param digits Integer specifying the number of decimal places for output.
#' Default is \code{max(3, getOption("digits") - 3)}.
#'
#' @details
#' The penalized information criteria, as proposed by Justino and
#' Cribari-Neto (2026), extend the classical Akaike, Schwarz and
#' Hannan--Quinn criteria with an additional penalty term for the link
#' parameter \eqn{\lambda}:
#' \deqn{AIC^{(\lambda)} = -2 \ell + (2 + \kappa \, |\log(\lambda)|)r}
#' \deqn{BIC^{(\lambda)} = -2 \ell + (\log(n) + \kappa \, |\log(\lambda)|)r}
#' \deqn{HQIC^{(\lambda)}  = -2 \ell + (2 \log(\log(n)) + \kappa \, |\log(\lambda)|)r}
#' where:
#' \itemize{
#'   \item \eqn{\ell} denotes the maximized log-likelihood function;
#'   \item \eqn{\kappa \geq 0} controls the additional penalty associated with
#'   the link parameter;
#'   \item \eqn{\lambda} is the parameter of the parametric mean link function;
#'   \item \eqn{r} indicate the dimension of their parameter vector;
#'   \item \eqn{n} is the number of observations.
#' }
#'
#' \strong{Important}: These penalized versions of the criteria should only be used
#' when the candidate models employ a parametric link function in the mean submodel
#' (use \code{kappa = 0.1}). When candidate models include specifications
#' with fixed link functions, the standard unpenalized versions of these criteria
#' should be applied instead (use \code{kappa = 0}).
#'
#' @return A data frame with rows named after the candidate models and four columns:
#' \describe{
#'   \item{\code{df}}{Number of estimated parameters.}
#'   \item{\code{AICc}}{Penalized AIC value.}
#'   \item{\code{BICc}}{Penalized BIC value.}
#'   \item{\code{HQICc}}{Penalized HQIC value.}
#' }
#' When \code{verbose = TRUE}, the results are also printed to the console and
#' the data frame is returned invisibly. When \code{verbose = FALSE}, the data
#' frame is returned visibly without printing.
#' When \code{kappa = 0}, the columns are named \code{AIC}, \code{BIC}, and
#' \code{HQIC} instead of \code{AICc}, \code{BICc}, and \code{HQICc}.
#'
#' @examples
#' # Simulate data
#' set.seed(2026)
#' n <- 100
#' x1 <- runif(n, 0, 1)
#' x2 <- runif(n, 0, 1)
#' z1 <- runif(n, 0, 1)
#' mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
#' sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
#' y <- rsimplex(n, mu, sigma2)
#' data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)
#'
#' # Fit two models with parametric mean link functions
#' fit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
#' fit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
#'
#' # Compute penalized criteria
#' penalized.ic(fit1, fit2)
#'
#' @references
#' Akaike, H. (1973). Information theory and an extension of the maximum
#' likelihood principle. \emph{Akadémiai Kiadó}, 267--281.
#'
#' Hannan, E. J. and Quinn, B. G. (1979). The determination of the order of an
#' autoregression. \emph{Journal of the Royal Statistical Society Series B:
#' Statistical Methodology}, \bold{41}(2), 190--195.
#' \doi{10.1111/j.2517-6161.1979.tb01072.x}
#'
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' Schwarz, G. E. (1978). Estimating the dimension of a model. \emph{Annals of
#' Statistics}, \bold{6}(2), 461--464. \doi{10.1214/aos/1176344136}
#'
#' @seealso \code{\link{penalized.ss}}
#' @export
penalized.ic <- function(..., kappa = 0.1, verbose = TRUE,
                         digits = max(3, getOption("digits") - 3)) {

  models <- list(...)
  M <- length(models)

  # Verify all objects are simplexregression models
  if (!all(sapply(models, function(x) inherits(x, "simplexregression")))) {
    stop("All arguments must be objects of class 'simplexregression'.")
  }

  if (!is.numeric(kappa) || length(kappa) != 1L || is.na(kappa) || kappa < 0) {
    stop("'kappa' must be a single non-negative numeric value.")
  }

  # Check parametric mean link
  has_lambda <- sapply(models, function(x) !is.null(x$lambda.fv))

  call <- match.call()
  user_specified_kappa <- "kappa" %in% names(call)

  # Prevent misuse of kappa
  if (!all(has_lambda)) {
    if (user_specified_kappa && kappa > 0) {
      warning("kappa > 0 is only valid when all models have parametric mean links. ",
              "Setting kappa = 0 automatically for this comparison.")
      kappa <- 0
    } else if (!user_specified_kappa && kappa > 0) {
      # User didn't specify kappa, using default but models don't support it
      kappa <- 0
      # No warning here - it's expected behavior
    }
  }

  # Get model names
  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- vapply(as.list(substitute(list(...)))[-1L], deparse, character(1))
  }

  # Initialize storage
  df_values <- aic_values <- bic_values <- hqic_values <- numeric(M)

  for (i in seq_len(M)) {
    model <- models[[i]]

    # Calculate degrees of freedom (number of parameters)
    df_values[i] <- length(model$coefficients$mean) +
      length(model$coefficients$dispersion) +
      as.integer(!is.null(model$lambda.fv))

    n <- length(model$fitted.values)
    ll <- model$loglik

    if (has_lambda[i]) {
      lambda <- model$lambda.fv
      penalty <- kappa * abs(log(lambda))
    } else {
      penalty <- 0
    }

    aic_values[i] <- -2 * ll + (2 + penalty) * df_values[i]
    bic_values[i] <- -2 * ll + (log(n) + penalty) * df_values[i]
    hqic_values[i] <- -2 * ll + (2 * log(log(n)) + penalty) * df_values[i]
  }

  result <- data.frame(
    df = df_values,
    AICc = aic_values,
    BICc = bic_values,
    HQICc = hqic_values,
    row.names = model_names
  )

  if (kappa == 0) {
    names(result)[-1] <- c("AIC", "BIC", "HQIC")
  }

  # Print message
  if (verbose) {
    cat("\n")
    if (kappa == 0) {
      cat("Information criteria values:\n")
    } else {
      cat(sprintf("Penalized information criteria values (kappa = %s):\n",
                  formatC(kappa, digits = 3, format = "f")))
    }

    result_print <- data.frame(
      df = formatC(result[, 1], digits = 0, format = "f"),
      AICc = formatC(result[, 2], digits = digits, format = "f"),
      BICc = formatC(result[, 3], digits = digits, format = "f"),
      HQICc = formatC(result[, 4], digits = digits, format = "f"),
      row.names = rownames(result)
    )

    names(result_print) <- names(result)

    print(result_print, quote = FALSE, right = TRUE)
    return(invisible(result))
  } else {
    return(result)
  }
}

################################################################################
#       SIMPLEX DISTRIBUTION FUNCTIONS - DENSITY, CDF, QUANTILE, RANDOM        #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Probability density, cumulative distribution, quantile, and     #
#              random generation functions for the simplex distribution        #
################################################################################

# ==============================================================================
# 1. DENSITY FUNCTION
# ==============================================================================

#' @title Simplex Distribution Functions
#' @name simplex_opt
#' @description
#' Density, distribution function, quantile function and random generation for the
#' simplex distribution with parameters mean \eqn{\mu} and dispersion \eqn{\sigma^2}.
#'
#' @param x,q Numeric vector of quantiles.
#' @param p Numeric vector of probabilities.
#' @param mu Mean parameter (\eqn{0 < \mu < 1}).
#' @param sigma2 Dispersion parameter (\eqn{\sigma^2 > 0}).
#' @param n Number of observations.
#' @param log,log.p Logical; if \code{TRUE}, probabilities/densities \eqn{p} are given
#' as \eqn{log(p)}.
#' @param lower.tail Logical; if \code{TRUE} (default), probabilities are \eqn{P[X <= x]},
#'        otherwise, \eqn{P[X > x]}.
#'
#' @details
#' The probability density function of the simplex distribution is given by:
#' \deqn{f(y; \mu, \sigma^2) = \frac{1}{\sqrt{2\pi\sigma^2[y(1-y)]^3}}
#'       \exp\left(-\frac{1}{2\sigma^2} d(y; \mu)\right),}
#' where \eqn{y \in (0, 1)}, and \eqn{d(y; \mu) = \frac{(y - \mu)^2}{y(1 - y)
#' \mu^2(1 - \mu)^2}} is the unit deviance.
#'
#' The cumulative distribution function and the quantile function of the simplex
#' distribution do not admit closed-form expressions. For small values of
#' \eqn{\sigma^2}, \code{psimplex()} and \code{qsimplex()} use the normal
#' approximation implied by the small-dispersion asymptotic theory (Jørgensen, 1997).
#' Otherwise, \code{psimplex()} is computed by numerical integration of the density and
#' \code{qsimplex()} is obtained by numerical root finding.
#'
#' Random generation in \code{rsimplex()} is based on the inverse Gaussian
#' mixture (M-IG) representation of the simplex distribution,
#' followed by the transformation \eqn{Y = X/(1 + X)}.
#'
#' @importFrom stats pnorm qnorm integrate rchisq runif uniroot
#'
#' @return \code{dsimplex} gives the density, \code{psimplex} the
#' distribution function, \code{qsimplex} the quantile function, and
#' \code{rsimplex} generates random deviates.
#'
#' For \code{sigma2} values requiring numerical root-finding (i.e., not small
#' enough for the normal approximation), \code{qsimplex} may return \code{NA}
#' if \code{uniroot} fails to find a root in the given interval.
#'
#' Invalid arguments (\code{mu} outside (0, 1) or non-positive \code{sigma2}) will
#' trigger an error.
#'
#' @references
#' Barndorff-Nielsen, O. E. and Jørgensen, B. (1991).
#' Some parametric models on the simplex.
#' \emph{Journal of Multivariate Analysis}, \bold{39}(1), 106--116.
#' \doi{10.1016/0047-259X(91)90008-P}
#'
#' Jørgensen B (1997). \emph{The Theory of Dispersion Models}.
#' Chapman and Hall, London.
#'
#' @examples
#' dsimplex(0.5, mu = 0.3, sigma2 = 0.5)
#' dsimplex(0.5, mu = 0.3, sigma2 = 0.5, log = TRUE)
#' psimplex(0.5, mu = 0.3, sigma2 = 0.5)
#' psimplex(0.5, mu = 0.3, sigma2 = 0.5, lower.tail = FALSE)
#' qsimplex(0.5, mu = 0.3, sigma2 = 0.5)
#' qsimplex(log(0.5), mu = 0.3, sigma2 = 0.5, log.p = TRUE)
#' rsimplex(5, mu = 0.5, sigma2 = 0.5)
#'
#' @export
dsimplex <- function(x, mu, sigma2, log = FALSE) {
  if (any(mu <= 0 | mu >= 1)) stop("'mu' must be in (0, 1)")
  if (any(sigma2 <= 0))       stop("'sigma2' must be positive")

  # Truncate extreme values to avoid numerical issues
  x <- pmin(pmax(x, 1e-8), 1 - 1e-8)

  # The distribution is only defined for x between 0 and 1.
  # Values outside this range should have a density of 0.

  # The vectorized `ifelse` function handles this condition for each element of 'x'.
  valid_x <- x > 0 & x < 1

  # The expression below is the simplex PDF, written in a vectorized way.
  # All calculations (numerator, denominator, exponential) are applied element-wise.
  # If mu and sig are scalars, R recycles them to match the length of x.

  # Calculate log-density to avoid numerical overflow/underflow
  log_numerator <- -((x - mu)^2) / (2 * sigma2 * x * (1 - x) * mu^2 * (1 - mu)^2)
  log_denominator <- 0.5 * log(2 * pi * sigma2) + 1.5 * log(x * (1 - x))

  log_density <- log_numerator - log_denominator

  # Set invalid values to -Inf (which becomes 0 on exp scale)
  log_density <- ifelse(valid_x, log_density, -Inf)

  # Return log-density or density depending on 'log' argument
  if (log) return(log_density)
  return(exp(log_density))
}

# ==============================================================================
# 2. CUMULATIVE DISTRIBUTION FUNCTION (CDF)
# ==============================================================================

#' @keywords internal
psimplex.norm <-  function (q, mu, sigma2, lower.tail = TRUE, log.p = FALSE) {
  prob <- pnorm(q, mean = mu, sd = sqrt(sigma2 * mu^3 * (1 - mu)^3),
                lower.tail = lower.tail, log.p = log.p)
  return(prob)
}

#' @rdname simplex_opt
#' @export
psimplex <- function(q, mu, sigma2, lower.tail = TRUE, log.p = FALSE) {
  if (any(mu <= 0 | mu >= 1)) stop("'mu' must be in (0, 1)")
  if (any(sigma2 <= 0))       stop("'sigma2' must be positive")

  sig <- sqrt(sigma2)
  # Ensures that all vectors have the same length
  n <- length(q)
  if (length(mu) != n) mu <- rep(mu, length.out = n)
  if (length(sig) != n) sig <- rep(sig, length.out = n)

  # Defines the internal density function
  dsimp <- function(x, mu_val, sig_val) {
    1 / sqrt(2 * pi * sig_val^2 * (x * (1 - x))^3) *
      exp(-0.5 / sig_val^2 * (x - mu_val)^2 / (x * (1 - x) * mu_val^2 * (1 - mu_val)^2))
  }

  # Uses `sapply` to iterate over indices, not directly over `q`
  pp <- sapply(1:n, function(i) {
    qi <- q[i]
    mui <- mu[i]
    sigi <- sig[i]

    # Applies the normal approximation if the condition is met
    if (sigi < 0.001 | (1 - mui) * sigi < 0.01) {
      return(psimplex.norm(qi, mui, sigi^2, lower.tail = lower.tail, log.p = log.p))
    } else {
      # Calls integration for each set of parameters
      result <- integrate(dsimp, lower = 1e-8, upper = qi,
                          mu_val = mui, sig_val = sigi)$value

      if (!lower.tail) {
        result <- 1 - result
      }
      if (log.p) {
        result <- log(result)
      }

      return(result)
    }
  })

  return(pp)
}

# ==============================================================================
# 3. QUANTILE FUNCTION
# ==============================================================================

#' @keywords internal
qsimplex.norm <- function(p, mu, sigma2, lower.tail = TRUE, log.p = FALSE) {
  # Ensures all input vectors have the same length.
  # This is crucial when 'mu' and 'sigma2' are vectors.
  n <- length(p)
  if (length(mu) != n) mu <- rep(mu, length.out = n)
  if (length(sigma2) != n) sigma2 <- rep(sigma2, length.out = n)

  # Calculates the standard deviation of the normal approximation in a vectorized way.
  # This operation works correctly even if 'mu' and 'sigma2' are vectors.
  sd_approx <- sqrt(sigma2 * mu^3 * (1 - mu)^3)

  # Returns the normal distribution quantile for each probability in 'p',
  # using the corresponding values of mean ('mu') and standard deviation ('sd_approx').
  return(qnorm(p, mean = mu, sd = sd_approx, lower.tail = lower.tail, log.p = log.p))
}

#' @rdname simplex_opt
#' @export
qsimplex <- function(p, mu, sigma2, lower.tail = TRUE, log.p = FALSE) {

  if (any(mu <= 0 | mu >= 1)) stop("'mu' must be in (0, 1)")
  if (any(sigma2 <= 0))       stop("'sigma2' must be positive")

  # Ensures that all vectors have the same length
  n <- length(p)
  if (length(mu) != n) mu <- rep(mu, length.out = n)
  if (length(sigma2) != n) sigma2 <- rep(sigma2, length.out = n)

  # Uses `sapply` to iterate over indices
  qq <- sapply(1:n, function(i) {
    pi <- p[i]
    mui <- mu[i]
    sigma2i <- sigma2[i]

    pi_transformed <- pi
    if (log.p) {
      pi_transformed <- exp(pi_transformed)
    }
    if (!lower.tail) {
      pi_transformed <- 1 - pi_transformed
    }

    # Handles the case of high dispersion
    if (sigma2i > 200) {
      warning("sigma2 > 200 capped at 200 for numerical stability in qsimplex.")
      sigma2i <- 200
    }

    if (sigma2i < 0.1) {
      return(qsimplex.norm(pi_transformed, mui, sigma2i, lower.tail = TRUE, log.p = FALSE))
    } else {
      # Calls `uniroot` with the correct scalar parameters for each iteration
      tryCatch({
        uniroot(
        f = function(x) psimplex(q = x, mu = mui, sigma2 = sigma2i,
                                 lower.tail = TRUE, log.p = FALSE) - pi_transformed,
        interval = c(1e-8, 1 - 1e-8),
        tol = 1e-6,
        extendInt = "no"
      )$root
      }, error = function(e) NA)
    }
  })

  return(qq)
}

# ==============================================================================
# 4. RANDOM GENERATION
# ==============================================================================

#' @keywords internal
rIG <- function(n, epsilon, Tau) {
  # Generates vectors of length 'n' for z and u1.
  z <- rchisq(n, 1)
  u1 <- runif(n, 0, 1)

  # The vectorized mathematical operations `sqrt`, `*`, `/` are applied element-wise.
  # If 'epsilon' and 'Tau' are scalars, R recycles them to match the length of 'z'.
  ss <- sqrt(4 * epsilon * z / Tau + (epsilon * z)^2)
  z1 <- epsilon + (epsilon^2) * Tau * z / 2 - (epsilon * Tau / 2) * ss

  xxx <- z1

  # The logical indexing handles recycling automatically.
  idx <- (u1 > (epsilon / (epsilon + z1)))

  # Correction: apply the `[idx]` indexing to both the numerator and the denominator.
  # This ensures that the replacement operation is performed with vectors of the
  # same length.
  # The operations on the right side of the assignment are applied only to the elements
  # selected by `idx`.
  # This line is correct as is: R's recycling behavior makes 'epsilon^2' work correctly here.
  # A simple check (if-else) can be added for clarity, but it's not strictly necessary.
  xxx[idx] <- ((epsilon^2) / z1)[idx]

  return(as.numeric(xxx))
}

#' @keywords internal
rMIG <- function(n, epsilon, Tau, mu) {
  # Passes the vectors (or recycled scalars) to rIG.
  x1 <- rIG(n, epsilon, Tau)

  # The other operations also work for vectors.
  x2 <- rchisq(n, 1)
  x3 <- x2 * Tau * (epsilon^2)
  u2 <- runif(n, 0, 1)

  xx <- x1

  # Logical indexing handles recycling automatically.
  idx <- which(u2 < mu)
  xx[idx] <- x1[idx] + x3[idx]

  return(as.numeric(xx))
}

#' @rdname simplex_opt
#' @export
rsimplex <- function(n, mu, sigma2) {
  if (any(mu <= 0 | mu >= 1)) stop("'mu' must be in (0, 1)")
  if (any(sigma2 <= 0))       stop("'sigma2' must be positive")

  # The operations below are vectorized.
  # If 'mu' and 'sigma2' are scalars, R recycles them to the length 'n'.
  # If 'mu' and 'sigma2' are vectors of length 'n', the operations are element-wise.
  epsilon <- mu / (1 - mu)
  Tau <- sigma2 * ((1 - mu)^2)

  # The rMIG function is now able to receive vectors for 'epsilon', 'Tau', and 'mu'.
  # This is handled by the next step.
  x <- rMIG(n, epsilon, Tau, mu)

  # The final transformation is also vectorized.
  yy <- x / (1 + x)

  return(as.vector(yy))
}

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_dist.R
# Tests for: dsimplex, psimplex, qsimplex, rsimplex
# ==============================================================================

# ==============================================================================
# 1. DENSITY FUNCTION — dsimplex
# ==============================================================================

test_that("dsimplex rejects invalid mu", {
  expect_error(dsimplex(0.5, mu = 0,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(dsimplex(0.5, mu = 1,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(dsimplex(0.5, mu = -0.1, sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(dsimplex(0.5, mu = 1.5,  sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
})

test_that("dsimplex rejects non-positive sigma2", {
  expect_error(dsimplex(0.5, mu = 0.3, sigma2 = 0),    "'sigma2' must be positive")
  expect_error(dsimplex(0.5, mu = 0.3, sigma2 = -0.1), "'sigma2' must be positive")
})

test_that("dsimplex returns non-negative values for x in (0, 1)", {
  x <- seq(0.1, 0.9, by = 0.1)
  res <- dsimplex(x, mu = 0.4, sigma2 = 0.5)
  expect_true(all(res >= 0))
})

test_that("dsimplex returns a numeric vector of the correct length", {
  x   <- c(0.1, 0.3, 0.5, 0.7, 0.9)
  res <- dsimplex(x, mu = 0.5, sigma2 = 0.5)
  expect_true(is.numeric(res))
  expect_length(res, 5)
})

test_that("dsimplex with log = TRUE equals log of density", {
  x  <- c(0.2, 0.5, 0.8)
  d  <- dsimplex(x, mu = 0.4, sigma2 = 0.3)
  ld <- dsimplex(x, mu = 0.4, sigma2 = 0.3, log = TRUE)
  expect_equal(ld, log(d), tolerance = 1e-10)
})

test_that("dsimplex integrates to approximately 1", {
  result <- integrate(dsimplex, lower = 1e-8, upper = 1 - 1e-8,
                      mu = 0.4, sigma2 = 0.5)$value
  expect_equal(result, 1, tolerance = 1e-4)
})

test_that("dsimplex integrates to approximately 1 for various parameters", {
  params <- list(
    list(mu = 0.2, sigma2 = 0.3),
    list(mu = 0.5, sigma2 = 1.0),
    list(mu = 0.8, sigma2 = 0.5)
  )
  for (p in params) {
    result <- integrate(dsimplex, lower = 1e-8, upper = 1 - 1e-8,
                        mu = p$mu, sigma2 = p$sigma2)$value
    expect_equal(result, 1, tolerance = 5e-4,
                 label = paste("mu =", p$mu, "sigma2 =", p$sigma2))
  }
})

test_that("dsimplex is maximised near mu", {
  mu    <- 0.4
  x_seq <- seq(0.05, 0.95, by = 0.05)
  dens  <- dsimplex(x_seq, mu = mu, sigma2 = 0.3)
  peak  <- x_seq[which.max(dens)]
  expect_true(abs(peak - mu) < 0.15)
})

test_that("dsimplex is symmetric about mu = 0.5 when mu = 0.5", {
  x   <- c(0.2, 0.3, 0.4)
  d1  <- dsimplex(x, mu = 0.5, sigma2 = 0.5)
  d2  <- dsimplex(1 - x, mu = 0.5, sigma2 = 0.5)
  expect_equal(d1, d2, tolerance = 1e-10)
})

# ==============================================================================
# 2. CUMULATIVE DISTRIBUTION FUNCTION — psimplex
# ==============================================================================

test_that("psimplex rejects invalid mu", {
  expect_error(psimplex(0.5, mu = 0,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(psimplex(0.5, mu = 1,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(psimplex(0.5, mu = -0.2, sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
})

test_that("psimplex rejects non-positive sigma2", {
  expect_error(psimplex(0.5, mu = 0.3, sigma2 = 0),    "'sigma2' must be positive")
  expect_error(psimplex(0.5, mu = 0.3, sigma2 = -1.0), "'sigma2' must be positive")
})

test_that("psimplex returns values in [0, 1]", {
  q   <- c(0.1, 0.3, 0.5, 0.7, 0.9)
  res <- psimplex(q, mu = 0.4, sigma2 = 0.5)
  expect_true(all(res >= 0 & res <= 1))
})

test_that("psimplex is non-decreasing", {
  q   <- seq(0.1, 0.9, by = 0.1)
  res <- psimplex(q, mu = 0.4, sigma2 = 0.5)
  expect_true(all(diff(res) >= 0))
})

test_that("psimplex(mu) is approximately 0.5", {
  mu  <- 0.5
  res <- psimplex(mu, mu = mu, sigma2 = 0.5)
  expect_equal(res, 0.5, tolerance = 0.05)
})

test_that("psimplex lower.tail = FALSE equals 1 - lower.tail = TRUE", {
  q <- c(0.2, 0.5, 0.8)
  p_lo <- psimplex(q, mu = 0.4, sigma2 = 0.5, lower.tail = TRUE)
  p_up <- psimplex(q, mu = 0.4, sigma2 = 0.5, lower.tail = FALSE)
  expect_equal(p_lo + p_up, rep(1, 3), tolerance = 1e-8)
})

test_that("psimplex log.p = TRUE equals log of probability", {
  q  <- c(0.3, 0.5, 0.7)
  p  <- psimplex(q, mu = 0.4, sigma2 = 0.5)
  lp <- psimplex(q, mu = 0.4, sigma2 = 0.5, log.p = TRUE)
  expect_equal(lp, log(p), tolerance = 1e-8)
})

test_that("psimplex uses normal approximation for small sigma2 without error", {
  expect_no_error(psimplex(0.5, mu = 0.4, sigma2 = 1e-5))
})

# ==============================================================================
# 3. QUANTILE FUNCTION — qsimplex
# ==============================================================================

test_that("qsimplex rejects invalid mu", {
  expect_error(qsimplex(0.5, mu = 0,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(qsimplex(0.5, mu = 1,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(qsimplex(0.5, mu = -0.2, sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
})

test_that("qsimplex rejects non-positive sigma2", {
  expect_error(qsimplex(0.5, mu = 0.3, sigma2 = 0),    "'sigma2' must be positive")
  expect_error(qsimplex(0.5, mu = 0.3, sigma2 = -0.5), "'sigma2' must be positive")
})

test_that("qsimplex returns values in (0, 1)", {
  p   <- c(0.1, 0.25, 0.5, 0.75, 0.9)
  res <- qsimplex(p, mu = 0.4, sigma2 = 0.5)
  expect_true(all(res > 0 & res < 1))
})

test_that("qsimplex returns non-decreasing values", {
  p   <- seq(0.1, 0.9, by = 0.1)
  res <- qsimplex(p, mu = 0.4, sigma2 = 0.5)
  expect_true(all(diff(res) >= 0))
})

test_that("qsimplex(0.5) is close to mu (median ≈ mean)", {
  mu  <- 0.5
  q50 <- qsimplex(0.5, mu = mu, sigma2 = 0.5)
  expect_equal(q50, mu, tolerance = 0.1)
})

test_that("qsimplex is the inverse of psimplex", {
  p    <- c(0.1, 0.3, 0.5, 0.7, 0.9)
  q    <- qsimplex(p, mu = 0.4, sigma2 = 0.5)
  p_back <- psimplex(q, mu = 0.4, sigma2 = 0.5)
  expect_equal(p_back, p, tolerance = 1e-4)
})

test_that("qsimplex log.p = TRUE accepts log-scale probabilities", {
  p    <- c(0.25, 0.5, 0.75)
  q1   <- qsimplex(p,      mu = 0.4, sigma2 = 0.5, log.p = FALSE)
  q2   <- qsimplex(log(p), mu = 0.4, sigma2 = 0.5, log.p = TRUE)
  expect_equal(q1, q2, tolerance = 1e-6)
})

test_that("qsimplex lower.tail = FALSE is consistent with lower.tail = TRUE", {
  p  <- 0.3
  q1 <- qsimplex(p,       mu = 0.4, sigma2 = 0.5, lower.tail = TRUE)
  q2 <- qsimplex(1 - p,   mu = 0.4, sigma2 = 0.5, lower.tail = FALSE)
  expect_equal(q1, q2, tolerance = 1e-6)
})

test_that("qsimplex uses normal approximation for small sigma2 without error", {
  expect_no_error(qsimplex(0.5, mu = 0.4, sigma2 = 0.05))
})

test_that("qsimplex warns and caps sigma2 > 200", {
  expect_warning(qsimplex(0.5, mu = 0.4, sigma2 = 300), "sigma2 > 200")
})

test_that("qsimplex.norm recycles mu and sigma2 to match length of p", {
  p      <- c(0.1, 0.3, 0.5, 0.7, 0.9)   # length 5
  result <- qsimplex.norm(p, mu = 0.4, sigma2 = 0.5)  # mu e sigma2 escalares → reciclados
  expect_length(result, 5L)
  expect_true(all(is.finite(result)))
})

test_that("qsimplex.norm recycles sigma2 when mu is vector", {
  p      <- c(0.25, 0.5, 0.75)
  mu     <- c(0.3, 0.4, 0.5)              # length 3
  result <- qsimplex.norm(p, mu = mu, sigma2 = 0.5)   # sigma2 escalar → reciclado
  expect_length(result, 3L)
  expect_true(all(is.finite(result)))
})

# ==============================================================================
# 4. RANDOM GENERATION — rsimplex
# ==============================================================================

test_that("rsimplex rejects invalid mu", {
  expect_error(rsimplex(5, mu = 0,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(rsimplex(5, mu = 1,    sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(rsimplex(5, mu = -0.5, sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
  expect_error(rsimplex(5, mu = 1.5,  sigma2 = 0.5), "'mu' must be in \\(0, 1\\)")
})

test_that("rsimplex rejects non-positive sigma2", {
  expect_error(rsimplex(5, mu = 0.5, sigma2 = 0),    "'sigma2' must be positive")
  expect_error(rsimplex(5, mu = 0.5, sigma2 = -1.0), "'sigma2' must be positive")
})

test_that("rsimplex returns a numeric vector of length n", {
  res <- rsimplex(10, mu = 0.5, sigma2 = 0.5)
  expect_true(is.numeric(res))
  expect_length(res, 10)
})

test_that("rsimplex returns values in (0, 1)", {
  res <- rsimplex(200, mu = 0.5, sigma2 = 0.5)
  expect_true(all(res > 0 & res < 1))
})

test_that("rsimplex sample mean is close to mu for large n", {
  set.seed(1)
  mu  <- 0.4
  res <- rsimplex(5000, mu = mu, sigma2 = 0.5)
  expect_equal(mean(res), mu, tolerance = 0.03)
})

test_that("rsimplex accepts vector mu and sigma2 of length n", {
  n      <- 20
  mu_vec <- rep(c(0.3, 0.7), n / 2)
  s2_vec <- rep(c(0.3, 0.6), n / 2)
  res    <- rsimplex(n, mu = mu_vec, sigma2 = s2_vec)
  expect_length(res, n)
  expect_true(all(res > 0 & res < 1))
})

test_that("rsimplex is reproducible with the same seed", {
  set.seed(7)
  r1 <- rsimplex(10, mu = 0.5, sigma2 = 0.3)
  set.seed(7)
  r2 <- rsimplex(10, mu = 0.5, sigma2 = 0.3)
  expect_equal(r1, r2)
})

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_r2.R
# Tests for: r2
# ==============================================================================

# ---------------------------------------------------------------------------
# Fixture
# ---------------------------------------------------------------------------
set.seed(42)
n   <- 100
x1  <- runif(n, 0, 1)
x2  <- runif(n, 0, 1)
mu  <- parametric_mean_link_inv(0.8 - 1.2*x1 - 1.5*x2, 0.25, "plogit2")
y   <- rsimplex(n, mu, 0.5)
dat <- data.frame(y = y, x1 = x1, x2 = x2)

fit_plogit2 <- simplexreg(y ~ x1 + x2 | 1, data = dat, link.mu = "plogit2")
fit_logit   <- simplexreg(y ~ x1 + x2 | 1, data = dat, link.mu = "logit")
fit_vardisp <- simplexreg(y ~ x1 + x2 | x1, data = dat, link.mu = "logit")

# ==============================================================================
# 1. Input validation
# ==============================================================================

test_that("r2 rejects non-simplexregression objects", {
  expect_error(r2(list()),    "class 'simplexregression'")
  expect_error(r2(fit_logit, list()), "class 'simplexregression'")
})

test_that("r2 rejects alpha1 outside [0, 1]", {
  expect_error(r2(fit_logit, alpha1 = -0.1), "alpha1")
  expect_error(r2(fit_logit, alpha1 =  1.1), "alpha1")
})

test_that("r2 rejects non-positive alpha2", {
  expect_error(r2(fit_logit, alpha2 = 0),    "alpha2")
  expect_error(r2(fit_logit, alpha2 = -1),   "alpha2")
})

# ==============================================================================
# 2. Single model — return structure
# ==============================================================================

test_that("r2 returns a named numeric vector for a single model", {
  res <- r2(fit_logit)
  expect_true(is.numeric(res))
  expect_named(res, c("R2_FC", "R2_FC_c", "R2_N", "R2_N_c",
                      "R2_Nw_c", "R2", "R2_HS"))
})

test_that("r2 returns a vector of length 7 for a single model", {
  expect_length(r2(fit_logit), 7L)
})

test_that("r2 all components are finite for parametric link", {
  res <- r2(fit_plogit2)
  expect_true(all(is.finite(res)))
})

test_that("r2 all components are finite for fixed link", {
  res <- r2(fit_logit)
  expect_true(all(is.finite(res)))
})

# ==============================================================================
# 3. Single model — value ranges
# ==============================================================================

test_that("r2 R2_FC is in [0, 1]", {
  res <- r2(fit_logit)
  expect_true(res["R2_FC"] >= 0 & res["R2_FC"] <= 1)
})

test_that("r2 R2_N is in [0, 1]", {
  res <- r2(fit_logit)
  expect_true(res["R2_N"] >= 0 & res["R2_N"] <= 1)
})

test_that("r2 R2 (conventional) is at most 1", {
  res <- r2(fit_logit)
  expect_true(res["R2"] <= 1)
})

test_that("r2 R2_HS is at most 1", {
  res <- r2(fit_logit)
  expect_true(res["R2_HS"] <= 1)
})

# ==============================================================================
# 4. Single model — consistency with fitted object
# ==============================================================================

test_that("r2 R2_FC matches value stored in fitted object", {
  res <- r2(fit_logit)
  expect_equal(unname(res["R2_FC"]), fit_logit$R2_FC, tolerance = 1e-10)
})

test_that("r2 R2_N matches value stored in fitted object", {
  res <- r2(fit_logit)
  expect_equal(unname(res["R2_N"]), fit_logit$R2_N, tolerance = 1e-10)
})

test_that("r2 R2 equals 1 minus ratio of SSres to SStot", {
  res  <- r2(fit_logit)
  y    <- fit_logit$y
  mu   <- fit_logit$mu.fv
  ref  <- 1 - sum((y - mu)^2) / sum((y - mean(y))^2)
  expect_equal(unname(res["R2"]), ref, tolerance = 1e-10)
})

# ==============================================================================
# 5. Corrections — direction and alpha sensibility
# ==============================================================================

test_that("r2 R2_FC_c is a finite-sample correction of R2_FC", {
  res <- r2(fit_logit)
  # correction moves R2_FC toward 1 when n >> k (both should be close)
  expect_true(is.finite(res["R2_FC_c"]))
})

test_that("r2 R2_N_c is close to R2_Nw_c when alpha1 = 0 and alpha2 = 1", {
  res_default <- r2(fit_logit, alpha1 = 0, alpha2 = 1)
  # With alpha1 = 0 and alpha2 = 1, R2_Nw_c reduces to R2_N_c
  expect_equal(unname(res_default["R2_N_c"]),
               unname(res_default["R2_Nw_c"]),
               tolerance = 1e-10)
})

test_that("r2 R2_Nw_c varies with alpha1", {
  res1 <- r2(fit_vardisp, alpha1 = 0.0)
  res2 <- r2(fit_vardisp, alpha1 = 0.8)
  # Different alpha1 should produce different corrections when p != q
  expect_false(isTRUE(all.equal(res1["R2_Nw_c"], res2["R2_Nw_c"])))
})

test_that("r2 R2_Nw_c varies with alpha2", {
  res1 <- r2(fit_logit, alpha2 = 1)
  res2 <- r2(fit_logit, alpha2 = 2)
  expect_false(isTRUE(all.equal(res1["R2_Nw_c"], res2["R2_Nw_c"])))
})

test_that("r2 alpha3 = 'log' gives different R2_HS than alpha3 = '1'", {
  res1 <- r2(fit_logit, alpha3 = "1")
  res2 <- r2(fit_logit, alpha3 = "log")
  expect_false(isTRUE(all.equal(res1["R2_HS"], res2["R2_HS"])))
})

# ==============================================================================
# 6. Multiple models — return structure
# ==============================================================================

test_that("r2 returns a data.frame for multiple models", {
  res <- r2(fit_plogit2, fit_logit)
  expect_s3_class(res, "data.frame")
})

test_that("r2 data.frame has correct dimensions for two models", {
  res <- r2(fit_plogit2, fit_logit)
  expect_equal(nrow(res), 2L)
  expect_equal(ncol(res), 7L)
})

test_that("r2 data.frame has correct column names", {
  res <- r2(fit_plogit2, fit_logit)
  expect_named(res, c("R2_FC", "R2_FC_c", "R2_N", "R2_N_c",
                      "R2_Nw_c", "R2", "R2_HS"))
})

test_that("r2 data.frame row values match single-model calls", {
  res_multi  <- r2(fit_plogit2, fit_logit)
  res_single <- r2(fit_logit)
  expect_equal(as.numeric(res_multi[2, ]), as.numeric(res_single),
               tolerance = 1e-10)
})

test_that("r2 data.frame all values are finite", {
  res <- r2(fit_plogit2, fit_logit)
  expect_true(all(is.finite(unlist(res))))
})

test_that("r2 works for three models", {
  res <- r2(fit_plogit2, fit_logit, fit_vardisp)
  expect_equal(nrow(res), 3L)
})

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_ic.R
# Tests for: ic
# ==============================================================================

# ---------------------------------------------------------------------------
# Fixture
# ---------------------------------------------------------------------------
set.seed(42)
n   <- 100
x1  <- runif(n, 0, 1)
x2  <- runif(n, 0, 1)
mu  <- parametric_mean_link_inv(0.8 - 1.2*x1 - 1.5*x2, 0.25, "plogit2")
y   <- rsimplex(n, mu, 0.5)
dat <- data.frame(y = y, x1 = x1, x2 = x2)

fit_plogit2 <- simplexreg(y ~ x1 + x2 | 1, data = dat, link.mu = "plogit2")
fit_logit   <- simplexreg(y ~ x1 + x2 | 1, data = dat, link.mu = "logit")
fit_vardisp <- simplexreg(y ~ x1 + x2 | x1, data = dat, link.mu = "logit")

# ==============================================================================
# 1. Input validation
# ==============================================================================

test_that("ic rejects non-simplexregression objects", {
  expect_error(ic(list()),    "class 'simplexregression'")
  expect_error(ic(fit_logit, list()), "class 'simplexregression'")
})

# ==============================================================================
# 2. Single model — return structure
# ==============================================================================

test_that("ic returns a named numeric vector for a single model", {
  res <- ic(fit_logit)
  expect_true(is.numeric(res))
  expect_named(res, c("AIC", "AICc", "BIC", "BICc", "HQIC", "HQICc"))
})

test_that("ic returns a vector of length 6 for a single model", {
  expect_length(ic(fit_logit), 6L)
})

test_that("ic all components are finite for parametric link", {
  res <- ic(fit_plogit2)
  expect_true(all(is.finite(res)))
})

test_that("ic all components are finite for fixed link", {
  res <- ic(fit_logit)
  expect_true(all(is.finite(res)))
})

# ==============================================================================
# 3. Single model — consistency with fitted object
# ==============================================================================

test_that("ic AIC matches -2*loglik + 2k computed from model dimensions", {
  res <- ic(fit_logit)

  n <- fit_logit$nobs
  p <- ncol(fit_logit$mu.x)
  q <- ncol(fit_logit$sigma2.x)
  parametric <- !is.na(fit_logit$coefficients$lambda)
  k <- if (parametric) p + q + 1 else p + q

  ref <- -2 * fit_logit$loglik + 2 * k
  expect_equal(unname(res["AIC"]), ref, tolerance = 1e-10)
})

test_that("ic BIC matches -2*loglik + k*log(n) computed from model dimensions", {
  res <- ic(fit_logit)

  n <- fit_logit$nobs
  p <- ncol(fit_logit$mu.x)
  q <- ncol(fit_logit$sigma2.x)
  parametric <- !is.na(fit_logit$coefficients$lambda)
  k <- if (parametric) p + q + 1 else p + q

  ref <- -2 * fit_logit$loglik + k * log(n)
  expect_equal(unname(res["BIC"]), ref, tolerance = 1e-10)
})

test_that("ic HQIC matches -2*loglik + 2k*log(log(n)) computed from model dimensions", {
  res <- ic(fit_logit)

  n <- fit_logit$nobs
  p <- ncol(fit_logit$mu.x)
  q <- ncol(fit_logit$sigma2.x)
  parametric <- !is.na(fit_logit$coefficients$lambda)
  k <- if (parametric) p + q + 1 else p + q

  ref <- -2 * fit_logit$loglik + 2 * k * log(log(n))
  expect_equal(unname(res["HQIC"]), ref, tolerance = 1e-10)
})

test_that("ic parametric link has larger k (and thus different AIC) than fixed link", {
  res_par   <- ic(fit_plogit2)
  res_fixed <- ic(fit_logit)
  expect_false(isTRUE(all.equal(unname(res_par["AIC"]), unname(res_fixed["AIC"]))))
})

# ==============================================================================
# 4. Corrections — direction relative to uncorrected criteria
# ==============================================================================

test_that("ic AICc is larger than AIC (correction inflates the penalty)", {
  res <- ic(fit_logit)
  expect_true(unname(res["AICc"]) > unname(res["AIC"]))
})

test_that("ic BICc is larger than BIC (correction inflates the penalty)", {
  res <- ic(fit_logit)
  expect_true(unname(res["BICc"]) > unname(res["BIC"]))
})

test_that("ic HQICc is larger than HQIC (correction inflates the penalty)", {
  res <- ic(fit_logit)
  expect_true(unname(res["HQICc"]) > unname(res["HQIC"]))
})

test_that("ic corrections shrink toward the uncorrected criteria as k/n decreases", {
  # fit_vardisp has an extra sigma2 parameter (larger k) than fit_logit
  res_small_k <- ic(fit_logit)
  res_large_k <- ic(fit_vardisp)

  gap_small_k <- unname(res_small_k["AICc"] - res_small_k["AIC"])
  gap_large_k <- unname(res_large_k["AICc"] - res_large_k["AIC"])

  expect_true(gap_large_k > gap_small_k)
})

# ==============================================================================
# 5. Multiple models — return structure
# ==============================================================================

test_that("ic returns a data.frame for multiple models", {
  res <- ic(fit_plogit2, fit_logit)
  expect_s3_class(res, "data.frame")
})

test_that("ic data.frame has correct dimensions for two models", {
  res <- ic(fit_plogit2, fit_logit)
  expect_equal(nrow(res), 2L)
  expect_equal(ncol(res), 6L)
})

test_that("ic data.frame has correct column names", {
  res <- ic(fit_plogit2, fit_logit)
  expect_named(res, c("AIC", "AICc", "BIC", "BICc", "HQIC", "HQICc"))
})

test_that("ic data.frame row values match single-model calls", {
  res_multi  <- ic(fit_plogit2, fit_logit)
  res_single <- ic(fit_logit)
  expect_equal(as.numeric(res_multi[2, ]), as.numeric(res_single),
               tolerance = 1e-10)
})

test_that("ic data.frame all values are finite", {
  res <- ic(fit_plogit2, fit_logit)
  expect_true(all(is.finite(unlist(res))))
})

test_that("ic works for three models", {
  res <- ic(fit_plogit2, fit_logit, fit_vardisp)
  expect_equal(nrow(res), 3L)
})

test_that("ic data.frame row names use deparsed model names when unnamed", {
  res <- ic(fit_plogit2, fit_logit)
  expect_equal(rownames(res), c("fit_plogit2", "fit_logit"))
})

test_that("ic respects supplied names when models are named arguments", {
  res <- ic(m1 = fit_plogit2, m2 = fit_logit)
  expect_equal(rownames(res), c("m1", "m2"))
})

#' Monthly Relative Humidity Data
#'
#' @description
#' Monthly meteorological data including relative humidity (rh),
#' temperature, insolation, precipitation, and wind speed for the
#' meteorological station of Brasília, Brazil (2000-2025), obtained from
#' the National Institute of Meteorology (INMET).
#'
#' The dataset contains two missing observations in the variables \code{ins}
#' and \code{pre}, corresponding to September 2020 and November 2025. To address
#' these missing values, imputation was performed via seasonal interpolation using
#' the \pkg{imputeTS} package, with the imputed versions stored as \code{ins2}
#' and \code{pre2}.
#'
#' @docType data
#' @usage data(RelativeHumidity)
#'
#' @format A data frame with 312 observations and 11 variables:
#' \describe{
#'   \item{date}{Date. Last day of the reference month for the monthly
#'         measurements (YYYY-MM-DD).}
#'   \item{rh}{Numeric. Monthly mean relative humidity, originally measured
#'         in percent and rescaled to the unit interval (0, 1).}
#'   \item{ins}{Numeric. Monthly total insolation (in hours), contains
#'         2 missing values.}
#'   \item{ins2}{Numeric. Monthly total insolation (in hours) with missing
#'         values imputed via seasonal interpolation.}
#'   \item{pre}{Numeric. Monthly total precipitation (in mm), contains
#'         2 missing values.}
#'   \item{pre2}{Numeric. Monthly total precipitation (in mm) with missing
#'         values imputed via seasonal interpolation.}
#'   \item{cld}{Numeric. Monthly mean cloudiness (in tenths).}
#'   \item{ap}{Numeric. Monthly mean atmospheric pressure (in hPa).}
#'   \item{tmax}{Numeric. Monthly mean of daily maximum temperature
#'         (in degrees Celsius).}
#'   \item{ws}{Numeric. Monthly mean wind speed (in m/s).}
#'   \item{wd}{Numeric. Monthly predominant wind direction, coded by
#'         INMET/BDMEP in tens of degrees (e.g., 9 = 90 degrees, East;
#'         32 = 320 degrees, Northwest). A value of 0 indicates calm
#'         conditions (no wind or undefined direction). Multiply by 10
#'         to obtain the direction in degrees (0-360).}
#' }
#'
#' @source
#' Instituto Nacional de Meteorologia (INMET). Banco de Dados
#' Meteorológicos para Ensino e Pesquisa (BDMEP), Estação de Brasília
#' (https://bdmep.inmet.gov.br). Data accessed in 2026.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the dataset
#' data(RelativeHumidity)
#'
#' # View first rows
#' head(RelativeHumidity)
#'
#' # Check structure
#' str(RelativeHumidity)
#'
#' # Insolation with and without imputation
#' plot(RelativeHumidity$ins, type = "l", col = "red",
#'      ylab = "Insolation", main = "Missing values visible as gaps")
#' points(RelativeHumidity$ins2, type = "l", col = "blue")
"RelativeHumidity"

################################################################################
#                  SIMPLEX REGRESSION - SCOUT SCORE CRITERION                  #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Scout Score criterion for simplex regression model selection    #
################################################################################

#' @title Scout Score Criterion for Simplex Regression Model Selection
#' @description Implements the Scout Score (SS) criterion for selecting among
#' competing simplex regression models with parametric and fixed mean link functions.
#'
#' @param ... Two or more objects of class \code{simplexregression} to be compared.
#' @param kappa A numeric value controlling the additional penalty for the link mean
#' parameter. Default is 0.1. Use \code{kappa = 0} for standard Scout Score.
#' @param verbose Logical. If \code{TRUE} (default), prints the SS values for
#' all models and the selected model. If \code{FALSE}, returns results silently.
#' @param digits Integer specifying the number of decimal places for output.
#' Default is \code{max(3, getOption("digits") - 3)}.
#'
#' @details
#' The Scout Score criterion, originally proposed by Costa et al. (2024) for
#' selecting link functions in the \eqn{\beta\text{ARMA}} (beta autoregressive
#' moving average) model, extends Vuong's test statistic to compare \eqn{M \geq 2}
#' competing non-nested models using their individual log-likelihood contributions.
#'
#' For each candidate model \eqn{j \in {1, \ldots, M}}, the Scout Score is defined as:
#' \deqn{SS_j = 1 - M + \sum_{k=1, k \neq j}^M (1 + \dot{\Delta}_{jk})^2,}
#'
#' where \eqn{\dot{\Delta}_{jk} = \max\{0, \Delta_{jk}\}} and \eqn{\Delta_{jk}}
#' denotes Vuong's test statistic comparing models \eqn{j} and \eqn{k}:
#' \deqn{\Delta_{jk} = n^{-1/2} \hat{\omega}_{jk}^{-1} \left[\sum_{i=1}^n
#' \log\left(\frac{f_{ji}(\hat{\boldsymbol{\theta}}_j)}{f_{ki}(\hat{
#' \boldsymbol{\theta}}_k)}\right) - \delta_{jk}\right],}
#'
#' with
#'
#' \eqn{\hat{\omega}_{jk}^2 = \frac{1}{n} \sum_{i=1}^n \left( \log
#' \left( \frac{f_{ji}(\boldsymbol{\hat{\theta}}_j)}{f_{ki}
#' (\boldsymbol{\hat{\theta}}_k)} \right) \right)^2 - \left( \frac{1}{n}
#' \sum_{i=1}^n \log \left( \frac{f_{ji}(\boldsymbol{\hat{\theta}}_j)}
#' {f_{ki}(\boldsymbol{\hat{\theta}}_k)} \right) \right)^2.}
#'
#' Here, \eqn{f_{ji}(\boldsymbol{\hat{\theta}}_j)} and
#' \eqn{f_{ki}(\boldsymbol{\hat{\theta}}_k)} denote the fitted densities under
#' models \eqn{j} and \eqn{k}, respectively.
#'
#' The penalization term \eqn{\delta_{jk}} for simplex models with parametric and
#' fixed mean links, as proposed by Justino and Cribari-Neto (2026), is given by:
#'
#' \deqn{\delta_{jk} = 0.5[(r_j - r_k) + \kappa(|\log(\hat{\lambda}_j)| -
#' |\log(\hat{\lambda}_k)|)]\log(n),}
#' where \eqn{r_j} and \eqn{r_k} indicate the dimensions of their parameter vectors,
#' and \eqn{\kappa \geq 0} controls the additional penalty associated with the link
#' parameter. The term \eqn{\kappa(|\log(\hat{\lambda}_j)| - |\log(\hat{\lambda}_k)|)}
#' measures link complexity on a logarithmic scale, ensuring symmetry around
#' \eqn{\lambda = 1}.
#'
#' The model with the highest Scout Score is selected as the most adequate.
#'
#' \strong{Important}: This penalized term \eqn{\delta_{jk}} should only be used
#' when all candidate models employ a parametric link function in the mean submodel
#' (use \code{kappa = 0.1}). When the set of candidate models includes specifications
#' with fixed link functions, the standard unpenalized versions of these criteria
#' should be applied instead (use \code{kappa = 0}).
#'
#' @return A data frame with rows named after the candidate models and two columns:
#' \describe{
#'   \item{\code{df}}{Number of estimated parameters in each model.}
#'   \item{\code{SS}}{Scout Score value. The model with the highest value is
#'   the selected one.}
#' }
#' When \code{verbose = TRUE}, the selected model is also printed to the console.
#' The data frame is returned invisibly in this case, and visibly when
#' \code{verbose = FALSE}.
#'
#' @references
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' Costa, E., Cribari-Neto, F., and Scher, V. T. (2024).
#' Test inferences and link function selection in dynamic beta modeling of seasonal
#' hydro-environmental time series with temporary abnormal regimes.
#' \emph{Journal of Hydrology}, \bold{638}, 131489.
#' \doi{10.1016/j.jhydrol.2024.131489}
#'
#' Vuong, Q. H. (1989). Likelihood ratio tests for model selection and non-nested
#' hypotheses. \emph{Econometrica}, \bold{57}(2), 307-–333. \doi{10.2307/1912557}
#'
#' @examples
#' # Simulate data
#' set.seed(2026)
#' n <- 100
#' x1 <- runif(n, 0, 1)
#' x2 <- runif(n, 0, 1)
#' z1 <- runif(n, 0, 1)
#' mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
#' sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
#' y <- rsimplex(n, mu, sigma2)
#' data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)
#'
#' # Fit models
#' fit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
#' fit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
#' fit3 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")
#' fit4 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "probit")
#'
#' # Compare models with verbose output
#' result <- penalized.ss(fit1, fit2, kappa = 0.1)
#'
#' # Compare models silently
#' result <- penalized.ss(fit1, fit2, kappa = 0.1, verbose = FALSE)
#'
#' # Use standard Scout Score (no parametric link penalty)
#' result <- penalized.ss(fit1, fit2, fit3, fit4, kappa = 0)
#'
#' @seealso \code{\link{penalized.ic}}
#' @export
penalized.ss <- function(..., kappa = 0.1, verbose = TRUE,
                         digits = max(3, getOption("digits") - 3)) {

  models <- list(...)
  M <- length(models)

  if (M < 2) stop("At least 2 models are required for comparison")

  # Verify all objects are simplexregression models
  if (!all(sapply(models, function(x) inherits(x, "simplexregression")))) {
    stop("All arguments must be objects of class 'simplexregression'")
  }

  # Get model names (supports named arguments)
  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- sapply(as.list(substitute(list(...)))[-1], deparse)
  }

  y <- models[[1]]$y
  n <- models[[1]]$nobs

  # Initialize storage lists
  mu_list <- sigma2_list <- lambda_list <- f_list <- vector("list", M)
  has_lambda <- logical(M)
  r_list <- numeric(M)

  # Extract values from all models
  for (i in 1:M) {
    mu_list[[i]] <- models[[i]]$mu.fv
    sigma2_list[[i]] <- models[[i]]$sigma2.fv
    f_list[[i]] <- dsimplex(y, mu_list[[i]], sigma2_list[[i]])
    r_list[i] <- n - models[[i]]$df.residual

    # check if model has lambda
    if (!is.null(models[[i]]$lambda.fv) && !all(is.na(models[[i]]$lambda.fv))) {
      lambda_list[[i]] <- models[[i]]$lambda.fv
      has_lambda[i] <- TRUE
    } else {
      lambda_list[[i]] <- NA
      has_lambda[i] <- FALSE
    }
  }

  all_parametric <- all(has_lambda)

  # Detect if user explicitly provided kappa
  call <- match.call()
  user_kappa <- "kappa" %in% names(call)

  # Adjust kappa intelligently
  if (!all_parametric) {
    if (user_kappa && kappa > 0) {
      warning("kappa > 0 is only valid when all models have parametric mean links. ",
              "Setting kappa = 0 automatically for this comparison.")
      kappa <- 0
    } else if (!user_kappa && kappa > 0) {
      # User didn't specify kappa, using default but models don't support it
      kappa <- 0
      # No warning - it's expected behavior
    }
  }

  # Calculate SS for each model
  SS_values <- numeric(M)

  for (i in 1:M) {
    delta_squared_sum <- 0

    for (j in 1:M) {
      if (i != j) {
        # Calculate log-likelihood ratio
        log_ratio <- log(f_list[[i]]/f_list[[j]])

        # Calculate omega_ij^2
        omega2_ij <- 1/n * sum((log_ratio)^2) - (mean(log_ratio)^2)

        # Protect against numerical issues
        if (omega2_ij < 1e-10) omega2_ij <- 1e-10

        # Penalization delta_jk
        if (all_parametric) {
          penalty <- kappa * (abs(log(lambda_list[[i]])) -
                                abs(log(lambda_list[[j]])))
        } else {
          penalty <- 0
        }

        # Calculate delta_ij
        delta_ij <- (sum(log_ratio) - 0.5 * ((r_list[i] - r_list[j]) + penalty)
                     * log(n)) / (sqrt(n) * sqrt(omega2_ij))

        # Apply max(0, delta_jk)
        delta_ij_max <- max(0, delta_ij)

        # Add to sum
        delta_squared_sum <- delta_squared_sum + (1 + delta_ij_max)^2
      }
    }

    # Calculate final SS value
    SS_values[i] <- 1 - M + delta_squared_sum
  }

  result <- data.frame(
    df = r_list,
    SS = SS_values,
    row.names = model_names
  )

  # Verbose output
  if (verbose) {
    cat("\n")
    if (kappa == 0) {
      cat("Scout Score values:\n")
    } else {
      cat(sprintf("Penalized Scout Score values (kappa = %s):\n",
                  formatC(kappa, digits = 3, format = "f")))
    }

    result_print <- data.frame(
      df = formatC(result$df, digits = 0, format = "f"),
      SS = formatC(result$SS, digits = digits, format = "f"),
      row.names = rownames(result)
    )
    print(result_print, quote = FALSE, right = TRUE)

    best_model_index <- which.max(SS_values)
    cat(sprintf("\nSelected model: %s\n", model_names[best_model_index]))

    return(invisible(result))
  } else {
    return(result)
  }
}

################################################################################
#               SIMPLEX REGRESSION - R2 MEASURES                              #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto            #
# Date: 2026-07-14                                                            #
# Description: Pseudo-R2 measures for simplex regression models               #
################################################################################

#' @title Pseudo-R2 Measures for Simplex Regression
#' @description Extracts the Ferrari and Cribari-Neto pseudo-\eqn{R^2}
#' (\eqn{R^2_{FC}}), the likelihood-ratio pseudo-\eqn{R^2}
#' (\eqn{R^2_N}), the conventional coefficient of determination
#' (\eqn{R^2}), and computes their finite-sample corrections. The
#' corrections for \eqn{R^2_{FC}} and \eqn{R^2_N} follow Bayer and
#' Cribari-Neto (2017), whereas the correction for the conventional
#' \eqn{R^2} follows Hu and Shao (2008).
#'
#' @param ... One or more objects of class \code{"simplexregression"} to be
#' evaluated.
#' @param alpha1 Numeric, \eqn{\alpha_1 \in [0, 1]} controlling the relative penalty
#' weight given to the mean and dispersion submodels in the weighted
#' correction \eqn{R^2_{Nw_c}}. Default is \code{0.4}, as
#' recommended by Bayer and Cribari-Neto (2017).
#' @param alpha2 Numeric, \eqn{\alpha_2 > 0}, controlling the penalization
#' intensity in \eqn{R^2_{Nw_c}}. Default is \code{1}, as recommended by
#' Bayer and Cribari-Neto (2017).
#' @param alpha3 Penalization constant for the correction of
#' \eqn{R^2_{HS}}. One of \code{"1"} (default, \eqn{\alpha_3 = 1}) or
#' \code{"log"} (\eqn{\alpha_3 = \log(n)}).
#'
#' @details
#' \eqn{R^2_{FC}} is the Ferrari and Cribari-Neto (2004) pseudo-R2, defined as
#' the squared correlation between the fitted mean linear predictor and the
#' link-transformed response, i.e.,
#' \deqn{R^2_{FC} = \mathrm{corr}^2\!\left(
#' \boldsymbol{\hat\eta_1}, g(\boldsymbol{y})\right),}
#' where \eqn{\boldsymbol{\hat\eta_1}} denotes the vector of fitted mean linear
#' predictors, \eqn{g(\cdot)} is the mean link function, and \eqn{\boldsymbol{y}}
#' is the vector of observed values of the response variable.
#'
#' \eqn{R^2_N} is a likelihood-ratio-based pseudo-R2 (Nagelkerke, 1991), defined as
#' \deqn{R^2_N = 1 - \left(\frac{L_{null}}{L_{fit}}\right)^{2/n},}
#' where \eqn{L_{null}} and \eqn{L_{fit}} are the maximized likelihoods of the
#' null (intercept-only) and fitted models, respectively.
#'
#' The conventional coefficient of determination is
#' \deqn{R^2 = 1 - \frac{\sum_{i=1}^n (y_i - \hat\mu_i)^2}{\sum_{i=1}^n (y_i - \bar y)^2},}
#' where \eqn{\bar y} is the sample mean of the responses.
#'
#' \strong{Corrections:}
#' Both finite-sample corrections implemented here for
#' \eqn{R^2_{FC}} and \eqn{R^2_N} were proposed by Bayer and Cribari-Neto
#' (2017) for beta regression with varying precision and are used here in
#' their direct simplex regression analogue.
#'
#' The simple correction, a function of the total number of estimated
#' parameters \eqn{r} only,
#' is applied to both \eqn{R^2_{FC}} and \eqn{R^2_N}:
#' \deqn{R^2_{FC_c} = 1 - (1 - R^2_{FC})\frac{n-1}{n-r} \qquad \text{and} \qquad
#' R^2_{N_c} = 1 - (1 - R^2_{N})\frac{n-1}{n-r}.}
#'
#' A second, weighted correction is defined for \eqn{R^2_N} only, penalizing
#' the mean and dispersion submodels asymmetrically through \code{alpha1} and
#' controlling penalization intensity through \code{alpha2}:
#' \deqn{R^2_{Nw_c} = 1 - (1 - R^2_N)\left(\frac{n-1}{n-(1+\alpha_1)p-
#' (1-\alpha_1)q}\right)^{\alpha2},}
#' where \eqn{p} and \eqn{q} being the number of parameters in the mean and
#' dispersion submodels, respectively.
#' Setting \code{alpha1 = 0} and \code{alpha2 = 1} reduces \eqn{R^2_{Nw_c}} to
#' the simple \eqn{R^2_{N_c}} above.
#' Bayer and Cribari-Neto (2017) recommend
#' \code{alpha1 = 0.4} and \code{alpha2 = 1} as sensible defaults; both arguments
#' are exposed here for users who wish to specify different values.
#'
#' Finally, the conventional coefficient of determination admits the
#' finite-sample correction proposed by Hu and Shao (2008):
#' \deqn{R^2_{HS} = 1 - \frac{n-1}{n - \alpha_3 r}
#' \frac{\sum_{i=1}^n (y_i - \hat\mu_i)^2}{\sum_{i=1}^n (y_i - \bar y)^2}.}
#' where \eqn{\alpha_3} is a penalization constant.
#' When \code{alpha_3 = "1"} (default), \eqn{\alpha_3 = 1}, \eqn{R^2_{HS}}
#' reduces to the modified R2 of Mittlböck and Schemper (2002). When
#' \code{alpha_3 = "log"}, \eqn{\alpha_3 = \log(n)}, as evaluated by Hu and Shao
#' (2008).
#'
#' The measures \eqn{R^2_{FC}} and \eqn{R^2_N} take values in \eqn{[0,1]}.
#' The conventional coefficient of determination \eqn{R^2}, as well as the
#' corrected measures \eqn{R^2_{FC_c}}, \eqn{R^2_{N_c}}, \eqn{R^2_{Nw_c}}, and
#' \eqn{R^2_{HS}}, take values in \eqn{(-\infty,1]}.
#' Larger values indicate better model fit.
#'
#' @return When a single model is provided, a named numeric vector with
#' components \code{R2_FC}, \code{R2_FC_c}, \code{R2_N}, \code{R2_N_c},
#' \code{R2_Nw_c}, \code{R2}, and \code{R2_HS}. When multiple models are
#' provided, a data frame with one row per model.
#'
#' @references
#' Bayer, F. M. and Cribari-Neto, F. (2017). Model selection criteria in beta
#' regression with varying dispersion. \emph{Communications in Statistics -
#' Simulation and Computation}, \bold{46}(1),
#' 729--746. \doi{10.1080/03610918.2014.977918}
#'
#' Ferrari, S. L. P. and Cribari-Neto, F. (2004). Beta regression for
#' modelling rates and proportions. \emph{Journal of Applied Statistics},
#' \bold{31}(7), 799--815. \doi{10.1080/0266476042000214501}
#'
#' Hu, B. and Shao, J. (2008). Generalized linear model selection using R2.
#' \emph{Journal of Statistical Planning and Inference}, \bold{138}(12),
#' 3705--3712. \doi{10.1016/j.jspi.2007.12.009}
#'
#' Mittlböck, M. and Schemper, M. (2002). Explained variation for logistic
#' regression -- small sample adjustments, confidence intervals and other
#' issues. \emph{Statistics in Medicine}, \bold{21}(23), 3547--3562.
#' \doi{10.1002/1521-4036(200204)44:3<263::AID-BIMJ263>3.0.CO;2-7}
#'
#' Nagelkerke, N. J. D. (1991). A note on a general definition of the
#' coefficient of determination. \emph{Biometrika}, \bold{78}(3), 691--692.
#' \doi{10.1093/biomet/78.3.691}
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                    data = ReadingSkills)
#'
#' # Single model, default alpha1, alpha2 and alpha3
#' r2(fit1)
#'
#' # Comparing multiple models
#' fit2 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                    data = ReadingSkills, link.mu = "loglog")
#' r2(fit1, fit2)
#'
#' # Custom alpha1/alpha2 for the weighted correction, and alpha3 = log(n)
#' r2(fit1, fit2, alpha1 = 0.6, alpha2 = 1.5, alpha3 = "log")
#'
#' @seealso \code{\link{simplexreg}}, \code{\link{press}}
#' @export
r2 <- function(..., alpha1 = 0.4, alpha2 = 1, alpha3 = c("1", "log")) {

  models <- list(...)
  M <- length(models)

  if (!all(sapply(models, function(x) inherits(x, "simplexregression")))) {
    stop("All arguments must be objects of class 'simplexregression'.")
  }
  if (alpha1 < 0 || alpha1 > 1) {
    stop("'alpha1' must be between 0 and 1.")
  }
  if (alpha2 <= 0) {
    stop("'alpha2' must be positive.")
  }

  alpha3 <- match.arg(alpha3)

  # Get model names (supports named arguments)
  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- vapply(as.list(substitute(list(...)))[-1L], deparse, character(1))
  }

  r2_fc <- r2_fc_c <- r2_n <- r2_n_c <- r2_nw_c <- r2 <- r2_hs <- numeric(M)

  for (i in seq_len(M)) {
    model <- models[[i]]

    n <- model$nobs
    p <- ncol(model$mu.x)
    q <- ncol(model$sigma2.x)
    parametric <- !is.na(model$coefficients$lambda)
    k <- if (parametric) p + q + 1 else p + q

    # --- R2 Ferrari-Cribari-Neto: pulled from the fitted object ---
    r2_fc[i] <- model$R2_FC
    r2_fc_c[i] <- 1 - (1 - r2_fc[i]) * ((n - 1) / (n - k))

    # --- R2 likelihood ratio: pulled from the fitted object ---
    r2_n[i] <- model$R2_N

    # Simple correction, function of k only (Bayer and Cribari-Neto, 2017)
    r2_n_c[i] <- 1 - (1 - r2_n[i]) * ((n - 1) / (n - k))

    # Weighted correction, separately penalizing mean (p) and dispersion (q)
    # submodels via alpha1/alpha2 (Bayer and Cribari-Neto, 2017)
    r2_nw_c[i] <- 1 - (1 - r2_n[i]) *
      ((n - 1) / (n - (1 + alpha1) * p - (1 - alpha1) * q))^alpha2

    # --- Coefficient of determination ---
    y    <- as.vector(model$y)
    mu   <- as.vector(model$mu.fv)
    ybar <- mean(y)

    r2[i] <- 1 - (sum((y - mu)^2) / sum((y - ybar)^2))

    # --- R2 Hu and Shao (2008) ---
    alpha3_value <- switch(alpha3, "1" = 1, "log" = log(n))

    r2_hs[i] <- 1 - ((n - 1) / (n - alpha3_value * k)) *
      (sum((y - mu)^2) / sum((y - ybar)^2))
  }

  if (M == 1) {
    return(c(R2_FC = r2_fc, R2_FC_c = r2_fc_c,
             R2_N = r2_n, R2_N_c = r2_n_c, R2_Nw_c = r2_nw_c,
             R2 = r2, R2_HS = r2_hs))
  } else {
    data.frame(R2_FC = r2_fc, R2_FC_c = r2_fc_c,
               R2_N = r2_n, R2_N_c = r2_n_c, R2_Nw_c = r2_nw_c,
               R2 = r2, R2_HS = r2_hs,
               row.names = model_names)
  }
}

#' Biomass Allocation in Two Grass Species Under Different Nitrate Supply
#'
#' @description
#' This dataset examines biomass allocation patterns in plants, specifically
#' the proportional distribution of biomass to different plant organs (stems,
#' leaves, and roots). The data come from an experiment manipulating nitrate
#' supply in fast-growing and slow-growing grass species.
#'
#' The response variables (stem, leaves, roots) are proportions bounded in the
#' (0, 1) interval, making them suitable for simplex regression analysis.
#'
#' @docType data
#' @usage data(Biomass)
#'
#' @format A data frame with 500 observations and 13 variables:
#' \describe{
#'   \item{group}{Factor. Combined species-by-nitrate-treatment code
#'         (\code{DfH}, \code{DfL}, \code{HlH}, \code{HlL}), corresponding to the combination of
#'         \code{species} and \code{trt} where:
#'     \itemize{
#'       \item \code{DfH} = D. flexuosa (slow-growing), high nitrate
#'       \item \code{DfL} = D. flexuosa (slow-growing), low nitrate
#'       \item \code{HlH} = H. lanatus (fast-growing), high nitrate
#'       \item \code{HlL} = H. lanatus (fast-growing), low nitrate
#'     }
#'   }
#'   \item{species}{Factor. Species scientific name (\code{D. flexuosa} or
#'         \code{H. lanatus}).}
#'   \item{trt}{Factor. Nitrate treatment level (\code{high} or \code{low}).}
#'   \item{day}{Numeric. Experimental day (0 to 49).}
#'   \item{pl_num}{Numeric. Plant number (individual plant identifier,
#'         6-8 replicates per treatment).}
#'   \item{ldm_mg}{Numeric. Leaf dry mass (in mg).}
#'   \item{sdm_mg}{Numeric. Stem dry mass (in mg).}
#'   \item{rdm_mg}{Numeric. Root dry mass (in mg).}
#'   \item{tdm_mg}{Numeric. Total dry mass (in mg).}
#'   \item{lmf}{Numeric. Leaf mass fraction, proportion of biomass allocated
#'         to leaves, bounded in the open interval (0, 1).}
#'   \item{smf}{Numeric. Stem mass fraction, proportion of biomass allocated
#'         to stems, bounded in the open interval (0, 1).}
#'   \item{rmf}{Numeric. Root mass fraction, proportion of biomass allocated
#'         to roots, bounded in the open interval (0, 1).}
#'   \item{ln_tdm}{Numeric. Natural log of total dry mass (log-transformed
#'         for allometric analysis).}
#' }
#'
#' @source
#' bobdouma (2019). \emph{bobdouma/proportions_beta_Dirichlet: v.01}.
#' \doi{10.5281/zenodo.3234670}.
#'
#' @references
#' bobdouma (2019). \emph{bobdouma/proportions_beta_Dirichlet: v.01}.
#' \doi{10.5281/zenodo.3234670}.
#'
#' Poorter, H.; van de Vijver, C. A. D. M.; Boot, R. G. A. and Lambers, H. (1995).
#' Growth and carbon economy of a fast-growing and a slow-growing grass
#' species as dependent on nitrate supply. \emph{Plant and Soil}, \bold{171},
#' 217--227. \doi{10.1007/BF00010275}
#'
#' Poorter, H. and Sack, L. (2012). Pitfalls and possibilities in the analysis
#' of biomass allocation patterns in plants. \emph{Frontiers in Plant
#' Science}, \bold{3}, 259. \doi{10.3389/fpls.2012.00259}
#'
#' @keywords datasets
#'
#' @examples
#' # Load the data
#' data(Biomass)
#'
#' # Quick overview
#' head(Biomass)
#' str(Biomass)
#'
#' # Check that proportions sum to 1 (within rounding error)
#' summary(rowSums(Biomass[, c("lmf", "smf", "rmf")]))
#'
#' # Simple plot of root mass fraction by treatment
#' boxplot(rmf ~ trt * species, data = Biomass,
#'         main = "Root Mass Fraction by Species and Nitrate Treatment",
#'         las = 2)
"Biomass"

################################################################################
#                 LINK FUNCTIONS - SIMPLEX REGRESSION MODELS                   #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Link functions, their inverses, and derivatives for simplex     #
#              regression models with parametric and fixed mean link functions #
################################################################################

# ==============================================================================
# 1. PARAMETRIC MEAN LINK FUNCTIONS (plogit1 and plogit2)
# ==============================================================================

#' @title Parametric Mean Link Functions and Derivatives
#'
#' @description
#' Provides the parametric mean link functions, their inverses, and derivatives
#' for the simplex regression models. Two parametric link types are supported:
#' \code{"plogit1"} and \code{"plogit2"}.
#'
#' @details
#' Two parametric mean link functions are available, as proposed by
#' Justino and Cribari-Neto (2026):
#' \itemize{
#'   \item Parametric logit type 1 (\code{"plogit1"}): \eqn{g(\mu; \lambda) = \log((1-\mu)^{-\lambda} - 1)};
#'   \item Parametric logit type 2 (\code{"plogit2"}): \eqn{g(\mu; \lambda) = \log(\mu^\lambda / (1 - \mu^\lambda))}.
#' }
#' Their inverses and derivatives with respect to \eqn{\mu} are also implemented.
#'
#' @param mu Mean parameter (numeric vector, \eqn{0 < \mu < 1}).
#' @param eta Linear predictor of mean (numeric vector).
#' @param lambda Power parameter (numeric scalar, \eqn{\lambda > 0}).
#' @param type Type of link function: \code{"plogit1"} or \code{"plogit2"}.
#'
#' @importFrom stats plogis qlogis
#'
#' @return Numeric vector with transformed values.
#'
#' @seealso \code{\link{simplexreg}}, \code{\link{fixed_mean_links}},
#' \code{\link{dispersion_links}}.
#'
#' @examples
#' parametric_mean_link(0.2, lambda = 1.2, type = "plogit2")
#' parametric_mean_link(c(0.2, 0.5, 0.8), lambda = 1.5, type = "plogit1")
#' parametric_mean_link_inv(0, lambda = 1, type = "plogit2")
#' parametric_mean_link_deriv1(0.5, lambda = 1, type = "plogit2")
#' parametric_mean_link_inv_deriv1(0, lambda = 1, type = "plogit2")
#' parametric_mean_link_deriv2(0.5, lambda = 1, type = "plogit2")
#'
#' @references
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' @name parametric_mean_links
NULL

#' @rdname parametric_mean_links
#' @export
parametric_mean_link <- function(mu, lambda, type = c("plogit1", "plogit2")) {
  type <- match.arg(type)

  # Input validation
  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  if (any(lambda <= 0)) {
    stop("'lambda' must be positive")
  }

  if(type == "plogit2") {
    mu_lambda <- pmin(pmax(mu^lambda, .Machine$double.eps), 1-.Machine$double.eps)
    result <- qlogis(mu_lambda)
  } else {
    one_minus_mu_lambda <- pmin(pmax(1-(1-mu)^lambda, .Machine$double.eps),
                                1-.Machine$double.eps)
    result <- qlogis(one_minus_mu_lambda)
  }

  return(result)
}

#' @rdname parametric_mean_links
#' @export
parametric_mean_link_inv <- function(eta, lambda, type = c("plogit1", "plogit2")) {
  type <- match.arg(type)

  if (any(lambda <= 0)) {
    stop("'lambda' must be positive")
  }

  if(type == "plogit2") {
    result <- pmin(pmax(plogis(eta)^(1/lambda), .Machine$double.eps), 1-.Machine$double.eps)
  } else {
    result <- pmin(pmax(1-(exp(eta)+1)^(-1/lambda), .Machine$double.eps), 1-.Machine$double.eps)
  }

  return(result)
}

#' @rdname parametric_mean_links
#' @export
parametric_mean_link_deriv1 <- function(mu, lambda, type = c("plogit1", "plogit2")) {
  type <- match.arg(type)

  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  if (any(lambda <= 0)) {
    stop("'lambda' must be positive")
  }

  if(type== "plogit2") {
    mu_lambda <- pmin(pmax(mu^lambda, .Machine$double.eps), 1-.Machine$double.eps)
    result <- lambda / (mu * (1 - mu_lambda))
  } else {
    one_minus_mu_lambda <- pmin(pmax(1-(1-mu)^lambda, .Machine$double.eps),
                                1-.Machine$double.eps)
    result <- lambda / ((1-mu) * one_minus_mu_lambda)
  }

  return(result)
}

#' @rdname parametric_mean_links
#' @export
parametric_mean_link_inv_deriv1 <- function(eta, lambda, type = c("plogit1", "plogit2")) {
  type <- match.arg(type)

  if (any(lambda <= 0)) {
    stop("'lambda' must be positive")
  }

  if(type == "plogit2") {
    result <- parametric_mean_link_inv(eta, lambda, type) / (lambda * (1 + exp(eta)))
  } else {
    result <- exp(eta) / (lambda * (1 + exp(eta))^(1/lambda + 1))
  }

  return(result)
}

#' @rdname parametric_mean_links
#' @export
parametric_mean_link_deriv2 <- function(mu, lambda, type = c("plogit1", "plogit2")) {
  type <- match.arg(type)

  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  if (any(lambda <= 0)) {
    stop("'lambda' must be positive")
  }

  if(type == "plogit2") {
    mu_lambda <- pmin(pmax(mu^lambda, .Machine$double.eps), 1-.Machine$double.eps)
    result <- (lambda * mu_lambda * (1+lambda) - lambda) / ((mu*(1-mu_lambda))^2)
  } else {
    one_minus_mu_lambda <- pmin(pmax(1-(1-mu)^lambda, .Machine$double.eps),
                                1-.Machine$double.eps)
    result <- (lambda * (one_minus_mu_lambda - lambda*(1-mu)^lambda )) /
      (((1-mu) * one_minus_mu_lambda)^2)
  }

  return(result)
}

# ==============================================================================
# 2. FIXED MEAN LINK FUNCTIONS
# ==============================================================================

#' @title Fixed Mean Link Functions and Derivatives
#'
#' @description
#' Provides the fixed mean link functions, their inverses, and derivatives
#' for the simplex regression model. Supported link types are:
#' \code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"}, and \code{"cauchit"}.
#'
#' @param mu Mean parameter (numeric vector, \eqn{0 < \mu < 1}).
#' @param eta Linear predictor of mean (numeric vector).
#' @param type Type of link function: \code{"logit"}, \code{"probit"},
#'   \code{"loglog"}, \code{"cloglog"}, or \code{"cauchit"}.
#'
#' @details
#' Available link functions:
#' \itemize{
#'   \item Logit (\code{"logit"}): \eqn{g(\mu) = \log(\mu/(1-\mu))};
#'   \item Probit (\code{"probit"}): \eqn{g(\mu) = \Phi^{-1}(\mu)};
#'   \item Log-log (\code{"loglog"}): \eqn{g(\mu) = -\log(-\log(\mu))};
#'   \item Complementary log-log (\code{"cloglog"}): \eqn{g(\mu) = \log(-\log(1-\mu))};
#'   \item Cauchit (\code{"cauchit"}): \eqn{g(\mu) = \tan(\pi(\mu - 0.5))}.
#' }
#'
#' @importFrom stats pnorm qnorm dnorm pcauchy qcauchy dcauchy plogis dlogis
#'
#' @return A numeric vector corresponding to the evaluated link,
#' its inverse, or derivative depending on the function.
#'
#' @seealso \code{\link{simplexreg}}, \code{\link{parametric_mean_links}},
#' \code{\link{dispersion_links}}.
#'
#' @examples
#' fixed_mean_link(0.5, type = "logit")
#' fixed_mean_link(c(0.2, 0.5, 0.8), type = "probit")
#' fixed_mean_link_inv(eta = 0.2, type = "logit")
#' fixed_mean_link_deriv1(mu = 0.5, type = "logit")
#'
#' @name fixed_mean_links
NULL

#' @rdname fixed_mean_links
#' @export
fixed_mean_link <- function(mu, type = c("logit", "probit", "loglog",
                                              "cloglog", "cauchit")) {
  type <- match.arg(type)

  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  result <- switch(type,
                   loglog = -log(-log(mu)),
                   cloglog = log(-log(1.0 - mu)),
                   probit = qnorm(mu),
                   cauchit = qcauchy(mu),
                   logit = log(mu/(1.0 - mu))
  )

  return(result)
}

#' @rdname fixed_mean_links
#' @export
fixed_mean_link_inv <- function(eta, type = c("logit", "probit", "loglog",
                                                   "cloglog", "cauchit")) {
  type <- match.arg(type)

  result <- switch(type,
                   loglog = exp(-exp(-eta)),
                   cloglog = 1 - exp(-exp(eta)),
                   probit = pnorm(eta),
                   cauchit = pcauchy(eta),
                   logit = plogis(eta)
  )

  return(result)
}

#' @rdname fixed_mean_links
#' @export
fixed_mean_link_deriv1 <- function(mu, type = c("logit", "probit", "loglog",
                                                     "cloglog", "cauchit")) {
  type <- match.arg(type)

  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  result <- switch(type,
                   loglog = -1 / (mu * log(mu)),
                   cloglog = -1 / ((1 - mu) * log(1 - mu)),
                   probit = 1 / dnorm(qnorm(mu)),
                   cauchit = 1 / dcauchy(qcauchy(mu)),
                   logit = 1 / (mu * (1 - mu))
  )

  return(result)
}

#' @rdname fixed_mean_links
#' @export
fixed_mean_link_deriv2 <- function(mu, type = c("logit", "probit", "loglog",
                                                     "cloglog", "cauchit")) {
  type <- match.arg(type)

  if (any(mu <= 0 | mu >= 1)) {
    stop("All values of 'mu' must be in the interval (0, 1)")
  }

  result <- switch(type,
                   loglog = (log(mu) + 1) / (mu^2 * log(mu)^2),
                   cloglog = - (log(1 - mu) + 1) / ((1 - mu)^2 * log(1 - mu)^2),
                   probit = qnorm(mu) / (dnorm(qnorm(mu))^2),
                   cauchit = 2*pi^2 * qcauchy(mu) * (1+qcauchy(mu)^2),
                   logit = (2*mu - 1) / (mu^2 * (1 - mu)^2)
  )

  return(result)
}

#' @rdname fixed_mean_links
#' @export
fixed_mean_link_inv_deriv1 <- function(eta, type = c("logit", "probit", "loglog",
                                                          "cloglog", "cauchit")) {
  type <- match.arg(type)

  result <- switch(type,
                   loglog = exp(-eta - exp(-eta)),
                   cloglog = exp(eta - exp(eta)),
                   probit = dnorm(eta),
                   cauchit = dcauchy(eta),
                   logit = dlogis(eta)
  )
  return(result)
}

# ==============================================================================
# 3. DISPERSION LINK FUNCTIONS
# ==============================================================================

#' @title Dispersion Link Functions and Their Derivatives
#' @description
#' Provides the link function, its inverse, and derivative for the dispersion
#' submodel in the simplex regression.
#' Supported link types are: \code{"log"}, \code{"sqrt"} and \code{"identity"}.
#'
#' @param sigma2 Dispersion parameter (numeric vector, \eqn{\sigma^2 > 0}).
#' @param eta Linear predictor of dispersion (numeric vector).
#' @param type Type of link: \code{"log"}, \code{"sqrt"} or \code{"identity"}.
#'
#' @details
#' Available link functions:
#' \itemize{
#'   \item Log (\code{"log"}): \eqn{h(\sigma^2) = \log(\sigma^2)} (ensures positivity);
#'   \item Sqrt (\code{"sqrt"}): \eqn{h(\sigma^2) = \sqrt{\sigma^2}};
#'   \item Identity (\code{"identity"}): \eqn{h(\sigma^2) = \sigma^2} (no transformation).
#' }
#'
#' @return Numeric vector with transformed values.
#'
#' @seealso \code{\link{simplexreg}}, \code{\link{fixed_mean_links}},
#' \code{\link{parametric_mean_links}}.
#'
#' @examples
#' dispersion_link(1.5, type = "log")
#' dispersion_link(c(0.5, 1, 2), type = "sqrt")
#' dispersion_link_inv(0, type = "log")
#' dispersion_link_deriv1(1, type = "log")
#' dispersion_link_inv_deriv1(0, type = "log")
#'
#' @name dispersion_links
NULL

#' @rdname dispersion_links
#' @export
dispersion_link <- function(sigma2, type = c("log", "sqrt", "identity")) {
  type <- match.arg(type)

  if (any(sigma2 <= 0)) {
    stop("All values of 'sigma2' must be positive")
  }

  result <- switch(type,
                   log = log(sigma2),
                   sqrt = sqrt(sigma2),
                   identity = sigma2
  )

  return(result)
}

#' @rdname dispersion_links
#' @export
dispersion_link_inv <- function(eta, type = c("log", "sqrt", "identity")) {
  type <- match.arg(type)

  result <- switch(type,
                   log = exp(eta),
                   sqrt = eta^2,
                   identity = eta
  )

  return(result)
}

#' @rdname dispersion_links
#' @export
dispersion_link_deriv1 <- function(sigma2, type = c("log", "sqrt", "identity")) {
  type <- match.arg(type)

  if (any(sigma2 <= 0)) {
    stop("All values of 'sigma2' must be positive")
  }

  result <- switch(type,
                   log = 1 / sigma2,
                   sqrt = 0.5 * sigma2^(-0.5),
                   identity = rep.int(1, length(sigma2))
  )

  return(result)
}

#' @rdname dispersion_links
#' @export
dispersion_link_inv_deriv1 <- function(eta, type = c("log", "sqrt", "identity")) {
  type <- match.arg(type)

  result <- switch(type,
                   log = exp(eta),
                   sqrt = 2 * eta,
                   identity = rep.int(1, length(eta))
  )

  return(result)
}

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu     <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y      <- rsimplex(n, mu, sigma2)
data   <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# Pre-compute results reused across tests
res_default  <- resettest(fit_plogit1)
res_nodisp   <- resettest(fit_plogit1, dispersion = FALSE)
res_power23  <- resettest(fit_plogit1, power = 2:3)
res_fitted   <- resettest(fit_plogit1, type = "fitted")
res_scalar   <- resettest(fit_plogit1, power = 2)   # scalar power → triggers is.vector branch

# ---------------------------------------------------------------------------

test_that("resettest requires an object of class simplexregression", {
  expect_error(resettest(list()),   "class 'simplexregression'")
  expect_error(resettest("model"),  "class 'simplexregression'")
})

test_that("resettest rejects power < 2", {
  expect_error(resettest(fit_plogit1, power = 1),    "integers >= 2")
  expect_error(resettest(fit_plogit1, power = 0),    "integers >= 2")
  expect_error(resettest(fit_plogit1, power = c(2, 1)), "integers >= 2")
})

test_that("resettest rejects non-integer power", {
  expect_error(resettest(fit_plogit1, power = 2.5),  "integers >= 2")
  expect_error(resettest(fit_plogit1, power = "2"),  "integers >= 2")
})

test_that("resettest returns an object of class htest", {
  expect_s3_class(res_default, "htest")
})

test_that("resettest returns all required htest components", {
  expect_named(res_default,
               c("statistic", "parameter", "p.value", "method", "data.name"),
               ignore.order = TRUE)
})

test_that("resettest method description is correct", {
  expect_equal(res_default$method, "RESET test")
})

test_that("resettest returns a finite non-negative test statistic", {
  expect_true(is.finite(res_default$statistic))
  expect_true(res_default$statistic >= 0)
})

test_that("resettest returns a p-value in [0, 1]", {
  expect_true(res_default$p.value >= 0 && res_default$p.value <= 1)
})

test_that("resettest degrees of freedom is positive", {
  expect_true(res_default$parameter > 0)
})

test_that("resettest works with dispersion = FALSE", {
  expect_s3_class(res_nodisp, "htest")
  expect_true(is.finite(res_nodisp$statistic))
  expect_true(res_nodisp$p.value >= 0 && res_nodisp$p.value <= 1)
})

test_that("resettest dispersion = TRUE has >= df than dispersion = FALSE", {
  expect_true(res_default$parameter >= res_nodisp$parameter)
})

test_that("resettest power = 2:3 returns larger df than power = 2", {
  expect_true(res_power23$parameter > res_scalar$parameter)
})

test_that("resettest power = 2:3 returns finite statistic and valid p-value", {
  expect_true(is.finite(res_power23$statistic))
  expect_true(res_power23$p.value >= 0 && res_power23$p.value <= 1)
})

test_that("resettest type = 'fitted' returns valid htest", {
  expect_s3_class(res_fitted, "htest")
  expect_true(is.finite(res_fitted$statistic))
  expect_true(res_fitted$p.value >= 0 && res_fitted$p.value <= 1)
})

test_that("resettest type = 'lp' and type = 'fitted' give different statistics", {
  expect_false(identical(res_default$statistic, res_fitted$statistic))
})

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_links.R
# Tests for: parametric_mean_links, fixed_mean_links, dispersion_links
# ==============================================================================

# ==============================================================================
# 1. PARAMETRIC MEAN LINK FUNCTIONS
# ==============================================================================

# --- parametric_mean_link -----------------------------------------------------

test_that("parametric_mean_link rejects mu outside (0, 1)", {
  expect_error(parametric_mean_link(0,    lambda = 1, type = "plogit2"), "interval \\(0, 1\\)")
  expect_error(parametric_mean_link(1,    lambda = 1, type = "plogit2"), "interval \\(0, 1\\)")
  expect_error(parametric_mean_link(-0.1, lambda = 1, type = "plogit1"), "interval \\(0, 1\\)")
  expect_error(parametric_mean_link(1.5,  lambda = 1, type = "plogit1"), "interval \\(0, 1\\)")
})

test_that("parametric_mean_link returns a finite numeric vector", {
  result <- parametric_mean_link(c(0.2, 0.5, 0.8), lambda = 1.5, type = "plogit2")
  expect_true(is.numeric(result))
  expect_length(result, 3)
  expect_true(all(is.finite(result)))
})

test_that("parametric_mean_link returns finite values for plogit1", {
  result <- parametric_mean_link(c(0.2, 0.5, 0.8), lambda = 1.5, type = "plogit1")
  expect_true(is.numeric(result))
  expect_length(result, 3)
  expect_true(all(is.finite(result)))
})

test_that("parametric_mean_link plogit2 equals logit when lambda = 1", {
  mu  <- c(0.2, 0.5, 0.8)
  res <- parametric_mean_link(mu, lambda = 1, type = "plogit2")
  ref <- log(mu / (1 - mu))
  expect_equal(res, ref, tolerance = 1e-8)
})

test_that("parametric_mean_link plogit1 equals logit when lambda = 1", {
  mu  <- c(0.2, 0.5, 0.8)
  res <- parametric_mean_link(mu, lambda = 1, type = "plogit1")
  ref <- log(mu / (1 - mu))
  expect_equal(res, ref, tolerance = 1e-8)
})

test_that("parametric_mean_link is monotone increasing in mu", {
  mu  <- seq(0.1, 0.9, by = 0.1)
  for (type in c("plogit1", "plogit2")) {
    res <- parametric_mean_link(mu, lambda = 1.5, type = type)
    expect_true(all(diff(res) > 0), label = paste(type, "is monotone increasing"))
  }
})

test_that("parametric_mean_link default type argument is matched correctly", {
  expect_no_error(parametric_mean_link(0.5, lambda = 1))
})

# --- parametric_mean_link_inv -------------------------------------------------

test_that("parametric_mean_link_inv returns values in (0, 1)", {
  eta <- c(-2, -1, 0, 1, 2)
  for (type in c("plogit1", "plogit2")) {
    res <- parametric_mean_link_inv(eta, lambda = 1.5, type = type)
    expect_true(all(res > 0 & res < 1), label = paste(type, "inverse in (0,1)"))
  }
})

test_that("parametric_mean_link_inv is the inverse of parametric_mean_link", {
  mu  <- c(0.2, 0.4, 0.6, 0.8)
  for (type in c("plogit1", "plogit2")) {
    eta <- parametric_mean_link(mu, lambda = 1.3, type = type)
    mu_back <- parametric_mean_link_inv(eta, lambda = 1.3, type = type)
    expect_equal(mu_back, mu, tolerance = 1e-8,
                 label = paste(type, "round-trip"))
  }
})

test_that("parametric_mean_link_inv plogit2 equals expit(eta) when lambda = 1", {
  eta <- c(-1, 0, 1)
  res <- parametric_mean_link_inv(eta, lambda = 1, type = "plogit2")
  ref <- 1 / (1 + exp(-eta))
  expect_equal(res, ref, tolerance = 1e-8)
})

# --- parametric_mean_link_deriv1 ----------------------------------------------

test_that("parametric_mean_link_deriv1 rejects mu outside (0, 1)", {
  expect_error(parametric_mean_link_deriv1(0,   lambda = 1, type = "plogit2"), "interval \\(0, 1\\)")
  expect_error(parametric_mean_link_deriv1(1.2, lambda = 1, type = "plogit1"), "interval \\(0, 1\\)")
})

test_that("parametric_mean_link_deriv1 returns positive finite values", {
  mu  <- c(0.2, 0.5, 0.8)
  for (type in c("plogit1", "plogit2")) {
    res <- parametric_mean_link_deriv1(mu, lambda = 1.5, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
    expect_true(all(res > 0),        label = paste(type, "positive"))
  }
})

test_that("parametric_mean_link_deriv1 plogit2 equals 1/(mu*(1-mu)) when lambda = 1", {
  mu  <- c(0.2, 0.5, 0.8)
  res <- parametric_mean_link_deriv1(mu, lambda = 1, type = "plogit2")
  ref <- 1 / (mu * (1 - mu))
  expect_equal(res, ref, tolerance = 1e-8)
})

test_that("parametric_mean_link_deriv1 approximates numerical derivative", {
  mu  <- 0.4
  lam <- 1.5
  h   <- 1e-6
  for (type in c("plogit1", "plogit2")) {
    num_deriv <- (parametric_mean_link(mu + h, lam, type) -
                    parametric_mean_link(mu - h, lam, type)) / (2 * h)
    ana_deriv <- parametric_mean_link_deriv1(mu, lam, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical deriv1"))
  }
})

# --- parametric_mean_link_inv_deriv1 ------------------------------------------

test_that("parametric_mean_link_inv_deriv1 returns positive finite values", {
  eta <- c(-2, 0, 2)
  for (type in c("plogit1", "plogit2")) {
    res <- parametric_mean_link_inv_deriv1(eta, lambda = 1.5, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
    expect_true(all(res > 0),        label = paste(type, "positive"))
  }
})

test_that("parametric_mean_link_inv_deriv1 approximates numerical derivative", {
  eta <- 0.3
  lam <- 1.5
  h   <- 1e-6
  for (type in c("plogit1", "plogit2")) {
    num_deriv <- (parametric_mean_link_inv(eta + h, lam, type) -
                    parametric_mean_link_inv(eta - h, lam, type)) / (2 * h)
    ana_deriv <- parametric_mean_link_inv_deriv1(eta, lam, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical inv_deriv1"))
  }
})

# --- parametric_mean_link_deriv2 ----------------------------------------------

test_that("parametric_mean_link_deriv2 rejects mu outside (0, 1)", {
  expect_error(parametric_mean_link_deriv2(0,   lambda = 1, type = "plogit2"), "interval \\(0, 1\\)")
  expect_error(parametric_mean_link_deriv2(1.1, lambda = 1, type = "plogit1"), "interval \\(0, 1\\)")
})

test_that("parametric_mean_link_deriv2 returns finite values", {
  mu  <- c(0.2, 0.5, 0.8)
  for (type in c("plogit1", "plogit2")) {
    res <- parametric_mean_link_deriv2(mu, lambda = 1.5, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
  }
})

test_that("parametric_mean_link_deriv2 approximates numerical second derivative", {
  mu  <- 0.4
  lam <- 1.5
  h   <- 1e-5
  for (type in c("plogit1", "plogit2")) {
    num_deriv2 <- (parametric_mean_link(mu + h, lam, type) -
                     2 * parametric_mean_link(mu, lam, type) +
                     parametric_mean_link(mu - h, lam, type)) / h^2
    ana_deriv2 <- parametric_mean_link_deriv2(mu, lam, type)
    expect_equal(ana_deriv2, num_deriv2, tolerance = 1e-4,
                 label = paste(type, "numerical vs analytical deriv2"))
  }
})

# ==============================================================================
# 2. FIXED MEAN LINK FUNCTIONS
# ==============================================================================

fixed_types <- c("logit", "probit", "loglog", "cloglog", "cauchit")

# --- fixed_mean_link ----------------------------------------------------------

test_that("fixed_mean_link rejects mu outside (0, 1)", {
  for (type in fixed_types) {
    expect_error(fixed_mean_link(0,    type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu = 0"))
    expect_error(fixed_mean_link(1,    type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu = 1"))
    expect_error(fixed_mean_link(-0.1, type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu < 0"))
  }
})

test_that("fixed_mean_link returns finite numeric values for all types", {
  mu <- c(0.2, 0.5, 0.8)
  for (type in fixed_types) {
    res <- fixed_mean_link(mu, type = type)
    expect_true(is.numeric(res),       label = paste(type, "numeric"))
    expect_length(res, 3)
    expect_true(all(is.finite(res)),   label = paste(type, "finite"))
  }
})

test_that("fixed_mean_link logit equals log(mu / (1 - mu))", {
  mu  <- c(0.2, 0.5, 0.8)
  res <- fixed_mean_link(mu, type = "logit")
  expect_equal(res, log(mu / (1 - mu)), tolerance = 1e-12)
})

test_that("fixed_mean_link is monotone increasing in mu for all types", {
  mu <- seq(0.1, 0.9, by = 0.1)
  for (type in fixed_types) {
    res <- fixed_mean_link(mu, type = type)
    expect_true(all(diff(res) > 0),
                label = paste(type, "monotone increasing"))
  }
})

# --- fixed_mean_link_inv ------------------------------------------------------

test_that("fixed_mean_link_inv returns values in (0, 1) for all types", {
  eta <- c(-3, -1, 0, 1, 3)
  for (type in fixed_types) {
    res <- fixed_mean_link_inv(eta, type = type)
    expect_true(all(res > 0 & res < 1), label = paste(type, "in (0,1)"))
  }
})

test_that("fixed_mean_link_inv is the inverse of fixed_mean_link", {
  mu <- c(0.2, 0.4, 0.6, 0.8)
  for (type in fixed_types) {
    eta    <- fixed_mean_link(mu, type = type)
    mu_back <- fixed_mean_link_inv(eta, type = type)
    expect_equal(mu_back, mu, tolerance = 1e-10,
                 label = paste(type, "round-trip"))
  }
})

# --- fixed_mean_link_deriv1 ---------------------------------------------------

test_that("fixed_mean_link_deriv1 rejects mu outside (0, 1)", {
  for (type in fixed_types) {
    expect_error(fixed_mean_link_deriv1(0,   type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu = 0"))
    expect_error(fixed_mean_link_deriv1(1.5, type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu > 1"))
  }
})

test_that("fixed_mean_link_deriv1 returns positive finite values for all types", {
  mu <- c(0.2, 0.5, 0.8)
  for (type in fixed_types) {
    res <- fixed_mean_link_deriv1(mu, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
    expect_true(all(res > 0),        label = paste(type, "positive"))
  }
})

test_that("fixed_mean_link_deriv1 logit equals 1 / (mu * (1 - mu))", {
  mu  <- c(0.2, 0.5, 0.8)
  res <- fixed_mean_link_deriv1(mu, type = "logit")
  expect_equal(res, 1 / (mu * (1 - mu)), tolerance = 1e-12)
})

test_that("fixed_mean_link_deriv1 approximates numerical derivative for all types", {
  mu <- 0.4
  h  <- 1e-6
  for (type in fixed_types) {
    num_deriv <- (fixed_mean_link(mu + h, type) -
                    fixed_mean_link(mu - h, type)) / (2 * h)
    ana_deriv <- fixed_mean_link_deriv1(mu, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical"))
  }
})

# --- fixed_mean_link_deriv2 ---------------------------------------------------

test_that("fixed_mean_link_deriv2 rejects mu outside (0, 1)", {
  for (type in fixed_types) {
    expect_error(fixed_mean_link_deriv2(0,   type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu = 0"))
    expect_error(fixed_mean_link_deriv2(1.2, type = type), "interval \\(0, 1\\)",
                 label = paste(type, "mu > 1"))
  }
})

test_that("fixed_mean_link_deriv2 returns finite values for all types", {
  mu <- c(0.2, 0.5, 0.8)
  for (type in fixed_types) {
    res <- fixed_mean_link_deriv2(mu, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
  }
})

test_that("fixed_mean_link_deriv2 approximates numerical second derivative", {
  mu <- 0.4
  h  <- 1e-5
  for (type in fixed_types) {
    num_d2 <- (fixed_mean_link(mu + h, type) -
                 2 * fixed_mean_link(mu, type) +
                 fixed_mean_link(mu - h, type)) / h^2
    ana_d2 <- fixed_mean_link_deriv2(mu, type)
    expect_equal(ana_d2, num_d2, tolerance = 1e-4,
                 label = paste(type, "numerical vs analytical"))
  }
})

# --- fixed_mean_link_inv_deriv1 -----------------------------------------------

test_that("fixed_mean_link_inv_deriv1 returns positive finite values for all types", {
  eta <- c(-2, -0.5, 0, 0.5, 2)
  for (type in fixed_types) {
    res <- fixed_mean_link_inv_deriv1(eta, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
    expect_true(all(res > 0),        label = paste(type, "positive"))
  }
})

test_that("fixed_mean_link_inv_deriv1 approximates numerical derivative of inverse", {
  eta <- 0.3
  h   <- 1e-6
  for (type in fixed_types) {
    num_deriv <- (fixed_mean_link_inv(eta + h, type) -
                    fixed_mean_link_inv(eta - h, type)) / (2 * h)
    ana_deriv <- fixed_mean_link_inv_deriv1(eta, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical"))
  }
})

# ==============================================================================
# 3. DISPERSION LINK FUNCTIONS
# ==============================================================================

disp_types <- c("log", "sqrt", "identity")

# --- dispersion_link ----------------------------------------------------------

test_that("dispersion_link rejects non-positive sigma2", {
  for (type in disp_types) {
    expect_error(dispersion_link(0,    type = type), "positive",
                 label = paste(type, "sigma2 = 0"))
    expect_error(dispersion_link(-0.5, type = type), "positive",
                 label = paste(type, "sigma2 < 0"))
  }
})

test_that("dispersion_link returns finite numeric values for all types", {
  sigma2 <- c(0.5, 1.0, 2.0)
  for (type in disp_types) {
    res <- dispersion_link(sigma2, type = type)
    expect_true(is.numeric(res),       label = paste(type, "numeric"))
    expect_length(res, 3)
    expect_true(all(is.finite(res)),   label = paste(type, "finite"))
  }
})

test_that("dispersion_link log equals log(sigma2)", {
  sigma2 <- c(0.5, 1.0, 2.0)
  expect_equal(dispersion_link(sigma2, type = "log"), log(sigma2), tolerance = 1e-12)
})

test_that("dispersion_link sqrt equals sqrt(sigma2)", {
  sigma2 <- c(0.5, 1.0, 4.0)
  expect_equal(dispersion_link(sigma2, type = "sqrt"), sqrt(sigma2), tolerance = 1e-12)
})

test_that("dispersion_link identity returns sigma2 unchanged", {
  sigma2 <- c(0.3, 1.0, 2.5)
  expect_equal(dispersion_link(sigma2, type = "identity"), sigma2)
})

# --- dispersion_link_inv ------------------------------------------------------

test_that("dispersion_link_inv returns appropriate values based on link type", {
  for (type in disp_types) {
    if (type == "log") {
      eta <- c(-2, -1, 0, 1, 2)
      res <- dispersion_link_inv(eta, type = type)
      expect_true(all(res > 0), label = paste(type, "returns positive values"))

    } else if (type == "sqrt") {
      eta <- c(-2, -1, 1, 2)
      res <- dispersion_link_inv(eta, type = type)
      expect_true(all(res > 0), label = paste(type, "returns positive values when eta != 0"))

      expect_equal(dispersion_link_inv(0, type = "sqrt"), 0)

    } else if (type == "identity") {
      eta <- c(0.1, 0.5, 1, 2)
      res <- dispersion_link_inv(eta, type = type)
      expect_true(all(res > 0), label = paste(type, "returns positive values when eta > 0"))

    }
  }
})

test_that("dispersion_link_inv is the inverse of dispersion_link for valid inputs", {
  sigma2 <- c(0.3, 0.8, 1.5, 3.0)
  for (type in disp_types) {
    eta <- dispersion_link(sigma2, type = type)
    sigma2_back <- dispersion_link_inv(eta, type = type)
    expect_equal(sigma2_back, sigma2, tolerance = 1e-10,
                 label = paste(type, "round-trip"))

    if (type %in% c("identity", "sqrt")) {
      expect_true(all(sigma2 > 0), label = paste(type, "sigma2 must be positive"))
    }
  }
})

test_that("dispersion_link_inv requires appropriate eta for identity and sqrt links", {
  expect_true(all(dispersion_link_inv(c(0.1, 0.5, 2), type = "identity") > 0))

  expect_true(all(dispersion_link_inv(c(-2, -1, 1, 2), type = "sqrt") > 0))
  expect_equal(dispersion_link_inv(0, type = "sqrt"), 0)
})

# --- dispersion_link_deriv1 ---------------------------------------------------

test_that("dispersion_link_deriv1 rejects non-positive sigma2", {
  for (type in disp_types) {
    expect_error(dispersion_link_deriv1(0,    type = type), "positive",
                 label = paste(type, "sigma2 = 0"))
    expect_error(dispersion_link_deriv1(-0.1, type = type), "positive",
                 label = paste(type, "sigma2 < 0"))
  }
})

test_that("dispersion_link_deriv1 returns finite values for all types", {
  sigma2 <- c(0.5, 1.0, 2.0)
  for (type in disp_types) {
    res <- dispersion_link_deriv1(sigma2, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "finite"))
  }
})

test_that("dispersion_link_deriv1 log equals 1 / sigma2", {
  sigma2 <- c(0.5, 1.0, 2.0)
  expect_equal(dispersion_link_deriv1(sigma2, type = "log"), 1 / sigma2,
               tolerance = 1e-12)
})

test_that("dispersion_link_deriv1 identity returns a vector of ones", {
  sigma2 <- c(0.3, 1.0, 2.5)
  res    <- dispersion_link_deriv1(sigma2, type = "identity")
  expect_equal(res, rep(1, 3))
})

test_that("dispersion_link_deriv1 approximates numerical derivative for all types", {
  sigma2 <- 1.0
  h      <- 1e-6
  for (type in disp_types) {
    num_deriv <- (dispersion_link(sigma2 + h, type) -
                    dispersion_link(sigma2 - h, type)) / (2 * h)
    ana_deriv <- dispersion_link_deriv1(sigma2, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical"))
  }
})

# --- dispersion_link_inv_deriv1 -----------------------------------------------

test_that("dispersion_link_inv_deriv1 returns appropriate values based on link type", {
  for (type in disp_types) {
    if (type == "log") {
      eta <- c(-2, -1, 0, 1, 2)
      res <- dispersion_link_inv_deriv1(eta, type = type)
      expect_true(all(is.finite(res)), label = paste(type, "finite"))
      expect_true(all(res > 0),        label = paste(type, "positive"))

    } else if (type == "sqrt") {
      eta <- c(-2, -1, 1, 2)
      res <- dispersion_link_inv_deriv1(eta, type = type)
      expect_true(all(is.finite(res)), label = paste(type, "finite"))

      expect_true(all(res != 0), label = paste(type, "non-zero derivative"))

      expect_equal(dispersion_link_inv_deriv1(0, type = "sqrt"), 0)

    } else if (type == "identity") {
      eta <- c(-2, -1, 0, 1, 2)
      res <- dispersion_link_inv_deriv1(eta, type = type)
      expect_true(all(is.finite(res)), label = paste(type, "finite"))
      expect_true(all(res == 1),       label = paste(type, "equals 1"))

    }
  }
})

test_that("dispersion_link_inv_deriv1 log equals exp(eta)", {
  eta <- c(-1, 0, 1, 2)
  expect_equal(dispersion_link_inv_deriv1(eta, type = "log"), exp(eta),
               tolerance = 1e-12)
})

test_that("dispersion_link_inv_deriv1 identity returns a vector of ones", {
  eta <- c(-1, 0, 1, 2)
  res <- dispersion_link_inv_deriv1(eta, type = "identity")
  expect_equal(res, rep(1, 4))
})

test_that("dispersion_link_inv_deriv1 approximates numerical derivative of inverse", {
  test_cases <- list(
    log = 0.5,
    sqrt = 2.0,
    identity = 0.5
  )

  h <- 1e-6
  for (type in disp_types) {
    eta <- test_cases[[type]]
    num_deriv <- (dispersion_link_inv(eta + h, type) -
                    dispersion_link_inv(eta - h, type)) / (2 * h)
    ana_deriv <- dispersion_link_inv_deriv1(eta, type)
    expect_equal(ana_deriv, num_deriv, tolerance = 1e-5,
                 label = paste(type, "numerical vs analytical"))
  }
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# ---------------------------------------------------------------------------

test_that("press requires objects of class simplexregression", {
  expect_error(press(list()),           "class 'simplexregression'")
  expect_error(press(fit_plogit1, "x"), "class 'simplexregression'")
})

test_that("press rejects invalid type", {
  expect_error(press(fit_plogit1, type = "invalid"))
})

test_that("press returns a named numeric vector for a single model", {
  result <- press(fit_plogit1)
  expect_true(is.numeric(result))
  expect_named(result, c("P2", "P2_c", "PRESS"))
})

test_that("press returns a data.frame for multiple models", {
  result <- press(fit_plogit1, fit_logit)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("P2", "P2_c", "PRESS"))
  expect_equal(nrow(result), 2)
})

test_that("press uses named arguments as row names", {
  result <- press(m1 = fit_plogit1, m2 = fit_logit)
  expect_equal(rownames(result), c("m1", "m2"))
})

test_that("press returns finite values for a single model", {
  result <- press(fit_plogit1)
  expect_true(all(is.finite(result)))
})

test_that("press returns finite values for multiple models", {
  result <- press(fit_plogit1, fit_logit)
  # data.frame: check each column
  expect_true(all(is.finite(result$P2)))
  expect_true(all(is.finite(result$P2_c)))
  expect_true(all(is.finite(result$PRESS)))
})

test_that("press PRESS statistic is positive", {
  result <- press(fit_plogit1)
  expect_true(result["PRESS"] > 0)
})

test_that("press works with type = 'biasvariance'", {
  result <- press(fit_plogit1, type = "biasvariance")
  expect_named(result, c("P2", "P2_c", "PRESS"))
  expect_true(all(is.finite(result)))
})

test_that("press works with fixed link model", {
  result <- press(fit_logit)
  expect_named(result, c("P2", "P2_c", "PRESS"))
  expect_true(all(is.finite(result)))
  expect_true(result["PRESS"] > 0)
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_plogit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# ---------------------------------------------------------------------------

test_that("penalized.ic requires objects of class simplexregression", {
  expect_error(penalized.ic(list()),          "class 'simplexregression'")
  expect_error(penalized.ic(fit_plogit1, "x"), "class 'simplexregression'")
})

test_that("penalized.ic returns a data.frame with correct columns (kappa > 0)", {
  result <- penalized.ic(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("df", "AICc", "BICc", "HQICc"))
  expect_equal(nrow(result), 2)
})

test_that("penalized.ic returns standard column names when kappa = 0", {
  result <- penalized.ic(fit_plogit1, fit_plogit2, kappa = 0, verbose = FALSE)
  expect_named(result, c("df", "AIC", "BIC", "HQIC"))
})

test_that("penalized.ic returns finite numeric values", {
  result <- penalized.ic(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expect_true(all(is.finite(result$AICc)))
  expect_true(all(is.finite(result$BICc)))
  expect_true(all(is.finite(result$HQICc)))
})

test_that("penalized.ic uses named arguments as row names", {
  result <- penalized.ic(m1 = fit_plogit1, m2 = fit_plogit2,
                         kappa = 0.1, verbose = FALSE)
  expect_equal(rownames(result), c("m1", "m2"))
})

test_that("penalized.ic works with a single model", {
  result <- penalized.ic(fit_plogit1, kappa = 0.1, verbose = FALSE)
  expect_equal(nrow(result), 1)
  expect_true(all(is.finite(unlist(result))))
})

test_that("penalized.ic warns and sets kappa = 0 with non-parametric models", {
  expect_warning(
    penalized.ic(fit_plogit1, fit_logit, kappa = 0.1, verbose = FALSE),
    "kappa > 0 is only valid"
  )
})

test_that("penalized.ic returns invisibly when verbose = TRUE", {
  result <- withVisible(
    penalized.ic(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = TRUE)
  )
  expect_false(result$visible)
})

test_that("penalized.ic returns visibly when verbose = FALSE", {
  result <- withVisible(
    penalized.ic(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  )
  expect_true(result$visible)
})

test_that("penalized.ic df column matches number of estimated parameters", {
  result <- penalized.ic(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expected_df <- length(fit_plogit1$coefficients$mean) +
                 length(fit_plogit1$coefficients$dispersion) + 1L
  expect_equal(result$df[1], expected_df)
})

test_that("penalized.ic silently sets kappa = 0 ...", {
  result <- expect_no_warning(penalized.ic(fit_logit, verbose = TRUE))
  expect_named(result, c("df", "AIC", "BIC", "HQIC"))
  expect_true(all(is.finite(unlist(result))))
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_logit <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

all_types <- c("quantile", "pearson", "deviance", "standardized",
               "weighted", "variance", "biasvariance", "score",
               "dualscore", "response")

# ---------------------------------------------------------------------------

test_that("residuals.simplexregression requires an object of class simplexregression", {
  # After adding inherits() check to the function, both lines work correctly
  expect_error(residuals.simplexregression(list()),  "class 'simplexregression'")
  expect_error(residuals.simplexregression("model"), "class 'simplexregression'")
})

test_that("residuals.simplexregression rejects invalid type", {
  expect_error(residuals(fit_logit, type = "invalid"))
})

test_that("residuals.simplexregression returns a numeric vector of length n for all types", {
  for (type in all_types) {
    res <- residuals(fit_plogit1, type = type)
    expect_true(is.numeric(res),   label = paste(type, "is numeric"))
    expect_equal(length(res), n,   label = paste(type, "has length n"))
  }
})

test_that("residuals.simplexregression returns finite values for all types", {
  for (type in all_types) {
    res <- residuals(fit_plogit1, type = type)
    expect_true(all(is.finite(res)), label = paste(type, "all finite"))
  }
})

test_that("residuals.simplexregression default type is quantile", {
  res_default  <- residuals(fit_plogit1)
  res_quantile <- residuals(fit_plogit1, type = "quantile")
  expect_equal(res_default, res_quantile)
})

test_that("residuals.simplexregression quantile residuals are approximately standard normal", {
  res <- residuals(fit_plogit1, type = "quantile")
  expect_true(abs(mean(res)) < 0.5)
  expect_true(abs(sd(res) - 1) < 0.5)
})

test_that("residuals.simplexregression response residuals equal y - fitted", {
  res  <- residuals(fit_plogit1, type = "response")
  diff <- as.vector(fit_plogit1$y) - as.vector(fit_plogit1$mu.fv)
  expect_equal(res, diff)
})

test_that("residuals.simplexregression pearson residuals have mean close to 0", {
  res <- residuals(fit_plogit1, type = "pearson")
  expect_true(abs(mean(res)) < 1)
})

test_that("residuals.simplexregression works with fixed link model", {
  for (type in all_types) {
    res <- residuals(fit_logit, type = type)
    expect_true(is.numeric(res),     label = paste("logit -", type, "is numeric"))
    expect_equal(length(res), n,     label = paste("logit -", type, "has length n"))
    expect_true(all(is.finite(res)), label = paste("logit -", type, "all finite"))
  }
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# ---------------------------------------------------------------------------
# plot.simplexregression
# ---------------------------------------------------------------------------

test_that("plot.simplexregression rejects invalid which values", {
  expect_error(plot(fit_plogit1, which = 0))
  expect_error(plot(fit_plogit1, which = 8))
})

test_that("plot.simplexregression runs without error for all plots (fixed link)", {
  expect_no_error({
    pdf(NULL)
    plot(fit_logit, which = 1:7, ask = FALSE)
    dev.off()
  })
})

test_that("plot.simplexregression runs without error for all plots (parametric link)", {
  expect_no_error({
    pdf(NULL)
    plot(fit_plogit1, which = 1:7, ask = FALSE)
    dev.off()
  })
})

test_that("plot.simplexregression works for each individual plot index", {
  for (i in 1:7) {
    expect_error(
      { pdf(NULL); plot(fit_plogit1, which = i, ask = FALSE); dev.off() },
      NA,
      info = paste("which =", i)
    )
  }
})

test_that("plot.simplexregression works with different residual types", {
  for (type in c("quantile", "pearson", "deviance", "response")) {
    expect_error(
      { pdf(NULL); plot(fit_plogit1, which = 1, type = type, ask = FALSE); dev.off() },
      NA,
      info = paste("type =", type)
    )
  }
})

test_that("plot.simplexregression returns invisibly", {
  pdf(NULL)
  result <- withVisible(plot(fit_plogit1, which = 1, ask = FALSE))
  dev.off()
  expect_false(result$visible)
})

test_that("plot.simplexregression threshold flags observations for Cook's distance (which = 6)", {
  # threshold = 0 garantees all observations exceed it, triggering the text() branch
  expect_no_error({
    pdf(NULL)
    plot(fit_plogit1, which = 6, ask = FALSE, threshold = 0.01)
    dev.off()
  })
})

test_that("plot.simplexregression threshold flags observations for gleverage (which = 7)", {
  expect_no_error({
    pdf(NULL)
    plot(fit_plogit1, which = 7, ask = FALSE, threshold = 0.1)
    dev.off()
  })
})

# ---------------------------------------------------------------------------
# halfnormal.plot
# ---------------------------------------------------------------------------

test_that("halfnormal.plot requires an object of class simplexregression", {
  expect_error(halfnormal.plot(list()),  "class 'simplexregression'")
  expect_error(halfnormal.plot("model"), "class 'simplexregression'")
})

test_that("halfnormal.plot runs without error with default arguments", {
  expect_no_error({
    pdf(NULL)
    halfnormal.plot(fit_plogit1, nsim = 10, seed = 42)
    dev.off()
  })
})

test_that("halfnormal.plot runs without error with parametric link", {
  expect_no_error({
    pdf(NULL)
    halfnormal.plot(fit_logit, nsim = 10, seed = 42)
    dev.off()
  })
})

test_that("halfnormal.plot works with different residual types", {
  for (type in c("weighted", "quantile", "pearson")) {
    expect_error(
      { pdf(NULL); halfnormal.plot(fit_plogit1, type = type, nsim = 10, seed = 42); dev.off() },
      NA,
      info = paste("type =", type)
    )
  }
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# ---------------------------------------------------------------------------

# --- print ---

test_that("print.simplexregression returns the object invisibly", {
  result <- withVisible(print(fit_plogit1))
  expect_false(result$visible)
  expect_s3_class(result$value, "simplexregression")
})

# --- summary ---

test_that("summary.simplexregression returns a summary object", {
  s <- summary(fit_plogit1)
  expect_s3_class(s, "summary.simplexregression")
})

test_that("summary.simplexregression contains required components", {
  s <- summary(fit_plogit1)
  expect_true(!is.null(s$coefficients$mean))
  expect_true(!is.null(s$coefficients$dispersion))
  expect_true(is.numeric(s$loglik))
  expect_true(is.numeric(s$nobs))
})

test_that("summary.simplexregression includes lambda for parametric model", {
  s <- summary(fit_plogit1)
  expect_true(s$parametric)
  expect_true(!is.null(s$coefficients$lambda))
})

test_that("summary.simplexregression lambda is NULL for fixed link model", {
  s <- summary(fit_logit)
  expect_false(s$parametric)
  expect_null(s$coefficients$lambda)
})

# --- coef ---

test_that("coef.simplexregression returns named numeric vector", {
  cf <- coef(fit_plogit1)
  expect_true(is.numeric(cf))
  expect_true(!is.null(names(cf)))
})

test_that("coef.simplexregression model = 'mean' returns mean coefficients only", {
  cf <- coef(fit_plogit1, model = "mean")
  expect_equal(length(cf), ncol(fit_plogit1$mu.x))
})

test_that("coef.simplexregression model = 'dispersion' returns dispersion coefficients only", {
  cf <- coef(fit_plogit1, model = "dispersion")
  expect_equal(length(cf), ncol(fit_plogit1$sigma2.x))
})

test_that("coef.simplexregression full model includes lambda for parametric", {
  cf <- coef(fit_plogit1, model = "full")
  expect_true("lambda" %in% names(cf))
})

test_that("coef.simplexregression full model for fixed link has no lambda", {
  cf <- coef(fit_logit, model = "full")
  expect_false("lambda" %in% names(cf))
})

# --- vcov ---

test_that("vcov.simplexregression returns a square matrix", {
  vc <- vcov(fit_plogit1)
  expect_true(is.matrix(vc))
  expect_equal(nrow(vc), ncol(vc))
})

test_that("vcov.simplexregression is positive definite (all eigenvalues > 0)", {
  vc <- vcov(fit_plogit1)
  expect_true(all(eigen(vc)$values > 0))
})

test_that("vcov.simplexregression model = 'mean' returns mean submatrix", {
  vc <- vcov(fit_plogit1, model = "mean")
  p  <- ncol(fit_plogit1$mu.x)
  expect_equal(dim(vc), c(p, p))
})

test_that("vcov.simplexregression dispersion model has correct names", {
  vc <- vcov(fit_plogit1, model = "dispersion")
  expect_equal(colnames(vc), names(fit_plogit1$coefficients$dispersion))
})

# --- logLik ---

test_that("logLik.simplexregression returns a logLik object", {
  ll <- logLik(fit_plogit1)
  expect_s3_class(ll, "logLik")
  expect_true(is.finite(as.numeric(ll)))
})

test_that("logLik.simplexregression df attribute equals number of parameters", {
  ll <- logLik(fit_plogit1)
  p  <- length(coef(fit_plogit1, model = "mean"))
  q  <- length(coef(fit_plogit1, model = "dispersion"))
  expect_equal(attr(ll, "df"), p + q + 1L)  # +1 for lambda
})

# --- fitted ---

test_that("fitted.simplexregression returns numeric vector of length n", {
  fv <- fitted(fit_plogit1)
  expect_true(is.numeric(fv))
  expect_length(fv, n)
  expect_true(all(fv > 0 & fv < 1))
})

# --- predict ---

test_that("predict.simplexregression with no newdata equals fitted values", {
  expect_equal(predict(fit_plogit1, type = "response"), fitted(fit_plogit1))
})

test_that("predict.simplexregression with no newdata type = 'dispersion'", {
  expect_equal(predict(fit_plogit1, type = "dispersion"), fit_plogit1$sigma2.fv)
})

test_that("predict.simplexregression with no newdata type = 'link'", {
  pred <- predict(fit_plogit1, type = "link")
  expect_type(pred, "list")
  expect_named(pred, c("mean", "dispersion"))
})

test_that("predict.simplexregression with newdata returns correct length", {
  newdat <- data.frame(x1 = c(0.2, 0.5, 0.8), x2 = c(0.3, 0.7, 0.9),
                       z1 = c(0.1, 0.4, 0.7))
  pred <- predict(fit_logit, newdata = newdat, type = "response")
  expect_length(pred, 3)
  expect_true(all(pred > 0 & pred < 1))
})

test_that("predict.simplexregression with newdata type = 'dispersion'", {
  newdat <- data.frame(x1 = c(0.2, 0.5), x2 = c(0.3, 0.7), z1 = c(0.1, 0.4))
  pred <- predict(fit_logit, newdata = newdat, type = "dispersion")
  expect_length(pred, 2)
  expect_true(all(pred > 0))
})

test_that("predict.simplexregression with newdata type = 'link'", {
  newdat <- data.frame(x1 = c(0.2, 0.5), x2 = c(0.3, 0.7), z1 = c(0.1, 0.4))
  pred <- predict(fit_logit, newdata = newdat, type = "link")
  expect_type(pred, "list")
  expect_length(pred$mean, 2)
  expect_length(pred$dispersion, 2)
})

test_that("predict.simplexregression with newdata returns parametric response", {
  # covers parametric_mean_link_inv branch with newdata
  newdat <- data.frame(x1 = c(0.2, 0.5), x2 = c(0.3, 0.7), z1 = c(0.1, 0.4))
  pred <- predict(fit_plogit1, newdata = newdat, type = "response")
  expect_length(pred, 2)
  expect_true(all(pred > 0 & pred < 1))
})

test_that("predict.simplexregression works for model without dispersion covariates", {
  # covers else branch: no pipe → z_new = matrix(1, ...)
  fit_nodispers <- simplexreg(y ~ x1 + x2, data = data, link.mu = "logit")
  newdat <- data.frame(x1 = c(0.2, 0.5), x2 = c(0.3, 0.7))
  pred <- predict(fit_nodispers, newdata = newdat, type = "response")
  expect_length(pred, 2)
  expect_true(all(pred > 0 & pred < 1))
})

test_that("predict.simplexregression works with intercept-only dispersion '| 1'", {
  # covers the trimws(parts[2]) == '1' branch → z_new = matrix(1, ...)
  fit_disp1 <- simplexreg(y ~ x1 + x2 | 1, data = data, link.mu = "logit")
  newdat <- data.frame(x1 = c(0.2, 0.5), x2 = c(0.3, 0.7))
  pred <- predict(fit_disp1, newdata = newdat, type = "response")
  expect_length(pred, 2)
  expect_true(all(pred > 0 & pred < 1))
})

# --- nobs ---

test_that("nobs.simplexregression returns correct number of observations", {
  expect_equal(nobs(fit_plogit1), n)
})

# --- deviance ---

test_that("deviance.simplexregression returns a positive finite scalar", {
  d <- deviance(fit_plogit1)
  expect_length(d, 1)
  expect_true(is.finite(d))
  expect_true(d > 0)
})

test_that("deviance.simplexregression works when y is NULL", {
  fit_no_y <- fit_plogit1
  fit_no_y$y <- NULL
  d <- deviance(fit_no_y)
  expect_true(is.finite(d))
  expect_true(d > 0)
})

# --- AIC / BIC / HQIC ---

test_that("AIC.simplexregression returns a finite scalar for a single model", {
  a <- AIC(fit_plogit1)
  expect_length(a, 1)
  expect_true(is.finite(a))
})

test_that("AIC.simplexregression returns a data.frame for multiple models", {
  result <- AIC(fit_plogit1, fit_logit)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("df", "AIC"))
  expect_equal(nrow(result), 2)
})

test_that("BIC.simplexregression returns a finite scalar for a single model", {
  b <- BIC(fit_plogit1)
  expect_length(b, 1)
  expect_true(is.finite(b))
})

test_that("HQIC.simplexregression returns a finite scalar for a single model", {
  h <- HQIC(fit_plogit1)
  expect_length(h, 1)
  expect_true(is.finite(h))
})

test_that("BIC >= AIC for the same model (penalty is larger)", {
  expect_true(BIC(fit_plogit1) >= AIC(fit_plogit1))
})

test_that("BIC.simplexregression returns data.frame for multiple models", {
  result <- BIC(fit_plogit1, fit_logit)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("df", "BIC"))
  expect_equal(nrow(result), 2)
})

test_that("HQIC.simplexregression returns data.frame for multiple models", {
  result <- HQIC(fit_plogit1, fit_logit)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("df", "HQIC"))
  expect_equal(nrow(result), 2)
})

# --- hatvalues ---

test_that("hatvalues.simplexregression returns numeric vector of length n", {
  hv <- hatvalues(fit_plogit1)
  expect_true(is.numeric(hv))
  expect_length(hv, n)
})

test_that("hatvalues.simplexregression values are in (0, 1)", {
  hv <- hatvalues(fit_plogit1)
  expect_true(all(hv > 0 & hv < 1))
})

# --- cooks.distance ---

test_that("cooks.distance.simplexregression returns non-negative numeric vector", {
  cd <- cooks.distance(fit_plogit1)
  expect_true(is.numeric(cd))
  expect_length(cd, n)
  expect_true(all(cd >= 0))
})

test_that("cooks.distance.simplexregression works with type = 'weighted'", {
  cd <- cooks.distance(fit_plogit1, type = "weighted")
  expect_true(is.numeric(cd))
  expect_length(cd, n)
})

test_that("cooks.distance type pearson returns finite values", {
  cd <- cooks.distance(fit_plogit1, type = "pearson")
  expect_true(all(is.finite(cd)))
  expect_true(all(cd >= 0))
})

# --- simulate ---

test_that("simulate.simplexregression returns a data.frame with correct dimensions", {
  sim <- simulate(fit_plogit1, nsim = 3, seed = 1)
  expect_s3_class(sim, "data.frame")
  expect_equal(nrow(sim), n)
  expect_equal(ncol(sim), 3)
})

test_that("simulate.simplexregression simulated values are in (0, 1)", {
  sim <- simulate(fit_plogit1, nsim = 1, seed = 1)
  expect_true(all(sim[[1]] > 0 & sim[[1]] < 1))
})

test_that("simulate.simplexregression works with seed = NULL", {
  sim <- simulate(fit_plogit1, nsim = 2, seed = NULL)
  expect_s3_class(sim, "data.frame")
  expect_equal(ncol(sim), 2)
})

test_that("simulate.simplexregression initializes RNG when .Random.seed does not exist", {
  if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
    old_seed <- .Random.seed
    rm(".Random.seed", envir = .GlobalEnv)
    on.exit(assign(".Random.seed", old_seed, envir = .GlobalEnv))
  }
  sim <- simulate(fit_plogit1, nsim = 1, seed = NULL)
  expect_s3_class(sim, "data.frame")
  expect_true(all(sim[[1]] > 0 & sim[[1]] < 1))
})

# --- formula / terms / model.matrix ---

test_that("formula.simplexregression returns the model formula", {
  f <- formula(fit_plogit1)
  expect_true(inherits(f, "formula") || inherits(f, "Formula"))
})

test_that("model.matrix.simplexregression returns the mean design matrix", {
  mm <- model.matrix(fit_plogit1, model = "mean")
  expect_equal(nrow(mm), n)
  expect_equal(ncol(mm), ncol(fit_plogit1$mu.x))
})

test_that("terms.simplexregression defaults to mean model", {
  trms <- terms(fit_plogit1)
  expect_true(!is.null(trms))
})

# --- update ---
test_that("update.simplexregression refits the model with new arguments", {
  fit_updated <- update(fit_logit, link.mu = "probit")
  expect_s3_class(fit_updated, "simplexregression")
  expect_equal(fit_updated$mu.link, "probit")
})

test_that("update.simplexregression errors when object has no call component", {
  obj_no_call <- fit_logit
  obj_no_call$call <- NULL
  expect_error(
    update.simplexregression(obj_no_call),
    "need an object with call component"
  )
})

test_that("update.simplexregression works with evaluate = FALSE", {
  result <- update(fit_logit, evaluate = FALSE)
  expect_true(is.call(result))
})

test_that("update.simplexregression updates formula", {
  fit_updated <- update(fit_logit, formula. = . ~ x1)
  expect_s3_class(fit_updated, "simplexregression")
})

test_that("update.simplexregression adds new argument not in original call", {
  fit_updated <- update(fit_logit, link.sigma2 = "sqrt")
  expect_s3_class(fit_updated, "simplexregression")
  expect_equal(fit_updated$sigma2.link, "sqrt")
})

# --- terms ---

test_that("terms.simplexregression works for mean model", {
  trms <- terms(fit_plogit1, model = "mean")
  expect_true(!is.null(trms))
})

test_that("terms.simplexregression works for dispersion model", {
  trms <- terms(fit_plogit1, model = "dispersion")
  expect_true(!is.null(trms))
})

# --- model.frame ---

test_that("model.frame.simplexregression returns stored model frame", {
  fit_with_model <- simplexreg(y ~ x1 + x2 | z1, data = data,
                               link.mu = "logit", model = TRUE)
  mf <- model.frame(fit_with_model)
  expect_s3_class(mf, "data.frame")
  expect_equal(nrow(mf), n)
})

test_that("model.frame.simplexregression reconstructs frame when not stored", {
  fit_no_model <- simplexreg(y ~ x1 + x2 | z1, data = data,
                             link.mu = "logit", model = FALSE)
  expect_null(fit_no_model$model)
  mf <- model.frame(fit_no_model)
  expect_s3_class(mf, "data.frame")
  expect_equal(nrow(mf), n)
})

# --- model.matrix for dispersion ---

test_that("model.matrix.simplexregression returns dispersion design matrix", {
  mm <- model.matrix(fit_plogit1, model = "dispersion")
  expect_equal(nrow(mm), n)
  expect_equal(ncol(mm), ncol(fit_plogit1$sigma2.x))
})

# --- df.residual ---

test_that("df.residual.simplexregression returns correct value", {
  dfr <- df.residual(fit_plogit1)
  expect_true(is.numeric(dfr))
  expect_equal(dfr, fit_plogit1$df.residual)
})

# --- print.summary ---

test_that("print.summary.simplexregression works without errors", {
  s <- summary(fit_plogit1)
  expect_output(print(s), "Call:")
  expect_output(print(s), "Coefficients")
})

# --- bread (sandwich) ---

test_that("bread.simplexregression returns a matrix", {
  skip_if_not_installed("sandwich")
  br <- bread(fit_plogit1)
  expect_true(is.matrix(br))
  expect_equal(dim(br), dim(vcov(fit_plogit1)))
})

test_that("bread.simplexregression returns correct dimension", {
  skip_if_not_installed("sandwich")
  br <- bread(fit_plogit1)
  npar <- length(coef(fit_plogit1, model = "full"))
  expect_equal(dim(br), c(npar, npar))
})

# --- estfun (sandwich) ---

test_that("estfun.simplexregression returns a matrix", {
  skip_if_not_installed("sandwich")
  ef <- estfun(fit_logit)
  expect_true(is.matrix(ef))
  expect_equal(nrow(ef), n)
})

test_that("estfun.simplexregression has correct column names", {
  skip_if_not_installed("sandwich")
  ef <- estfun(fit_plogit1)
  expected_names <- names(coef(fit_plogit1, model = "full"))
  expect_equal(colnames(ef), expected_names)
})

test_that("estfun.simplexregression works with plogit2 parametric link", {
  skip_if_not_installed("sandwich")
  fit_plogit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
  ef <- estfun(fit_plogit2)
  expect_true(is.matrix(ef))
  expect_equal(nrow(ef), n)
  expect_equal(ncol(ef), length(coef(fit_plogit2, model = "full")))
})

# --- coeftest (lmtest) ---

test_that("coeftest.simplexregression works", {
  skip_if_not_installed("lmtest")
  ct <- coeftest(fit_plogit1)
  expect_s3_class(ct, "coeftest")
})

# --- lrtest (lmtest) ---

test_that("lrtest.simplexregression works", {
  skip_if_not_installed("lmtest")
  lt <- lrtest(fit_plogit1, fit_logit)
  expect_s3_class(lt, "anova")
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_plogit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")
fit_probit  <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "probit")

# ---------------------------------------------------------------------------

test_that("ss.penalized requires at least 2 models", {
  expect_error(ss.penalized(fit_plogit1), "At least 2 models are required")
})

test_that("ss.penalized requires objects of class simplexregression", {
  expect_error(ss.penalized(fit_plogit1, list()),    "class 'simplexregression'")
  expect_error(ss.penalized(fit_plogit1, "model"),   "class 'simplexregression'")
})

test_that("ss.penalized returns a data.frame with correct columns", {
  result <- ss.penalized(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("df", "SS"))
  expect_equal(nrow(result), 2)
})

test_that("ss.penalized uses named arguments as row names", {
  result <- ss.penalized(m1 = fit_plogit1, m2 = fit_plogit2,
                         kappa = 0.1, verbose = FALSE)
  expect_equal(rownames(result), c("m1", "m2"))
})

test_that("ss.penalized returns finite numeric SS values", {
  result <- ss.penalized(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expect_true(all(is.finite(result$SS)))
  expect_true(all(is.numeric(result$SS)))
})

test_that("ss.penalized works with kappa = 0 (standard Scout Score)", {
  result <- ss.penalized(fit_logit, fit_probit, kappa = 0, verbose = FALSE)
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_true(all(is.finite(result$SS)))
})

test_that("ss.penalized works with more than 2 models", {
  result <- ss.penalized(fit_plogit1, fit_plogit2, fit_logit, fit_probit,
                         kappa = 0, verbose = FALSE)
  expect_equal(nrow(result), 4)
  expect_true(all(is.finite(result$SS)))
})

test_that("ss.penalized warns and sets kappa = 0 when models lack parametric link", {
  expect_warning(
    ss.penalized(fit_plogit1, fit_logit, kappa = 0.1, verbose = FALSE),
    "kappa > 0 is only valid"
  )
})

test_that("ss.penalized returns invisibly when verbose = TRUE", {
  result <- withVisible(
    ss.penalized(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = TRUE)
  )
  expect_false(result$visible)
})

test_that("ss.penalized returns visibly when verbose = FALSE", {
  result <- withVisible(
    ss.penalized(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  )
  expect_true(result$visible)
})

test_that("ss.penalized df column matches number of estimated parameters", {
  result <- ss.penalized(fit_plogit1, fit_plogit2, kappa = 0.1, verbose = FALSE)
  expect_equal(result$df[1], n - fit_plogit1$df.residual)
  expect_equal(result$df[2], n - fit_plogit2$df.residual)
})

test_that("ss.penalized silently sets kappa = 0 when models lack parametric link and kappa not specified", {
  result <- expect_no_warning(
    ss.penalized(fit_logit, fit_probit, verbose = TRUE)
  )
  expect_true(all(is.finite(result$SS)))
})

library(testthat)
library(SimplexRegression)

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = data,
                           link.mu = "logit")
fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit_plogit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")

# ---------------------------------------------------------------------------

test_that("scoretest requires an object of class simplexregression", {
  expect_error(scoretest(list()),   "class 'simplexregression'")
  expect_error(scoretest("model"),  "class 'simplexregression'")
})

test_that("scoretest requires logit link in the model under H0", {
  expect_error(
    scoretest(fit_plogit1, link.mu = "plogit2"),
    "only valid for models fitted with the logit mean link"
  )
  expect_error(
    scoretest(fit_plogit2, link.mu = "plogit1"),
    "only valid for models fitted with the logit mean link"
  )
})

test_that("scoretest returns an object of class htest", {
  result <- scoretest(fit_logit, link.mu = "plogit1")
  expect_s3_class(result, "htest")
})

test_that("scoretest returns all required htest components", {
  result <- scoretest(fit_logit, link.mu = "plogit1")
  expect_named(result, c("statistic", "parameter", "p.value", "method", "data.name"),
               ignore.order = TRUE)
})

test_that("scoretest returns a finite and non-negative test statistic", {
  result1 <- scoretest(fit_logit, link.mu = "plogit1")
  result2 <- scoretest(fit_logit, link.mu = "plogit2")

  expect_true(is.finite(result1$statistic))
  expect_true(result1$statistic >= 0)

  expect_true(is.finite(result2$statistic))
  expect_true(result2$statistic >= 0)
})

test_that("scoretest returns a p-value between 0 and 1", {
  result1 <- scoretest(fit_logit, link.mu = "plogit1")
  result2 <- scoretest(fit_logit, link.mu = "plogit2")

  expect_true(result1$p.value >= 0 && result1$p.value <= 1)
  expect_true(result2$p.value >= 0 && result2$p.value <= 1)
})

test_that("scoretest has 1 degree of freedom", {
  result <- scoretest(fit_logit, link.mu = "plogit1")
  expect_equal(unname(result$parameter), 1L)
})

test_that("scoretest data.name contains the alternative link function", {
  result1 <- scoretest(fit_logit, link.mu = "plogit1")
  result2 <- scoretest(fit_logit, link.mu = "plogit2")

  expect_true(grepl("plogit1", result1$data.name))
  expect_true(grepl("plogit2", result2$data.name))
})

test_that("scoretest produces a finite p-value when data come from a non-logit link", {
  # Data generated with plogit1 — the test should detect departure from logit
  result <- scoretest(fit_logit, link.mu = "plogit1")
  expect_true(is.finite(result$p.value))
})

################################################################################
#                     SIMPLEX REGRESSION - S3 METHODS                          #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: S3 methods for simplexregression objects including summary,     #
#              print, predict, coef, vcov, logLik, AIC, BIC, and others        #
################################################################################

# ==============================================================================
# HELPERS
# ==============================================================================
is_parametric <- function(object) {
  !is.na(object$coefficients$lambda)
}

.npar <- function(object) {
  length(object$coefficients$mean) +
    length(object$coefficients$dispersion) +
    if(is_parametric(object)) 1L else 0L
}

#' @title Methods for simplexregression Objects
#' @description
#' Methods for extracting information from fitted simplex regression model objects
#' of class \code{simplexregression}.
#'
#' @param object,x An object of class \code{simplexregression}.
#' @param model Character specifying for which component of the model
#' coefficients/covariance should be extracted.
#' @param digits Number of digits to printing.
#' @param newdata Optional data frame for prediction.
#' @param formula A model formula or terms object.
#' @param type Character indicating type of predictions: fitted means of the
#' response (default, "response"), corresponding linear
#' predictor ("link") or fitted dispersion parameter ("dispersion").
#' @param formula. Changes to the formula.
#' @param evaluate If true evaluate the new call else return the call.
#' @param nsim number of response vectors to simulate. Defaults to 1.
#' @param seed an object specifying if and how the random number generator
#' should be initialized.
#' @param k weight of the penalty term in AIC. Default is 2.
#' @param vcov. a specification of the covariance matrix of the estimated coefficients.
#' @param df the degrees of freedom to be used.
#' @param ... Additional arguments.
#'
#' @name simplexreg.methods
#'
#' @return The return value depends on the method called:
#' \itemize{
#'   \item \code{print}: returns \code{x} invisibly;
#'   \item \code{summary}: returns an object of class
#'     \code{"summary.simplexregression"};
#'   \item \code{print.summary}: returns \code{x} invisibly;
#'   \item \code{coef}: named numeric vector of coefficients;
#'   \item \code{vcov}: numeric matrix (variance-covariance);
#'   \item \code{logLik}: object of class \code{"logLik"};
#'   \item \code{fitted}: numeric vector of fitted mean values;
#'   \item \code{predict}: numeric vector or list depending on \code{type};
#'   \item \code{nobs}: integer scalar (number of observations);
#'   \item \code{df.residual}: integer scalar (residual degrees of freedom);
#'   \item \code{deviance}: numeric scalar (total deviance);
#'   \item \code{formula}: the model formula;
#'   \item \code{terms}: the model terms for the selected submodel;
#'   \item \code{model.frame}: a data frame;
#'   \item \code{model.matrix}: numeric matrix of regressors;
#'   \item \code{update}: fitted model or call depending on \code{evaluate};
#'   \item \code{simulate}: a data frame with \code{nsim} columns;
#'   \item \code{AIC}, \code{BIC}, \code{HQIC}: numeric scalar (single model)
#'     or data frame (multiple models);
#'   \item \code{hatvalues}: numeric vector of leverage values;
#'   \item \code{cooks.distance}: numeric vector of Cook's distances;
#'   \item \code{bread}: numeric matrix;
#'   \item \code{estfun}: numeric matrix of score contributions;
#'   \item \code{coeftest}: object of class \code{"coeftest"};
#'   \item \code{lrtest}: object of class \code{"anova"}.
#' }
#'
#' @seealso \code{\link{simplexreg}}.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Extract information
#' summary(fit)
#' coef(fit)
#' vcov(fit)
#' logLik(fit)
#' fitted(fit)
#' AIC(fit)
#' BIC(fit)
#' HQIC(fit)
#' hatvalues(fit)
#' cooks.distance(fit)
#'
#' @aliases print.simplexregression summary.simplexregression coef.simplexregression
#' @aliases vcov.simplexregression logLik.simplexregression fitted.simplexregression
#' @aliases predict.simplexregression nobs.simplexregression df.residual.simplexregression
#' @aliases deviance.simplexregression formula.simplexregression terms.simplexregression
#' @aliases model.frame.simplexregression model.matrix.simplexregression update.simplexregression
#' @aliases simulate.simplexregression AIC.simplexregression BIC.simplexregression
#' @aliases hatvalues.simplexregression cooks.distance.simplexregression bread.simplexregression
#' @aliases estfun.simplexregression coeftest.simplexregression lrtest.simplexregression

NULL

#' @rdname simplexreg.methods
#' @export
print.simplexregression <- function(x, digits = max(3, getOption("digits") - 3),
                                    ...) {

  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)),
      "", sep = "\n")

  converged <- is.null(x$optim$convergence) || x$optim$convergence == 0

  if(!converged) {
    cat("model did not converge\n")
  } else {
    # Coefficients (Mean model)
    if(length(x$coefficients$mean)) {
      cat(sprintf("Coefficients (mean model with %s link):\n", x$mu.link))
      coef_mean <- x$coefficients$mean
      if (!is.null(x$x_names)) names(coef_mean) <- x$x_names
      print.default(format(coef_mean, digits = digits), print.gap = 2, quote = FALSE)
      cat("\n")
    } else {
      cat("No coefficients (in mean model)\n\n")
    }

    # Coefficients (Dispersion model)
    if(length(x$coefficients$dispersion)) {
      cat(sprintf("Coefficients (dispersion model with %s link):\n",
                  x$sigma2.link))
      coef_disp <- x$coefficients$dispersion
      if (!is.null(x$z_names)) names(coef_disp) <- x$z_names
      print.default(format(coef_disp, digits = digits), print.gap = 2, quote = FALSE)
      cat("\n")
    } else {
      cat("No coefficients (in dispersion model)\n\n")
    }

    if (is_parametric(x)) {
      cat(sprintf("Link function parameter (parametric %s model)\nlambda: %s\n\n",
                  x$mu.link,
                  format(x$coefficients$lambda, digits = digits)))
    }
  }

  invisible(x)
}

#' @rdname simplexreg.methods
#' @importFrom stats pnorm na.omit
#' @export
summary.simplexregression <- function(object, ...) {

  parametric <- is_parametric(object)

  P2 <- press(object)[1]

  # Extract coefficients
  coef_mean <- object$coefficients$mean
  coef_disp <- object$coefficients$dispersion
  coef_lambda <- object$coefficients$lambda

  # Get standard errors from variance-covariance matrix
  vcov_matrix <- object$vcov
  se <- sqrt(diag(vcov_matrix))

  # Dimensions
  p <- length(coef_mean)
  q <- length(coef_disp)

  # Standard errors
  se_mean <- se[1:p]
  se_disp <- se[(p+1):(p+q)]

  cf <- c(coef_mean, coef_disp)
  if(parametric) cf <- c(cf, coef_lambda)

  se_all <- se[seq_len(length(cf))]
  cf_table <- cbind(cf, se_all, cf/se_all, 2 * pnorm(-abs(cf/se_all)))
  colnames(cf_table) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")

  coef_table_mean <- cf_table[1:p, , drop = FALSE]
  coef_table_disp <- cf_table[(p+1):(p+q), , drop = FALSE]

  # Use regressor names if available
  if (!is.null(object$x_names)) {
    rownames(coef_table_mean) <- object$x_names
  } else {
    rownames(coef_table_mean) <- names(coef_mean)
  }

  if (!is.null(object$z_names)) {
    rownames(coef_table_disp) <- object$z_names
  } else {
    rownames(coef_table_disp) <- names(coef_disp)
  }

  # Lambda parameter (if parametric)
  if (parametric) {
    lambda_table <- cf_table[p+q+1, , drop = FALSE]
    rownames(lambda_table) <- "lambda"
  } else {
    lambda_table <- NULL
  }

  mytail <- function(x) x[length(x)]
  if (!is.null(object$optim$counts)) {
    object$iterations <- c("optim" = as.vector(mytail(na.omit(object$optim$counts))))
  } else {
    object$iterations <- NA
  }

  converged <- is.null(object$optim$convergence) || (object$optim$convergence == 0)

  object$fitted.values <- object$terms <- object$model <- object$y <-
    object$x <- object$z <- object$levels <- object$contrasts <-
    object$start <- NULL

  result <- list(
    call = object$call,
    coefficients = list(
      mean = coef_table_mean,
      dispersion = coef_table_disp,
      lambda = lambda_table
    ),
    parametric = parametric,
    mu.link = object$mu.link,
    sigma2.link = object$sigma2.link,
    loglik = object$loglik,
    aic = object$AIC,
    bic = object$BIC,
    hqic = object$HQ,
    nobs = object$nobs,
    df.residual = object$df.residual,
    iterations = c("optim" = object$iterations, "scoring" = object$scoring),
    method = ifelse(is.null(object$method), "BFGS", object$method),
    R2_N = object$R2_N,
    R2_FC = object$R2_FC,
    P2 = P2,
    residuals = if(!is.null(object$residuals)) object$residuals else NULL,
    residuals.type = if(!is.null(object$residuals)) "quantile" else NULL,
    converged = converged
  )

  class(result) <- "summary.simplexregression"
  return(result)
}

#' @rdname simplexreg.methods
#' @importFrom stats printCoefmat
#' @export
print.summary.simplexregression <- function(x, digits = max(3, getOption("digits") - 3),
                                            ...) {
  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)),
      "", sep = "\n")

  if(x$converged) {

    if(!is.null(x$residuals)) {
      cat(sprintf("%s:\n", "Quantile residuals"))
      qres <- quantile(x$residuals)
      names(qres) <- c("Min", "1Q", "Median", "3Q", "Max")
      print(formatC(qres, digits = digits, format = "f"), quote = FALSE)
    }

    # Coefficients (Mean model)
    if(NROW(x$coefficients$mean)) {
      cat(sprintf("\nCoefficients (mean model with %s link):\n", x$mu.link))
      coef_mean_print <- x$coefficients$mean
      printCoefmat(coef_mean_print, digits = digits, signif.legend = FALSE)
    } else {
      cat("\nNo coefficients (in mean model)\n")
    }

    # Coefficients (Dispersion model)
    if(NROW(x$coefficients$dispersion)) {
      cat(sprintf("\nDispersion coefficients (dispersion model with %s link):\n",
                  x$sigma2.link))
      coef_disp_print <- x$coefficients$dispersion
      printCoefmat(coef_disp_print, digits = digits, signif.legend = FALSE)
    } else {
      cat("\nNo coefficients (in dispersion model)\n")
    }

    # Lambda parameter (if parametric)
    if(x$parametric && !is.null(x$coefficients$lambda)) {
      cat(sprintf("\nLink function parameter (parametric %s model):\n", x$mu.link))
      lambda_print <- x$coefficients$lambda[, c("Estimate", "Std. Error"), drop = FALSE]
      print(formatC(lambda_print, digits = digits, format = "f"), quote = FALSE)
    }

    if(getOption("show.signif.stars") &&
       any(do.call("rbind", x$coefficients)[, 4L] < 0.1, na.rm = TRUE)) {
      cat("---\nSignif. codes: ",
          "0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1", "\n")
    }

    # Model fit statistics
    cat("\nLog-likelihood:", formatC(x$loglik, digits = digits, format = "f"),
        "on", sum(sapply(x$coefficients, NROW)), "Df")
    cat("\nAIC:", formatC(x$aic, digits = digits, format = "f"))
    cat("  BIC:", formatC(x$bic, digits = digits, format = "f"))
    cat("  HQIC:", formatC(x$hqic, digits = digits, format = "f"))
    cat("\nPseudo R-squared (Nagelkerke):", formatC(x$R2_N, digits = digits, format = "f"))
    cat("\nPseudo R-squared (Ferrari and Cribari-Neto):", formatC(x$R2_FC, digits = digits, format = "f"))
    cat("\nP-squared (Espinheira, Silva and Lima):", formatC(x$P2, digits = digits, format = "f"))
    cat("\nNumber of observations:", x$nobs)
    cat(paste("\nNumber of iterations:", x$iterations[1L],
              sprintf("(%s) +", x$method), x$iterations[2L], paste("(Fisher scoring)", "\n")))
  } else{
    cat("model did not converge\n")
   }

  invisible(x)
}

#' @rdname simplexreg.methods
#' @importFrom stats coef
#' @export
coef.simplexregression <- function(object,
                                   model = c("full", "mean", "dispersion"),
                                   ...) {

  model <- match.arg(model)
  cf <- object$coefficients

  switch(model,
         "mean" = {
           cf$mean
         },
         "dispersion" = {
           cf$dispersion
         },
         "full" = {
           nam1 <- names(cf$mean)
           nam2 <- names(cf$dispersion)

           coefs <- c(cf$mean, cf$dispersion)
           names(coefs) <- c(
             nam1,
             if(identical(nam2, "(dispersion)")) "(dispersion)"
             else paste("(dispersion)", nam2, sep = "_")
           )

           if (is_parametric(object)) {
             coefs <- c(coefs, lambda = cf$lambda)
           }

           coefs
         }
  )
}

#' @rdname simplexreg.methods
#' @importFrom stats vcov
#' @export
vcov.simplexregression <- function(object,
                                   model = c("full", "mean", "dispersion"),
                                   ...) {

  model <- match.arg(model)

  vc <- object$vcov
  p <- length(object$coefficients$mean)
  q <- length(object$coefficients$dispersion)

  switch(model,
         "mean" = {
           vc[seq.int(length.out = p), seq.int(length.out = p), drop = FALSE]
         },
         "dispersion" = {
           vc_disp <- vc[seq.int(length.out = q) + p,
                         seq.int(length.out = q) + p,
                         drop = FALSE]
           colnames(vc_disp) <- rownames(vc_disp) <- names(object$coefficients$dispersion)
           vc_disp
         },
         "full" = {
           vc
         }
  )
}

#' @rdname simplexreg.methods
#' @importFrom stats logLik
#' @export
logLik.simplexregression <- function(object, ...) {
  structure(object$loglik,
            df = .npar(object),
            nobs = object$nobs,
            class = "logLik")
}

#' @rdname simplexreg.methods
#' @method fitted simplexregression
#' @export
fitted.simplexregression <- function(object, ...) {
  object$fitted.values
}

#' @rdname simplexreg.methods
#' @importFrom stats na.pass
#' @importFrom Formula as.Formula
#' @method predict simplexregression
#' @export
predict.simplexregression <- function(object, newdata = NULL,
                                      type = c("response", "link", "dispersion"),
                                      ...) {
  type <- match.arg(type)
  parametric <- is_parametric(object)

  # Use fitted values if no new data
  if (is.null(newdata)) {
    return(switch(type,
                  response   = object$fitted.values,
                  link       = list(mean = object$mu.lp, dispersion = object$sigma2.lp),
                  dispersion = object$sigma2.fv
    ))
  }

  formula_full <- object$formula

  deparse1 <- function(x) paste(deparse(x), collapse = " ")
  has_pipe <- function(f) grepl("\\|", deparse1(f), fixed = FALSE)

  # ---- Extract X (mean submodel) -------------------------------------------
  if (has_pipe(formula_full)) {
    f_str       <- deparse1(formula_full)
    rhs_str     <- sub("^.*~\\s*", "", f_str)
    lhs_part    <- trimws(strsplit(rhs_str, "\\|")[[1]][1])
    rhs_formula <- as.formula(paste("~", lhs_part))
  } else {
    rhs_formula <- formula_full
    if (length(rhs_formula) == 3L) rhs_formula <- rhs_formula[-2]
  }

  Terms <- terms(rhs_formula, data = newdata)
  mf    <- model.frame(Terms, data = newdata, na.action = na.pass)
  x_new <- model.matrix(Terms, data = mf)

  # ---- Extract Z (dispersion submodel) -------------------------------------
  if (has_pipe(formula_full)) {
    f_str   <- deparse1(formula_full)
    rhs_str <- sub("^.*~\\s*", "", f_str)
    parts   <- strsplit(rhs_str, "\\|")[[1]]
    if (length(parts) > 1L && trimws(parts[2]) != "1") {
      rhs_disp   <- as.formula(paste("~", trimws(parts[2])))
      Terms_disp <- terms(rhs_disp, data = newdata)
      mf_disp    <- model.frame(Terms_disp, data = newdata, na.action = na.pass)
      z_new      <- model.matrix(Terms_disp, data = mf_disp)
    } else {
      z_new <- matrix(1, nrow = nrow(newdata), ncol = 1L)
    }
  } else {
    z_new <- matrix(1, nrow = nrow(newdata), ncol = 1L)
  }

  # ---- Coefficients --------------------------------------------------------
  beta  <- object$coefficients$mean
  delta <- object$coefficients$dispersion

  # ---- Prediction ----------------------------------------------------------
  if (type %in% c("response", "link")) eta1 <- as.vector(x_new %*% beta)
  if (type %in% c("dispersion", "link")) eta2 <- as.vector(z_new %*% delta)

  switch(type,
         response   = {
           if (parametric)
             parametric_mean_link_inv(eta1, object$lambda.fv, object$mu.link)
           else
             fixed_mean_link_inv(eta1, object$mu.link)
         },
         dispersion = dispersion_link_inv(eta2, object$sigma2.link),
         link       = list(mean = eta1, dispersion = eta2)
  )
}

#' @rdname simplexreg.methods
#' @importFrom stats nobs
#' @export
nobs.simplexregression <- function(object, ...) {
  object$nobs
}

#' @rdname simplexreg.methods
#' @export
df.residual.simplexregression <- function(object, ...) {
  object$nobs - .npar(object)
}

#' @rdname simplexreg.methods
#' @importFrom stats deviance
#' @export
deviance.simplexregression <- function(object, ...) {
  y <- if(is.null(object$y)) model.response(model.frame(object)) else object$y
  mu <- object$mu.fv
  yoneminy <- y * (1 - y)
  muonemu <- mu * (1 - mu)
  sum(((y - mu) / muonemu)^2 / yoneminy)
}

#' @rdname simplexreg.methods
#' @importFrom stats as.formula
#' @export
formula.simplexregression <- function(x, ...) {
  x$formula
}

#' @rdname simplexreg.methods
#' @export
terms.simplexregression <- function(x, model = c("mean", "dispersion"), ...) {
  model <- match.arg(model)
  x$terms[[model]]
}

#' @rdname simplexreg.methods
#' @export
model.frame.simplexregression <- function(formula, ...) {
  if (!is.null(formula$model)) return(formula$model)

  # Reconstruct model frame from the original call when model was not stored
  call          <- formula$call
  call[[1L]]    <- quote(stats::model.frame)
  call$formula  <- formula$formula
  call$link.mu     <- NULL
  call$link.sigma2 <- NULL
  call$model       <- NULL
  eval(call, parent.frame())
}

#' @rdname simplexreg.methods
#' @export
model.matrix.simplexregression <- function(object, model = c("mean", "dispersion"),
                                           ...) {
  model <- match.arg(model)

  switch(model,
         mean = object$mu.x,
         dispersion = object$sigma2.x)
}

#' @rdname simplexreg.methods
#' @importFrom stats formula update
#' @export
update.simplexregression <- function(object, formula., ..., evaluate = TRUE) {
  call <- object$call

  if(is.null(call)) {
    stop("need an object with call component")
  }

  extras <- match.call(expand.dots = FALSE)$...

  if(!missing(formula.)) {
    call$formula <- formula(update(Formula::Formula(formula(object)), formula.))
  }

  if(length(extras)) {
    existing <- !is.na(match(names(extras), names(call)))
    for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
    if(!all(existing)) {
      call <- c(as.list(call), extras[!existing])
      call <- as.call(call)
    }
  }

  if(evaluate) {
    eval(call, parent.frame())
  } else {
    call
  }
}

#' @rdname simplexreg.methods
#' @importFrom stats simulate
#' @export
simulate.simplexregression <- function(object, nsim = 1, seed = NULL, ...) {

  if (!is.null(seed)) set.seed(seed)
  if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
  RNGstate <- get(".Random.seed", envir = .GlobalEnv)

  mu <- object$mu.fv
  sigma2 <- object$sigma2.fv
  n <- length(mu)
  nm <- names(mu)

  s <- replicate(nsim, rsimplex(n, mu = mu, sigma2 = sigma2))

  s <- as.data.frame(s)
  names(s) <- paste("sim", seq_len(nsim), sep = "_")

  if (!is.null(nm)) {
    row.names(s) <- nm
  }

  attr(s, "seed") <- RNGstate
  return(s)
}

#' @rdname simplexreg.methods
#' @importFrom stats AIC
#' @export
AIC.simplexregression <- function(object, ..., k = 2) {
  objects <- list(object, ...)

  aic_fn <- function(obj) -2 * obj$loglik + k * .npar(obj)

  if (length(objects) == 1) return(aic_fn(object))

  model_names <- c(deparse(substitute(object)),
                   sapply(substitute(list(...)), deparse)[-1])
  data.frame(
    df  = sapply(objects, .npar),
    AIC = sapply(objects, aic_fn),
    row.names = model_names
  )
}

#' @rdname simplexreg.methods
#' @importFrom stats BIC
#' @export
BIC.simplexregression <- function(object, ...) {
  objects <- list(object, ...)

  bic_fn <- function(obj) -2 * obj$loglik + log(obj$nobs) * .npar(obj)

  if (length(objects) == 1) return(bic_fn(object))

  model_names <- c(deparse(substitute(object)),
                   sapply(substitute(list(...)), deparse)[-1])
  data.frame(
    df  = sapply(objects, .npar),
    BIC = sapply(objects, bic_fn),
    row.names = model_names
  )
}

#' @rdname simplexreg.methods
#' @export
HQIC <- function(object, ...) UseMethod("HQIC")

#' @rdname simplexreg.methods
#' @export
HQIC.simplexregression <- function(object, ...) {
  objects <- list(object, ...)

  hqic_fn <- function(obj) -2 * obj$loglik + 2 * .npar(obj) * log(log(obj$nobs))

  if (length(objects) == 1) return(hqic_fn(object))

  model_names <- c(deparse(substitute(object)),
                   sapply(substitute(list(...)), deparse)[-1])
  data.frame(
    df   = sapply(objects, .npar),
    HQIC = sapply(objects, hqic_fn),
    row.names = model_names
  )
}

#' @rdname simplexreg.methods
#' @importFrom stats hatvalues
#' @export
hatvalues.simplexregression <- function(model, ...) {
  parametric <- is_parametric(model)

  X <- model$mu.x
  mu <- as.vector(model$mu.fv)
  sigma2 <- as.vector(model$sigma2.fv)
  muonemu <- mu * (1 - mu)

  if(parametric){
    weights <- (1/sigma2) * ((3*sigma2 / muonemu) + ( 1 / (muonemu^3))) *
      (parametric_mean_link_inv_deriv1(model$mu.lp, model$lambda.fv, model$mu.link)^2)
  } else {
    weights <- (1/sigma2) * ((3*sigma2 / muonemu) + ( 1 / (muonemu^3))) *
      (fixed_mean_link_inv_deriv1(model$mu.lp, model$mu.link)^2)
  }

  Xw <- X * sqrt(weights)
  XtX <- crossprod(Xw)
  Inv <- tryCatch(
    chol2inv(chol(XtX)),
    error = function(e) stop("Hat matrix computation failed: design matrix may be singular.")
  )

  rowSums((Xw %*% Inv) * Xw)
}

#' @rdname simplexreg.methods
#' @importFrom stats residuals hatvalues cooks.distance
#' @export
cooks.distance.simplexregression <- function(model, type = c("pearson", "weighted"), ...) {
  type <- match.arg(type)
  h <- hatvalues(model)

  switch(type,
         "pearson" = {
           h * (residuals(model, type = "pearson") ^ 2) / (2*((1-h)^2))
         },
         "weighted" = {
           (residuals(model, type = "weighted") ^ 2) * (h / (1 - h))
         }
  )
}

#' @rdname simplexreg.methods
#' @importFrom sandwich bread
#' @export
bread.simplexregression <- function(x, ...) {
  x$nobs * vcov(x)
}

#' @rdname simplexreg.methods
#' @importFrom sandwich estfun
#' @importFrom stats plogis
#' @export
estfun.simplexregression <- function(x, ...) {

  y <- if(is.null(x$y)) model.response(model.frame(x)) else x$y
  mu <- x$mu.fv
  sigma2 <- x$sigma2.fv
  eta1 <- x$mu.lp
  eta2 <- x$sigma2.lp
  X <- x$mu.x
  Z <- x$sigma2.x
  link.mu <- x$mu.link
  n <- length(y)
  wts <- if(!is.null(x$weights)) x$weights else rep(1, n)
  parametric <- is_parametric(x)

  # Derivative of mu with respect to eta1
  if(parametric) {
    lambda <- x$lambda.fv
    dmu_deta <- as.vector(parametric_mean_link_inv_deriv1(eta1, lambda, link.mu))

    # Compute rho (derivative w.r.t. lambda)
    if(link.mu == "plogit2") {
      exp_aval_frac <- plogis(eta1)^(1/lambda)
      rho <- as.vector(-exp_aval_frac * log(plogis(eta1)) / (lambda^2))
    } else {
      rho <- as.vector((-1/(lambda^2)) * ((exp(eta1) + 1)^(-1/lambda)) *
                         log(exp(eta1) + 1))
    }
  } else {
    dmu_deta <- fixed_mean_link_inv_deriv1(eta1, link.mu)
  }

  # Derivative of sigma2 with respect to eta2
  dsigma2_deta <- dispersion_link_inv_deriv1(eta2, x$sigma2.link)

  diff <- as.vector(y - mu)
  muonemu <- as.vector(mu * (1 - mu))

  yoneminy <- y * (1 - y)
  dev <- ((y - mu) / muonemu)^2 / yoneminy

  U <- wts * (dev / (sigma2 * muonemu) + 1 / (sigma2 * (muonemu)^3))
  a <- wts * ((-1 / (2 * sigma2)) + (dev / (2 * sigma2^2)))

  score_beta <- (dmu_deta * U * diff) * X
  score_delta <- (dsigma2_deta * a) * Z

  if(parametric) {
    score_lambda <- U * rho * diff
    rval <- cbind(score_beta, score_delta, score_lambda)
  } else {
    rval <- cbind(score_beta, score_delta)
  }

  colnames(rval) <- names(coef(x, model = "full"))
  rval[wts <= 0, ] <- 0

  return(rval)
}

#' @rdname simplexreg.methods
#' @importFrom lmtest coeftest
#' @export
coeftest.simplexregression <- function(x, vcov. = NULL, df = Inf, ...) {
  if (is.null(vcov.)) vcov. <- x$vcov
  lmtest::coeftest.default(x, vcov. = vcov., df = df, ...)
}

#' @rdname simplexreg.methods
#' @importFrom lmtest lrtest
#' @export
lrtest.simplexregression <- function(object, ...) {
  lmtest::lrtest.default(object, ...)
}

#' Reading Accuracy in Dyslexic and Non-Dyslexic Children
#'
#' @description
#' This dataset examines the relationship between non-verbal IQ and reading
#' accuracy in children diagnosed with dyslexia and in typical readers.
#' The reading scores are proportions bounded in the open interval (0, 1), making
#' them suitable for beta regression and simplex regression analysis.
#'
#' @docType data
#' @usage data(ReadingSkills)
#'
#' @format A data frame with 44 observations and 4 variables:
#' \describe{
#'   \item{accuracy}{Numeric. Reading score transformed to the open (0, 1) interval.
#'         Values originally equal to 1 were replaced with 0.99. Suitable for
#'         standard beta regression.}
#'   \item{dyslexia}{Factor. Indicates whether the child has dyslexia
#'         (levels: \code{"no"}, \code{"yes"}). Note that sum contrasts are typically used
#'         instead of treatment contrasts in beta regression analyses of this data.}
#'   \item{iq}{Numeric. Non-verbal intelligence quotient, transformed to
#'         \eqn{z}-scores (mean = 0, standard deviation = 1).}
#'   \item{accuracy1}{Numeric. Unrestricted reading score in the [0, 1] interval.
#'         This version preserves the original maximum value of 1 and can be used
#'         with extended--support beta mixture regression models.}
#' }
#'
#' @details
#' The data were originally collected by Pammer and Kevan (2004) and later
#' analyzed by Smithson and Verkuilen (2006) to demonstrate beta regression.
#'
#' The transformation procedure for \code{accuracy} was as follows:
#' \enumerate{
#'   \item The original test scores were scaled to the [0, 1] interval using
#'         the minimum and maximum possible scores in the reading test,
#'         resulting in \code{accuracy1}.
#'   \item To avoid boundary values (0 and 1) that are problematic for standard
#'         beta regression, all observations with value 1 were replaced with 0.99,
#'         creating the \code{accuracy} variable.
#' }
#'
#' The unrestricted \code{accuracy1} variable can be analyzed using extended--support
#' beta regression methods (Kosmidis & Zeileis, 2025), which naturally accommodate
#' boundary observations.
#'
#' @source
#' Pammer, K. and Kevan, A. (2007). The contribution of visual sensitivity,
#' phonological processing and non-verbal IQ to children's reading.
#' \emph{Scientific Studies of Reading}, \bold{11}(1), 33--53.
#' \doi{10.1080/10888430709336633}
#'
#' Smithson, M. and Verkuilen, J. (2006). A Better lemon squeezer?
#' Maximum-likelihood regression with beta-distributed dependent variables.
#' \emph{Psychological Methods}, \bold{11}(1), 54--71. \doi{10.1037/1082-989X.11.1.54}
#'
#' @references
#' Cribari-Neto, F. and Zeileis, A. (2010). Beta regression in R.
#' \emph{Journal of Statistical Software}, \bold{34}(2), 1--24.
#' \doi{10.18637/jss.v034.i02}
#'
#' Kosmidis, I. and Zeileis, A. (2025). Extended-support beta regression for
#' [0, 1] responses. \emph{Journal of the Royal Statistical Society C},
#' \bold{75}(1), 139--157. \doi{10.1093/jrsssc/qlaf039}
#'
#' @keywords datasets
#'
#' @examples
#' # Load the data
#' data(ReadingSkills)
#'
#' # Quick overview
#' head(ReadingSkills)
#' str(ReadingSkills)
#'
#' # Summary statistics by dyslexia status
#' aggregate(accuracy ~ dyslexia, data = ReadingSkills, summary)
#' aggregate(iq ~ dyslexia, data = ReadingSkills, summary)
#'
#' # Visualize the relationship between IQ and reading accuracy
#' plot(accuracy ~ iq, data = ReadingSkills,
#'      col = c(4, 2)[dyslexia], pch = 19,
#'      main = "Reading Accuracy vs. IQ by Dyslexia Status",
#'      xlab = "IQ (z-scored)", ylab = "Reading Accuracy")
#' legend("topleft", legend = c("Non-dyslexic", "Dyslexic"),
#'        col = c(4, 2), pch = 19, bty = "n")
#'
#' # Check for boundary values
#' table(ReadingSkills$accuracy == 0.99)  # Values replaced from 1
#' table(ReadingSkills$accuracy1 == 1)    # Original boundary values
"ReadingSkills"

################################################################################
#          SIMPLEX REGRESSION - DIAGNOSTIC PLOTS AND SIMULATED ENVELOPES       #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Visual diagnostic tools: residuals plots, half-normal plots and #
#              simulated envelopes.                                            #
################################################################################

# ==============================================================================
# 1. PLOTs RESIDUALS
# ==============================================================================

#' @title Diagnostic Plots for Simplex Regression Models
#' @description Produces diagnostic plots for fitted simplex regression models with
#' parametric or fixed mean link function.
#'
#' @param x An object of class \code{simplexregression}.
#' @param which Numeric vector indicating which plots to display (\code{1:7}).
#' @param type Character string specifying the residual type (default: \code{"quantile"}).
#' \cr See \code{\link{residuals.simplexregression}} for available options.
#' @param ask Logical; if \code{TRUE}, the user is asked before each plot.
#' Default is \code{TRUE} when multiple plots are requested.
#' @param reset.par Logical; if \code{TRUE}, resets graphical parameters before plotting.
#' Set to \code{FALSE} to preserve user-defined \code{par()} settings such as \code{mfrow}.
#' Default is \code{TRUE}.
#' @param threshold Numeric threshold for identifying influential or outlying
#' observations in plots 1--3 (residuals plots), 6 (Cook's distance), and
#' 7 (generalized leverage). If \code{NULL} (default), no observations are
#' highlighted.
#' @param label.pos Position(s) for outlier labels in plots 1--3, 6, and 7.
#' Can be a single value (applied to all labels) or a vector. Values:
#' 1 = below, 2 = left, 3 = above, 4 = right. Default is \code{3} (above).
#' See \code{\link[graphics]{text}} for details.
#' @param plot.type Controls the plot symbol/type for scatter plots and index plots.
#' If \code{NULL} (default), uses \code{pch = 1} (open circles) for residual plots
#' (which = 1--5) and \code{type = "h"} (vertical lines) for Cook's distance and
#' generalized leverage plots (which = 6--7). Otherwise, the value is passed
#' directly to \code{pch} (for scatter plots) or \code{type} (for index plots).
#' @param ... Additional graphical parameters.
#'
#' @details
#' Eight diagnostic plots are available:
#' \itemize{
#'   \item Residuals vs observation index (\code{which = 1}): Identifies outliers
#'   and temporal patterns;
#'
#'   \item Residuals vs fitted values (\code{which = 2}): Checks for heteroscedasticity
#'   and patterns;
#'
#'   \item Residuals vs linear predictor (\code{which = 3}): Evaluates link function
#'   adequacy;
#'
#'   \item Observed vs fitted values (\code{which = 4}): Assesses overall model fit;
#'
#'   \item Normal Q–Q plot (\code{which = 5}): Evaluates residual normality (especially
#'   useful for quantile residuals);
#'
#'   \item Cook’s distance vs indices of observations (\code{which = 6}): Identifies
#'   influential observations;
#'
#'   \item Generalized leverage vs indices of observations (\code{which = 7}): Identifies
#'   influential observations.
#' }
#'
#' @return Called for its side effects (diagnostic plots). Returns \code{NULL}
#' invisibly.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Display all diagnostic plots
#' oldpar <- par(mfrow = c(3, 3))
#' plot(fit, which = 1:7)
#' par(oldpar)
#'
#' @importFrom stats qqnorm qqline residuals cooks.distance
#' @importFrom graphics par abline text lines
#' @importFrom utils modifyList
#'
#' @seealso \code{\link{residuals.simplexregression}},
#' \code{\link{cooks.distance.simplexregression}},
#' \code{\link{halfnormal.plot}}.
#'
#' @export
plot.simplexregression <- function(x, which = 1:7,
                                   type = c("quantile", "pearson",
                                            "deviance", "standardized",
                                            "weighted", "variance",
                                            "biasvariance", "score",
                                            "dualscore", "response"),
                                   ask = prod(par("mfcol")) < length(which) && interactive(),
                                   reset.par = TRUE, threshold = NULL,
                                   label.pos = 3, plot.type = NULL, ...) {

  if (!is.numeric(which) || any(which < 1) || any(which > 7))
    stop("`which' must be in 1:7")

  type <- match.arg(type)

  resid <- residuals(x, type = type)

  n <- length(resid)
  show <- rep(FALSE, 7)
  show[which] <- TRUE
  one.fig <- prod(par("mfcol")) == 1

  # Automatic plot.type defaults
  pt_scatter <- if (is.null(plot.type)) 1    else plot.type   # pch for plots 1-5
  pt_index   <- if (is.null(plot.type)) "h"  else plot.type   # type for plots 6-7

  # Configure ask mode
  op <- par(no.readonly = TRUE)
  on.exit(par(op), add = TRUE)
  if (ask) par(ask = TRUE)
  if (reset.par) par(mar = c(3,3,2,3), oma = c(0.5,0.5,0.5,0.5), mgp = c(2,0.6,0))

  # Helper to merge defaults with user ...
  plot_args <- function(..., defaults) modifyList(defaults, list(...))

  scatter_defaults <- list(pch = pt_scatter, cex = 1, cex.axis = 0.8, cex.lab = 1.2)
  index_defaults   <- list(type = pt_index,  cex = 1, cex.axis = 0.8, cex.lab = 1.2)
  user_args <- list(...)

  # 1. Residuals vs indices
  if (show[1]) {
    args <- modifyList(c(list(x = 1:n, y = resid,
                              xlab = "Observation index", ylab = "Residuals"),
                         scatter_defaults), user_args)
    do.call(plot, args)
    abline(h = c(-3, -2, 0, 2, 3), lty = 2, col = "gray60")
    if (!is.null(threshold)) {
      idx <- which(abs(resid) > threshold)
      if (length(idx) > 0)
        text(idx, resid[idx], labels = idx, pos = label.pos, cex = 0.8, col = "red")
    }
  }

  # 2. Residuals vs fitted values
  if (show[2]) {
    args <- modifyList(c(list(x = x$fitted.values, y = resid,
                              xlab = "Fitted values", ylab = "Residuals"),
                         scatter_defaults), user_args)
    do.call(plot, args)
    abline(h = c(-3, -2, 0, 2, 3), lty = 2, col = "gray60")
    if (!is.null(threshold)) {
      idx <- which(abs(resid) > threshold)
      if (length(idx) > 0)
        text(x$fitted.values[idx], resid[idx], labels = idx, pos = label.pos, cex = 0.8, col = "red")
    }
  }

  # 3. Residuals vs linear predictor
  if (show[3]) {
    args <- modifyList(c(list(x = x$mu.lp, y = resid,
                              xlab = "Linear predictor", ylab = "Residuals"),
                         scatter_defaults), user_args)
    do.call(plot, args)
    abline(h = c(-3, -2, 0, 2, 3), lty = 2, col = "gray60")
    if (!is.null(threshold)) {
      idx <- which(abs(resid) > threshold)
      if (length(idx) > 0)
        text(x$mu.lp[idx], resid[idx], labels = idx, pos = label.pos, cex = 0.8, col = "red")
    }
  }

  # 4. Observed vs fitted
  if (show[4]) {
    args <- modifyList(c(list(x = x$y, y = x$fitted.values,
                              xlab = "Observed values", ylab = "Fitted values"),
                         scatter_defaults), user_args)
    do.call(plot, args)
    abline(a = 0, b = 1, lty = 1, col = "gray60")
  }

  # 5. Q-Q plot (qqnorm does not accept x/y, filter valid args)
  if (show[5]) {
    qq_valid <- c("pch", "cex", "cex.axis", "cex.lab", "col", "lwd")
    args <- modifyList(list(cex = 1, cex.axis = 0.8, cex.lab = 1.2,
                            xlab = "Normal quantiles", ylab = "Empirical quantiles",
                            main = NULL),
                       user_args[names(user_args) %in% qq_valid])
    do.call(qqnorm, c(list(y = resid), args))
    qqline(resid, col = "gray60")
  }

  # 6. Cook's distance
  if (show[6]) {
    cook <- cooks.distance(x, type = "pearson")
    args <- modifyList(c(list(x = cook,
                              xlab = "Observation index", ylab = "Cook's distance",
                              ylim = c(min(cook), max(cook, na.rm = TRUE) * 1.05)),
                         index_defaults), user_args)
    do.call(plot, args)
    if (!is.null(threshold)) {
      idx <- which(cook > threshold)
      if (length(idx) > 0)
        text(idx, cook[idx], labels = idx, pos = label.pos, cex = 0.8, col = "red")
    }
  }

  # 7. Generalized leverage
  if (show[7]) {
    glev <- gleverage(x)
    args <- modifyList(c(list(x = glev,
                              xlab = "Observation index", ylab = "Generalized leverage",
                              ylim = c(min(glev), max(glev, na.rm = TRUE) * 1.05)),
                         index_defaults), user_args)
    do.call(plot, args)
    if (!is.null(threshold)) {
      idx <- which(glev > threshold)
      if (length(idx) > 0)
        text(idx, glev[idx], labels = idx, pos = label.pos, cex = 0.8, col = "red")
    }
  }

  invisible()
}

# ==============================================================================
# 2. SIMULATED ENVELOPE - HALF-NORMAL PLOT
# ==============================================================================

#' @title Half-Normal Plots with Simulated Envelopes for Simplex Regression
#' @description Produces half-normal plots with simulated envelopes for simplex
#' regression model with parametric or fixed mean link function.
#'
#' @param model An object of class \code{simplexregression}.
#' @param type Character string specifying the residual type (default: \code{"weighted"}).
#' \cr See \code{\link{residuals.simplexregression}} for available options.
#' @param nsim Number of simulations for envelope construction (default: 100).
#' @param seed Integer setting the random seed for reproducibility (default: 1987).
#' @param level Confidence level for envelope bounds (default: 0.95).
#' @param ... Additional graphical parameters.
#'
#' @details
#' The envelope is based on the following steps:
#' \enumerate{
#'   \item Simulate \code{nsim} response vectors from the fitted model
#'         using its estimated mean and dispersion parameter;
#'   \item Refitting the model to each simulated dataset;
#'   \item Computing absolute residuals and their order statistics;
#'   \item Obtaining envelope bounds from empirical quantiles.
#' }
#'
#' Points outside the envelope may indicate model inadequacy.
#'
#' @return Called for its side effects (half-normal plot with simulated envelope).
#' Returns \code{NULL} invisibly.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' \donttest{
#' halfnormal.plot(fit, seed = 2008)
#' }
#'
#' @importFrom stats qnorm quantile median residuals
#' @importFrom graphics matplot points legend lines
#' @importFrom utils modifyList
#'
#' @seealso \code{\link{residuals.simplexregression}},
#' \code{\link{plot.simplexregression}}.
#'
#' @export
halfnormal.plot <- function (model, type = c("weighted", "quantile",
                                             "pearson", "deviance",
                                             "standardized", "variance",
                                             "biasvariance", "score",
                                             "dualscore", "response"),
                             nsim = 100, level = 0.95, seed = 1987, ...) {

  if (!inherits(model, "simplexregression")) {
    stop("'model' must be an object of class 'simplexregression'")
  }

  set.seed(seed)
  type <- match.arg(type)

  n <- model$nobs
  alpha <- (1 - level)/2

  # Original residuals
  td <- residuals(model, type)

  X <- model$mu.x
  Z <- model$sigma2.x
  mu <- as.vector(model$mu.fv)
  sigma2 <- as.vector(model$sigma2.fv)

  re <- matrix(0, nrow = n, ncol = nsim)

  e1 <- numeric(n)
  e2 <- numeric(n)

  ctrl <- model$control
  ctrl$hessian <- FALSE
  ctrl$fsmaxit <- 0  # skip Fisher scoring in simulations for speed

  max_tries <- nsim * 5
  tries <- 0

  i <- 1
  while (i <= nsim) {
    tries <- tries + 1
    if (tries > max_tries)
      stop("Could not obtain ", nsim, " converged fits after ", max_tries, " attempts.")

    ysim <- rsimplex(n, mu, sigma2)

    fit <- suppressWarnings(
        simplexreg.fit(
          y = ysim,
          x = X[, colnames(X) != "(Intercept)", drop = FALSE],
          z = Z[, colnames(Z) != "(Intercept)", drop = FALSE],
          link.mu = model$mu.link,
          link.sigma2 = model$sigma2.link,
          x_names = model$x_names,
          z_names = model$z_names,
          control = ctrl,
          weights = model$weights
        )
      )

    if(fit$optim$convergence == 0){
      re[, i] <- sort(abs(residuals(fit, type)))
      i <- i + 1
      }
  }

  for (j in 1:n) {
    eo <- sort(re[j, ])
    e1[j] <- quantile(eo, alpha)
    e2[j] <- quantile(eo, 1 - alpha)
  }

  e0 <- apply(re, 1, median, na.rm = TRUE)

  qq <- qnorm((n + 1:n + 0.5)/(2 * n + 1.125))
  xx <- cbind(qq, qq)
  yy <- cbind(e1, e2)

  op <- par(no.readonly = TRUE)
  on.exit(par(op), add = TRUE)
  par(mar = c(3, 3, 2, 3), oma = c(0.5, 0.5, 0.5, 0.5), mgp = c(2, 0.6, 0))

  # Plot defaults (can be overridden via ...)
  plot_args <- modifyList(
    list(type = "l", lty = c(1, 1), col = c("black", "black"),
         xlab = "Normal quantiles", ylab = "Empirical quantiles",
         cex = 1, cex.axis = 0.8, cex.lab = 1.2),
    list(...)
  )

  do.call(matplot, c(list(xx, yy), plot_args))

  res.sorted.abs <- sort(abs(td))

  # arguments valid for points() only
  points_valid <- c("pch", "cex", "col", "bg", "lwd")
  point_args <- modifyList(
    list(pch = 1, cex = plot_args$cex),
    list(...)[names(list(...)) %in% points_valid]
  )
  do.call(points, c(list(qq, res.sorted.abs), point_args))

  lines(qq, e0, lty = 2, col = "black")

  outside_bands <- (res.sorted.abs < e1) | (res.sorted.abs > e2)
  cOut <- sum(outside_bands)
  prop95 <- round(cOut /n*100, 2)

  legend("topleft",
         legend = c(paste("Points outside:", cOut, "(", prop95, "%)"),
                    paste("Total points:", n)),
         bty="n", cex = 0.8)
}

################################################################################
#                SIMPLEX REGRESSION - DEVIANCE FUNCTION                        #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Computes the unit deviance for the simplex distribution         #
################################################################################

#' @title Unit Deviance Function of the Simplex Distribution
#' @description Computes the unit deviance (scaled deviance component) for the
#' simplex distribution.
#'
#' @param y Numeric scalar or vector of observed values (\eqn{0 < y < 1}).
#' If a vector, must be the same length as \code{mu} or recyclable.
#' @param mu Numeric scalar or vector of mean values (\eqn{0 < \mu < 1}).
#' If a vector, must be the same length as \code{y} or recyclable.
#'
#' @details
#' The unit deviance for the simplex distribution is defined as:
#' \deqn{d(y, \mu) = \frac{(y - \mu)^2}{y(1-y)[\mu(1-\mu)]^2}.}
#'
#' This function is used internally in maximum likelihood estimation and
#' model diagnostics for simplex regression.
#'
#' @return A numeric scalar or vector of unit deviance values.
#'
#' @examples
#' # Single value
#' dev.unit.simplex(y = 0.6, mu = 0.5)
#'
#' # Vector of values
#' y_vec <- c(0.2, 0.5, 0.8)
#' mu_vec <- c(0.3, 0.5, 0.7)
#' dev.unit.simplex(y = y_vec, mu = mu_vec)
#'
#' # Perfect fit returns zero deviance
#' dev.unit.simplex(y = 0.5, mu = 0.5)
#'
#' @references
#' Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
#' Chapman and Hall, London.
#'
#' Song, P. X.-K. and Tan, M. (2000).
#' Marginal models for longitudinal continuous proportional data.
#' \emph{Biometrics}, \bold{56}(2), 496--502.
#' \doi{10.1111/j.0006-341X.2000.00496.x}
#'
#' @seealso \code{\link{variance.simplex}}, \code{\link{dsimplex}}.
#' @export
dev.unit.simplex <- function(y, mu){
  # Input validation
  stopifnot(
    "'y' must always be in (0, 1)" = all(y > 0 & y < 1),
    "parameter 'mu' must always be in (0, 1)" = all(mu > 0 & mu < 1)
  )

  # Compute unit deviance
  diff <- y - mu
  yoneminy <- y * (1 - y)
  muonemu <- mu * (1 - mu)
  deviance <- (diff / muonemu)^2 / yoneminy

  return(deviance)
}

library(testthat)
library(SimplexRegression)

# ==============================================================================
# Tests for dev.unit.simplex function
# ==============================================================================

test_that("dev.unit.simplex validates inputs correctly", {
  # y must be in (0, 1)
  expect_error(dev.unit.simplex(y = 0, mu = 0.5), "'y' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = 1, mu = 0.5), "'y' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = -0.5, mu = 0.5), "'y' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = 1.5, mu = 0.5), "'y' must always be in \\(0, 1\\)")

  # mu must be in (0, 1)
  expect_error(dev.unit.simplex(y = 0.5, mu = 0), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = 0.5, mu = 1), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = 0.5, mu = -0.5), "parameter 'mu' must always be in \\(0, 1\\)")
  expect_error(dev.unit.simplex(y = 0.5, mu = 1.5), "parameter 'mu' must always be in \\(0, 1\\)")
})

test_that("dev.unit.simplex accepts valid inputs without error", {
  expect_no_error(dev.unit.simplex(y = 0.5, mu = 0.5))
  expect_no_error(dev.unit.simplex(y = 0.3, mu = 0.7))
  expect_no_error(dev.unit.simplex(y = 0.8, mu = 0.2))
  expect_no_error(dev.unit.simplex(y = 0.1, mu = 0.9))
})

test_that("dev.unit.simplex returns correct type and dimensions", {
  # scalar input
  result <- dev.unit.simplex(y = 0.5, mu = 0.5)
  expect_type(result, "double")
  expect_length(result, 1)

  # vector input same length
  result <- dev.unit.simplex(y = c(0.2, 0.5, 0.8), mu = c(0.3, 0.5, 0.7))
  expect_type(result, "double")
  expect_length(result, 3)

  # recycling (y vector, mu scalar)
  result <- dev.unit.simplex(y = c(0.2, 0.5, 0.8), mu = 0.5)
  expect_length(result, 3)

  # recycling (y scalar, mu vector)
  result <- dev.unit.simplex(y = 0.5, mu = c(0.2, 0.5, 0.8))
  expect_length(result, 3)
})

test_that("dev.unit.simplex returns zero when y equals mu", {
  # Perfect fit should return zero deviance
  expect_equal(dev.unit.simplex(y = 0.5, mu = 0.5), 0)
  expect_equal(dev.unit.simplex(y = 0.3, mu = 0.3), 0)
  expect_equal(dev.unit.simplex(y = 0.7, mu = 0.7), 0)
  expect_equal(dev.unit.simplex(y = 0.1, mu = 0.1), 0)
  expect_equal(dev.unit.simplex(y = 0.9, mu = 0.9), 0)
})

test_that("dev.unit.simplex returns non-negative values", {
  # All deviances should be >= 0
  expect_true(dev.unit.simplex(y = 0.6, mu = 0.5) >= 0)
  expect_true(dev.unit.simplex(y = 0.4, mu = 0.5) >= 0)
  expect_true(all(dev.unit.simplex(y = c(0.2, 0.5, 0.8), mu = c(0.3, 0.5, 0.7)) >= 0))
})

test_that("dev.unit.simplex handles numeric precision near boundaries", {
  # Values close to boundaries (but still valid)
  expect_true(is.finite(dev.unit.simplex(y = 1e-8, mu = 0.5)))
  expect_true(is.finite(dev.unit.simplex(y = 1 - 1e-8, mu = 0.5)))
  expect_true(is.finite(dev.unit.simplex(y = 0.5, mu = 1e-8)))
  expect_true(is.finite(dev.unit.simplex(y = 0.5, mu = 1 - 1e-8)))

  # Should still be positive
  expect_true(dev.unit.simplex(y = 1e-8, mu = 0.5) > 0)
  expect_true(dev.unit.simplex(y = 0.5, mu = 1e-8) > 0)
})

test_that("dev.unit.simplex is consistent with known values", {
  # Test some specific values for correctness
  # For y = 0.6, mu = 0.5
  # diff = 0.1
  # muonemu = 0.5 * 0.5 = 0.25
  # yoneminy = 0.6 * 0.4 = 0.24
  # deviance = (0.1 / 0.25)^2 / 0.24 = (0.4)^2 / 0.24 = 0.16 / 0.24 = 0.6666667

  result <- dev.unit.simplex(y = 0.6, mu = 0.5)
  expect_equal(result, 0.16 / 0.24, tolerance = 1e-12)
  expect_equal(result, 2/3, tolerance = 1e-12)

  # For y = 0.7, mu = 0.5
  # diff = 0.2
  # muonemu = 0.25
  # yoneminy = 0.7 * 0.3 = 0.21
  # deviance = (0.2 / 0.25)^2 / 0.21 = (0.8)^2 / 0.21 = 0.64 / 0.21 ≈ 3.047619

  result2 <- dev.unit.simplex(y = 0.7, mu = 0.5)
  expect_equal(result2, 0.64 / 0.21, tolerance = 1e-12)
})

test_that("dev.unit.simplex handles NA values appropriately", {
  # Note: This behavior depends on your stopifnot implementation
  # If you want to allow NA, adjust accordingly
  expect_error(dev.unit.simplex(y = NA, mu = 0.5))
  expect_error(dev.unit.simplex(y = 0.5, mu = NA))
})

test_that("dev.unit.simplex formula matches expected mathematical properties", {
  # The deviance should be scale invariant in a specific way
  y <- 0.6
  mu <- 0.4

  # Direct calculation
  direct <- dev.unit.simplex(y, mu)

  # Alternative calculation using formula components
  diff <- y - mu
  mu_term <- mu * (1 - mu)
  y_term <- y * (1 - y)
  alt <- (diff / mu_term)^2 / y_term

  expect_equal(direct, alt, tolerance = 1e-12)
})

################################################################################
#                      SIMPLEX REGRESSION - RESIDUALS                          #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Diagnostic residuals                                            #
################################################################################

#' @title Residuals for Simplex Regression Models
#' @description Extracts various types of residuals for diagnostic analysis in
#' simplex regression models with parametric or fixed mean link functions.
#'
#' @param object An object of class \code{simplexregression}.
#' @param type Character string specifying the type of residual to extract.
#' Options:
#' \code{"quantile"} (default), \code{"pearson"}, \code{"deviance"},
#' \code{"standardized"}, \code{"weighted"}, \code{"variance"},
#' \code{"biasvariance"}, \code{"score"}, \code{"dualscore"}, \code{"response"}.
#' @param ... Additional arguments (currently not used).
#'
#' @return A numeric vector of residuals.
#'
#' @details
#' Several types of residuals are available for model diagnostics:
#'
#' \strong{Quantile residuals} (\code{"quantile"}): Proposed by Dunn and Smyth
#' (1996) as \eqn{r_i^Q = \Phi^{-1}(F(y_i; \hat{\mu}_i, \hat{\sigma}^2_i))},
#' where \eqn{\Phi(\cdot)} is the standard normal CDF and
#' \eqn{F(\cdot; \cdot)} is the simplex CDF (see \code{\link{psimplex}}).
#' Under correct model specification, these residuals are approximately standard
#' normal and are therefore recommended for general diagnostic use.
#'
#' \strong{Pearson residuals} (\code{"pearson"}): Defined in McCullagh and
#' Nelder (1989) as \eqn{r_i^P = (y_i - \hat{\mu}_i) /
#' \sqrt{\widehat{\text{Var}}(y_i)}}, where \eqn{\widehat{\text{Var}}(y_i)} is
#' the estimated variance of the response
#' (see \code{\link{variance.simplex}}).
#'
#' \strong{Deviance residuals} (\code{"deviance"}): Defined in Jørgensen (1997,
#' p. 115) as \eqn{r_i^D = (y_i - \hat{\mu}_i) /
#' (\hat{\mu}_i(1-\hat{\mu}_i)\sqrt{y_i(1-y_i)})}.
#'
#' \strong{Standardized residuals} (\code{"standardized"}): Proposed by Espinheira
#' and Silva (2020, Eq. 15) as \eqn{r_i^\beta = \hat{u}_i(y_i - \hat{\mu}_i) /
#' \sqrt{\hat{\sigma}^2_i \hat{w}_i}}, where
#' \eqn{\hat{w}_i} and \eqn{\hat{u}_i} are weight functions.
#'
#' \strong{Weighted residuals} (\code{"weighted"}): Proposed by Espinheira and
#' Silva (2020, Eq. 16) as \eqn{r_i^{\beta*} = \hat{u}_i(y_i - \hat{\mu}_i) /
#' \sqrt{\hat{\sigma}^2_i \hat{w}_i(1-\hat{h}_{ii})}}, where \eqn{\hat{h}_{ii}}
#' are the diagonal elements of the hat matrix (see \cr
#' \code{\link{hatvalues.simplexregression}}).
#' These residuals are recommended for simulated envelope plots.
#'
#' \strong{Variance residuals} (\code{"variance"}): Proposed by Espinheira et al.
#' (2021, Eq. 7) as \eqn{r_i^\gamma = (\hat{d}_i - \hat{\sigma}^2_i) /
#' (\hat{\sigma}^2_i\sqrt{2})}, where \eqn{\hat{d}_i} is the estimated unit
#' deviance (see \code{\link{dev.unit.simplex }}).
#'
#' \strong{Bias–variance residuals} (\code{"biasvariance"}): Proposed by Espinheira
#' et al. (2021, Eq. 8) as \eqn{r_i^{\beta \gamma} =
#' (\hat{u_i}(y_i - \hat{\mu}_i) + \hat{a}_i) / \sqrt{\hat{\sigma}^2_i
#' \hat{w}_i + 1/(2\hat{\sigma}^4_i)}}, where \eqn{\hat{a}_i} is a correction term.
#'
#' \strong{Score residuals} (\code{"score"}): Defined in Jørgensen (1997, p. 115) as
#' \eqn{r_i^{S} = (y_i - \hat{\mu}_i)(\hat{\mu}_i^2 + y_i - 2y_i\hat{\mu}_i) /
#' (y_i(1-y_i)\hat{\mu}_i^{1.5}(1-\hat{\mu}_i)^{1.5})}.
#'
#' \strong{Dual score residuals} (\code{"dualscore"}): Defined in Jørgensen
#' (1997, p. 115) as \eqn{r_i^{DS} = (y_i - \hat{\mu}_i)(y_i + \hat{\mu}_i -
#' 2y_i\hat{\mu}_i) / (2\sqrt{y_i(1-y_i)}\hat{\mu}_i^2(1-\hat{\mu}_i)^2)}.
#'
#' \strong{Response residuals} (\code{"response"}): Simple difference between
#' observed and fitted values, \eqn{r_i^R = y_i - \hat{\mu}_i}.
#'
#' \strong{Recommendations}: Quantile residuals are recommended for general model
#' diagnostics due to their theoretical properties. Weighted residuals are
#' particularly useful for constructing simulated envelope plots, as they account
#' for both the variance structure and leverage effects.
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Compute different types of residuals
#' res_quantile <- residuals(fit, type = "quantile")
#' res_pearson <- residuals(fit, type = "pearson")
#' res_weighted <- residuals(fit, type = "weighted")
#'
#' @importFrom stats pnorm qnorm
#'
#' @references
#'
#' McCullagh, P. and Nelder, J. A. (1989). \emph{Generalized Linear Models}.
#' 2nd ed.  Chapman and Hall, London.

#' Jørgensen, B. (1997).
#' \emph{The Theory of Dispersion Models}.
#' Chapman and Hall, London.
#'
#' Dunn, P. K. and Smyth, G. K. (1996). Randomized Quantile Residuals.
#' \emph{Journal of Computational and Graphical Statistics}, \bold{5}(3),
#' 236-–244. \doi{10.2307/1390802}
#'
#' Espinheira, P. L., Silva, L. C. M. and Cribari-Neto, F. (2021). Bias and
#' variance residuals for machine learning nonlinear simplex regressions.
#' \emph{Expert Systems With Applications}, \bold{185}, 115656.
#' \doi{10.1016/j.eswa.2021.115656}
#'
#' Espinheira, P. L., Silva, A. O. (2020). Residual and influence analysis to a
#' general class of simplex regression. \emph{TEST}, \bold{29}, 523–-552.
#' \doi{10.1007/s11749-019-00665-3}
#'
#' @seealso \code{\link{plot.simplexregression}}, \code{\link{halfnormal.plot}},
#' \code{\link{hatvalues.simplexregression}}.
#'
#' @export
residuals.simplexregression <- function(object, type = c("quantile", "pearson",
                                                     "deviance", "standardized",
                                                     "weighted", "variance",
                                                     "biasvariance", "score",
                                                     "dualscore", "response"), ...) {

  if (!inherits(object, "simplexregression")) {
    stop("'object' must be an object of class 'simplexregression'")
  }

  type <- match.arg(type)

  # Quantile residuals are already stored in the object
  if (type == "quantile") {
    return(object$residuals)
  }

  y <- as.vector(object$y)
  mu <- as.vector(object$mu.fv)
  sigma2 <- as.vector(object$sigma2.fv)

  diff <- y - mu
  muonemu <- mu * (1 - mu)
  yoneminy <- y * (1 - y)
  dev <- (diff / muonemu)^2 / yoneminy

  # Calculate weights for weighted residuals
  if(type %in% c("standardized", "weighted", "biasvariance")) {
    wi <- (3*sigma2 / muonemu) + (1 / (muonemu^3))
    ui <- (dev/muonemu) + (1/(muonemu^3))
  }

  res <- switch(type,

                "pearson" = {
                  var_hat <- variance.simplex(mu, sigma2)
                  diff / sqrt(var_hat)
                },

                "deviance" = {
                  diff / (muonemu*sqrt(yoneminy))
                },

                "standardized" = {
                  (ui*diff) / sqrt(sigma2*wi)
                },

                "weighted" = {
                  (ui*diff) / sqrt(sigma2*wi*(1-hatvalues(object)))
                },

                "variance" = {
                  (dev - sigma2) / (sigma2*sqrt(2))
                },

                "biasvariance" = {
                  ai <- - 1/(2*sigma2) + dev/(2*sigma2^2)
                  (ui*diff + ai) / sqrt(sigma2*wi + 1/(2*sigma2^2))
                },

                "response" = {
                  diff
                },

                "score" = {
                  (diff * (mu^2 + y - 2*y*mu)) / (yoneminy * muonemu^1.5)
                },

                "dualscore" = {
                  (diff * (y + mu - 2*y*mu)) / (2 * sqrt(yoneminy) * muonemu^2)
                })

  return(res)
}

################################################################################
#               SIMPLEX REGRESSION - INFORMATION CRITERIA                      #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-08-10                                                             #
# Description: Information criteria and their finite-sample corrections for    #
#              simplex regression models                                       #
################################################################################

#' @title Information Criteria for Simplex Regression
#' @description Extracts the Akaike information criterion (AIC), the Schwarz
#' (Bayesian) information criterion (BIC), and the Hannan-Quinn criterion
#' (HQIC), along with their finite-sample corrections AICc, BICc, and HQICc follow
#' Bayer and Cribari-Neto (2017).
#'
#' @param ... One or more objects of class \code{"simplexregression"} to be
#' evaluated.
#'
#' @details
#' Let \eqn{\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})}
#' denote the maximized log-likelihood of the fitted model and \eqn{k} the
#' total number of estimated parameters. The AIC (Akaike, 1973) is
#' \deqn{AIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + 2k.}
#' Sugiura (1978) proposed an unbiased estimator of the Kullback-Leibler
#' distance in linear regressions, later generalized by Hurvich and Tsai
#' (1989) to nonlinear regressions and autoregressive models. The resulting
#' finite-sample-corrected AIC is
#' \deqn{AICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + \frac{2nk}{n-k-1}.}
#'
#' The Schwarz (1978) information criterion, also known as the Bayesian
#' information criterion (BIC), is a consistent criterion given by
#' \deqn{BIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + k\log(n).}
#' McQuarrie (1999) derived a finite-sample-corrected version, which remains
#' consistent:
#' \deqn{BICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + \frac{nk\log(n)}{n-k-1}.}
#'
#' The Hannan and Quinn (1979) criterion, proposed for autoregressive model
#' selection, is
#' \deqn{HQIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + 2k\log(\log(n)).}
#' McQuarrie and Tsai (1998) proposed a finite-sample-corrected variant:
#' \deqn{HQICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
#' + \frac{2nk\log(\log(n))}{n-k-1}.}
#'
#' Smaller values indicate better model fit.
#'
#' @return When a single model is provided, a named numeric vector with
#' components \code{AIC}, \code{AICc}, \code{BIC}, \code{BICc}, \code{HQIC},
#' and \code{HQICc}. When multiple models are provided, a data frame with one
#' row per model.
#'
#' @references
#' Akaike, H. (1973). Information theory and an extension of the maximum
#' likelihood principle. In \emph{Proceedings of the 2nd International
#' Symposium on Information Theory}, 267--281.
#'
#' Akaike, H. (1978). A Bayesian analysis of the minimum AIC procedure.
#' \emph{Annals of the Institute of Statistical Mathematics}, \bold{30}(1),
#' 9--14. \doi{10.1007/BF02480194}
#'
#' Bayer, F. M. and Cribari-Neto, F. (2017). Model selection criteria in beta
#' regression with varying dispersion. \emph{Communications in Statistics -
#' Simulation and Computation}, \bold{46}(1), 729--746.
#' \doi{10.1080/03610918.2014.977918}
#'
#' Hannan, E. J. and Quinn, B. G. (1979). The determination of the order of
#' an autoregression. \emph{Journal of the Royal Statistical Society, Series
#' B}, \bold{41}(2), 190--195. \doi{10.1111/j.2517-6161.1979.tb01072.x}
#'
#' Hurvich, C. M. and Tsai, C.-L. (1989). Regression and time series model
#' selection in small samples. \emph{Biometrika}, \bold{76}(2), 297--307.
#' \doi{10.1093/biomet/76.2.297}
#'
#' McQuarrie, A. D. R. (1999). A small-sample correction for the Schwarz SIC
#' model selection criterion. \emph{Statistics & Probability Letters},
#' \bold{44}(1), 79--86. \doi{10.1016/S0167-7152(98)00294-6}
#'
#' McQuarrie, A. D. R. and Tsai, C.-L. (1998). \emph{Regression and Time
#' Series Model Selection}. Singapore: World Scientific.
#'
#' Schwarz, G. (1978). Estimating the dimension of a model. \emph{The Annals
#' of Statistics}, \bold{6}(2), 461--464. \doi{10.1214/aos/1176344136}
#'
#' Sugiura, N. (1978). Further analysis of the data by Akaike's information
#' criterion and the finite corrections. \emph{Communications in Statistics -
#' Theory and Methods}, \bold{7}(1), 13--26.
#' \doi{10.1080/03610927808827599}
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                    data = ReadingSkills)
#'
#' # Single model
#' ic(fit1)
#'
#' # Comparing multiple models
#' fit2 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                    data = ReadingSkills, link.mu = "loglog")
#' ic(fit1, fit2)
#'
#' @seealso \code{\link{r2}}, \code{\link{press}}, \code{\link{ic.penalized}}
#' @export
ic <- function(...) {

  models <- list(...)
  M <- length(models)

  if (!all(sapply(models, function(x) inherits(x, "simplexregression")))) {
    stop("All arguments must be objects of class 'simplexregression'.")
  }

  # Get model names (supports named arguments)
  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- vapply(as.list(substitute(list(...)))[-1L], deparse, character(1))
  }

  aic <- aic_c <- bic <- bic_c <- hq <- hq_c <- numeric(M)

  for (i in seq_len(M)) {
    model <- models[[i]]

    n <- model$nobs
    p <- ncol(model$mu.x)
    q <- ncol(model$sigma2.x)
    parametric <- !is.na(model$coefficients$lambda)
    k <- if (parametric) p + q + 1 else p + q

    ll <- model$loglik

    # --- AIC (Akaike, 1973) and its finite-sample correction ---
    # (Sugiura, 1978; Hurvich and Tsai, 1989)
    aic[i]   <- -2 * ll + 2 * k
    aic_c[i] <- -2 * ll + (2 * n * k) / (n - k - 1)

    # --- SIC/BIC (Schwarz, 1978) and its finite-sample correction ---
    # (McQuarrie, 1999)
    bic[i]   <- -2 * ll + k * log(n)
    bic_c[i] <- -2 * ll + (n * k * log(n)) / (n - k - 1)

    # --- HQIC (Hannan and Quinn, 1979) and its finite-sample correction ---
    # (McQuarrie and Tsai, 1998)
    hq[i]   <- -2 * ll + 2 * k * log(log(n))
    hq_c[i] <- -2 * ll + (2 * n * k * log(log(n))) / (n - k - 1)
  }

  if (M == 1) {
    return(c(AIC = aic, AICc = aic_c,
             BIC = bic, BICc = bic_c,
             HQIC = hq, HQICc = hq_c))
  } else {
    data.frame(AIC = aic, AICc = aic_c,
               BIC = bic, BICc = bic_c,
               HQIC = hq, HQICc = hq_c,
               row.names = model_names)
  }
}

################################################################################
#                 SIMPLEX REGRESSION - MAIN FITTING FUNCTIONS                  #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2025-11-08                                                             #
# Description: Main functions for fitting simplex regression models with       #
#              parametric or fixed mean link functions.                        #
################################################################################

# ==============================================================================
# 0. CONTROL FUNCTION
# ==============================================================================

#' @title Control Parameters for Simplex Regression
#' @description Auxiliary function for controlling simplex regression fitting.
#'
#' @param method Character string specifying the optimization method
#' passed to \code{optim} (default: \code{"BFGS"}).
#' @param maxit Integer specifying the maximum number of iterations for \code{optim}
#' (default: \code{5000}).
#' @param gradient Logical; use analytical gradient? (default: \code{TRUE}).
#' @param hessian Logical; compute Hessian via \code{optim}? (default: \code{FALSE}).
#' @param trace Logical; trace optimization? (default: \code{FALSE}).
#' @param start An optional vector with starting values for all parameters.
#' @param fsmaxit Integer specifying maximal number of additional Fisher scoring
#' iterations (default: \code{500}).
#' @param fstol Numeric tolerance for convergence in Fisher scoring (default: \code{1e-8}).
#' @param reltol Relative convergence tolerance (default: \code{sqrt(.Machine$double.eps)}).
#' @param ... Additional parameters passed to \code{optim}.
#'
#' @details
#' All parameters in \code{\link{simplexreg}} are estimated by maximum likelihood using
#' \code{\link{optim}} with control options set in \code{\link{simplexreg.control}}.
#' Most arguments are passed on directly to \code{optim}, and \code{start} controls
#' how \code{optim} is called.
#'
#' After the \code{optim} maximization, an additional Fisher scoring iteration can
#' be performed to further enhance the result by moving the gradient even closer to zero.
#' If \code{fsmaxit} is greater than zero, this additional optimization is performed and
#' it converges if the threshold \code{fstol} is attained for the absolute value of the
#' step size.
#'
#' Starting values can be supplied via \code{start} or estimated by \code{\link{lm.wfit}},
#' using the link-transformed response. For parametric mean link functions (\code{"plogit1"},
#' \code{"plogit2"}), the link parameter \eqn{\lambda} is jointly estimated with the regression
#' coefficients. Covariances are derived analytically using the expected Fisher information
#' matrix. The Fisher scoring uses analytical gradients and the expected information matrix to
#' refine the maximum likelihood estimates obtained from \code{optim}.
#'
#' The main parameters of interest are the coefficient vector \eqn{\boldsymbol{\beta}}
#' in the linear predictor of the mean submodel and the coefficient vector
#' \eqn{\boldsymbol{\gamma}} in the linear predictor of the dispersion submodel.
#' For parametric links, the additional link parameter \eqn{\lambda} is also
#' estimated and reported. The dispersion parameter \eqn{\sigma^2} can be modeled either
#' as constant (when the dispersion formula contains only an intercept) or as varying
#' across observations through a linear predictor.
#'
#' @seealso \code{\link{simplexreg}}
#'
#' @return A list of control parameters.
#' @export
simplexreg.control <- function(method = "BFGS",
                               maxit = 5000,
                               gradient = TRUE,
                               hessian = FALSE,
                               trace = FALSE,
                               start = NULL,
                               fsmaxit = 500,
                               fstol = 1e-8,
                               reltol = .Machine$double.eps^(0.5),
                               ...) {

  method <- match.arg(method, c("BFGS", "Nelder-Mead", "CG", "L-BFGS-B", "SANN"))

  rval <- list(
    method = method,
    maxit = maxit,
    gradient = gradient,
    hessian = hessian,
    trace = trace,
    start = start,
    fsmaxit = fsmaxit,
    fstol = fstol,
    reltol = reltol
  )

  rval <- c(rval, list(...))

  # Configuration specific for optim (maximization)
  if(!is.null(rval$fnscale)) warning("fnscale must not be modified")
  rval$fnscale <- -1  # Maximize

  if(method == "L-BFGS-B") {
    rval$reltol <- NULL
    if(is.null(rval$factr)) rval$factr <- 1e7
    if(is.null(rval$pgtol)) rval$pgtol <- 0
  } else {
    if(is.null(rval$reltol)) rval$reltol <- .Machine$double.eps^(0.5)
  }

  return(rval)
}

# ==============================================================================
# 1. MAIN USER-FACING FUNCTION
# ==============================================================================

#' @title Simplex Regression with Parametric or Fixed Mean Link
#' @description Fit simplex regression models for rates and proportions via
#' maximum likelihood estimation, modeling both the mean (via parametric
#' or fixed link function) and the dispersion parameter.
#'
#' @param formula A two-part formula: \code{y ~ x} or \code{y ~ x | 1}
#' (mean submodel, constant dispersion), or \code{y ~ x | z} (submodels for both
#' mean and dispersion).
#' @param data A data frame containing the variables in formula.
#' @param subset A specification of the rows/observations to be used: defaults to all.
#' @param na.action An optional (name of a) function for treating missing values (NAs).
#' @param weights An optional numeric vector of case weights.
#' @param offset Optional numeric vector specifying a known component to be
#' included in the linear predictor of the mean submodel during fitting.
#' One or more \code{\link{offset}} terms can be included in the formula instead
#' or as well, and if more than one is specified their sum is used. See
#' \code{\link{model.offset}}.
#' @param link.mu Character specification of the link function in the mean submodel
#' (parametric functions: \code{"plogit1"}, \code{"plogit2"}; or fixed functions:
#' \code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"}, \code{"cauchit"}).
#' @param link.sigma2 Character specification of the link function in the dispersion
#' submodel (\code{"log"}, \code{"sqrt"}, \code{"identity"}).
#' @param control A list of control arguments specified via \code{\link{simplexreg.control}}.
#' @param contrasts An optional list. See the \code{contrasts.arg} argument of
#' \code{\link[stats]{model.matrix.default}}.
#' @param model,y,x  Logicals. If \code{TRUE} the corresponding components of the fit
#' (model frame, response, model matrix) are returned. For \code{\link{simplexreg.fit}},
#' \code{x} should be a numeric regressor matrix and \code{y} should be the numeric response
#' vector (with values in (0, 1)).
#' @param x_names Column names for mean design matrix (includes intercept).
#' @param z_names Column names for dispersion design matrix (includes intercept).
#' @param z Design matrix for dispersion model (without intercept).
#' @param ... Additional arguments passed to \code{\link{simplexreg.control}}.
#'
#' @name simplexreg.fit
#'
#' @aliases simplexreg simplexreg.fit
#'
#' @details
#' Simplex regression, introduced by Song and Tan (2000) and extended by
#' Song, Qiu and Tan (2004), is useful for modeling continuous response variables
#' restricted to the unit interval (0, 1), such as rates and proportions. The model
#' assumes that the dependent variable follows the simplex distribution, originally
#' proposed by Barndorff-Nielsen and Jørgensen (1991), which is indexed by the mean
#' \eqn{\mu} and the dispersion parameter \eqn{\sigma^2}.
#'
#' Similar to generalized linear models (GLMs), simplex regression relates
#' the mean of the response variable and the dispersion parameter to their respective
#' linear predictors through link functions. This package implements five fixed link
#' functions (\code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"},
#' \code{"cauchit"}) and two parametric link functions (\code{"plogit1"},
#' \code{"plogit2"}) for the mean submodel. For the dispersion submodel, the
#' links \code{"log"}, \code{"sqrt"} and \code{"identity"} are supported.
#'
#' The model is specified through a two-part formula separated by \code{|}. The
#' left side contains the predictors for the mean submodel and the right side contains
#' the predictors for the dispersion submodel:
#' \itemize{
#'   \item Mean submodel: \eqn{g(\mu_i, \lambda) = \boldsymbol{x}_i'\boldsymbol{\beta}}
#'   (parametric link) or \eqn{g(\mu_i) = \boldsymbol{x}_i'\boldsymbol{\beta}} (fixed link),
#'   where \eqn{g(\cdot)} is the mean link function, \eqn{\lambda} is the
#'   extra shape parameter of the parametric link,
#'   \eqn{\boldsymbol{x}_i} is the vector of covariates for the \eqn{i}-th observation in
#'   the mean submodel, and \eqn{\boldsymbol{\beta}} is the corresponding
#'   vector of regression coefficients.
#'   \item Dispersion submodel: \eqn{h(\sigma^2_i) = \boldsymbol{z}_i'\boldsymbol{\gamma}},
#'   where \eqn{h(\cdot)} is the dispersion link function, \eqn{\boldsymbol{z}_i} is the
#'   vector of covariates for the \eqn{i}-th observation in the dispersion
#'   submodel, and \eqn{\boldsymbol{\gamma}} is the corresponding vector of
#'   regression coefficients.
#' }
#'
#' Formula examples: \code{y ~ x1 + x2 | z1 + z2} (variable dispersion) or
#' \code{y ~ x1 + x2} (constant dispersion). The link functions for both
#' submodels are specified using \code{link.mu} and \code{link.sigma2}.
#'
#' The parametric mean link functions include a parameter \eqn{\lambda} that
#' is estimated along with other model parameters. Parameter estimation is performed
#' via maximum likelihood using the \code{optim} function with analytical gradient
#' and initial values obtained from an auxiliary linear regression of the transformed
#' response. Subsequently, the \code{optim} result may be enhanced by an additional Fisher
#' scoring iteration using analytical gradients and expected information. The Fisher
#' scoring is just a refinement to move the gradients even closer to zero and can be
#' disabled by setting \code{fsmaxit = 0} in the control arguments.
#'
#' Methods for extracting and analyzing results are implemented for objects
#' of class \code{"simplexregression"}, allowing the use of generic functions such as
#' \code{\link{summary}}, \code{\link{print}}, \code{\link{fitted}}, \code{\link{coef}},
#' \code{\link{formula}}, \code{\link{logLik}}, \code{\link{vcov}}, \code{\link{predict}},
#' \code{\link{terms}}, \code{\link{model.frame}}, \code{\link{model.matrix}},
#' \code{\link{plot}}, \code{\link{residuals}}, \code{\link{cooks.distance}},
#' \code{\link{gleverage}}, \code{\link{hatvalues}}, \code{\link{update}},
#' \code{\link{simulate}}, \code{\link{AIC}}, \code{\link[lmtest]{coeftest}} (from the
#' lmtest package), \code{\link[sandwich]{bread}} and \code{\link[sandwich]{estfun}} (from the sandwich package).
#'
#' @seealso \code{\link{summary.simplexregression}},
#' \code{\link{predict.simplexregression}}, \code{\link{residuals.simplexregression}},
#' \code{\link{penalized.ic}}, \code{\link{penalized.ss}},
#' \code{\link{scoretest}}
#'
#' @return An object of class \code{"simplexregression"}, i.e.,
#' a list with components as follows:
#' \itemize{
#'   \item \code{coefficients}: A list with elements \code{mean} and \code{dispersion},
#'   containing the estimated regression coefficients \eqn{\hat{\boldsymbol{\beta}}} and
#'   \eqn{\hat{\boldsymbol{\gamma}}} of the mean and dispersion submodels, respectively.
#'   For parametric mean link functions, the list also includes an additional element,
#'   \code{lambda}, containing the estimated link parameter \eqn{\hat{\lambda}}.
#'   \item \code{fitted.values}: a vector of fitted mean values,
#'   \item \code{optim}: a list containing \code{start} (initial values),
#'   \code{convergence} (convergence code), \code{counts} (number of iterations) and
#'   \code{method} (optimization method) from the optimization procedure,
#'   \item \code{scoring}: number of iterations from the optimization procedure via
#'   Fisher scoring,
#'   \item \code{mu.fv}: a vector of fitted mean values,
#'   \item \code{mu.lp}: a vector of fitted mean linear predictor,
#'   \item \code{mu.x}: design matrix for the mean model (with intercept),
#'   \item \code{mu.link}: character string specifying the mean link function,
#'   \item \code{mu.df}: degrees of freedom for the mean model,
#'   \item \code{sigma2.fv}: a vector of fitted dispersion values,
#'   \item \code{sigma2.lp}: a vector of fitted dispersion linear predictor,
#'   \item \code{sigma2.x}: design matrix for the dispersion model (with intercept),
#'   \item \code{sigma2.link}: character string specifying the dispersion link function,
#'   \item \code{sigma2.df}: degrees of freedom for the dispersion model,
#'   \item \code{lambda.fv}: estimated value of the parametric link function parameter
#'   (\code{NA} for mean fixed links),
#'   \item \code{df.residual}: residual degrees of freedom,
#'   \item \code{nobs}: number of observations,
#'   \item \code{loglik}: maximized log-likelihood value,
#'   \item \code{vcov}: variance-covariance matrix of the parameter estimates,
#'   \item \code{residuals}: a vector of quantile residuals,
#'   \item \code{AIC}, \code{BIC}, \code{HQIC}: Akaike, Schwarz, and Hannan-Quinn
#'   information criteria,
#'   \item \code{R2_N}, \code{R2_FC}: Nagelkerke, and Ferrari and Cribari-Neto
#'   pseudo R-squared measures,
#'   \item \code{zstat}: \eqn{z}-statistics for the coefficient tests,
#'   \item \code{pvalues}: \eqn{p}-values for the coefficient tests,
#'   \item \code{y}: the response vector,
#'   \item \code{x_names}: column names of the mean design matrix,
#'   \item \code{z_names}: column names of the dispersion design matrix,
#'   \item \code{control}: the control arguments passed to the \code{optim} call,
#'   \item \code{converged}: logical indicating successful convergence of \code{optim},
#'   \item \code{call}: the original function call,
#'   \item \code{formula}: the original two-part formula,
#'   \item \code{formula_mean}: formula for the mean submodel,
#'   \item \code{formula_disp}: formula for the dispersion submodel,
#'   \item \code{terms}: a list with \code{mean} and \code{dispersion} terms
#'   objects,
#'   \item \code{weights}: the weights used in fitting (if any),
#'   \item \code{offset}: the offset used in fitting (if any),
#'   \item \code{na.action}: the na.action attribute from the model frame,
#'   \item \code{subset}: the subset used in fitting (if any),
#'   \item \code{model}: the full model frame.
#'
#' }
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#' summary(fit)
#'
#' @references
#' Barndorff-Nielsen, O. E. and Jørgensen, B. (1991).
#' Some parametric models on the simplex.
#' \emph{Journal of Multivariate Analysis}, \bold{39}(1), 106--116.
#' \doi{10.1016/0047-259X(91)90008-P}
#'
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
#' Chapman and Hall, London.
#'
#' Song, P. X.-K. and Tan, M. (2000).
#' Marginal models for longitudinal continuous proportional data.
#' \emph{Biometrics}, \bold{56}(2), 496--502.
#' \doi{10.1111/j.0006-341X.2000.00496.x}
#'
#' Song, P. X.-K.; Qiu, Z. and Tan, M. (2004).
#' Modelling heterogeneous dispersion in marginal models for longitudinal
#' proportional data.
#' \emph{Biometrical Journal}, \bold{46}(5), 540--553.
#' \doi{10.1002/bimj.200110052}
#'
#' Song, P. X.-K. (2009).
#' Dispersion models in regression analysis.
#' \emph{Pakistan Journal of Statistics}, \bold{25}(4), 529--551.
#'
#' Zhang, P. and Qiu, Z. G. (2014).
#' Regression analysis of proportional data using simplex distribution.
#' \emph{SCIENTIA SINICA Mathematica}, \bold{44}(1), 89--104.
#' \doi{10.1360/012013-200}
NULL

#' @rdname simplexreg.fit
#' @importFrom stats model.frame model.response model.matrix terms plogis qnorm
#' @importFrom stats pnorm cor optim lm.fit weighted.mean delete.response model.weights model.offset
#' @export
simplexreg <- function(formula, data, subset, na.action, weights, offset,
                       link.mu = c("logit", "probit", "loglog", "cloglog", "cauchit",
                                   "plogit1", "plogit2"),
                       link.sigma2 = NULL, contrasts = NULL,
                       control = simplexreg.control(...),
                       model = TRUE, y = TRUE, x = FALSE, ...) {

  # Call
  cl <- match.call()

  # Set up model.frame call similar to betareg
  if(missing(data)) data <- environment(formula)
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "subset", "na.action", "weights",
               "offset", "contrasts"), names(mf), 0L)
  mf <- mf[c(1L, m)]
  mf$drop.unused.levels <- TRUE

  # Store original formula
  oformula <- as.formula(formula)

  # Handle formula - convert to formula object if needed
  if (!inherits(formula, "formula")) {
    stop("The 'formula' argument must be a formula object")
  }

  # Check for separator |
  formula_str <- as.character(formula)
  has_separator <- length(formula_str) >= 3 && grepl("\\|", formula_str[3])

  if (has_separator) {
    # Create Formula object for two-part formula
    formula <- Formula::as.Formula(formula)
    mf$formula <- formula

    # Evaluate model.frame
    mf[[1L]] <- quote(stats::model.frame)
    mf <- eval(mf, parent.frame())

    # Extract components
    Y <- model.response(mf, "numeric")

    if(any(Y <= 0 | Y >= 1, na.rm = TRUE)) {
      stop("Response variable must be in the open interval (0, 1)")
    }

    # Extract design matrices using Formula package
    X <- model.matrix(formula, data = mf, rhs = 1, contrasts.arg = contrasts)
    Z <- model.matrix(formula, data = mf, rhs = 2, contrasts.arg = contrasts)

    # Extract terms
    mtX <- terms(formula, data = mf, rhs = 1)
    mtZ <- delete.response(terms(formula, data = mf, rhs = 2))

    # Create formula objects for mean and dispersion
    formula_mean <- formula(formula, rhs = 1)
    formula_disp <- formula(formula, rhs = 2)

  } else {
    # Simple formula without separator
    mf$formula <- formula

    # Evaluate model.frame
    mf[[1L]] <- quote(stats::model.frame)
    mf <- eval(mf, parent.frame())

    # Extract components
    Y <- model.response(mf, "numeric")

    if(any(Y <= 0 | Y >= 1, na.rm = TRUE)) {
      stop("Response variable must be in the open interval (0, 1)")
    }

    # Extract design matrices
    X <- model.matrix(formula, data = mf)
    Z <- matrix(1, nrow = length(Y), ncol = 1)
    colnames(Z) <- "(Intercept)"
    rownames(Z) <- rownames(mf)

    # Extract terms
    mtX <- terms(formula, data = mf)
    mtZ <- terms(~ 1, data = mf)

    # Create formula objects
    formula_mean <- formula
    formula_disp <- as.formula("~ 1")
  }

  # Extract weights
  weights <- model.weights(mf)
  if(is.null(weights)) weights <- 1
  if(length(weights) == 1) weights <- rep.int(weights, length(Y))
  weights <- as.vector(weights)
  names(weights) <- rownames(mf)

  # Extract offset
  offset <- model.offset(mf)
  if(is.null(offset)) {
    offset <- rep(0, length(Y))
  } else {
    offset <- as.vector(offset)
  }
  names(offset) <- rownames(mf)

  # Save column names
  x_names <- colnames(X)
  z_names <- colnames(Z)

  is_constant_disp <- (ncol(Z) == 1 && all(Z[,1] == 1))
  # Set default link for sigma2 if not specified
  if (is.null(link.sigma2)) {
    if (!has_separator && is_constant_disp) {
      link.sigma2 <- "identity"
    } else {
      link.sigma2 <- "log"
    }
  } else {
    link.sigma2 <- match.arg(link.sigma2, c("log", "sqrt", "identity"))
  }

  parametric <- link.mu %in% c("plogit1","plogit2")

  # Handle null model with special initialization
  is_null_model <- (ncol(X) == 1 && all(X[,1] == 1)) && is_constant_disp
  if (is_null_model) {
    if(parametric) {
      ystar <- log(Y / (1 - Y))
      betaols_null <- weighted.mean(ystar, weights)
      muols_null <- exp(betaols_null) / (1 + exp(betaols_null))
      devtrans_null <- (Y - muols_null)^2 / (Y * (1 - Y) * muols_null^2 *
                                               (1 - muols_null)^2)
      deltaols_null <- weighted.mean(devtrans_null, weights)
      ini_nul <- c(betaols_null, deltaols_null, 1)
    } else {
      ystar <- fixed_mean_link(Y, link.mu)
      betaols_null <- weighted.mean(ystar, weights)
      muols_null <- fixed_mean_link_inv(betaols_null, link.mu)
      devtrans_null <- (Y - muols_null)^2 / (Y * (1 - Y) * muols_null^2 *
                                               (1 - muols_null)^2)
      deltaols_null <- weighted.mean(devtrans_null, weights)
      ini_nul <- c(betaols_null, deltaols_null)
    }

    rval <- simplexreg.fit(Y, X[, -1, drop = FALSE], Z[, -1, drop = FALSE],
                           weights = weights, offset = offset,
                           link.mu = link.mu, link.sigma2 = link.sigma2,
                           x_names = x_names, z_names = z_names,
                           start = ini_nul, control = control)

  } else {
    # Call standard fitting function
    rval <- simplexreg.fit(Y, X[, -1, drop = FALSE], Z[, -1, drop = FALSE],
                           weights = weights, offset = offset,
                           link.mu = link.mu, link.sigma2 = link.sigma2,
                           x_names = x_names, z_names = z_names, control = control)
  }

  # Further model information
  rval$call <- cl
  rval$formula <- oformula
  rval$formula_mean <- formula_mean
  rval$formula_disp <- formula_disp
  rval$terms <- list(mean = mtX, dispersion = mtZ)
  rval$weights <- if(identical(as.vector(weights), rep.int(1, length(Y)))) NULL else weights
  rval$offset <- if(identical(as.vector(offset), rep.int(0, length(Y)))) NULL else offset
  rval$na.action <- attr(mf, "na.action")
  rval$subset <- if(missing(subset)) NULL else subset
  if(model) rval$model <- mf
  if(y) rval$y <- Y
  if(x) rval$x <- list(mean = X, dispersion = Z)

  return(rval)
}

#' @rdname simplexreg.fit
#' @importFrom stats lm.wfit sd
#' @export
simplexreg.fit <- function(y, x, z, weights = NULL, offset = NULL,
                           link.mu = c("logit", "probit", "loglog", "cloglog", "cauchit",
                                       "plogit1", "plogit2"),
                           link.sigma2 = c("log", "sqrt", "identity"),
                           x_names = NULL, z_names = NULL,
                           control = simplexreg.control(...), ...){

  y <- as.vector(y)
  n <- length(y)

  # Handle weights
  if(is.null(weights)) weights <- rep.int(1, n)
  if(length(weights) == 1) weights <- rep.int(weights, n)
  weights <- as.vector(weights)
  nobs <- sum(weights > 0)

  # Handle offset
  if(is.null(offset)) {
    offset <- rep(0, n)
  } else {
    if(length(offset) == 1) offset <- rep.int(offset, n)
    offset <- as.vector(offset)
    if(length(offset) != n) stop("offset must have length equal to number of observations")
  }

  # Ensure x is a matrix
  if (is.null(x)) {
    x <- matrix(0, nrow = n, ncol = 0)
  } else if (!is.matrix(x)) {
    x <- as.matrix(x)
  }

  # Create x1 (with intercept)
  if (ncol(x) > 0) {
    x1 <- cbind(rep(1, n), x)
    colnames(x1) <- if(!is.null(x_names)) x_names else c("(Intercept)", paste0("X", seq_len(ncol(x))))
  } else {
    x1 <- matrix(1, nrow = n, ncol = 1)
    colnames(x1) <- if(!is.null(x_names)) x_names else "(Intercept)"
  }

  # Ensure z is a matrix
  if (is.null(z)) {
    z <- matrix(0, nrow = n, ncol = 0)
  } else if (!is.matrix(z)) {
    z <- as.matrix(z)
  }

  # Create z1 (with intercept)
  if (ncol(z) > 0) {
    z1 <- cbind(1, z)
    colnames(z1) <- if(!is.null(z_names)) z_names else c("(Intercept)", paste0("Z", seq_len(ncol(z))))
  } else {
    z1 <- matrix(1, nrow = n, ncol = 1)
    colnames(z1) <- if(!is.null(z_names)) z_names else "(Intercept)"
  }

  link.mu <- match.arg(link.mu)
  link.sigma2 <- match.arg(link.sigma2)
  parametric <- link.mu %in% c("plogit1","plogit2")

  # Model dimensions
  p <- ncol(x1)  # Number of mean parameters (including intercept)
  q <- ncol(z1)  # Number of dispersion parameters (including intercept)
  r <- if(parametric) (p + q + 1) else (p + q)

  # Control parameters
  ocontrol <- control
  method <- control$method
  gradient <- control$gradient
  hessian <- control$hessian
  start <- control$start
  fsmaxit <- control$fsmaxit
  fstol <- control$fstol
  trace <- control$trace

  # Remove control parameters not needed by optim
  control_optim <- control
  control_optim$method <- control_optim$gradient <- control_optim$hessian <- NULL
  control_optim$start <- control_optim$fsmaxit <- control_optim$fstol <- NULL
  control_optim$trace <- NULL

  # Set x_names and z_names if not provided
  if(is.null(x_names)) x_names <- colnames(x1)
  if(is.null(z_names)) z_names <- colnames(z1)

  # ============================
  # FITTED VALUES FUNCTION
  # ============================
  fitfun <- function(par, deriv = 0L) {
    beta <- par[1:p]
    delta <- par[(p+1):(p+q)]

    eta1 <- as.vector(x1 %*% beta) + offset
    eta2 <- as.vector(z1 %*% delta)

    # Soft safeguards for eta2
    if (link.sigma2 == "log") {
      eta2 <- pmin(pmax(eta2, -20), 20)
    } else if (link.sigma2 == "sqrt") {
      eta2 <- pmax(eta2, 0.01)
    } else if (link.sigma2 == "identity") {
      eta2 <- pmax(eta2, 1e-6)
    }

    if(parametric) {
      lambda <- pmax(par[r], 0.001)
      mu <- as.vector(parametric_mean_link_inv(eta1, lambda, link.mu))
    } else {
      lambda <- NULL
      mu <- as.vector(fixed_mean_link_inv(eta1, link.mu))
    }

    sigma2 <- as.vector(dispersion_link_inv(eta2, link.sigma2))

    # Standard safeguards
    mu <- pmin(pmax(mu, 1e-6), 1 - 1e-6)
    sigma2 <- pmax(sigma2, 1e-6)

    result <- list(
      beta = beta,
      delta = delta,
      lambda = lambda,
      eta1 = eta1,
      eta2 = eta2,
      mu = mu,
      sigma2 = sigma2
    )

    # First derivatives (for score function)
    if(deriv >= 1L) {
      if(parametric) {
        result$dmu_deta <- as.vector(parametric_mean_link_inv_deriv1(eta1, lambda, link.mu))

        # Compute rho (derivative w.r.t. lambda)
        if(link.mu == "plogit2") {
          exp_aval_frac <- plogis(eta1)^(1/lambda)
          result$rho <- as.vector(-exp_aval_frac * log(plogis(eta1)) / (lambda^2))
        } else {
          result$rho <- as.vector((-1/(lambda^2)) * ((exp(eta1) + 1)^(-1/lambda)) *
                                    log(exp(eta1) + 1))
        }
      } else {
        result$dmu_deta <- as.vector(fixed_mean_link_inv_deriv1(eta1, link.mu))
      }
      result$dsigma2_deta <- as.vector(dispersion_link_inv_deriv1(eta2, link.sigma2))
    }

    # Second derivatives (for Fisher information)
    if(deriv >= 2L) {
      result$muonemu <- as.vector(mu * (1 - mu))
      result$diff <- as.vector(y - mu)
    }

    return(result)
  }

  # ============================
  # LOG-LIKELIHOOD FUNCTION
  # ============================
  loglikfun <- function(par, fit = NULL){
    # Extract fitted parameters
    if(is.null(fit)) fit <- fitfun(par)

    with(fit, {
      # Check for problematic values
      if(!all(is.finite(mu)) || !all(is.finite(sigma2))) {
        return(-1e10)
      }

      # Safeguard y
      y_safe <- pmin(pmax(y, 1e-6), 1 - 1e-6)

      # Compute deviance
      diff <- y_safe - mu
      yoneminy <- y_safe * (1 - y_safe)
      muonemu <- mu * (1 - mu)
      dev <- (diff / muonemu)^2 / yoneminy

      if(!all(is.finite(dev))) {
        return(-1e10)
      }

      # Log-likelihood
      ll <- -0.5 * sum( weights *  (log(2 * pi) + log(sigma2) + 3 * log(y_safe * (1 - y_safe)) +
                         dev / sigma2))

      if(!is.finite(ll)) return(-1e10)

      return(ll)
    })
  }

  # ============================
  # SCORE FUNCTION (GRADIENT)
  # ============================
  gradfun <- function(par, sum = FALSE, fit = NULL){
    # Extract fitted means/precisions
    if(is.null(fit)) fit <- fitfun(par, deriv = 1L)

    with(fit, {
      # Safeguard y
      y_safe <- pmin(pmax(y, 1e-6), 1 - 1e-6)

      diff <- y_safe - mu
      yoneminy <- y_safe * (1 - y_safe)
      muonemu <- mu * (1 - mu)
      dev <- (diff / muonemu)^2 / yoneminy

      # Check for non-finite values
      if(!all(is.finite(c(mu, sigma2, diff, muonemu, dev)))) {
        warning("Non-finite values in score vector estimates.")
        if(parametric) {
          return(rep(0, p + q + 1))
        } else {
          return(rep(0, p + q))
        }
      }

      U <- weights * (dev / (sigma2 * muonemu) + 1 / (sigma2 * (muonemu)^3))
      a <- weights * ((-1 / (2 * sigma2)) + (dev / (2 * sigma2^2)))

      Ubeta <- crossprod(x1, dmu_deta * U * diff)
      Udelta <- crossprod(z1, dsigma2_deta * a)

      if(parametric) {
        if(!all(is.finite(rho))) {
          warning("Non-finite values in rho.")
          return(rep(0, p + q + 1))
        }
        Ulambda <- sum(U * rho * diff)
        rval <- c(Ubeta, Udelta, Ulambda)
      } else {
        rval <- c(Ubeta, Udelta)
      }

      return(as.vector(rval))
    })
  }

  # ============================
  # FISHER INFORMATION MATRIX
  # ============================
  hessfun <- function(par, fit) {
    with(fit, {
      # Safeguard y
      y_safe <- pmin(pmax(y, 1e-6), 1 - 1e-6)

      diff <- y_safe - mu
      yoneminy <- y_safe * (1 - y_safe)
      muonemu <- mu * (1 - mu)
      dev <- (diff / muonemu)^2 / yoneminy

      wi <- weights * ((3.0 * sigma2) / muonemu + (1 / (muonemu^3)))
      vi <- weights * (1 / (2 * sigma2^2))

      # Information matrix components
      Wbetabeta <- (1.0 / sigma2) * wi * (dmu_deta^2)
      Wdeltadelta <- vi * (dsigma2_deta^2)

      Kbetabeta <- crossprod(x1, Wbetabeta * x1)
      Kdeltadelta <- crossprod(z1, Wdeltadelta * z1)

      if(parametric) {
        Wbetalambda <- (1.0 / sigma2) * wi * dmu_deta
        Wlambdalambda <- (1.0 / sigma2) * wi

        Kbetalambda <- crossprod(x1, Wbetalambda * rho)
        Klambdabeta <- t(Kbetalambda)
        Klambdalambda <- sum(Wlambdalambda * rho^2)

        # Assemble information matrix
        K <- rbind(
          cbind(Kbetabeta, matrix(0, p, q), Kbetalambda),
          cbind(matrix(0, q, p), Kdeltadelta, matrix(0, q, 1)),
          cbind(Klambdabeta, matrix(0, 1, q), Klambdalambda)
        )
      } else {
        K <- rbind(
          cbind(Kbetabeta, matrix(0, p, q)),
          cbind(matrix(0, q, p), Kdeltadelta)
        )
      }
      chol2inv(chol(K))
    })
  }

  # ============================
  # STARTING VALUES
  # ============================
  if (!is.null(start)) {
    ini <- start
  } else {
    # Standard initialization with OLS
    if(parametric){
      ystar <- log(y / (1 - y))
      auxreg <- lm.wfit(x1, ystar, weights, offset)
      betaols <- auxreg$coefficients
      muols <- exp(x1 %*% betaols) / (1 + exp(x1 %*% betaols))
      devtrans <- (y - muols)^2 / (y * (1 - y) * muols^2 * (1 - muols)^2)
      deltaols <- lm.wfit(z1, dispersion_link(devtrans, link.sigma2),
                          weights)$coefficients

      lambdaini <- 1
      ini <- c(as.vector(betaols), as.vector(deltaols), as.numeric(lambdaini))
    } else {
      ystar <- fixed_mean_link(y, link.mu)
      auxreg <- lm.wfit(x1, ystar, weights, offset)
      betaols <- auxreg$coefficients
      muols <- fixed_mean_link_inv(x1 %*% betaols, link.mu)
      devtrans <- (y - muols)^2 / (y * (1 - y) * muols^2 * (1 - muols)^2)
      deltaols <- lm.wfit(z1, dispersion_link(devtrans, link.sigma2),
                          weights)$coefficients

      ini <- c(as.vector(betaols), as.vector(deltaols))
    }
  }

  # ============================
  # OPTIMIZATION (STEP 1: BFGS)
  # ============================

  opt <- optim(par = ini,
               fn = loglikfun,
               gr = if(gradient) gradfun else NULL,
               method = method,
               hessian = hessian,
               control = control_optim)

  par <- opt$par
  ll_opt <- opt$value

  # ============================
  # FISHER SCORING (STEP 2)
  # ============================

  iter <- 0
  converged_fs <- TRUE

  if(fsmaxit > 0) {
    if(trace) cat("Starting Fisher scoring iterations...\n")

    for(iter in 1:fsmaxit) {
      # Compute step
      fit <- fitfun(par, deriv = 2L)
      scores <- gradfun(par, fit = fit)
      InfoInv <- try(hessfun(par, fit = fit), silent = TRUE)

      if(inherits(InfoInv, "try-error")) {
        warning("Failed to invert information matrix")
        converged_fs <- FALSE
        break
      }

      step <- InfoInv %*% scores
      par_new <- par + step

      # Check improvement with backtracking
      step_factor <- 0
      improved <- FALSE

      while(step_factor <= 10 && !improved) {
        ll_new <- loglikfun(par_new)

        if(ll_new > ll_opt) {
          par <- par_new
          ll_opt <- ll_new
          improved <- TRUE
        } else {
          # Reduce step size
          par_new <- par + (0.5^step_factor) * step
          step_factor <- step_factor + 1
        }
      }

      # Convergence check
      if(all(abs(step) < fstol)) {
        if(trace) cat(sprintf("Converged after %d Fisher scoring iterations\n", iter))
        break
      }

      if(!improved && step_factor > 10) {
        if(trace) cat("No improvement after backtracking\n")
        break
      }
    }

    if(iter >= fsmaxit) {
      converged_fs <- FALSE
      warning(sprintf("Fisher scoring did not converge in %d iterations", fsmaxit))
    }
  }

  # ============================
  # EXTRACT FINAL ESTIMATES
  # ============================
  fit <- fitfun(par, deriv = 2L)
  beta <- fit$beta
  delta <- fit$delta
  lambda <- fit$lambda
  mu <- fit$mu
  sigma2 <- fit$sigma2
  eta1 <- fit$eta1
  eta2 <- fit$eta2

  # Final log-likelihood
  ll <- loglikfun(par, fit = fit)

  # Variance-covariance matrix
  vcov <- hessfun(par, fit = fit)

  # ============================
  # NAMING AND DIAGNOSTICS
  # ============================
  names(beta) <- x_names
  names(delta) <- z_names

  if(parametric) {
    vcov_names <- c(
      x_names,
      if(q == 1L && z_names[1] == "(Intercept)") {
        "(dispersion)"
      } else {
        paste("(dispersion)", z_names, sep = "_")
      },
      "lambda"
    )
  } else {
    vcov_names <- c(
      x_names,
      if(q == 1L && z_names[1] == "(Intercept)") {
        "(dispersion)"
      } else {
        paste("(dispersion)", z_names, sep = "_")
      }
    )
  }

  rownames(vcov) <- colnames(vcov) <- vcov_names
  stderror <- sqrt(diag(vcov))

  # Test statistics
  zstat <- abs(par / stderror)
  pvalues <- 2 * (1 - pnorm(zstat))

  # Residuals
  quantile_res <- qnorm(psimplex(y, mu, sigma2))

  # Pseudo R-squared
  R2_N <- 1 - exp((-2/sum(weights)) * (ll - simplexreg.nul(y, link.mu, weights)))

  gy <- if(parametric) {
    parametric_mean_link(y, lambda, link.mu)
  } else {
    fixed_mean_link(y, link.mu)
  }

  if(sd(eta1) > 0 && sd(gy) > 0) {
    R2_FC <- cor(eta1, gy)^2
  } else {
    R2_FC <- NA
  }

  # Information criteria
  aic <- -2 * ll + 2 * r
  bic <- -2 * ll + log(n) * r
  hqic <- -2 * ll + 2 * r * log(log(n))

  # ============================
  # RETURN OBJECT
  # ============================
  mean_coefs <- beta
  names(mean_coefs) <- x_names

  disp_coefs <- delta
  names(disp_coefs) <- z_names

  # Construct result object
  result <- list(
    coefficients = list(
      mean = mean_coefs,
      dispersion = disp_coefs,
      lambda = if(parametric) lambda else NA
    ),
    fitted.values = structure(mu, names = seq_len(n)),
    optim = list(
      start = ini,
      convergence = opt$convergence,
      counts = opt$counts, # mudei aqui
      method = method
    ),
    scoring = iter,
    mu.fv = structure(mu, names = seq_len(n)),
    mu.lp = structure(eta1, names = seq_len(n)),
    mu.x = x1,
    mu.link = link.mu,
    mu.df = p,
    sigma2.fv = structure(sigma2, names = seq_len(n)),
    sigma2.lp = structure(eta2, names = seq_len(n)),
    sigma2.x = z1,
    sigma2.link = link.sigma2,
    sigma2.df = q,
    lambda.fv = lambda,
    df.residual = n - r,
    nobs = n,
    loglik = ll,
    vcov = vcov,
    residuals = structure(quantile_res, names = seq_len(n)),
    AIC = aic,
    BIC = bic,
    HQIC = hqic,
    R2_FC = R2_FC,
    R2_N = R2_N,
    zstat = zstat,
    pvalues = pvalues,
    y = structure(y, names = seq_len(n)),
    x_names = x_names,
    z_names = z_names,
    control = ocontrol,
    converged = (opt$convergence == 0) && converged_fs
  )

  class(result) <- "simplexregression"

  return(result)
}

# ==============================================================================
# 3. NULL MODEL LOG-LIKELIHOOD
# ==============================================================================

#' @title Null Model Log-Likelihood for Simplex Regression
#' @description Computes the log-likelihood for the null (intercept-only) model in
#' simplex regression with a parametric or fixed mean link.
#'
#' @param y Numeric response vector \eqn{(0 < y < 1)}.
#' @param link.mu Mean link function: parametric ("plogit1", "plogit2") or fixed
#'   ("logit", "probit", "loglog", "cloglog", "cauchit").
#' @param weights Optional vector of weights (default: \code{NULL}).
#'
#' @return Numeric value of the null model log-likelihood
#' @keywords internal
simplexreg.nul <- function(y, link.mu, weights = NULL){
  y <- as.vector(y)
  n <- length(y)

  parametric <- link.mu %in% c("plogit1","plogit2")

  # Null model log-likelihood function
  fLogLiknull <- function(theta){
    sigma2 <- theta[2]
    eta1 <- cbind(rep(1, n)) %*% theta[1]

    if(parametric){
      lambda = pmax(theta[3], 0.001)
      mu <- parametric_mean_link_inv(eta1, lambda, link.mu)
    } else {
      mu <- fixed_mean_link_inv(eta1, link.mu)
    }

    y <- pmin(pmax(y, 1e-6), 1 - 1e-6)
    mu <- pmin(pmax(mu, 1e-6), 1 - 1e-6)
    sigma2 <- pmax(sigma2, 1e-6)

    diff <- as.vector(y - mu)
    yoneminy <- as.vector(y * (1 - y))
    muonemu <- as.vector(mu * (1 - mu))
    dev <- (diff / muonemu)^2 / yoneminy

    # Log-likelihood function
    adFunc <- -0.5 * sum( weights * (log(2*pi) + log(sigma2) + 3*log(yoneminy) + dev/sigma2))
    adFunc
  }

  # Initial values
  if(parametric){
    ystar <- log(y / (1 - y))
    betaols_null <- weighted.mean(ystar, weights)
    muols_null <- exp(betaols_null) / (1 + exp(betaols_null))
    devtrans_null <- (y - muols_null)^2 / (y * (1 - y) * muols_null^2 *
                                             (1 - muols_null)^2)
    deltaols_null <- weighted.mean(devtrans_null, weights)

    ini_nul <- c(betaols_null, deltaols_null, 1)
  } else {
    ystar <- fixed_mean_link(y, link.mu)
    betaols_null <- weighted.mean(ystar, weights)
    muols_null <- fixed_mean_link_inv(betaols_null, link.mu)
    devtrans_null <- (y - muols_null)^2 / (y * (1 - y) * muols_null^2 *
                                             (1 - muols_null)^2)
    deltaols_null <- weighted.mean(devtrans_null, weights)

    ini_nul <- c(betaols_null, deltaols_null)
  }

  # Optimize
  opt_nul <- optim(ini_nul, fLogLiknull, method = "BFGS",
                   control=list(fnscale = -1, maxit = 5000, reltol = .Machine$double.eps^(0.5)))

  k <- opt_nul$value
  return(k)
}

library(testthat)
library(SimplexRegression)

# ==============================================================================
# test-simplexreg_fit.R
# Tests for: simplexreg, simplexreg.fit, simplexreg.control
# ==============================================================================

# ---------------------------------------------------------------------------
# Fixture: simulate data and fit models once for all tests
# ---------------------------------------------------------------------------
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
dat <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

fit_plogit1 <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "plogit1")
fit_plogit2 <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "plogit2")
fit_logit   <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit")
fit_probit  <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "probit")
fit_fixdisp <- simplexreg(y ~ x1 + x2,  data = dat, link.mu = "plogit1")
fit_null    <- simplexreg(y ~ 1,  data = dat, link.mu = "logit")

# ==============================================================================
# 1. simplexreg.control
# ==============================================================================

test_that("simplexreg.control returns a list with expected fields", {
  ctrl <- simplexreg.control()
  expect_type(ctrl, "list")
  expect_true(all(c("method", "maxit", "gradient", "hessian", "start",
                    "fsmaxit", "fstol", "reltol", "fnscale") %in% names(ctrl)))
})

test_that("simplexreg.control defaults are correct", {
  ctrl <- simplexreg.control()
  expect_equal(ctrl$method,   "BFGS")
  expect_equal(ctrl$maxit,    5000L)
  expect_true(ctrl$gradient)
  expect_false(ctrl$hessian)
  expect_equal(ctrl$fsmaxit,  500L)
  expect_equal(ctrl$fstol,    1e-8)
  expect_equal(ctrl$fnscale, -1)   # must be -1 (maximization)
})

test_that("simplexreg.control rejects invalid method", {
  expect_error(simplexreg.control(method = "Newton"))
})

test_that("simplexreg.control accepts valid methods", {
  for (m in c("BFGS", "Nelder-Mead", "CG", "L-BFGS-B", "SANN")) {
    expect_no_error(simplexreg.control(method = m))
  }
})

test_that("simplexreg.control warns if fnscale is modified", {
  expect_warning(simplexreg.control(fnscale = 1), "fnscale must not be modified")
})

test_that("simplexreg.control L-BFGS-B drops reltol and sets factr/pgtol", {
  ctrl <- simplexreg.control(method = "L-BFGS-B")
  expect_null(ctrl$reltol)
  expect_false(is.null(ctrl$factr))
  expect_false(is.null(ctrl$pgtol))
})

test_that("simplexreg.control accepts user-supplied start values", {
  ctrl <- simplexreg.control(start = c(0, 1, 2))
  expect_equal(ctrl$start, c(0, 1, 2))
})

test_that("simplexreg.control sets default reltol when reltol = NULL and method != L-BFGS-B", {
  ctrl <- simplexreg.control(method = "BFGS", reltol = NULL)
  expect_equal(ctrl$reltol, .Machine$double.eps^(0.5))
})

# ==============================================================================
# 2. simplexreg — object class and structure
# ==============================================================================

test_that("simplexreg returns an object of class simplexregression", {
  expect_s3_class(fit_plogit1, "simplexregression")
  expect_s3_class(fit_logit,   "simplexregression")
})

test_that("simplexreg object contains all required components", {
  required <- c("coefficients", "fitted.values", "optim", "scoring",
                "mu.fv", "mu.lp", "mu.x", "mu.link", "mu.df",
                "sigma2.fv", "sigma2.lp", "sigma2.x", "sigma2.link", "sigma2.df",
                "lambda.fv", "df.residual", "nobs", "loglik", "vcov",
                "residuals", "AIC", "BIC", "HQIC", "R2_FC", "R2_N",
                "zstat", "pvalues", "y", "converged")
  for (comp in required) {
    expect_true(comp %in% names(fit_plogit1),
                label = paste("component", comp, "present"))
  }
})

test_that("simplexreg coefficients list has correct sub-elements", {
  cf <- fit_plogit1$coefficients
  expect_true(!is.null(cf$mean))
  expect_true(!is.null(cf$dispersion))
  expect_true(!is.null(cf$lambda))    # parametric: lambda present
})

test_that("simplexreg coefficients lambda is NULL or NA for fixed link", {
  lam <- fit_logit$coefficients$lambda
  expect_true(is.null(lam) || (length(lam) == 1L && is.na(lam)))
})

test_that("simplexreg coefficients lambda is numeric for parametric link", {
  expect_true(is.numeric(fit_plogit1$coefficients$lambda))
  expect_true(is.finite(fit_plogit1$coefficients$lambda))
  expect_true(fit_plogit1$coefficients$lambda > 0)
})

# ==============================================================================
# 3. simplexreg — fitted values and linear predictors
# ==============================================================================

test_that("simplexreg mu.fv is in (0, 1) for all link types", {
  for (fit in list(fit_plogit1, fit_plogit2, fit_logit, fit_probit)) {
    expect_true(all(fit$mu.fv > 0 & fit$mu.fv < 1),
                label = paste(fit$mu.link, "mu.fv in (0,1)"))
  }
})

test_that("simplexreg sigma2.fv is positive for all link types", {
  for (fit in list(fit_plogit1, fit_logit)) {
    expect_true(all(fit$sigma2.fv > 0),
                label = paste(fit$mu.link, "sigma2.fv > 0"))
  }
})

test_that("simplexreg mu.fv and fitted.values are identical", {
  expect_equal(fit_plogit1$mu.fv, fit_plogit1$fitted.values)
  expect_equal(fit_logit$mu.fv,   fit_logit$fitted.values)
})

test_that("simplexreg mu.fv has length n", {
  expect_length(fit_plogit1$mu.fv, n)
  expect_length(fit_logit$mu.fv,   n)
})

test_that("simplexreg sigma2.fv has length n", {
  expect_length(fit_plogit1$sigma2.fv, n)
})

test_that("simplexreg linear predictor mu.lp has length n", {
  expect_length(fit_plogit1$mu.lp, n)
  expect_true(all(is.finite(fit_plogit1$mu.lp)))
})

# ==============================================================================
# 4. simplexreg — design matrices
# ==============================================================================

test_that("simplexreg mu.x has correct dimensions", {
  expect_equal(nrow(fit_plogit1$mu.x), n)
  expect_equal(ncol(fit_plogit1$mu.x), 3L)  # intercept + x1 + x2
})

test_that("simplexreg sigma2.x has correct dimensions (constant dispersion)", {
  expect_equal(nrow(fit_fixdisp$sigma2.x), n)
  expect_equal(ncol(fit_fixdisp$sigma2.x), 1L)  # intercept only
})

test_that("simplexreg sigma2.x has correct dimensions (variable dispersion)", {
  expect_equal(ncol(fit_plogit1$sigma2.x), 2L)  # intercept + z1
})

test_that("simplexreg mu.x first column is all ones (intercept)", {
  expect_true(all(fit_plogit1$mu.x[, 1] == 1))
})

# ==============================================================================
# 5. simplexreg — degrees of freedom and model dimensions
# ==============================================================================

test_that("simplexreg nobs equals n", {
  expect_equal(fit_plogit2$nobs, n)
  expect_equal(fit_logit$nobs,   n)
})

test_that("simplexreg df.residual is correct for parametric link", {
  r <- ncol(fit_plogit2$mu.x) + ncol(fit_plogit2$sigma2.x) + 1L  # +1 for lambda
  expect_equal(fit_plogit2$df.residual, n - r)
})

test_that("simplexreg df.residual is correct for fixed link", {
  r <- ncol(fit_logit$mu.x) + ncol(fit_logit$sigma2.x)
  expect_equal(fit_logit$df.residual, n - r)
})

test_that("simplexreg mu.df equals number of mean parameters", {
  expect_equal(fit_plogit2$mu.df, ncol(fit_plogit2$mu.x))
})

test_that("simplexreg sigma2.df equals number of dispersion parameters", {
  expect_equal(fit_plogit2$sigma2.df, ncol(fit_plogit2$sigma2.x))
})

# ==============================================================================
# 6. simplexreg — log-likelihood and information criteria
# ==============================================================================

test_that("simplexreg loglik is finite", {
  expect_true(is.finite(fit_plogit1$loglik))
  expect_true(is.finite(fit_logit$loglik))
})

test_that("simplexreg AIC, BIC, HQIC are finite", {
  for (ic in c("AIC", "BIC", "HQIC")) {
    expect_true(is.finite(fit_plogit1[[ic]]),
                label = paste(ic, "is finite"))
  }
})

# ==============================================================================
# 7. simplexreg — variance-covariance matrix
# ==============================================================================

test_that("simplexreg vcov is a square numeric matrix", {
  vc <- fit_plogit1$vcov
  expect_true(is.matrix(vc))
  expect_equal(nrow(vc), ncol(vc))
})

test_that("simplexreg vcov dimension matches number of parameters", {
  r_par <- ncol(fit_plogit1$mu.x) + ncol(fit_plogit1$sigma2.x) + 1L  # +1 lambda
  expect_equal(dim(fit_plogit1$vcov), c(r_par, r_par))
})

test_that("simplexreg vcov is positive definite (all eigenvalues > 0)", {
  ev <- eigen(fit_plogit1$vcov, only.values = TRUE)$values
  expect_true(all(ev > 0))
})

test_that("simplexreg vcov is symmetric", {
  vc <- fit_plogit1$vcov
  expect_equal(vc, t(vc), tolerance = 1e-12)
})

test_that("simplexreg vcov has named rows and columns", {
  vc <- fit_plogit1$vcov
  expect_false(is.null(rownames(vc)))
  expect_false(is.null(colnames(vc)))
})

# ==============================================================================
# 8. simplexreg — residuals, R-squared, zstat
# ==============================================================================

test_that("simplexreg residuals have length n", {
  expect_length(fit_plogit1$residuals, n)
  expect_length(fit_logit$residuals,   n)
})

test_that("simplexreg quantile residuals are approximately standard normal", {
  res <- fit_plogit1$residuals
  expect_true(abs(mean(res)) < 0.5)
  expect_true(abs(sd(res) - 1) < 0.5)
})

test_that("simplexreg R2_N is in (0, 1)", {
  expect_true(fit_plogit1$R2_N > 0 & fit_plogit1$R2_N < 1)
  expect_true(fit_logit$R2_N   > 0 & fit_logit$R2_N   < 1)
})

test_that("simplexreg R2_FC is in (0, 1)", {
  expect_true(fit_plogit1$R2_FC > 0 & fit_plogit1$R2_FC < 1)
})

test_that("simplexreg zstat and pvalues have correct length", {
  r_par <- ncol(fit_plogit2$mu.x) + ncol(fit_plogit2$sigma2.x) + 1L
  expect_length(fit_plogit2$zstat,   r_par)
  expect_length(fit_plogit2$pvalues, r_par)
})

test_that("simplexreg zstat values are non-negative", {
  expect_true(all(fit_plogit2$zstat >= 0))
})

test_that("simplexreg pvalues are in [0, 1]", {
  expect_true(all(fit_plogit2$pvalues >= 0 & fit_plogit2$pvalues <= 1))
})

# ==============================================================================
# 9. simplexreg — formula and call
# ==============================================================================

test_that("simplexreg stores the call", {
  expect_true(!is.null(fit_plogit2$call))
  expect_true(inherits(fit_plogit2$call, "call"))
})

test_that("simplexreg stores the formula", {
  expect_true(!is.null(fit_plogit2$formula))
  expect_true(inherits(fit_plogit2$formula, "formula"))
})

test_that("simplexreg mu.link is stored correctly", {
  expect_equal(fit_plogit2$mu.link, "plogit2")
  expect_equal(fit_logit$mu.link,   "logit")
})

test_that("simplexreg sigma2.link is stored", {
  expect_true(!is.null(fit_plogit2$sigma2.link))
  expect_true(fit_plogit2$sigma2.link %in%
                c("log", "sqrt", "identity"))
})

# ==============================================================================
# 10. simplexreg — convergence and optim info
# ==============================================================================

test_that("simplexreg converged flag is logical", {
  expect_true(is.logical(fit_plogit1$converged))
  expect_true(is.logical(fit_logit$converged))
})

test_that("simplexreg optim list contains required fields", {
  opt <- fit_plogit1$optim
  expect_true(all(c("start", "convergence", "counts", "method") %in% names(opt)))
})

test_that("simplexreg optim method is a valid string", {
  expect_true(fit_plogit1$optim$method %in%
                c("BFGS", "Nelder-Mead", "CG", "L-BFGS-B", "SANN"))
})

test_that("simplexreg scoring field is a non-negative integer", {
  expect_true(is.numeric(fit_plogit1$scoring))
  expect_true(fit_plogit1$scoring >= 0)
})

# ==============================================================================
# 11. simplexreg — input validation
# ==============================================================================

test_that("simplexreg rejects response outside (0, 1)", {
  dat_bad      <- dat
  dat_bad$y[1] <- 0
  expect_error(simplexreg(y ~ x1 + x2 | z1, data = dat_bad, link.mu = "logit"),
               "open interval \\(0, 1\\)")

  dat_bad$y[1] <- 1
  expect_error(simplexreg(y ~ x1 + x2 | z1, data = dat_bad, link.mu = "logit"),
               "open interval \\(0, 1\\)")
})

test_that("simplexreg requires a formula object", {
  expect_error(simplexreg("y ~ x1", data = dat), "formula")
})

# ==============================================================================
# 12. simplexreg — all mean link functions fit without error
# ==============================================================================

all_links <- c("logit", "probit", "loglog", "cloglog", "cauchit",
               "plogit1", "plogit2")

test_that("simplexreg fits without error for all mean link functions", {
  for (lnk in all_links) {
    expect_no_error(
      simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = lnk)
    )
  }
})

test_that("simplexreg produces finite log-likelihood for all mean link functions", {
  for (lnk in all_links) {
    fit <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = lnk)
    expect_true(is.finite(fit$loglik),
                label = paste("link.mu =", lnk, "loglik finite"))
  }
})

# ==============================================================================
# 13. simplexreg — all dispersion link functions fit without error
# ==============================================================================

disp_links <- c("log", "sqrt", "identity")

test_that("simplexreg fits without error for all dispersion link functions", {
  for (lnk in disp_links) {
    expect_no_error(
      simplexreg(y ~ x1 + x2 | z1, data = dat,
                 link.mu = "logit", link.sigma2 = lnk)
    )
  }
})

test_that("simplexreg produces finite log-likelihood for all dispersion links", {
  for (lnk in disp_links) {
    fit <- simplexreg(y ~ x1 + x2 | z1, data = dat,
                      link.mu = "logit", link.sigma2 = lnk)
    expect_true(is.finite(fit$loglik),
                label = paste("link.sigma2 =", lnk, "loglik finite"))
  }
})

# ==============================================================================
# 14. simplexreg — null (intercept-only) model
# ==============================================================================

test_that("simplexreg fits null model without error", {
  expect_no_error(simplexreg(y ~ 1, data = dat, link.mu = "logit"))
})

test_that("simplexreg null model has 1 mean coefficient", {
  expect_equal(length(fit_null$coefficients$mean), 1L)
})

test_that("simplexreg null model mu.x has 1 column (intercept only)", {
  expect_equal(ncol(fit_null$mu.x), 1L)
})

test_that("simplexreg null model loglik is less than full model loglik", {
  # Adding predictors should improve the log-likelihood
  expect_true(fit_null$loglik <= fit_logit$loglik)
})

# ==============================================================================
# 15. simplexreg — variable dispersion model
# ==============================================================================

test_that("simplexreg variable-dispersion model has correct sigma2.df", {
  expect_equal(fit_plogit1$sigma2.df, 2L)  # intercept + z1
})

test_that("simplexreg variable-dispersion sigma2.fv values are not all equal", {
  expect_true(length(unique(round(fit_plogit1$sigma2.fv, 6))) > 1L)
})

test_that("simplexreg variable-dispersion loglik >= constant-dispersion loglik", {
  # More flexible model should have at least as good a log-likelihood
  expect_true(fit_plogit1$loglik >= fit_fixdisp$loglik - 0.5)
})

# ==============================================================================
# 16. simplexreg — formula without | separator (simple formula)
# ==============================================================================

test_that("simplexreg accepts simple formula without | separator", {
  expect_no_error(simplexreg(y ~ x1 + x2, data = dat, link.mu = "logit"))
})

test_that("simplexreg simple formula produces constant dispersion model", {
  fit_simple <- simplexreg(y ~ x1 + x2, data = dat, link.mu = "logit")
  expect_equal(ncol(fit_simple$sigma2.x), 1L)
  expect_equal(length(unique(round(fit_simple$sigma2.fv, 8))), 1L)
})

# ==============================================================================
# 17. simplexreg — starting values via control
# ==============================================================================

test_that("simplexreg accepts user-supplied starting values via control", {
  # Use estimated parameters from a converged fit as starting values
  beta0  <- fit_logit$coefficients$mean
  delta0 <- fit_logit$coefficients$dispersion
  start  <- c(beta0, delta0)

  expect_no_error(
    simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit",
               control = simplexreg.control(start = start))
  )
})

test_that("simplexreg with user start produces consistent log-likelihood", {
  beta0  <- fit_logit$coefficients$mean
  delta0 <- fit_logit$coefficients$dispersion
  start  <- c(beta0, delta0)

  fit_warm <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit",
                         control = simplexreg.control(start = start))
  expect_equal(fit_warm$loglik, fit_logit$loglik, tolerance = 1e-4)
})

# ==============================================================================
# 18. simplexreg — Fisher scoring can be disabled
# ==============================================================================

test_that("simplexreg with fsmaxit = 0 runs without error", {
  expect_no_error(
    simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit",
               control = simplexreg.control(fsmaxit = 0L))
  )
})

test_that("simplexreg with fsmaxit = 0 has scoring = 0", {
  fit_no_fs <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit",
                          control = simplexreg.control(fsmaxit = 0L))
  expect_equal(fit_no_fs$scoring, 0L)
})

# ==============================================================================
# 19. simplexreg — parametric vs fixed link: lambda presence
# ==============================================================================

test_that("simplexreg lambda.fv is numeric and positive for plogit1", {
  expect_true(is.numeric(fit_plogit1$lambda.fv))
  expect_true(fit_plogit1$lambda.fv > 0)
})

test_that("simplexreg lambda.fv is NA or NULL for fixed link", {
  expect_true(is.null(fit_logit$lambda.fv) || is.na(fit_logit$lambda.fv))
  expect_true(is.null(fit_probit$lambda.fv) || is.na(fit_probit$lambda.fv))
})

test_that("simplexreg works when data is not supplied (uses environment)", {
  y_env  <- dat$y
  x1_env <- dat$x1
  x2_env <- dat$x2
  z1_env <- dat$z1
  expect_no_error(simplexreg(y_env ~ x1_env + x2_env | z1_env, link.mu = "logit"))
})

test_that("simplexreg rejects response outside (0,1) for simple formula", {
  dat_bad      <- dat
  dat_bad$y[1] <- 0
  expect_error(
    simplexreg(y ~ x1 + x2, data = dat_bad, link.mu = "logit"),
    "open interval \\(0, 1\\)"
  )
})

test_that("simplexreg accepts offset argument", {
  off <- rep(0, nrow(dat))
  expect_no_error(
    simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit", offset = off)
  )
})

test_that("simplexreg stores design matrices when x = TRUE", {
  fit_x <- simplexreg(y ~ x1 + x2 | z1, data = dat, link.mu = "logit", x = TRUE)
  expect_true(!is.null(fit_x$x))
  expect_true(!is.null(fit_x$x$mean))
  expect_true(!is.null(fit_x$x$dispersion))
})

# test_that("simplexreg fits null model with parametric link", {
#   expect_no_error(
#     simplexreg(y ~ 1, data = dat, link.mu = "plogit1")
#   )
# })
#
# test_that("simplexreg null model with parametric link has finite loglik", {
#   fit_null_par <- simplexreg(y ~ 1, data = dat, link.mu = "plogit1")
#   expect_true(is.finite(fit_null_par$loglik))
#   expect_true(!is.null(fit_null_par$coefficients$lambda))
# })

# ==============================================================================
# 20. simplexreg.fit — low-level interface
# ==============================================================================

test_that("simplexreg.fit returns an object of class simplexregression", {
  X_mat <- cbind(x1, x2)
  Z_mat <- matrix(0, nrow = n, ncol = 0)
  fit_low <- simplexreg.fit(y, X_mat, Z_mat, link.mu = "logit",
                            link.sigma2 = "log")
  expect_s3_class(fit_low, "simplexregression")
})

test_that("simplexreg.fit produces finite log-likelihood", {
  X_mat <- cbind(x1, x2)
  Z_mat <- matrix(0, nrow = n, ncol = 0)
  fit_low <- simplexreg.fit(y, X_mat, Z_mat, link.mu = "logit",
                            link.sigma2 = "log")
  expect_true(is.finite(fit_low$loglik))
})

test_that("simplexreg.fit results are consistent with simplexreg", {
  X_mat <- cbind(x1, x2)
  Z_mat <- cbind(z1)
  fit_low <- simplexreg.fit(y, X_mat, Z_mat, link.mu = "logit",
                            link.sigma2 = "log",
                            x_names = c("(Intercept)", "x1", "x2"),
                            z_names = c("(Intercept)", "z1"))
  # Log-likelihoods should be very close
  expect_equal(fit_low$loglik, fit_logit$loglik, tolerance = 1e-3)
})

test_that("simplexreg.fit handles NULL x (intercept-only mean model)", {
  expect_no_error(
    simplexreg.fit(y, x = NULL, z = NULL,
                   link.mu = "logit", link.sigma2 = "log")
  )
})

test_that("simplexreg.fit accepts scalar weights", {
  X_mat <- cbind(x1, x2)
  Z_mat <- cbind(z1)
  expect_no_error(
    simplexreg.fit(y, X_mat, Z_mat, weights = 1,
                   link.mu = "logit", link.sigma2 = "log")
  )
})

test_that("simplexreg.fit accepts data.frame as x", {
  X_df <- as.data.frame(cbind(x1, x2))
  Z_mat <- cbind(z1)
  expect_no_error(
    simplexreg.fit(y, X_df, Z_mat, link.mu = "logit", link.sigma2 = "log")
  )
})

test_that("simplexreg.fit accepts data.frame as z", {
  X_mat <- cbind(x1, x2)
  Z_df  <- as.data.frame(cbind(z1))
  expect_no_error(
    simplexreg.fit(y, X_mat, Z_df, link.mu = "logit", link.sigma2 = "log")
  )
})

test_that("simplexreg.fit accepts scalar offset", {
  X_mat <- cbind(x1, x2)
  Z_mat <- cbind(z1)
  expect_no_error(
    simplexreg.fit(y, X_mat, Z_mat, offset = 1,
                   link.mu = "logit", link.sigma2 = "log")
  )
})

test_that("simplexreg.fit rejects offset with wrong length", {
  X_mat <- cbind(x1, x2)
  Z_mat <- cbind(z1)
  expect_error(
    simplexreg.fit(y, X_mat, Z_mat, offset = rep(0, n - 1),
                   link.mu = "logit", link.sigma2 = "log"),
    "offset must have length"
  )
})

################################################################################
#                     SIMPLEX REGRESSION - RESET TEST                          #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: RESET (Regression Equation Specification Error Test) for        #
#              testing functional form misspecification                        #
################################################################################

#' @title RESET Test for Simplex Regression
#' @description Performs the Ramsey's RESET misspecification test in simplex
#' regression models.
#'
#' @param model An object of class \code{"simplexregression"}.
#' @param dispersion Logical. If \code{TRUE}, includes the augmented terms
#' in the dispersion submodel as well. Default is \code{TRUE}.
#' @param power Integer vector specifying which powers of the fitted mean linear predictor
#' (or fitted mean values) to include as additional regressors. Default is \code{2}
#' (squared term only). Use \code{power = 2:3} to include both squared and
#' cubic terms, following the convention of \code{lmtest::resettest}.
#' @param type Character string specifying the base for the augmented terms.
#' \code{"lp"} (default) uses the fitted mean linear predictor \eqn{\boldsymbol{\hat{\eta}_1}};
#' \code{"fitted"} uses the fitted mean values \eqn{\boldsymbol{\hat{\mu}}} on the (0, 1)
#' scale.
#'
#' @details
#' The RESET test augments the original model by adding powers of the fitted mean linear
#' predictor (or fitted mean values) as additional covariates. Under the null
#' hypothesis of correct functional form, these additional terms should not
#' be significant.
#'
#' Under H0, the likelihood ratio statistic is asymptotically distributed
#' as chi-squared with degrees of freedom equal to the number of augmented
#' terms added (i.e., \code{length(power)} if \code{dispersion = FALSE}, or
#' \code{2 * length(power)} if \code{dispersion = TRUE}).
#'
#' If \code{dispersion = TRUE}, the augmented terms are added to both the
#' mean and dispersion submodels. If \code{FALSE}, they are only added to
#' the mean submodel.
#'
#' @return An object of class \code{"htest"} containing:
#' \itemize{
#'   \item \code{statistic}: The likelihood ratio test statistic,
#'   \item \code{parameter}: Degrees of freedom,
#'   \item \code{p.value}: The \eqn{p}-value of the test,
#'   \item \code{method}: Description of the test,
#'   \item \code{data.name}: Model formula.
#' }
#'
#' @references
#' Ramsey, J. B. (1969). Tests for specification errors in classical linear
#' least-squares regression analysis. \emph{Journal of the Royal Statistical
#' Society: Series B}, \bold{31}(2), 350--371.
#' \doi{10.1111/j.2517-6161.1969.tb00796.x}
#'
#' @examples
#' data(ReadingSkills, package = "SimplexRegression")
#' fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
#'                  data = ReadingSkills)
#'
#' # Default: squared linear predictor in both submodels
#' resettest(fit)
#'
#' # RESET test only for mean submodel
#' resettest(fit, dispersion = FALSE)
#'
#' # Include squared and cubic terms
#' resettest(fit, power = 2:3)
#'
#' # Use fitted values instead of linear predictor
#' resettest(fit, type = "fitted")
#'
#' @importFrom stats pchisq
#'
#' @seealso \code{\link{simplexreg}}.
#' @export
resettest <- function(model, dispersion = TRUE, power = 2,
                      type = c("lp", "fitted")) {

  if (!inherits(model, "simplexregression"))
    stop("'model' must be an object of class 'simplexregression'")

  if (!is.numeric(power) || any(power < 2) || any(power != floor(power)))
    stop("'power' must be a vector of integers >= 2 (e.g., 2 or 2:3).")

  type <- match.arg(type)

  METHOD <- "RESET test"
  DNAME  <- paste(deparse(model$formula), collapse = "\n")

  y <- as.vector(model$y)

  # Base for augmented terms: linear predictor or fitted values
  base <- if (type == "lp") model$mu.lp else model$mu.fv

  # Build matrix of augmented terms: base^2, base^3, ...
  aug_cols <- matrix(
    sapply(power, function(pw) base^pw),
    ncol = length(power)
  )
  colnames(aug_cols) <- paste0(if (type == "lp") "lp^" else "mu^", power)

  x <- cbind(model$mu.x, aug_cols)
  z <- if (dispersion) cbind(model$sigma2.x, aug_cols) else model$sigma2.x

  ctrl          <- model$control
  ctrl$hessian  <- FALSE

  modelh1 <- tryCatch(
    simplexreg.fit(y,
                   x[, colnames(x) != "(Intercept)", drop = FALSE],
                   z[, colnames(z) != "(Intercept)", drop = FALSE],
                   link.mu     = model$mu.link,
                   link.sigma2 = model$sigma2.link,
                   x_names     = colnames(x),
                   z_names     = colnames(z),
                   control     = ctrl,
                   weights     = model$weights),
    error = function(e) {
      stop("Failed to fit the augmented model for the RESET test. ",
           "The information matrix is singular. ",
           "This may occur with parametric link functions or near-collinear predictors. ",
           "Try fitting the model with a fixed link function (e.g., link.mu = 'logit').")
    }
  )

  lH0  <- model$loglik
  lH1  <- modelh1$loglik
  LR   <- 2 * (lH1 - lH0)
  df   <- model$df.residual - modelh1$df.residual
  PVAL <- pchisq(LR, df, lower.tail = FALSE)

  names(df) <- "df"
  names(LR) <- "RESET"

  RVAL <- list(statistic = LR, parameter = df, p.value = PVAL,
               method = METHOD, data.name = DNAME)
  class(RVAL) <- "htest"

  return(RVAL)
}

#' Body Composition Data for Australian Rowers
#'
#' @description
#' Hematological, body composition, and anthropometric measurements on
#' 37 elite rowers (22 female, 15 male) at the Australian Institute of
#' Sport (AIS), a subset of a larger dataset collected across multiple
#' sports. The data are useful for investigating sex-based differences
#' in blood and body composition among highly trained athletes.
#'
#' @docType data
#' @usage data(AISRowing)
#'
#' @format A data frame with 37 observations and 12 variables:
#' \describe{
#'   \item{sex}{Factor. Sex of the athlete (\code{female} or \code{male}).}
#'   \item{rcc}{Numeric. Red blood cell count (in \eqn{10^{12}} per litre).}
#'   \item{wcc}{Numeric. White blood cell count (in \eqn{10^{12}} per litre).}
#'   \item{hc}{Numeric. Hematocrit (in percent).}
#'   \item{hb}{Numeric. Hemoglobin concentration (in g per decilitre).}
#'   \item{ferr}{Numeric. Plasma ferritin concentration (in ng per millilitre).}
#'   \item{bmi}{Numeric. Body mass index (in kg per metre-squared).}
#'   \item{ssf}{Numeric. Sum of skin folds (in mm).}
#'   \item{bfat}{Numeric. Body fat proportion, originally measured
#'         in percent and rescaled to the unit interval (0, 1).}
#'   \item{lbm}{Numeric. Lean body mass (in kg).}
#'   \item{ht}{Numeric. Height (in cm).}
#'   \item{wt}{Numeric. Weight (in kg).}
#' }
#'
#' @details
#' The original measurements were collected in the late 1980s by Richard
#' Telford and Ross Cunningham at the Australian Institute of Sport (AIS),
#' and were later compiled and popularized by Cook and Weisberg (1994).
#' \code{AISRowing} is the 37-athlete subset corresponding to rowing,
#' out of 202 athletes across all sports in the original dataset. The
#' full dataset (covering all sports, not just rowing) is also discussed
#' in Weisberg (2005, Section 6.4).
#'
#' @source
#' Telford, R. D. and Cunningham, R. B. (1991). Sex, sport, and body-size
#' dependency of hematology in highly trained athletes. \emph{Medicine &
#' Science in Sports & Exercise}, \bold{23}(7), 788--794.
#'
#' @references
#' Cook, R. D. and Weisberg, S. (1994). \emph{An Introduction to Regression
#' Graphics}. John Wiley & Sons, New York.
#'
#' Weisberg, S. (2005). \emph{Applied Linear Regression}, 3rd edition.
#' New York: Wiley, Section 6.4.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the data
#' data(AISRowing)
#'
#' # Quick overview
#' head(AISRowing)
#' str(AISRowing)
#'
#' # Summary statistics
#' summary(AISRowing)
#'
#' # Sex-based comparison
#' aggregate(bfat ~ sex, data = AISRowing, summary)
#' boxplot(bfat ~ sex, data = AISRowing,
#'         main = "Body Fat Proportion by Sex",
#'         ylab = "Body fat (proportion)")
"AISRowing"

#' Public Opinion on Abortion Across U.S. States
#'
#' @description
#' This dataset examines the relationship between public opposition to abortion
#' and demographic, religious, and socioeconomic characteristics across the
#' 50 U.S. states and the District of Columbia. The response variable is the
#' proportion of adults who believe abortion should be illegal in all or most
#' cases, bounded in the open interval (0, 1), making the data suitable for simplex
#' regression and beta regression analyses.
#'
#' @docType data
#' @usage data(AbortionOpposition)
#'
#' @format A data frame with 51 observations and 9 variables:
#' \describe{
#'   \item{state}{Factor. U.S. state or the District of Columbia.}
#'   \item{abortion_opp}{Numeric. Proportion of adults who believe
#'         abortion should be illegal in all or most cases, based on the Pew
#'         Research Center 2023--24 Religious Landscape Study.}
#'   \item{relig_attend}{Numeric. Percentage of adults who report
#'         attending religious services at least once a week, based on the
#'         Pew Research Center 2023--24 Religious Landscape Study.}
#'   \item{income}{Numeric. Mean annual household income (USD), 2024.}
#'   \item{sex_ratio}{Numeric. Number of males per 100 females, 2024.}
#'   \item{female}{Numeric. Percentage of the population that is female, 2024.}
#'   \item{pop_18_24}{Numeric. Percentage of the population aged 18--24 years,
#'         obtained directly from the 2024 American Community Survey (ACS)
#'         1-Year Estimates.}
#'   \item{bachelors}{Numeric. Percentage of adults aged 25 and older with a
#'         bachelor's degree or higher, 2024.}
#'   \item{urban}{Numeric. Percentage of the population living in urban areas,
#'         based on 2020 U.S. Census Bureau urban area criteria (a densely
#'         settled core of census blocks meeting minimum housing unit and/or
#'         population density thresholds, requiring at least 2,000 housing
#'         units or 5,000 people).}
#' }
#'
#' @details
#' The dataset was assembled by the package authors from multiple publicly
#' available sources. The response variable, \code{abortion_opp},
#' was obtained from the Pew Research Center 2023--24 Religious Landscape
#' Study, originally published as a percentage (0--100 scale), and rescaled
#' to the (0, 1) interval by dividing by 100.
#'
#' The explanatory variables were obtained as follows:
#' \enumerate{
#'   \item \code{relig_attend} was obtained from the Pew Research Center
#'         2023--24 Religious Landscape Study.
#'   \item \code{income}, \code{sex_ratio}, and \code{bachelors} were obtained from
#'         World Population Review, which itself compiles estimates from the
#'         U.S. Census Bureau/American Community Survey. World Population
#'         Review is cited here as the immediate data source, following
#'         standard practice for reproducibility.
#'   \item \code{female} was obtained from the U.S. Census Bureau's American
#'         Community Survey (ACS) 1-Year Estimates, Table S0101 (Age and
#'         Sex), the same table used for \code{pop_18_24}. Note that
#'         \code{sex_ratio} and \code{female} are closely related (both
#'         describe the sex composition of each state's population, from
#'         different sources) and should generally not be included
#'         together as covariates in the same regression model, as doing
#'         so may introduce near-perfect collinearity.
#'   \item \code{pop_18_24} corresponds to the percent estimate of the
#'         population aged 18--24 years reported directly in Table S0101
#'         (Age and Sex) of the 2024 American Community Survey (ACS)
#'         1-Year Estimates.
#'   \item \code{urban} was obtained from World Population Review and is
#'         based on 2020 U.S. Census Bureau data, the most recent Census
#'         estimate of urbanization by state available at the time this
#'         dataset was compiled. Unlike the other explanatory variables,
#'         which reflect 2023--24 estimates, \code{urban} reflects the 2020
#'         Census urban area delineation (see \code{@format} above for
#'         definition details).
#' }
#'
#' @source
#' Pew Research Center (2025). 2023-24 Religious Landscape Study (RLS)
#' Dataset. \doi{10.58094/3kwb-bf52}. Data accessed in 2026.
#'
#' World Population Review (2024). Per Capita Income by State.
#' \url{https://worldpopulationreview.com/state-rankings/per-capita-income-by-state}.
#' Data accessed in 2026.
#'
#' World Population Review (2024). Sex Ratio by State.
#' \url{https://worldpopulationreview.com/state-rankings/sex-ratio-by-state}.
#' Data accessed in 2026.
#'
#' World Population Review (2024). Educational Attainment by State.
#' \url{https://worldpopulationreview.com/state-rankings/educational-attainment-by-state}.
#' Data accessed in 2026.
#'
#' World Population Review (2024). Most Urban States.
#' \url{https://worldpopulationreview.com/state-rankings/most-urban-states}.
#' Data accessed in 2026.
#'
#' U.S. Census Bureau (2024). Age and Sex. American Community Survey,
#' ACS 1-Year Estimates Subject Tables, Table S0101.
#' \url{https://data.census.gov/table/ACSST1Y2024.S0101?q=age+and+sex+by+state&moe=false}.
#' Data accessed in 2026.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the data
#' data(AbortionOpposition)
#'
#' # Quick overview
#' head(AbortionOpposition)
#' str(AbortionOpposition)
#'
#' # Summary statistics
#' summary(AbortionOpposition)
#'
#' # Relationship between religious attendance and abortion opposition
#' plot(abortion_opp ~ relig_attend,
#'      data = AbortionOpposition,
#'      pch = 19,
#'      xlab = "Religious attendance (%)",
#'      ylab = "Abortion opposition (proportion)")
#'
#' # Correlation among numeric variables
#' cor(AbortionOpposition[, c("abortion_opp",
#'                         "relig_attend",
#'                         "income",
#'                         "sex_ratio",
#'                         "female",
#'                         "pop_18_24",
#'                         "bachelors",
#'                         "urban")])
"AbortionOpposition"

################################################################################
#                     SIMPLEX REGRESSION - RAO SCORE TEST                      #
# Author: Maria Eduarda da Cruz Justino and Francisco Cribari-Neto             #
# Date: 2026-05-26                                                             #
# Description: Rao score test for testing the link parameter lambda = 1        #
################################################################################

#' @title Rao's Score Test for Simplex Regression with Parametric Mean Link Function
#' @description Performs a Rao's score test to test whether the link parameter
#' \eqn{\lambda} equals 1, which corresponds to evaluating the null hypothesis
#' that the mean link function is the standard logit.
#'
#' @param model An object of class \code{"simplexregression"} fitted with the
#' fixed logit mean link (\code{link.mu = "logit"})
#' @param link.mu Character string specifying the parametric link function under the
#' alternative hypothesis. Options are \code{"plogit1"} or \code{"plogit2"}.
#'
#' @details
#' Given that the fixed logit link function is a particular case of the parametric
#' logit link functions when \eqn{\lambda = 1}, it is possible to test whether the
#' mean link function is logit by testing \eqn{H_0: \lambda = 1} against
#' \eqn{H_1: \lambda \neq 1}.
#'
#' The score test statistic is computed from the component of the score vector
#' associated with \eqn{\lambda} and the corresponding element of the inverse
#' Fisher information matrix, both evaluated at the maximum likelihood estimator
#' under the null hypothesis (i.e., with \eqn{\lambda} fixed at 1). For the full
#' expression of the test statistic, see Justino and Cribari-Neto (2026).
#'
#' Under regularity conditions, the null hypothesis, and when \eqn{n} is large,
#' the test statistic follows a chi-squared distribution with 1 degree of freedom.
#'
#' @return An object of class \code{"htest"} containing:
#' \itemize{
#'   \item \code{statistic}: The score test statistic,
#'   \item \code{parameter}: Degrees of freedom (always 1),
#'   \item \code{p.value}: The \eqn{p}-value of the test,
#'   \item \code{method}: Description of the test,
#'   \item \code{data.name}: Description of the link comparison being tested
#'   (e.g., \code{"Logit vs plogit1"})
#' }
#'
#' @references
#' Justino, M. E. C. and Cribari-Neto, F. (2026).
#' Simplex regression with a flexible logit link: Inference and application
#' to cross-country impunity data.
#' \emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
#'
#' Rao, C. R. (1948). Large sample tests of statistical hypotheses concerning
#' several parameters with applications to problems of estimation.
#' \emph{Mathematical Proceedings of the Cambridge Philosophical Society},
#'  \bold{44}(1), 50--57. \doi{10.1017/S0305004100023987}
#'
#' @examples
#' # Simulate data
#' set.seed(2026)
#' n <- 100
#' x1 <- runif(n, 0, 1)
#' x2 <- runif(n, 0, 1)
#' z1 <- runif(n, 0, 1)
#' mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
#' sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
#' y <- rsimplex(n, mu, sigma2)
#' data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)
#'
#' # Fit model with logit
#' model <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")
#'
#' # Test if lambda = 1
#' scoretest(model, link.mu = "plogit1")
#'
#' @seealso \code{\link{parametric_mean_link}}
#'
#' @importFrom stats pchisq plogis
#'
#' @export
scoretest <- function(model, link.mu = c("plogit1", "plogit2")) {

  if (!inherits(model, "simplexregression")) {
    stop("'model' must be an object of class 'simplexregression'")
  }

  link.mu <- match.arg(link.mu)

  # Validate that the fitted model uses logit link
  if (model$mu.link != "logit") {
    stop("The score test is only valid for models fitted with the logit mean link function. ",
         "The null hypothesis H0: lambda = 1 corresponds to the logit link, which must ",
         "be the link function of the model under H0. ",
         "Model provided uses link: '", model$mu.link, "'.")
  }

  METHOD <- "Rao score test"
  DNAME <- paste("Logit vs", link.mu)

  y <- as.vector(model$y)
  mu <- as.vector(model$mu.fv)
  sigma2 <- as.vector(model$sigma2.fv)
  lambda <- 1 # Under H0
  eta1 <- as.vector(model$mu.lp)
  mu_x <- model$mu.x

  diff <- as.vector(y - mu)

  yoneminy <- as.vector(y * (1 - y))
  muonemu <- as.vector(mu * (1 - mu))
  dev <- (diff / muonemu)^2 / yoneminy

  if(link.mu == "plogit2"){
    exp_aval_frac <- plogis(eta1)^(1/lambda)
    rho <- as.vector(- exp_aval_frac * (log(plogis(eta1))) / (lambda^2))
  } else{
    rho <- as.vector((-1/(lambda^2)) * ((exp(eta1) + 1) ^ (-1/lambda)) *
                       (log(exp(eta1) + 1)))
  }

  Ui <- (dev / muonemu) + (1 / (muonemu^3))
  Ulambda <- sum(diff * Ui * rho / sigma2)

  l1id1 <- as.vector(fixed_mean_link_inv_deriv1(eta1, model$mu.link))
  wi <- (3 * sigma2) / muonemu + (1 / (muonemu^3))

  Klambdalambda <- sum(wi * rho^2 / sigma2)
  Klambdabeta <- colSums(wi * rho * l1id1 / sigma2 * mu_x)
  Kbetabeta <- crossprod(mu_x, (wi * l1id1^2 / sigma2) * mu_x)

  Kbetabeta_inv_Klambdabeta <- tryCatch(
    solve(Kbetabeta, Klambdabeta),
    error = function(e) {
      stop("Failed to compute the score test statistic. ",
           "The information matrix block for the mean submodel is singular. ",
           "This may occur with near-collinear predictors in the mean submodel. ",
           "Try removing redundant covariates or refitting with a reduced formula.")
    }
  )

  vcovlambda_inv <- Klambdalambda - sum(Klambdabeta * Kbetabeta_inv_Klambdabeta)

  S <- Ulambda^2 / vcovlambda_inv
  df <- 1L
  PVAL <- pchisq(S, df, lower.tail = FALSE)

  names(df) <- "df"
  names(S) <- "S"

  RVAL <- list(statistic = S, parameter = df, p.value = PVAL,
               method = METHOD, data.name = DNAME)
  class(RVAL) <- "htest"

  return(RVAL)
}

---
title: "Modeling Relative Air Humidity with Simplex Regression"
output:
  rmarkdown::html_vignette:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Modeling Relative Air Humidity with Simplex Regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  fig.align  = "center",
  message    = FALSE,
  warning    = FALSE
)
old_opts <- options(width = 70, prompt = "R> ", continue = "+  ", digits = 5)
```

# Introduction

## The simplex distribution

Regression models for continuous responses restricted to the unit interval
$(0, 1)$ play a central role in empirical research across economics,
biostatistics, political science, and environmental studies. Typical examples
include rates, proportions, indices, and concentration measures. The
**simplex regression model** provides a flexible and theoretically appealing
framework for such data. It is based on the simplex distribution, introduced
by Barndorff-Nielsen and Jørgensen (1991), which is indexed by a mean
parameter $\mu \in (0, 1)$ and a dispersion parameter $\sigma^2 > 0$.

A random variable $Y$ follows a simplex distribution,
$Y \sim S^-(\mu, \sigma^2)$, if its density is

$$f(y; \mu, \sigma^2) = \left\{2\pi\sigma^2[y(1-y)]^3\right\}^{-1/2}
\exp\!\left(-\frac{d(y;\mu)}{2\sigma^2}\right), \quad y \in (0,1),$$

where $d(y;\mu) = (y - \mu)^2 / [y(1-y)\mu^2(1-\mu)^2]$ is the unit
deviance. The mean and variance are $\mathrm{E}(Y) = \mu$ and
$\mathrm{Var}(Y) < \mu(1-\mu)$, with variance function
$\mathrm{V}(\mu) = \mu^3(1-\mu)^3$.

The simplex distribution can assume a wide variety of shapes — symmetric,
left- or right-skewed, J-shaped, U-shaped, and even bimodal — making it
particularly flexible for modeling bounded responses. Unlike the beta
distribution, which is indexed by a precision parameter, the simplex
distribution is indexed by a dispersion parameter $\sigma^2$: smaller values
correspond to higher concentration around the mean, whereas larger values
indicate greater variability.

## The simplex regression model

The simplex regression model with variable dispersion relates the mean and
dispersion to covariates through link functions:

$$g(\mu_i) = \mathbf{x}_i^\top \boldsymbol{\beta} = \eta_{1i}
\quad \text{and} \quad
h(\sigma^2_i) = \mathbf{z}_i^\top \boldsymbol{\gamma} = \eta_{2i},$$

where $\boldsymbol{\beta}$ and $\boldsymbol{\gamma}$ are unknown parameter
vectors. For models with a **parametric mean link**, an additional shape
parameter $\lambda > 0$ is estimated jointly with $\boldsymbol{\beta}$ and
$\boldsymbol{\gamma}$:

$$g(\mu_i, \lambda) = \mathbf{x}_i^\top \boldsymbol{\beta}.$$

The `SimplexRegression` package supports five fixed link functions for the
mean submodel — `"logit"`, `"probit"`, `"loglog"`, `"cloglog"`,
`"cauchit"` — and two parametric links, `"plogit1"` and `"plogit2"`,
defined as

$$\text{plogit1:}\; g(\mu_i,\lambda) = \log\!\left[(1-\mu_i)^{-\lambda}-1\right],
\qquad
\text{plogit2:}\; g(\mu_i,\lambda) = \log\!\left(\frac{\mu_i^\lambda}{1-\mu_i^\lambda}\right).$$

Both reduce to the standard logit when $\lambda = 1$. For the dispersion
submodel, the links `"log"`, `"sqrt"`, and `"identity"` are available.
All parameters are estimated by maximum likelihood using a mixed algorithm
that combines the BFGS quasi-Newton method with Fisher scoring steps.

## Advantages over beta regression

Compared to beta regression, simplex regression has several distinctive
advantages:

- It can represent **bimodal densities**, which the beta distribution
  cannot;
- Maximum likelihood estimators for the mean and dispersion parameters are
  **asymptotically orthogonal**, simplifying inference and improving
  numerical stability;
- Likelihood-based inferences are typically **less sensitive to atypical
  observations**;
- Its higher-order variance function $\mu^3(1-\mu)^3$ allows **more
  flexible variance patterns** than the quadratic variance function of
  the beta distribution.

## This vignette

This vignette illustrates the main functionalities of the `SimplexRegression`
package through the analysis of monthly average relative humidity data
recorded in Brasília, Brazil ($n = 312$ observations). The analysis covers:

- data preparation and exploratory summary;
- fitting simplex regression models with **parametric** and **fixed** mean
  link functions;
- link function selection via the **penalized Scout Score** (`penalized.ss`)
  and **penalized information criteria** (`penalized.ic`);
- **likelihood ratio test** for nested models (`lrtest`);
- score test for $H_0\!: \lambda = 1$ (`scoretest`);
- specification testing with the **RESET test** (`resettest`);
- fitted values, residuals, and predictive performance (`press`);
- prediction and simulation;
- influence measures: hat values, Cook's distance, generalized leverage;
- diagnostic plots (`plot`, `halfnormal.plot`);
- **local influence** and **diagnostic distance** measures
  (`local.influence`, `diag.im`, `diag.distances`).

# The Data

Relative air humidity (RH) is defined as the ratio of the partial pressure
of water vapor in the air to the saturation vapor pressure at the same
temperature. In Brasília, low RH levels during dry months are associated with
adverse health outcomes (asthma, nosebleeds, dehydration), increased
forest-fire risk, and pressure on water resources. The dataset contains
monthly averages from January 2000 to December 2025, obtained from the
National Institute of Meteorology (INMET).

```{r setup}
library(SimplexRegression)
data(RelativeHumidity, package = "SimplexRegression")
head(RelativeHumidity, 5)
```

| Variable | Description |
|----------|-------------|
| `date`   | Observation date (end of month) |
| `rh`     | Monthly average relative humidity, rescaled to $(0, 1)$ |
| `ins`    | Total insolation (hours), with two missing values |
| `ins2`   | Total insolation with imputed missing values |
| `pre`    | Total precipitation (mm), with two missing values |
| `pre2`   | Total precipitation with imputed missing values |
| `cld`    | Average cloudiness (tenths) |
| `ap`     | Average atmospheric pressure (hPa) |
| `tmax`   | Average maximum temperature (°C) |
| `ws`     | Average wind speed (m/s) |
| `wd`     | Predominant wind direction (degrees) |

## Data preparation

To capture the 12-month seasonal cycle we add harmonic regressors
$s_i = \sin(2\pi i/12)$ and $c_i = \cos(2\pi i/12)$, and a rainy-season
dummy equal to 1 for October, November, and December (the rainy season in
Brasília) and 0 otherwise.

```{r data-prep}
RH       <- RelativeHumidity
RH$hs    <- sin(2 * pi * seq_len(nrow(RH)) / 12)
RH$hc    <- cos(2 * pi * seq_len(nrow(RH)) / 12)
RH$dummy <- as.integer(as.integer(format(RH$date, "%m")) %in% 10:12)
```

## Descriptive summary

```{r summary-rh}
summary(RH$rh)
cat(sprintf(
  "Std. dev.: %.4f  |  Skewness: %.4f\n",
  sd(RH$rh),
  mean(((RH$rh - mean(RH$rh)) / sd(RH$rh))^3)
))
```

The response ranges from 0.301 to 0.851, with mean 0.635 and moderate left
skewness ($-0.466$), consistent with a higher concentration of RH values
above the mean during the wetter months. These features support the use of a
unit-interval distribution such as the simplex.

# Model Formula

All models fitted in this vignette share the same formula structure. The
two-part formula `y ~ x | z` specifies the mean submodel (left of `|`) and
the dispersion submodel (right of `|`) simultaneously. The mean submodel
includes insolation (`ins2`), maximum temperature (`tmax`), wind speed (`ws`),
harmonic terms (`hs`, `hc`) to capture seasonality, a rainy-season dummy,
and its interaction with wind speed. The dispersion submodel contains only
precipitation (`pre2`), allowing the variability of RH to depend on rainfall.

```{r formula}
formula <- rh ~ ins2 + tmax + ws + hs + hc + dummy + I(dummy * ws) | pre2
```

# Parametric Mean Link Functions

## Fitting and model selection

We first fit models with the two parametric mean links available in the
package, `plogit1` and `plogit2`. These links include a shape parameter
$\lambda$ estimated jointly with the regression coefficients. When $\lambda = 1$,
both reduce to the standard logit link; values $\lambda \neq 1$ introduce
asymmetry in the link, adding flexibility to capture non-standard
relationships between the mean response and the linear predictor.

```{r models-plogit}
fit_p1 <- simplexreg(formula, data = RH, link.mu = "plogit1")
fit_p2 <- simplexreg(formula, data = RH, link.mu = "plogit2")
```

## Penalized Scout Score and information criteria

When comparing models that all use a parametric link, the **penalized Scout
Score** (`penalized.ss`) and **penalized information criteria**
(`penalized.ic`) are recommended. These functions incorporate an additional
penalty controlled by `kappa` (default `kappa = 0.1`) that accounts for the
complexity introduced by $\lambda$ relative to its deviation from the
standard logit. The model with the highest Scout Score or lowest penalized
criterion is selected. 

```{r penalized-ss-param}
penalized.ss(fit_p1, fit_p2, kappa = 0.1)
```

```{r penalized-ic}
penalized.ic(fit_p1, fit_p2, kappa = 0.1)
```

All penalized criteria unanimously select the `plogit1` link. The Scout
Score of `fit_p1` is substantially higher than that of `fit_p2`, and all
three penalized information criteria are lower for `fit_p1`.

# Fixed Mean Link Functions

## Fitting

We also fit models with all five fixed mean links supported by the package.
Fixed links do not include an additional parameter and are therefore always
comparable using standard (unpenalized) criteria.

```{r models-fixed}
fit_loglog  <- simplexreg(formula, data = RH, link.mu = "loglog")
fit_logit   <- simplexreg(formula, data = RH, link.mu = "logit")
fit_probit  <- simplexreg(formula, data = RH, link.mu = "probit")
fit_cauchit <- simplexreg(formula, data = RH, link.mu = "cauchit")
fit_cloglog <- simplexreg(formula, data = RH, link.mu = "cloglog")
```

## Comparing all candidates

When comparing models with both fixed and parametric links simultaneously,
the **unpenalized** version (`kappa = 0`) must be used, since the penalty
for $\lambda$ is only meaningful when all candidates use a parametric link.

```{r penalized-ss-all}
penalized.ss(
  fit_loglog, fit_logit, fit_probit,
  fit_cauchit, fit_cloglog, fit_p1,
  kappa = 0
)
```

The log-log link achieves the highest Scout Score and is selected as the
best-fitting specification. We proceed with `fit_loglog` for all remaining
analyses.

# Model Summary

The `summary` method provides a standard regression output: estimated
coefficients for the mean and dispersion submodels, standard errors, Wald
$z$-statistics and $p$-values, the maximized log-likelihood, information
criteria (AIC, BIC, HQIC), pseudo-$R^2$ measures, and details on the
numerical optimization procedure.

```{r summary-fit}
summary(fit_loglog)
```

All regression coefficients are statistically significant at the 5% level.
The harmonic terms (`hs`, `hc`) and the rainy-season dummy capture the
strong seasonal pattern in RH, while insolation (`ins2`), maximum temperature
(`tmax`), and wind speed (`ws`) explain the within-season variation. The
Nagelkerke and Ferrari–Cribari-Neto pseudo-$R^2$ values both exceed 0.95,
indicating an excellent fit. The model converged in 18 BFGS iterations
followed by 7 Fisher scoring steps.

# Standard Information Criteria

The standard (unpenalized) AIC, BIC, and HQIC are accessible via the usual
S3 generics and compare multiple models simultaneously. Lower values
indicate a better trade-off between fit and complexity.

```{r aic-bic}
AIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
BIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
HQIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
```

The log-log model consistently achieves the lowest values across all three
criteria, confirming the Scout Score selection.

# Extracting Model Components

## Coefficients and covariance matrix

The `coef` method extracts estimated coefficients for the full model or for
each submodel separately, controlled by the `model` argument
(`"full"`, `"mean"`, or `"dispersion"`). The `vcov` method returns the
corresponding variance-covariance matrix.

```{r coef-vcov}
coef(fit_loglog)                          # full coefficient vector
coef(fit_loglog, model = "mean")          # mean submodel only
coef(fit_loglog, model = "dispersion")    # dispersion submodel only
round(vcov(fit_loglog, model = "mean"), 6)  # vcov of mean submodel
```

## Log-likelihood

The `logLik` method returns the maximized log-likelihood together with the
number of estimated parameters (`df`) and the number of observations. This
is compatible with `AIC()` and `BIC()` from the `stats` package.

```{r loglik}
logLik(fit_loglog)
```

# Hypothesis Tests

## Likelihood ratio test for nested models

The `lrtest` method from the `lmtest` package performs likelihood ratio tests
between nested simplex regression models. A natural application is testing
whether the dispersion is constant against a model with covariate-dependent
dispersion. The restricted model (constant dispersion) is obtained via
`update` by replacing the dispersion submodel with an intercept only
(`| 1`).

```{r lrtest}
fit_loglog_null <- update(fit_loglog, . ~ . | 1)
lmtest::lrtest(fit_loglog, fit_loglog_null)
```

The very small $p$-value ($< 0.001$) provides strong evidence against
constant dispersion, confirming that RH variability is significantly
associated with precipitation (`pre2`). The model with variable dispersion
is therefore preferred.

## Score test for $H_0\!: \lambda = 1$

The `scoretest` function implements Rao's score test for the hypothesis that
the standard logit link is adequate, i.e., $H_0\!: \lambda = 1$. The test
requires a model fitted under $H_0$ (with the logit link) and tests against
either `"plogit1"` or `"plogit2"` as the alternative. Rejection of $H_0$
suggests that a parametric link provides a better fit than the standard
logit.

```{r scoretest}
fit_logit_h0 <- simplexreg(formula, data = RH, link.mu = "logit")
scoretest(fit_logit_h0, link.mu = "plogit1")
scoretest(fit_logit_h0, link.mu = "plogit2")
```

## RESET test for functional form

The `resettest` function implements Ramsey's RESET test, which augments the
original model with the squared fitted linear predictor as an additional
covariate and tests its significance. By default (`dispersion = TRUE`), the
squared predictor is added to both the mean and dispersion submodels.
Setting `dispersion = FALSE` restricts the augmentation to the mean submodel
only. Failure to reject $H_0$ supports the adequacy of the assumed
functional form.

```{r resettest}
resettest(fit_loglog)                      # both submodels augmented
resettest(fit_loglog, dispersion = FALSE)  # mean submodel only
```

The large $p$-values in both cases (0.82 and 0.97) provide no
evidence of functional form misspecification in either submodel.

# Fitted Values, Residuals, and Predictive Performance

## Fitted values and residuals

The `fitted` method returns fitted mean values $\hat{\mu}_i$. The
`residuals` method supports ten residual types. Quantile residuals
(default) have an approximately standard normal distribution under correct
model specification and are recommended for general diagnostics. Weighted
residuals are particularly useful for half-normal plots.

```{r fitted-resid}
head(fitted(fit_loglog))
head(residuals(fit_loglog, type = "quantile"))  # approx. N(0,1)
head(residuals(fit_loglog, type = "pearson"))
head(residuals(fit_loglog, type = "weighted"))  # for halfnormal.plot
```

## Predictive performance: PRESS and $P^2$

The `press` function computes the PRESS (Predicted Residual Error Sum of
Squares) statistic and the associated cross-validation measures $P^2$ and
adjusted $P^2_c$. The $P^2$ statistic is a cross-validation analog of the
coefficient of determination, computed from the hat matrix diagonal without
refitting the model $n$ times. Values close to 1 indicate strong predictive
accuracy. The function accepts multiple models simultaneously, enabling
direct comparison of predictive performance across competing specifications.

```{r press}
press(fit_loglog)                            # single model
press(fit_loglog, fit_logit, fit_probit)     # comparing models
```

The log-log model achieves the highest $P^2$ and $P^2_c$ values, confirming
its superior predictive performance. The PRESS statistic reflects the total
cumulative squared prediction error across all leave-one-out fits.

# Prediction and Simulation

## Prediction

The `predict` method computes predicted values for new or original data. The
`type` argument controls the output: `"response"` returns fitted means
$\hat{\mu}_i$; `"link"` returns the linear predictors $\hat{\eta}_{1i}$ and
$\hat{\eta}_{2i}$; `"dispersion"` returns fitted dispersion values
$\hat{\sigma}^2_i$.

```{r predict}
head(predict(fit_loglog, type = "response"))        # fitted means
head(predict(fit_loglog, type = "link")$mean)       # mean linear predictor
head(predict(fit_loglog, type = "link")$dispersion) # dispersion predictor
head(predict(fit_loglog, type = "dispersion"))      # fitted sigma^2

# Out-of-sample prediction
new_obs <- RH[1:3, ]
predict(fit_loglog, newdata = new_obs, type = "response")
```

## Simulation

The `simulate` method generates response vectors from the fitted simplex
distribution using the estimated $\hat{\mu}_i$ and $\hat{\sigma}^2_i$. The
`nsim` argument controls the number of replicates and `seed` ensures
reproducibility. The result is a data frame with `nsim` columns.

```{r simulate}
set.seed(2026)
sims <- simulate(fit_loglog, nsim = 3)
head(sims)
```

# Influence Measures

## Hat values and Cook's distance

The `hatvalues` method returns the diagonal elements $h_{ii}$ of the hat
matrix, measuring the leverage of each observation on the fitted values.
Observations with $h_{ii}$ much larger than the average $r/n$ (where $r$
is the number of estimated parameters) deserve closer inspection. The
`cooks.distance` method computes approximate Cook's distances, which combine
leverage and residual size to measure overall influence on the parameter
estimates. The `type` argument selects the residual type used
(`"pearson"` or `"weighted"`).

```{r influence}
hii  <- hatvalues(fit_loglog)
cook <- cooks.distance(fit_loglog, type = "pearson")
cat(sprintf("Leverages  — max: %.4f  mean: %.4f\n", max(hii),  mean(hii)))
cat(sprintf("Cook's D   — max: %.4f\n", max(cook)))
```

No observation has a disproportionately large leverage or Cook's distance,
suggesting no single point dominates the fit.

## Generalized leverage

The `gleverage` function computes the generalized leverage values, which
extend the classical hat values to account for both the mean and dispersion
submodels. High generalized leverage indicates observations that may exert
disproportionate influence on the fitted values across the entire model.

```{r gleverage}
gl <- gleverage(fit_loglog)
cat(sprintf("Generalized leverage — max: %.4f  mean: %.4f\n",
            max(gl), mean(gl)))
```

# Diagnostic Plots

## Residual and fit plots

The `plot` method produces up to eight diagnostic plots selected via `which`:
(1) residuals vs. observation index; (2) residuals vs. fitted values;
(3) residuals vs. linear predictor; (4) observed vs. fitted values;
(5) normal Q-Q plot; (6) Cook's distances; (7) generalized
leverages. The default residual type is `"quantile"`. For plots 6 and 7,
`threshold` highlights influential observations and `label.pos` controls
label placement (1 = below, 2 = left, 3 = above, 4 = right).

```{r plots-1-7, fig.height = 8, fig.cap = "Diagnostic plots (1–7) for the fitted simplex regression model with log-log link."}
oldpar <- par(mfrow = c(3, 3))
plot(fit_loglog, which = 1:5, ask = FALSE)
plot(fit_loglog, which = 6, threshold = 0.15, label.pos = c(4,2,3))
plot(fit_loglog, which = 7, threshold = 0.08, label.pos = c(3,3,3,4))
par(oldpar)
```

The quantile residuals are approximately standard normal, with no systematic
patterns across fitted values or the linear predictor. Cook's distances and
generalized leverages are uniformly small, with no evidence of unduly
influential observations.

## Local influence

Local influence analysis quantifies how small perturbations to the model
assumptions affect the parameter estimates. Two perturbation schemes are
supported: `"case.weight"` perturbs observation weights and `"response"`
perturbs the response values. The `parameter` argument selects the
parameter block of interest — `"theta"` (all parameters), `"beta"` (mean
submodel), or `"gamma"` (dispersion submodel). The `type` argument controls
the influence measure: `"Ci"` for the total local influence index $C_i$,
or `"dmax"` for the direction of maximum curvature. Observations exceeding
`threshold` are labeled in the index plot.

```{r local-influence-cw, fig.height = 4.5, fig.cap = "Total local influence $C_i$ under case-weight and response perturbation for all parameters."}
oldpar <- par(mfrow = c(1, 2))
local.influence(
  fit_loglog,
  scheme    = "case.weight",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.5,
  label.pos = c(3, 2, 4)
)
local.influence(
  fit_loglog,
  scheme    = "response",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.4,
  label.pos = 2
)
par(oldpar)
```

A small set of observations is identified as locally influential under both
perturbation schemes, but none produces an extreme influence value.

## Half-normal plot with simulated envelope

The `halfnormal.plot` function produces a half-normal plot of the absolute
residuals together with a simulated envelope based on `nsim` Monte Carlo
replications of the fitted model (default `nsim = 100`). Points outside the
envelope may indicate model inadequacy. The `type` argument selects the
residual type (default `"weighted"`) and `seed` ensures reproducibility.

```{r halfnormal, fig.height = 5, fig.cap = "Half-normal plot of absolute weighted residuals with 95% simulated envelope (100 replications)."}
halfnormal.plot(fit_loglog, nsim = 19, type = "weighted", seed = 2026)
```

All diagnostic plots reveal no evidence against the adequacy of the fitted
model. Quantile residuals are approximately standard normal, no observation
has disproportionate influence, and nearly all points in the half-normal
plot fall within the simulated envelope.

# Fitted vs. Observed: Time Series

```{r timeseries, fig.height = 4, fig.cap = "Observed (solid black) and fitted (dashed red) monthly relative humidity in Brasília, January 2000 to December 2025."}
plot(RH$date, RH$rh,
     type = "l", col = "black", lwd = 1.2,
     xlab = "Date", ylab = "Relative humidity",
     main = "Observed vs Fitted RH — Brasília (2000–2025)")
lines(RH$date, fitted(fit_loglog), col = "red", lwd = 1.5, lty = 2)
legend("bottomleft",
       legend = c("Observed", "Fitted"),
       col    = c("black", "red"),
       lty    = c(1, 2), lwd = c(1.2, 1.5),
       bty    = "n", cex = 0.85)
```

The simplex regression model with log-log link accurately captures the
strong seasonal pattern in relative humidity, with fitted values tracking
the observed series closely throughout the entire 26-year period.

# Summary

This vignette demonstrated the main functionalities of the
`SimplexRegression` package through the analysis of monthly relative
humidity data from Brasília. The selected simplex regression model uses the
log-log mean link with variable dispersion driven by precipitation. It
achieved excellent goodness of fit (pseudo-$R^2 > 0.95$) and strong
cross-validated predictive performance ($P^2 > 0.95$), with no evidence of
misspecification from the RESET test or departures from model assumptions
in the diagnostic plots.

The package provides a comprehensive and self-consistent workflow — from
data preparation and model fitting to inference, model selection,
diagnostics, and influence analysis — following the standard conventions
of formula-based regression modeling in R.

# References

- Barndorff-Nielsen, O. E. and Jørgensen, B. (1991). Some parametric models
  on the simplex. *Journal of Multivariate Analysis*, **39**(1), 106--116.
- Cribari-Neto, F., Vasconcellos, K. L. P. and Santana e Silva, J. J.
  (2025). New strategies for detecting atypical observations based on the
  information matrix equality. *Journal of Applied Statistics*, **52**,
  2873--2893.
- Espinheira, P. L. and Silva, A. O. (2020). Residual and influence
  analysis to a general class of simplex regression. *TEST*, **29**,
  523--552.
- Justino, M. E. C. and Cribari-Neto, F. (2026). Simplex regression with a
  flexible logit link: Inference and application to cross-country impunity
  data. *Applied Mathematical Modelling*, **154**, 116713.

```{r restore-options, include = FALSE}
options(old_opts)
```

# Session Info

```{r session}
sessionInfo()
```

---
title: "Modeling Relative Air Humidity with Simplex Regression"
output:
  rmarkdown::html_vignette:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{relative-humidity}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  fig.align  = "center",
  message    = FALSE,
  warning    = FALSE
)
old_opts <- options(width = 70, prompt = "R> ", continue = "+  ", digits = 5)
```

# Introduction

## The simplex distribution

Regression models for continuous responses restricted to the unit interval
$(0, 1)$ play a central role in empirical research across economics,
biostatistics, political science, and environmental studies. Typical examples
include rates, proportions, indices, and concentration measures. The
**simplex regression model** provides a flexible and theoretically appealing
framework for such data. It is based on the simplex distribution, introduced
by Barndorff-Nielsen and Jørgensen (1991), which is indexed by a mean
parameter $\mu \in (0, 1)$ and a dispersion parameter $\sigma^2 > 0$.

A random variable $Y$ follows a simplex distribution,
$Y \sim S^-(\mu, \sigma^2)$, if its density is

$$f(y; \mu, \sigma^2) = \left\{2\pi\sigma^2[y(1-y)]^3\right\}^{-1/2}
\exp\!\left(-\frac{d(y;\mu)}{2\sigma^2}\right), \quad y \in (0,1),$$

where $d(y;\mu) = (y - \mu)^2 / [y(1-y)\mu^2(1-\mu)^2]$ is the unit
deviance. The mean and variance are $\mathrm{E}(Y) = \mu$ and
$\mathrm{Var}(Y) < \mu(1-\mu)$, with variance function
$\mathrm{V}(\mu) = \mu^3(1-\mu)^3$.

The simplex distribution can assume a wide variety of shapes — symmetric,
left- or right-skewed, J-shaped, U-shaped, and even bimodal — making it
particularly flexible for modeling bounded responses. Unlike the beta
distribution, which is indexed by a precision parameter, the simplex
distribution is indexed by a dispersion parameter $\sigma^2$: smaller values
correspond to higher concentration around the mean, whereas larger values
indicate greater variability.

## The simplex regression model

The simplex regression model with variable dispersion relates the mean and
dispersion to covariates through link functions:

$$g(\mu_i) = \mathbf{x}_i^\top \boldsymbol{\beta} = \eta_{1i}
\quad \text{and} \quad
h(\sigma^2_i) = \mathbf{z}_i^\top \boldsymbol{\gamma} = \eta_{2i},$$

where $\boldsymbol{\beta}$ and $\boldsymbol{\gamma}$ are unknown parameter
vectors. For models with a **parametric mean link**, an additional shape
parameter $\lambda > 0$ is estimated jointly with $\boldsymbol{\beta}$ and
$\boldsymbol{\gamma}$:

$$g(\mu_i, \lambda) = \mathbf{x}_i^\top \boldsymbol{\beta}.$$

The `SimplexRegression` package supports five fixed link functions for the
mean submodel — `"logit"`, `"probit"`, `"loglog"`, `"cloglog"`,
`"cauchit"` — and two parametric links, `"plogit1"` and `"plogit2"`,
defined as

$$\text{plogit1:}\; g(\mu_i,\lambda) = \log\!\left[(1-\mu_i)^{-\lambda}-1\right],
\qquad
\text{plogit2:}\; g(\mu_i,\lambda) = \log\!\left(\frac{\mu_i^\lambda}{1-\mu_i^\lambda}\right).$$

Both reduce to the standard logit when $\lambda = 1$. For the dispersion
submodel, the links `"log"`, `"sqrt"`, and `"identity"` are available.
All parameters are estimated by maximum likelihood using a mixed algorithm
that combines the BFGS quasi-Newton method with Fisher scoring steps.

## Advantages over beta regression

Compared to beta regression, simplex regression has several distinctive
advantages:

- It can represent **bimodal densities**, which the beta distribution
  cannot;
- Maximum likelihood estimators for the mean and dispersion parameters are
  **asymptotically orthogonal**, simplifying inference and improving
  numerical stability;
- Likelihood-based inferences are typically **less sensitive to atypical
  observations**;
- Its higher-order variance function $\mu^3(1-\mu)^3$ allows **more
  flexible variance patterns** than the quadratic variance function of
  the beta distribution.

## This vignette

This vignette illustrates the main functionalities of the `SimplexRegression`
package through the analysis of monthly average relative humidity data
recorded in Brasília, Brazil ($n = 312$ observations). The analysis covers:

- data preparation and exploratory summary;
- fitting simplex regression models with **parametric** and **fixed** mean
  link functions;
- link function selection via the **penalized Scout Score** (`penalized.ss`)
  and **penalized information criteria** (`penalized.ic`);
- **likelihood ratio test** for nested models (`lrtest`);
- score test for $H_0\!: \lambda = 1$ (`scoretest`);
- specification testing with the **RESET test** (`resettest`);
- fitted values, residuals, and predictive performance (`press`);
- prediction and simulation;
- influence measures: hat values, Cook's distance, generalized leverage;
- diagnostic plots (`plot`, `halfnormal.plot`);
- **local influence** and **diagnostic distance** measures
  (`local.influence`, `diag.im`, `diag.distances`).

# The Data

Relative air humidity (RH) is defined as the ratio of the partial pressure
of water vapor in the air to the saturation vapor pressure at the same
temperature. In Brasília, low RH levels during dry months are associated with
adverse health outcomes (asthma, nosebleeds, dehydration), increased
forest-fire risk, and pressure on water resources. The dataset contains
monthly averages from January 2000 to December 2025, obtained from the
National Institute of Meteorology (INMET).

```{r setup}
library(SimplexRegression)
data(RelativeHumidity, package = "SimplexRegression")
head(RelativeHumidity, 5)
```

| Variable | Description |
|----------|-------------|
| `date`   | Observation date (end of month) |
| `rh`     | Monthly average relative humidity, rescaled to $(0,1)$ |
| `ins`    | Total insolation (hours), with two missing values |
| `ins2`   | Total insolation with imputed missing values |
| `pre`    | Total precipitation (mm), with two missing values |
| `pre2`   | Total precipitation with imputed missing values |
| `neb`    | Average cloudiness (tenths) |
| `ap`     | Average atmospheric pressure (hPa) |
| `mt`     | Average maximum temperature (°C) |
| `ws`     | Average wind speed (m/s) |
| `dir`    | Predominant wind direction (degrees) |

## Data preparation

To capture the 12-month seasonal cycle we add harmonic regressors
$s_i = \sin(2\pi i/12)$ and $c_i = \cos(2\pi i/12)$, and a rainy-season
dummy equal to 1 for October, November, and December (the rainy season in
Brasília) and 0 otherwise.

```{r data-prep}
RH       <- RelativeHumidity
RH$hs    <- sin(2 * pi * seq_len(nrow(RH)) / 12)
RH$hc    <- cos(2 * pi * seq_len(nrow(RH)) / 12)
RH$dummy <- as.integer(as.integer(format(RH$date, "%m")) %in% 10:12)
```

## Descriptive summary

```{r summary-rh}
summary(RH$rh)
cat(sprintf(
  "Std. dev.: %.4f  |  Skewness: %.4f\n",
  sd(RH$rh),
  mean(((RH$rh - mean(RH$rh)) / sd(RH$rh))^3)
))
```

The response ranges from 0.301 to 0.851, with mean 0.635 and moderate left
skewness ($-0.466$), consistent with a higher concentration of RH values
above the mean during the wetter months. These features support the use of a
unit-interval distribution such as the simplex.

# Model Formula

All models fitted in this vignette share the same formula structure. The
two-part formula `y ~ x | z` specifies the mean submodel (left of `|`) and
the dispersion submodel (right of `|`) simultaneously. The mean submodel
includes insolation (`ins2`), maximum temperature (`mt`), wind speed (`ws`),
harmonic terms (`hs`, `hc`) to capture seasonality, a rainy-season dummy,
and its interaction with wind speed. The dispersion submodel contains only
precipitation (`pre2`), allowing the variability of RH to depend on rainfall.

```{r formula}
formula <- rh ~ ins2 + mt + ws + hs + hc + dummy + I(dummy * ws) | pre2
```

# Parametric Mean Link Functions

## Fitting and model selection

We first fit models with the two parametric mean links available in the
package, `plogit1` and `plogit2`. These links include a shape parameter
$\lambda$ estimated jointly with the regression coefficients. When $\lambda = 1$,
both reduce to the standard logit link; values $\lambda \neq 1$ introduce
asymmetry in the link, adding flexibility to capture non-standard
relationships between the mean response and the linear predictor.

```{r models-plogit}
fit_p1 <- simplexreg(formula, data = RH, link.mu = "plogit1")
fit_p2 <- simplexreg(formula, data = RH, link.mu = "plogit2")
```

## Penalized Scout Score and information criteria

When comparing models that all use a parametric link, the **penalized Scout
Score** (`penalized.ss`) and **penalized information criteria**
(`penalized.ic`) are recommended. These functions incorporate an additional
penalty controlled by `kappa` (default `kappa = 0.1`) that accounts for the
complexity introduced by $\lambda$ relative to its deviation from the
standard logit. The model with the highest Scout Score or lowest penalized
criterion is selected. 

```{r penalized-ss-param}
penalized.ss(fit_p1, fit_p2, kappa = 0.1)
```

```{r penalized-ic}
penalized.ic(fit_p1, fit_p2, kappa = 0.1)
```

All penalized criteria unanimously select the `plogit1` link. The Scout
Score of `fit_p1` is substantially higher than that of `fit_p2`, and all
three penalized information criteria are lower for `fit_p1`.

# Fixed Mean Link Functions

## Fitting

We also fit models with all five fixed mean links supported by the package.
Fixed links do not include an additional parameter and are therefore always
comparable using standard (unpenalized) criteria.

```{r models-fixed}
fit_loglog  <- simplexreg(formula, data = RH, link.mu = "loglog")
fit_logit   <- simplexreg(formula, data = RH, link.mu = "logit")
fit_probit  <- simplexreg(formula, data = RH, link.mu = "probit")
fit_cauchit <- simplexreg(formula, data = RH, link.mu = "cauchit")
fit_cloglog <- simplexreg(formula, data = RH, link.mu = "cloglog")
```

## Comparing all candidates

When comparing models with both fixed and parametric links simultaneously,
the **unpenalized** version (`kappa = 0`) must be used, since the penalty
for $\lambda$ is only meaningful when all candidates use a parametric link.

```{r penalized-ss-all}
penalized.ss(
  fit_loglog, fit_logit, fit_probit,
  fit_cauchit, fit_cloglog, fit_p1,
  kappa = 0
)
```

The log-log link achieves the highest Scout Score and is selected as the
best-fitting specification. We proceed with `fit_loglog` for all remaining
analyses.

# Model Summary

The `summary` method provides a standard regression output: estimated
coefficients for the mean and dispersion submodels, standard errors, Wald
$z$-statistics and $p$-values, the maximized log-likelihood, information
criteria (AIC, BIC, HQIC), pseudo-$R^2$ measures, and details on the
numerical optimization procedure.

```{r summary-fit}
summary(fit_loglog)
```

All regression coefficients are statistically significant at the 5% level.
The harmonic terms (`hs`, `hc`) and the rainy-season dummy capture the
strong seasonal pattern in RH, while insolation (`ins2`), maximum temperature
(`mt`), and wind speed (`ws`) explain the within-season variation. The
Nagelkerke and Ferrari–Cribari-Neto pseudo-$R^2$ values both exceed 0.95,
indicating an excellent fit. The model converged in 18 BFGS iterations
followed by 7 Fisher scoring steps.

# Standard Information Criteria

The standard (unpenalized) AIC, BIC, and HQIC are accessible via the usual
S3 generics and compare multiple models simultaneously. Lower values
indicate a better trade-off between fit and complexity.

```{r aic-bic}
AIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
BIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
HQIC(fit_loglog, fit_logit, fit_probit, fit_cauchit, fit_cloglog, fit_p1)
```

The log-log model consistently achieves the lowest values across all three
criteria, confirming the Scout Score selection.

# Extracting Model Components

## Coefficients and covariance matrix

The `coef` method extracts estimated coefficients for the full model or for
each submodel separately, controlled by the `model` argument
(`"full"`, `"mean"`, or `"dispersion"`). The `vcov` method returns the
corresponding variance-covariance matrix.

```{r coef-vcov}
coef(fit_loglog)                          # full coefficient vector
coef(fit_loglog, model = "mean")          # mean submodel only
coef(fit_loglog, model = "dispersion")    # dispersion submodel only
round(vcov(fit_loglog, model = "mean"), 6)  # vcov of mean submodel
```

## Log-likelihood

The `logLik` method returns the maximized log-likelihood together with the
number of estimated parameters (`df`) and the number of observations. This
is compatible with `AIC()` and `BIC()` from the `stats` package.

```{r loglik}
logLik(fit_loglog)
```

# Hypothesis Tests

## Likelihood ratio test for nested models

The `lrtest` method from the `lmtest` package performs likelihood ratio tests
between nested simplex regression models. A natural application is testing
whether the dispersion is constant against a model with covariate-dependent
dispersion. The restricted model (constant dispersion) is obtained via
`update` by replacing the dispersion submodel with an intercept only
(`| 1`).

```{r lrtest}
fit_loglog_null <- update(fit_loglog, . ~ . | 1)
lmtest::lrtest(fit_loglog, fit_loglog_null)
```

The very small $p$-value ($< 0.001$) provides strong evidence against
constant dispersion, confirming that RH variability is significantly
associated with precipitation (`pre2`). The model with variable dispersion
is therefore preferred.

## Score test for $H_0\!: \lambda = 1$

The `scoretest` function implements Rao's score test for the hypothesis that
the standard logit link is adequate, i.e., $H_0\!: \lambda = 1$. The test
requires a model fitted under $H_0$ (with the logit link) and tests against
either `"plogit1"` or `"plogit2"` as the alternative. Rejection of $H_0$
suggests that a parametric link provides a better fit than the standard
logit.

```{r scoretest}
fit_logit_h0 <- simplexreg(formula, data = RH, link.mu = "logit")
scoretest(fit_logit_h0, link.mu = "plogit1")
scoretest(fit_logit_h0, link.mu = "plogit2")
```

## RESET test for functional form

The `resettest` function implements Ramsey's RESET test, which augments the
original model with the squared fitted linear predictor as an additional
covariate and tests its significance. By default (`dispersion = TRUE`), the
squared predictor is added to both the mean and dispersion submodels.
Setting `dispersion = FALSE` restricts the augmentation to the mean submodel
only. Failure to reject $H_0$ supports the adequacy of the assumed
functional form.

```{r resettest}
resettest(fit_loglog)                      # both submodels augmented
resettest(fit_loglog, dispersion = FALSE)  # mean submodel only
```

The large $p$-values in both cases (0.82 and 0.97) provide no
evidence of functional form misspecification in either submodel.

# Fitted Values, Residuals, and Predictive Performance

## Fitted values and residuals

The `fitted` method returns fitted mean values $\hat{\mu}_i$. The
`residuals` method supports ten residual types. Quantile residuals
(default) have an approximately standard normal distribution under correct
model specification and are recommended for general diagnostics. Weighted
residuals are particularly useful for half-normal plots.

```{r fitted-resid}
head(fitted(fit_loglog))
head(residuals(fit_loglog, type = "quantile"))  # approx. N(0,1)
head(residuals(fit_loglog, type = "pearson"))
head(residuals(fit_loglog, type = "weighted"))  # for halfnormal.plot
```

## Predictive performance: PRESS and $P^2$

The `press` function computes the PRESS (Predicted Residual Error Sum of
Squares) statistic and the associated cross-validation measures $P^2$ and
adjusted $P^2_c$. The $P^2$ statistic is a cross-validation analog of the
coefficient of determination, computed from the hat matrix diagonal without
refitting the model $n$ times. Values close to 1 indicate strong predictive
accuracy. The function accepts multiple models simultaneously, enabling
direct comparison of predictive performance across competing specifications.

```{r press}
press(fit_loglog)                            # single model
press(fit_loglog, fit_logit, fit_probit)     # comparing models
```

The log-log model achieves the highest $P^2$ and $P^2_c$ values, confirming
its superior predictive performance. The PRESS statistic reflects the total
cumulative squared prediction error across all leave-one-out fits.

# Prediction and Simulation

## Prediction

The `predict` method computes predicted values for new or original data. The
`type` argument controls the output: `"response"` returns fitted means
$\hat{\mu}_i$; `"link"` returns the linear predictors $\hat{\eta}_{1i}$ and
$\hat{\eta}_{2i}$; `"dispersion"` returns fitted dispersion values
$\hat{\sigma}^2_i$.

```{r predict}
head(predict(fit_loglog, type = "response"))        # fitted means
head(predict(fit_loglog, type = "link")$mean)       # mean linear predictor
head(predict(fit_loglog, type = "link")$dispersion) # dispersion predictor
head(predict(fit_loglog, type = "dispersion"))      # fitted sigma^2

# Out-of-sample prediction
new_obs <- RH[1:3, ]
predict(fit_loglog, newdata = new_obs, type = "response")
```

## Simulation

The `simulate` method generates response vectors from the fitted simplex
distribution using the estimated $\hat{\mu}_i$ and $\hat{\sigma}^2_i$. The
`nsim` argument controls the number of replicates and `seed` ensures
reproducibility. The result is a data frame with `nsim` columns.

```{r simulate}
set.seed(2026)
sims <- simulate(fit_loglog, nsim = 3)
head(sims)
```

# Influence Measures

## Hat values and Cook's distance

The `hatvalues` method returns the diagonal elements $h_{ii}$ of the hat
matrix, measuring the leverage of each observation on the fitted values.
Observations with $h_{ii}$ much larger than the average $r/n$ (where $r$
is the number of estimated parameters) deserve closer inspection. The
`cooks.distance` method computes approximate Cook's distances, which combine
leverage and residual size to measure overall influence on the parameter
estimates. The `type` argument selects the residual type used
(`"pearson"` or `"weighted"`).

```{r influence}
hii  <- hatvalues(fit_loglog)
cook <- cooks.distance(fit_loglog, type = "pearson")
cat(sprintf("Leverages  — max: %.4f  mean: %.4f\n", max(hii),  mean(hii)))
cat(sprintf("Cook's D   — max: %.4f\n", max(cook)))
```

No observation has a disproportionately large leverage or Cook's distance,
suggesting no single point dominates the fit.

## Generalized leverage

The `gleverage` function computes the generalized leverage values, which
extend the classical hat values to account for both the mean and dispersion
submodels. High generalized leverage indicates observations that may exert
disproportionate influence on the fitted values across the entire model.

```{r gleverage}
gl <- gleverage(fit_loglog)
cat(sprintf("Generalized leverage — max: %.4f  mean: %.4f\n",
            max(gl), mean(gl)))
```

# Diagnostic Plots

## Residual and fit plots

The `plot` method produces up to eight diagnostic plots selected via `which`:
(1) residuals vs. observation index; (2) residuals vs. fitted values;
(3) residuals vs. linear predictor; (4) observed vs. fitted values;
(5) normal Q-Q plot; (6) Cook's distances; (7) generalized
leverages. The default residual type is `"quantile"`. For plots 6 and 7,
`threshold` highlights influential observations and `label.pos` controls
label placement (1 = below, 2 = left, 3 = above, 4 = right).

```{r plots-1-5, fig.height = 8, fig.cap = "Diagnostic plots (1–6) for the fitted simplex regression model with log-log link."}
oldpar <- par(mfrow = c(3, 2))
plot(fit_loglog, which = 1:5, reset.par = FALSE)
par(oldpar)
```

```{r plot-cook, fig.height = 4, fig.cap = "Cook's distances. Observations exceeding the threshold of 0.15 are labeled."}
plot(fit_loglog, which = 6, threshold = 0.15, label.pos = 4)
```

```{r plot-glev, fig.height = 4, fig.cap = "Generalized leverage values. Observations exceeding 0.08 are labeled."}
plot(fit_loglog, which = 7, threshold = 0.08, label.pos = 3)
```

The quantile residuals are approximately standard normal, with no systematic
patterns across fitted values or the linear predictor. Cook's distances and
generalized leverages are uniformly small, with no evidence of unduly
influential observations.

## Local influence

Local influence analysis quantifies how small perturbations to the model
assumptions affect the parameter estimates. Two perturbation schemes are
supported: `"case.weight"` perturbs observation weights and `"response"`
perturbs the response values. The `parameter` argument selects the
parameter block of interest — `"theta"` (all parameters), `"beta"` (mean
submodel), or `"gamma"` (dispersion submodel). The `type` argument controls
the influence measure: `"Ci"` for the total local influence index $C_i$,
or `"dmax"` for the direction of maximum curvature. Observations exceeding
`threshold` are labeled in the index plot.

```{r local-influence-cw, fig.height = 4.5, fig.cap = "Total local influence $C_i$ under case-weight perturbation for all parameters."}
local.influence(
  fit_loglog,
  scheme    = "case.weight",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.5,
  label.pos = c(3, 4, 3, 2, 2)
)
```

```{r local-influence-resp, fig.height = 4.5, fig.cap = "Total local influence $C_i$ under response perturbation for all parameters."}
local.influence(
  fit_loglog,
  scheme    = "response",
  parameter = "theta",
  type      = "Ci",
  plot      = TRUE,
  threshold = 0.4,
  label.pos = 2
)
```

A small set of observations is identified as locally influential under both
perturbation schemes, but none produces an extreme influence value.

## Half-normal plot with simulated envelope

The `halfnormal.plot` function produces a half-normal plot of the absolute
residuals together with a simulated envelope based on `nsim` Monte Carlo
replications of the fitted model (default `nsim = 100`). Points outside the
envelope may indicate model inadequacy. The `type` argument selects the
residual type (default `"weighted"`) and `seed` ensures reproducibility.

```{r halfnormal, fig.height = 5, fig.cap = "Half-normal plot of absolute weighted residuals with 95% simulated envelope (100 replications)."}
halfnormal.plot(fit_loglog, nsim = 19, type = "weighted", seed = 2026)
```

All diagnostic plots reveal no evidence against the adequacy of the fitted
model. Quantile residuals are approximately standard normal, no observation
has disproportionate influence, and nearly all points in the half-normal
plot fall within the simulated envelope.

# Fitted vs. Observed: Time Series

```{r timeseries, fig.height = 4, fig.cap = "Observed (solid black) and fitted (dashed red) monthly relative humidity in Brasília, January 2000 to December 2025."}
plot(RH$date, RH$rh,
     type = "l", col = "black", lwd = 1.2,
     xlab = "Date", ylab = "Relative humidity",
     main = "Observed vs Fitted RH — Brasília (2000–2025)")
lines(RH$date, fitted(fit_loglog), col = "red", lwd = 1.5, lty = 2)
legend("bottomleft",
       legend = c("Observed", "Fitted"),
       col    = c("black", "red"),
       lty    = c(1, 2), lwd = c(1.2, 1.5),
       bty    = "n", cex = 0.85)
```

The simplex regression model with log-log link accurately captures the
strong seasonal pattern in relative humidity, with fitted values tracking
the observed series closely throughout the entire 26-year period.

# Summary

This vignette demonstrated the main functionalities of the
`SimplexRegression` package through the analysis of monthly relative
humidity data from Brasília. The selected simplex regression model uses the
log-log mean link with variable dispersion driven by precipitation. It
achieved excellent goodness of fit (pseudo-$R^2 > 0.95$) and strong
cross-validated predictive performance ($P^2 > 0.95$), with no evidence of
misspecification from the RESET test or departures from model assumptions
in the diagnostic plots.

The package provides a comprehensive and self-consistent workflow — from
data preparation and model fitting to inference, model selection,
diagnostics, and influence analysis — following the standard conventions
of formula-based regression modeling in R.

# References

- Barndorff-Nielsen, O. E. and Jørgensen, B. (1991). Some parametric models
  on the simplex. *Journal of Multivariate Analysis*, **39**(1), 106--116.
- Cribari-Neto, F., Vasconcellos, K. L. P. and Santana e Silva, J. J.
  (2025). New strategies for detecting atypical observations based on the
  information matrix equality. *Journal of Applied Statistics*, **52**,
  2873--2893.
- Espinheira, P. L. and Silva, A. O. (2020). Residual and influence
  analysis to a general class of simplex regression. *TEST*, **29**,
  523--552.
- Justino, M. E. C. and Cribari-Neto, F. (2026). Simplex regression with a
  flexible logit link: Inference and application to cross-country impunity
  data. *Applied Mathematical Modelling*, **154**, 116713.

```{r restore-options, include = FALSE}
options(old_opts)
```

# Session Info

```{r session}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_scoretest.R
\name{scoretest}
\alias{scoretest}
\title{Rao's Score Test for Simplex Regression with Parametric Mean Link Function}
\usage{
scoretest(model, link.mu = c("plogit1", "plogit2"))
}
\arguments{
\item{model}{An object of class \code{"simplexregression"} fitted with the
fixed logit mean link (\code{link.mu = "logit"})}

\item{link.mu}{Character string specifying the parametric link function under the
alternative hypothesis. Options are \code{"plogit1"} or \code{"plogit2"}.}
}
\value{
An object of class \code{"htest"} containing:
\itemize{
  \item \code{statistic}: The score test statistic,
  \item \code{parameter}: Degrees of freedom (always 1),
  \item \code{p.value}: The \eqn{p}-value of the test,
  \item \code{method}: Description of the test,
  \item \code{data.name}: Description of the link comparison being tested
  (e.g., \code{"Logit vs plogit1"})
}
}
\description{
Performs a Rao's score test to test whether the link parameter
\eqn{\lambda} equals 1, which corresponds to evaluating the null hypothesis
that the mean link function is the standard logit.
}
\details{
Given that the fixed logit link function is a particular case of the parametric
logit link functions when \eqn{\lambda = 1}, it is possible to test whether the
mean link function is logit by testing \eqn{H_0: \lambda = 1} against
\eqn{H_1: \lambda \neq 1}.

The score test statistic is computed from the component of the score vector
associated with \eqn{\lambda} and the corresponding element of the inverse
Fisher information matrix, both evaluated at the maximum likelihood estimator
under the null hypothesis (i.e., with \eqn{\lambda} fixed at 1). For the full
expression of the test statistic, see Justino and Cribari-Neto (2026).

Under regularity conditions, the null hypothesis, and when \eqn{n} is large,
the test statistic follows a chi-squared distribution with 1 degree of freedom.
}
\examples{
# Simulate data
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

# Fit model with logit
model <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")

# Test if lambda = 1
scoretest(model, link.mu = "plogit1")

}
\references{
Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Rao, C. R. (1948). Large sample tests of statistical hypotheses concerning
several parameters with applications to problems of estimation.
\emph{Mathematical Proceedings of the Cambridge Philosophical Society},
 \bold{44}(1), 50--57. \doi{10.1017/S0305004100023987}
}
\seealso{
\code{\link{parametric_mean_link}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_influence.R
\name{diag.distances}
\alias{diag.distances}
\title{Distance-Based Influence Diagnostics for Simplex Regression}
\usage{
diag.distances(
  model,
  data,
  type = c("W1", "W2", "H"),
  plot = FALSE,
  verbose = TRUE,
  ncores = 1,
  label.pos = 3,
  plot.type = NULL,
  ...
)
}
\arguments{
\item{model}{An object of class \code{simplexregression}.}

\item{data}{The data frame used to fit \code{model}.}

\item{type}{Character string or integer specifying the distance measure:
\code{"W1"} (default, Wasserstein with \eqn{p_W = 1}),
\code{"W2"} (Wasserstein with \eqn{p_W = 2}), \code{"H"} (Hellinger).}

\item{plot}{Logical; if \code{FALSE} (default), returns the numeric vector
of distances. If \code{TRUE}, produces an index plot with the ad hoc
threshold and flagged-observation labels.}

\item{verbose}{Logical; if \code{TRUE} (default), prints progress during
leave-one-out refitting. Ignored when \code{ncores > 1}, since output
from parallel workers does not reach the main console.}

\item{ncores}{Positive integer specifying the number of CPU cores to use
for the leave-one-out loop. Default is \code{1} (sequential). Values
greater than \code{1} activate parallel computation via
\code{\link[parallel]{parLapply}}. If \code{ncores} exceeds
\code{parallel::detectCores() - 1}, it is clamped to that value with a
warning to avoid overloading the system. A safe explicit choice is
\code{parallel::detectCores() - 1}.}

\item{label.pos}{Position(s) for outlier labels in the plot. Can be a
single value (applied to all labels) or a vector. Values: 1 = below,
2 = left, 3 = above, 4 = right.}

\item{plot.type}{Character string controlling the plot style when
\code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} for
\eqn{n \le 150} and \code{"p"} for \eqn{n > 150} (automatic). Passed
to the \code{type} argument of \code{plot()}.}

\item{...}{Additional graphical parameters passed to \code{plot()}.}
}
\value{
If \code{plot = FALSE}, a list containing:
\describe{
  \item{\code{distances}}{Numeric vector of length \eqn{n} with the
    leave-one-out distances.}
  \item{\code{threshold}}{Named numeric scalar with the ad hoc upper
    threshold.}
  \item{\code{outliers}}{Data frame of flagged observations (index and
    distance value). An empty data frame if no observations are flagged.}
  \item{\code{type}}{Distance type used (full label).}
  \item{\code{n}}{Number of observations.}
}
If \code{plot = TRUE}, the same list is returned invisibly.
}
\description{
Computes leave-one-out influence measures based on
distributional distances (Wasserstein with \eqn{p_W = 1} and \eqn{p_W = 2} or
Hellinger) for simplex regression models, as proposed by Justino and
Cribari-Neto (2026).
}
\details{
For each observation \eqn{i}, the model is refit on the dataset with
observation \eqn{i} removed. The chosen distance between the full-model
and leave-one-out fitted distributions is then computed pointwise across
all \eqn{n} observations and summed to produce the scalar influence
measure \eqn{d_i}.

\strong{Ad hoc threshold} uses an asymmetric IQR spread adjusted for
skewness, as proposed by Justino and Cribari-Neto (2026). Let \eqn{Q(p)} denote the
empirical \eqn{p}-th quantile of the distances \eqn{d_i}, the threshold is
\deqn{\text{threshold} = Q(0.75) + (1 + a)(Q(0.75) - Q(0.25))}
where \eqn{a} is the sample skewness of the distances. Observations with
\eqn{d_i} above this threshold are flagged as potentially influential.

\strong{Warning:} for \eqn{n > 500}, the numerical integration used
internally by the distance functions may be slow. Consider using a subset
or a faster integration method.

\strong{Parallel computation:} when \code{ncores > 1}, the
leave-one-out loop is distributed across workers using
\code{\link[parallel]{parLapply}} from the \pkg{parallel} package
(included in base R). Each worker receives the necessary objects and
loads the \pkg{simplexregression} package. The random-number stream is
initialized with \code{\link[parallel]{clusterSetRNGStream}} to ensure
reproducibility across runs. Progress messages (\code{verbose}) are
suppressed in parallel mode because worker output does not reach the
main console.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

\donttest{
# Sequential (default) — Wasserstein W1
dd <- diag.distances(fit, data = ReadingSkills, type = "W1")

# Index plot with flagged observations
diag.distances(fit, data = ReadingSkills, type = "W1", plot = TRUE)

# Hellinger distance
diag.distances(fit, data = ReadingSkills, type = "H", plot = TRUE)
}


}
\references{
Justino, M. E. C. and Cribari-Neto, F. (2026).
Influence diagnostics in beta regression via Hellinger and Wasserstein
distances. \emph{Statistical Papers}, \bold{67}(3), 50.
\doi{10.1007/s00362-026-01823-0}
}
\seealso{
\code{\link{diag.im}}, \code{\link{local.influence}},
\code{\link{gleverage}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_penalized_ic.R
\name{ic.penalized}
\alias{ic.penalized}
\title{Penalized Information Criteria for Simplex Regression Model Selection}
\usage{
ic.penalized(
  ...,
  kappa = 0.1,
  verbose = TRUE,
  digits = max(3, getOption("digits") - 3)
)
}
\arguments{
\item{...}{One or more objects of class \code{"simplexregression"} fitted with a
parametric mean link functions (\code{"plogit1"} or \code{"plogit2"}).}

\item{kappa}{A numeric value controlling the additional penalty for the link
mean parameter, \eqn{\kappa \geq 0}. Default is \code{0.1}.}

\item{verbose}{Logical. If \code{TRUE} (default), prints the criteria values.
If \code{FALSE}, returns results silently.}

\item{digits}{Integer specifying the number of decimal places for output.
Default is \code{max(3, getOption("digits") - 3)}.}
}
\value{
A data frame with rows named after the candidate models and four columns:
\describe{
  \item{\code{df}}{Number of estimated parameters.}
  \item{\code{AICp}}{Penalized AIC value.}
  \item{\code{BICp}}{Penalized BIC value.}
  \item{\code{HQICp}}{Penalized HQIC value.}
}
When \code{verbose = TRUE}, the results are also printed to the console and
the data frame is returned invisibly. When \code{verbose = FALSE}, the data
frame is returned visibly without printing.
When \code{kappa = 0}, the columns are named \code{AIC}, \code{BIC}, and
\code{HQIC} instead of \code{AICp}, \code{BICp}, and \code{HQICp}.
}
\description{
Implements the Akaike, Schwarz, and Hannan–Quinn information
criteria with a penalty term for selecting among competing simplex regression
models with parametric mean link functions.
}
\details{
The penalized information criteria, as proposed by Justino and
Cribari-Neto (2026), extend the classical Akaike, Schwarz and
Hannan--Quinn criteria with an additional penalty term for the link
parameter \eqn{\lambda}:
\deqn{AIC^{(\lambda)} = -2 \ell + (2 + \kappa \, |\log(\lambda)|)r}
\deqn{BIC^{(\lambda)} = -2 \ell + (\log(n) + \kappa \, |\log(\lambda)|)r}
\deqn{HQIC^{(\lambda)}  = -2 \ell + (2 \log(\log(n)) + \kappa \, |\log(\lambda)|)r}
where:
\itemize{
  \item \eqn{\ell} denotes the maximized log-likelihood function;
  \item \eqn{\kappa \geq 0} controls the additional penalty associated with
  the link parameter;
  \item \eqn{\lambda} is the parameter of the parametric mean link function;
  \item \eqn{r} indicate the dimension of their parameter vector;
  \item \eqn{n} is the number of observations.
}

\strong{Important}: These penalized versions of the criteria should only be used
when the candidate models employ a parametric link function in the mean submodel
(use \code{kappa = 0.1}). When candidate models include specifications
with fixed link functions, the standard unpenalized versions of these criteria
should be applied instead (use \code{kappa = 0}).
}
\examples{
# Simulate data
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

# Fit two models with parametric mean link functions
fit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")

# Compute penalized criteria
ic.penalized(fit1, fit2)

}
\references{
Akaike, H. (1973). Information theory and an extension of the maximum
likelihood principle. \emph{Akadémiai Kiadó}, 267--281.

Hannan, E. J. and Quinn, B. G. (1979). The determination of the order of an
autoregression. \emph{Journal of the Royal Statistical Society Series B:
Statistical Methodology}, \bold{41}(2), 190--195.
\doi{10.1111/j.2517-6161.1979.tb01072.x}

Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Schwarz, G. E. (1978). Estimating the dimension of a model. \emph{Annals of
Statistics}, \bold{6}(2), 461--464. \doi{10.1214/aos/1176344136}
}
\seealso{
\code{\link{ss.penalized}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abortionopposition.R
\docType{data}
\name{AbortionOpposition}
\alias{AbortionOpposition}
\title{Public Opinion on Abortion Across U.S. States}
\format{
A data frame with 51 observations and 9 variables:
\describe{
  \item{state}{Factor. U.S. state or the District of Columbia.}
  \item{abortion_opp}{Numeric. Proportion of adults who believe
        abortion should be illegal in all or most cases, based on the Pew
        Research Center 2023--24 Religious Landscape Study.}
  \item{relig_attend}{Numeric. Percentage of adults who report
        attending religious services at least once a week, based on the
        Pew Research Center 2023--24 Religious Landscape Study.}
  \item{income}{Numeric. Mean annual household income (USD), 2024.}
  \item{sex_ratio}{Numeric. Number of males per 100 females, 2024.}
  \item{female}{Numeric. Percentage of the population that is female, 2024.}
  \item{pop_18_24}{Numeric. Percentage of the population aged 18--24 years,
        obtained directly from the 2024 American Community Survey (ACS)
        1-Year Estimates.}
  \item{bachelors}{Numeric. Percentage of adults aged 25 and older with a
        bachelor's degree or higher, 2024.}
  \item{urban}{Numeric. Percentage of the population living in urban areas,
        based on 2020 U.S. Census Bureau urban area criteria (a densely
        settled core of census blocks meeting minimum housing unit and/or
        population density thresholds, requiring at least 2,000 housing
        units or 5,000 people).}
}
}
\source{
Pew Research Center (2025). 2023-24 Religious Landscape Study (RLS)
Dataset. \doi{10.58094/3kwb-bf52}. Data accessed in 2026.

World Population Review (2024). Per Capita Income by State.
\url{https://worldpopulationreview.com/state-rankings/per-capita-income-by-state}.
Data accessed in 2026.

World Population Review (2024). Sex Ratio by State.
\url{https://worldpopulationreview.com/state-rankings/sex-ratio-by-state}.
Data accessed in 2026.

World Population Review (2024). Educational Attainment by State.
\url{https://worldpopulationreview.com/state-rankings/educational-attainment-by-state}.
Data accessed in 2026.

World Population Review (2024). Most Urban States.
\url{https://worldpopulationreview.com/state-rankings/most-urban-states}.
Data accessed in 2026.

U.S. Census Bureau (2024). Age and Sex. American Community Survey,
ACS 1-Year Estimates Subject Tables, Table S0101.
\url{https://data.census.gov/table/ACSST1Y2024.S0101?q=age+and+sex+by+state&moe=false}.
Data accessed in 2026.
}
\usage{
data(AbortionOpposition)
}
\description{
This dataset examines the relationship between public opposition to abortion
and demographic, religious, and socioeconomic characteristics across the
50 U.S. states and the District of Columbia. The response variable is the
proportion of adults who believe abortion should be illegal in all or most
cases, bounded in the open interval (0, 1), making the data suitable for simplex
regression and beta regression analyses.
}
\details{
The dataset was assembled by the package authors from multiple publicly
available sources. The response variable, \code{abortion_opp},
was obtained from the Pew Research Center 2023--24 Religious Landscape
Study, originally published as a percentage (0--100 scale), and rescaled
to the (0, 1) interval by dividing by 100.

The explanatory variables were obtained as follows:
\enumerate{
  \item \code{relig_attend} was obtained from the Pew Research Center
        2023--24 Religious Landscape Study.
  \item \code{income}, \code{sex_ratio}, and \code{bachelors} were obtained from
        World Population Review, which itself compiles estimates from the
        U.S. Census Bureau/American Community Survey. World Population
        Review is cited here as the immediate data source, following
        standard practice for reproducibility.
  \item \code{female} was obtained from the U.S. Census Bureau's American
        Community Survey (ACS) 1-Year Estimates, Table S0101 (Age and
        Sex), the same table used for \code{pop_18_24}. Note that
        \code{sex_ratio} and \code{female} are closely related (both
        describe the sex composition of each state's population, from
        different sources) and should generally not be included
        together as covariates in the same regression model, as doing
        so may introduce near-perfect collinearity.
  \item \code{pop_18_24} corresponds to the percent estimate of the
        population aged 18--24 years reported directly in Table S0101
        (Age and Sex) of the 2024 American Community Survey (ACS)
        1-Year Estimates.
  \item \code{urban} was obtained from World Population Review and is
        based on 2020 U.S. Census Bureau data, the most recent Census
        estimate of urbanization by state available at the time this
        dataset was compiled. Unlike the other explanatory variables,
        which reflect 2023--24 estimates, \code{urban} reflects the 2020
        Census urban area delineation (see \code{@format} above for
        definition details).
}
}
\examples{
# Load the data
data(AbortionOpposition)

# Quick overview
head(AbortionOpposition)
str(AbortionOpposition)

# Summary statistics
summary(AbortionOpposition)

# Relationship between religious attendance and abortion opposition
plot(abortion_opp ~ relig_attend,
     data = AbortionOpposition,
     pch = 19,
     xlab = "Religious attendance (\%)",
     ylab = "Abortion opposition (proportion)")

# Correlation among numeric variables
cor(AbortionOpposition[, c("abortion_opp",
                        "relig_attend",
                        "income",
                        "sex_ratio",
                        "female",
                        "pop_18_24",
                        "bachelors",
                        "urban")])
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_sscriteria.R
\name{penalized.ss}
\alias{penalized.ss}
\title{Scout Score Criterion for Simplex Regression Model Selection}
\usage{
penalized.ss(
  ...,
  kappa = 0.1,
  verbose = TRUE,
  digits = max(3, getOption("digits") - 3)
)
}
\arguments{
\item{...}{Two or more objects of class \code{"simplexregression"} to be compared.}

\item{kappa}{A numeric value controlling the additional penalty for the link mean
parameter, \eqn{\kappa \geq 0}. Default is \code{0.1}. Use \code{kappa = 0} for
standard Scout Score.}

\item{verbose}{Logical. If \code{TRUE} (default), prints the SS values for
all models and the selected model. If \code{FALSE}, returns results silently.}

\item{digits}{Integer specifying the number of decimal places for output.
Default is \code{max(3, getOption("digits") - 3)}.}
}
\value{
A data frame with rows named after the candidate models and two columns:
\describe{
  \item{\code{df}}{Number of estimated parameters in each model.}
  \item{\code{SS}}{Scout Score value. The model with the highest value is
  the selected one.}
}
When \code{verbose = TRUE}, the selected model is also printed to the console.
The data frame is returned invisibly in this case, and visibly when
\code{verbose = FALSE}.
}
\description{
Implements the Scout Score (SS) criterion for selecting among
competing simplex regression models with parametric and fixed mean link functions.
}
\details{
The Scout Score criterion, originally proposed by Costa et al. (2024) for
selecting link functions in the \eqn{\beta\text{ARMA}} (beta autoregressive
moving average) model, extends Vuong's test statistic to compare \eqn{M \geq 2}
competing non-nested models using their individual log-likelihood contributions.

For each candidate model \eqn{j \in {1, \ldots, M}}, the Scout Score is defined as:
\deqn{SS_j = 1 - M + \sum_{k=1, k \neq j}^M (1 + \dot{\Delta}_{jk})^2,}

where \eqn{\dot{\Delta}_{jk} = \max\{0, \Delta_{jk}\}} and \eqn{\Delta_{jk}}
is Vuong's (1989) test statistic comparing models \eqn{j} and \eqn{k}, penalized
by a term \eqn{\delta_{jk}} that combines the difference in parameter-vector
dimensions with, when applicable, a link-complexity penalty controlled by
\code{kappa}. The model with the highest Scout Score is selected as the most
adequate. For the full derivation of \eqn{\Delta_{jk}}, \eqn{\delta_{jk}}, and
the rationale behind the link-complexity penalty, see Justino and
Cribari-Neto (2026) and Costa et al. (2024).

When at least one of the candidate models does not use a parametric mean
link function, \eqn{\kappa} is internally set to \code{0} (see \strong{Important}
below), so that \eqn{\delta_{jk}} reduces to the classical dimension penalty
with no link-complexity term.

The model with the highest Scout Score is selected as the most adequate.

\strong{Important}: The penalty term \eqn{\delta_{jk}} is only applied when
\emph{all} candidate models employ a parametric mean link function, in which
case \code{kappa} (default \code{0.1}) controls the additional penalty. In
any other case --- whether all candidate models use fixed mean links, or the
set of candidate models mixes parametric and fixed mean links --- the penalty
is disabled and the standard, unpenalized Scout Score is computed for all
models (\code{kappa} is internally reset to \code{0}). If the user explicitly
requested \code{kappa > 0} in either of these situations, a warning is issued;
if \code{kappa} was left at its default value, no warning is issued, since
falling back to the standard Scout Score is the expected behavior.

\strong{Note}: all candidate models must be fitted to the same response
vector \code{y}; the function verifies this and stops with an error if the
response vectors differ.
}
\examples{
# Simulate data
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

# Fit models
fit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")
fit3 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "logit")
fit4 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "probit")

# Compare models with verbose output
result <- penalized.ss(fit1, fit2, kappa = 0.1)

# Compare models silently
result <- penalized.ss(fit1, fit2, kappa = 0.1, verbose = FALSE)

# Use standard Scout Score (no parametric link penalty)
result <- penalized.ss(fit1, fit2, fit3, fit4, kappa = 0)

}
\references{
Costa, E., Cribari-Neto, F. and Scher, V. T. (2024).
Test inferences and link function selection in dynamic beta modeling of seasonal
hydro-environmental time series with temporary abnormal regimes.
\emph{Journal of Hydrology}, \bold{638}, 131489.
\doi{10.1016/j.jhydrol.2024.131489}

Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Vuong, Q. H. (1989). Likelihood ratio tests for model selection and non-nested
hypotheses. \emph{Econometrica}, \bold{57}(2), 307--333. \doi{10.2307/1912557}
}
\seealso{
\code{\link{penalized.ic}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_fit.R
\name{simplexreg.control}
\alias{simplexreg.control}
\title{Control Parameters for Simplex Regression}
\usage{
simplexreg.control(
  method = "BFGS",
  maxit = 5000,
  gradient = TRUE,
  hessian = FALSE,
  trace = FALSE,
  start = NULL,
  fsmaxit = 500,
  fstol = 1e-08,
  reltol = .Machine$double.eps^(0.5),
  ...
)
}
\arguments{
\item{method}{Characters string specifying the method argument passed to \code{optim}
(default: "BFGS").}

\item{maxit}{Integer specifying the maximum number of iterations for \code{optim}
(default: 5000).}

\item{gradient}{Logical; use analytical gradient? (default: TRUE).}

\item{hessian}{Logical; compute Hessian via \code{optim}? (default: FALSE).}

\item{trace}{Logical; trace optimization? (default: FALSE).}

\item{start}{An optional vector with starting values for all parameters.}

\item{fsmaxit}{Integer specifying maximal number of additional Fisher scoring
iterations (default: 500).}

\item{fstol}{Numeric tolerance for convergence in Fisher scoring (default: 1e-8).}

\item{reltol}{Relative convergence tolerance (default: 1e-8).}

\item{...}{Additional parameters passed to \code{optim}.}
}
\value{
A list of control parameters.
}
\description{
Auxiliary function for controlling simplex regression fitting.
}
\details{
All parameters in \code{\link{simplexreg}} are estimated by maximum likelihood using
\code{\link{optim}} with control options set in \code{\link{simplexreg.control}}.
Most arguments are passed on directly to \code{optim}, and \code{start} controls
how \code{optim} is called.

After the \code{optim} maximization, an additional Fisher scoring iteration can
be performed to further enhance the result by moving the gradient even closer to zero.
If \code{fsmaxit} is greater than zero, this additional optimization is performed and
it converges if the threshold \code{fstol} is attained for the absolute value of the
step size.

Starting values can be supplied via \code{start} or estimated by \code{\link{lm.wfit}},
using the link-transformed response. For parametric mean link functions (\code{plogit1},
\code{plogit2}), the link parameter \eqn{\lambda} is jointly estimated with the regression
coefficients. Covariances are derived analytically using the expected Fisher information
matrix. The Fisher scoring uses analytical gradients and the expected information matrix to
refine the maximum likelihood estimates obtained from \code{optim}.

The main parameters of interest are the coefficients \eqn{\beta} in the linear predictor of
the mean submodel and the coefficients \eqn{\delta} in the linear predictor of the dispersion
submodel. For parametric links, the additional link parameter \eqn{\lambda} is also
estimated and reported. The dispersion parameter \eqn{\sigma^2} can be modeled either
as constant (when the dispersion formula contains only an intercept) or as varying
across observations through a linear predictor.
}
\seealso{
\code{\link{simplexreg}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_dist.R
\name{simplex_opt}
\alias{simplex_opt}
\alias{dsimplex}
\alias{psimplex}
\alias{qsimplex}
\alias{rsimplex}
\title{Simplex Distribution Functions}
\usage{
dsimplex(x, mu, sigma2, log = FALSE)

psimplex(q, mu, sigma2, lower.tail = TRUE, log.p = FALSE)

qsimplex(p, mu, sigma2, lower.tail = TRUE, log.p = FALSE)

rsimplex(n, mu, sigma2)
}
\arguments{
\item{x, q}{Numeric vector of quantiles.}

\item{mu}{Mean parameter (\eqn{0 < \mu < 1}).}

\item{sigma2}{Dispersion parameter (\eqn{\sigma^2 > 0}).}

\item{log, log.p}{Logical; if TRUE, probabilities/densities p are given as log(p).}

\item{lower.tail}{Logical; if TRUE (default), probabilities are \eqn{P[X <= x]},
otherwise, \eqn{P[X > x]}.}

\item{p}{Numeric vector of probabilities.}

\item{n}{Number of observations.}
}
\value{
\code{dsimplex} gives the density, \code{psimplex} the
distribution function, \code{qsimplex} the quantile function, and
\code{rsimplex} generates random deviates.

Invalid arguments will result in return value NaN, with a warning.
}
\description{
Density, distribution function, quantile function and random generation for the
simplex distribution with parameters mean \eqn{\mu} and dispersion
\eqn{\sigma^2 > 0}.
}
\details{
The probability density function of the simplex distribution is:
\deqn{f(y; \mu, \sigma^2) = \frac{1}{\sqrt{2\pi\sigma^2[y(1-y)]^3}}
      \exp\left(-\frac{1}{2\sigma^2} d(y; \mu)\right),}
where \eqn{y \in (0,1)}, and \eqn{d(x; \mu) = \frac{(y - \mu)^2}{y(1 - y)
\mu^2(1 - \mu)^2}}
is the deviance component of the simplex model.
}
\examples{
dsimplex(0.5, mu = 0.3, sigma2 = 0.5)
dsimplex(0.5, mu = 0.3, sigma2 = 0.5, log = TRUE)
psimplex(0.5, mu = 0.3, sigma2 = 0.5)
psimplex(0.5, mu = 0.3, sigma2 = 0.5, lower.tail = FALSE)
qsimplex(0.5, mu = 0.3, sigma2 = 0.5)
qsimplex(log(0.5), mu = 0.3, sigma2 = 0.5, log.p = TRUE)
rsimplex(5, mu = 0.5, sigma2 = 0.5)

}
\references{
Barndorff-Nielsen, O. E. and Jørgensen, B. (1991).
Some parametric models on the simplex.
\emph{Journal of Multivariate Analysis}, \bold{39}(1), 106--116.
\doi{10.1016/0047-259X(91)90008-P}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_fit.R
\name{simplexreg.nul}
\alias{simplexreg.nul}
\title{Null Model Log-Likelihood for Simplex Regression}
\usage{
simplexreg.nul(y, link.mu, weights = NULL)
}
\arguments{
\item{y}{Numeric response vector (0 < y < 1).}

\item{link.mu}{Mean link function: parametric ("plogit1", "plogit2") or fixed
("logit", "probit", "loglog", "cloglog", "cauchit").}

\item{weights}{Optional vector of weights (default: NULL).}
}
\value{
Numeric value of the null model log-likelihood
}
\description{
Computes the log-likelihood for the null (intercept-only) for a
simplex regression with a parametric or fixed mean link.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_methods.R
\name{simplexreg.methods}
\alias{simplexreg.methods}
\alias{print.simplexregression}
\alias{summary.simplexregression}
\alias{coef.simplexregression}
\alias{vcov.simplexregression}
\alias{logLik.simplexregression}
\alias{fitted.simplexregression}
\alias{predict.simplexregression}
\alias{nobs.simplexregression}
\alias{df.residual.simplexregression}
\alias{deviance.simplexregression}
\alias{formula.simplexregression}
\alias{terms.simplexregression}
\alias{model.frame.simplexregression}
\alias{model.matrix.simplexregression}
\alias{update.simplexregression}
\alias{simulate.simplexregression}
\alias{AIC.simplexregression}
\alias{BIC.simplexregression}
\alias{hatvalues.simplexregression}
\alias{bread.simplexregression}
\alias{estfun.simplexregression}
\alias{coeftest.simplexregression}
\alias{lrtest.simplexregression}
\alias{print.summary.simplexregression}
\alias{HQIC}
\alias{HQIC.simplexregression}
\title{Methods for simplexregression Objects}
\usage{
\method{print}{simplexregression}(x, digits = max(3, getOption("digits") - 3), ...)

\method{summary}{simplexregression}(object, ...)

\method{print}{summary.simplexregression}(x, digits = max(3, getOption("digits") - 3), ...)

\method{coef}{simplexregression}(object, model = c("full", "mean", "dispersion"), ...)

\method{vcov}{simplexregression}(object, model = c("full", "mean", "dispersion"), ...)

\method{logLik}{simplexregression}(object, ...)

\method{fitted}{simplexregression}(object, ...)

\method{predict}{simplexregression}(
  object,
  newdata = NULL,
  type = c("response", "link", "dispersion"),
  ...
)

\method{nobs}{simplexregression}(object, ...)

\method{df.residual}{simplexregression}(object, ...)

\method{deviance}{simplexregression}(object, ...)

\method{formula}{simplexregression}(x, ...)

\method{terms}{simplexregression}(x, model = c("mean", "dispersion"), ...)

\method{model.frame}{simplexregression}(formula, ...)

\method{model.matrix}{simplexregression}(object, model = c("mean", "dispersion"), ...)

\method{update}{simplexregression}(object, formula., ..., evaluate = TRUE)

\method{simulate}{simplexregression}(object, nsim = 1, seed = NULL, ...)

\method{AIC}{simplexregression}(object, ..., k = 2)

\method{BIC}{simplexregression}(object, ...)

HQIC(object, ...)

\method{HQIC}{simplexregression}(object, ...)

\method{hatvalues}{simplexregression}(model, ...)

\method{bread}{simplexregression}(x, ...)

\method{estfun}{simplexregression}(x, ...)

\method{coeftest}{simplexregression}(x, vcov. = NULL, df = Inf, ...)

\method{lrtest}{simplexregression}(object, ...)
}
\arguments{
\item{digits}{Number of digits for printing.}

\item{...}{Additional arguments.}

\item{object, x}{An object of class \code{"simplexregression"}.}

\item{model}{Character specifying for which component of the model
coefficients/covariance should be extracted.}

\item{newdata}{Optional data frame for prediction.}

\item{type}{Character indicating type of predictions: fitted means of the
response (default, \code{"response"}), corresponding linear
predictor (\code{"link"}) or fitted dispersion parameter (\code{"dispersion"}).}

\item{formula}{A model formula or terms object.}

\item{formula.}{Changes to the formula.}

\item{evaluate}{If true evaluate the new call else return the call.}

\item{nsim}{number of response vectors to simulate. Defaults to \code{1}.}

\item{seed}{an object specifying if and how the random number generator
should be initialized.}

\item{k}{weight of the penalty term in AIC. Default is \code{2}.}

\item{vcov.}{a specification of the covariance matrix of the estimated coefficients.}

\item{df}{the degrees of freedom to be used.}
}
\value{
The return value depends on the method called:
\itemize{
  \item \code{print}: returns \code{x} invisibly;
  \item \code{summary}: returns an object of class
    \code{"summary.simplexregression"};
  \item \code{print.summary}: returns \code{x} invisibly;
  \item \code{coef}: named numeric vector of coefficients;
  \item \code{vcov}: numeric matrix (variance-covariance);
  \item \code{logLik}: object of class \code{"logLik"};
  \item \code{fitted}: numeric vector of fitted mean values;
  \item \code{predict}: numeric vector or list depending on \code{type};
  \item \code{nobs}: integer scalar (number of observations);
  \item \code{df.residual}: integer scalar (residual degrees of freedom);
  \item \code{deviance}: numeric scalar (total deviance);
  \item \code{formula}: the model formula;
  \item \code{terms}: the model terms for the selected submodel;
  \item \code{model.frame}: a data frame;
  \item \code{model.matrix}: numeric matrix of regressors;
  \item \code{update}: fitted model or call depending on \code{evaluate};
  \item \code{simulate}: a data frame with \code{nsim} columns;
  \item \code{AIC}, \code{BIC}, \code{HQIC}: numeric scalar (single model)
    or data frame (multiple models);
  \item \code{hatvalues}: numeric vector of hat values;
  \item \code{bread}: numeric matrix;
  \item \code{estfun}: numeric matrix of score contributions;
  \item \code{coeftest}: object of class \code{"coeftest"};
  \item \code{lrtest}: object of class \code{"anova"}.
}
}
\description{
Methods for extracting information from fitted model objects
of class \code{"simplexregression"}.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Extract information
summary(fit)
coef(fit)
vcov(fit)
logLik(fit)
fitted(fit)
AIC(fit)
BIC(fit)
HQIC(fit)
hatvalues(fit)

}
\seealso{
\code{\link{simplexreg}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_influence.R
\name{gleverage}
\alias{gleverage}
\alias{gleverage.simplexregression}
\title{Generalized Leverage Values for Simplex Regression Models}
\usage{
gleverage(model)

\method{gleverage}{simplexregression}(model)
}
\arguments{
\item{model}{An object of class \code{"simplexregression"}.}
}
\value{
A numeric vector of generalized leverage values.
}
\description{
Compute the generalized leverage values for simplex regression
models with parametric or fixed mean link function.
}
\details{
\code{gleverage} computes generalized leverage values as suggested by Wei, Hu,
and Fung (1998). Generalized leverage extends the concept of hat values to account for both
mean and dispersion parameters. High leverage values indicate observations
that have potentially large influence on parameter estimates.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Compute generalized leverage
glev <- gleverage(fit)

# Plot leverage values
plot(glev, type = "h", ylab = "Generalized leverage",
     xlab = "Observation index")
abline(h = 2 * mean(glev), lty = 2, col = "red")

}
\references{
Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Wei, B. C., Hu, Y. Q. and Fung, W. K. (1998).
Generalized leverage and its applications.
\emph{Scandinavian Journal of Statistics}, \bold{25}, 25--37.
}
\seealso{
\code{\link{hatvalues.simplexregression}},
\code{\link{cooks.distance.simplexregression}},
\code{\link{local.influence}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_resettest.R
\name{resettest}
\alias{resettest}
\title{RESET Test for Simplex Regression}
\usage{
resettest(model, dispersion = TRUE, power = 2, type = c("lp", "fitted"))
}
\arguments{
\item{model}{An object of class \code{"simplexregression"}.}

\item{dispersion}{Logical. If \code{TRUE}, includes the augmented terms
in the dispersion submodel as well. Default is \code{TRUE}.}

\item{power}{Integer vector specifying which powers of the fitted mean linear predictor
(or fitted mean values) to include as additional regressors. Default is \code{2}
(squared term only). Use \code{power = 2:3} to include both squared and
cubic terms, following the convention of \code{lmtest::resettest}.}

\item{type}{Character string specifying the base for the augmented terms.
\code{"lp"} (default) uses the fitted mean linear predictor \eqn{\boldsymbol{\hat{\eta}_1}};
\code{"fitted"} uses the fitted mean values \eqn{\boldsymbol{\hat{\mu}}} on the (0, 1)
scale.}
}
\value{
An object of class \code{"htest"} containing:
\itemize{
  \item \code{statistic}: The likelihood ratio test statistic,
  \item \code{parameter}: Degrees of freedom,
  \item \code{p.value}: The \eqn{p}-value of the test,
  \item \code{method}: Description of the test,
  \item \code{data.name}: Model formula.
}
}
\description{
Performs the Ramsey's RESET misspecification test in simplex
regression models.
}
\details{
The RESET test augments the original model by adding powers of the fitted mean linear
predictor (or fitted mean values) as additional covariates. Under the null
hypothesis of correct functional form, these additional terms should not
be significant.

Under H0, the likelihood ratio statistic is asymptotically distributed
as chi-squared with degrees of freedom equal to the number of augmented
terms added (i.e., \code{length(power)} if \code{dispersion = FALSE}, or
\code{2 * length(power)} if \code{dispersion = TRUE}).

If \code{dispersion = TRUE}, the augmented terms are added to both the
mean and dispersion submodels. If \code{FALSE}, they are only added to
the mean submodel.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Default: squared linear predictor in both submodels
resettest(fit)

# RESET test only for mean submodel
resettest(fit, dispersion = FALSE)

# Include squared and cubic terms
resettest(fit, power = 2:3)

# Use fitted values instead of linear predictor
resettest(fit, type = "fitted")

}
\references{
Ramsey, J. B. (1969). Tests for specification errors in classical linear
least-squares regression analysis. \emph{Journal of the Royal Statistical
Society: Series B}, \bold{31}(2), 350--371.
\doi{10.1111/j.2517-6161.1969.tb00796.x}
}
\seealso{
\code{\link{simplexreg}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_r2.R
\name{r2}
\alias{r2}
\title{Pseudo-R2 Measures for Simplex Regression}
\usage{
r2(..., alpha1 = 0.4, alpha2 = 1, alpha3 = c("1", "log"))
}
\arguments{
\item{...}{One or more objects of class \code{"simplexregression"} to be
evaluated.}

\item{alpha1}{Numeric, \eqn{\alpha_1 \in [0, 1]} controlling the relative penalty
weight given to the mean and dispersion submodels in the weighted
correction \eqn{R^2_{Nw_c}}. Default is \code{0.4}, as
recommended by Bayer and Cribari-Neto (2017).}

\item{alpha2}{Numeric, \eqn{\alpha_2 > 0}, controlling the penalization
intensity in \eqn{R^2_{Nw_c}}. Default is \code{1}, as recommended by
Bayer and Cribari-Neto (2017).}

\item{alpha3}{Penalization constant for the correction of
\eqn{R^2_{HS}}. One of \code{"1"} (default, \eqn{\alpha_3 = 1}) or
\code{"log"} (\eqn{\alpha_3 = \log(n)}).}
}
\value{
When a single model is provided, a named numeric vector with
components \code{R2_FC}, \code{R2_FC_c}, \code{R2_N}, \code{R2_N_c},
\code{R2_Nw_c}, \code{R2}, and \code{R2_HS}. When multiple models are
provided, a data frame with one row per model.
}
\description{
Extracts the Ferrari and Cribari-Neto pseudo-\eqn{R^2}
(\eqn{R^2_{FC}}), the likelihood-ratio pseudo-\eqn{R^2}
(\eqn{R^2_N}), the conventional coefficient of determination
(\eqn{R^2}), and computes their finite-sample corrections. The
corrections for \eqn{R^2_{FC}} and \eqn{R^2_N} follow Bayer and
Cribari-Neto (2017), whereas the correction for the conventional
\eqn{R^2} follows Hu and Shao (2008).
}
\details{
\eqn{R^2_{FC}} is the Ferrari and Cribari-Neto (2004) pseudo-R2, defined as
the squared correlation between the fitted mean linear predictor and the
link-transformed response, i.e.,
\deqn{R^2_{FC} = \mathrm{corr}^2\!\left(
\boldsymbol{\hat\eta_1}, g(\boldsymbol{y})\right),}
where \eqn{\boldsymbol{\hat\eta_1}} denotes the vector of fitted mean linear
predictors, \eqn{g(\cdot)} is the mean link function, and \eqn{\boldsymbol{y}}
is the vector of observed values of the response variable.

\eqn{R^2_N} is a likelihood-ratio-based pseudo-R2 (Nagelkerke, 1991), defined as
\deqn{R^2_N = 1 - \left(\frac{L_{null}}{L_{fit}}\right)^{2/n},}
where \eqn{L_{null}} and \eqn{L_{fit}} are the maximized likelihoods of the
null (intercept-only) and fitted models, respectively.

The conventional coefficient of determination is
\deqn{R^2 = 1 - \frac{\sum_{i=1}^n (y_i - \hat\mu_i)^2}{\sum_{i=1}^n (y_i - \bar y)^2},}
where \eqn{\bar y} is the sample mean of the responses.

\strong{Corrections:}
Both finite-sample corrections implemented here for
\eqn{R^2_{FC}} and \eqn{R^2_N} were proposed by Bayer and Cribari-Neto
(2017) for beta regression with varying precision and are used here in
their direct simplex regression analogue.

The simple correction, a function of the total number of estimated
parameters \eqn{r} only,
is applied to both \eqn{R^2_{FC}} and \eqn{R^2_N}:
\deqn{R^2_{FC_c} = 1 - (1 - R^2_{FC})\frac{n-1}{n-r} \qquad \text{and} \qquad
R^2_{N_c} = 1 - (1 - R^2_{N})\frac{n-1}{n-r}.}

A second, weighted correction is defined for \eqn{R^2_N} only, penalizing
the mean and dispersion submodels asymmetrically through \code{alpha1} and
controlling penalization intensity through \code{alpha2}:
\deqn{R^2_{Nw_c} = 1 - (1 - R^2_N)\left(\frac{n-1}{n-(1+\alpha_1)p-
(1-\alpha_1)q}\right)^{\alpha2},}
where \eqn{p} and \eqn{q} being the number of parameters in the mean and
dispersion submodels, respectively.
Setting \code{alpha1 = 0} and \code{alpha2 = 1} reduces \eqn{R^2_{Nw_c}} to
the simple \eqn{R^2_{N_c}} above.
Bayer and Cribari-Neto (2017) recommend
\code{alpha1 = 0.4} and \code{alpha2 = 1} as sensible defaults; both arguments
are exposed here for users who wish to specify different values.

Finally, the conventional coefficient of determination admits the
finite-sample correction proposed by Hu and Shao (2008):
\deqn{R^2_{HS} = 1 - \frac{n-1}{n - \alpha_3 r}
\frac{\sum_{i=1}^n (y_i - \hat\mu_i)^2}{\sum_{i=1}^n (y_i - \bar y)^2}.}
where \eqn{\alpha_3} is a penalization constant.
When \code{alpha_3 = "1"} (default), \eqn{\alpha_3 = 1}, \eqn{R^2_{HS}}
reduces to the modified R2 of Mittlböck and Schemper (2002). When
\code{alpha_3 = "log"}, \eqn{\alpha_3 = \log(n)}, as evaluated by Hu and Shao
(2008).

The measures \eqn{R^2_{FC}} and \eqn{R^2_N} take values in \eqn{[0,1]}.
The conventional coefficient of determination \eqn{R^2}, as well as the
corrected measures \eqn{R^2_{FC_c}}, \eqn{R^2_{N_c}}, \eqn{R^2_{Nw_c}}, and
\eqn{R^2_{HS}}, take values in \eqn{(-\infty,1]}.
Larger values indicate better model fit.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                   data = ReadingSkills)

# Single model, default alpha1, alpha2 and alpha3
r2(fit1)

# Comparing multiple models
fit2 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                   data = ReadingSkills, link.mu = "loglog")
r2(fit1, fit2)

# Custom alpha1/alpha2 for the weighted correction, and alpha3 = log(n)
r2(fit1, fit2, alpha1 = 0.6, alpha2 = 1.5, alpha3 = "log")

}
\references{
Bayer, F. M. and Cribari-Neto, F. (2017). Model selection criteria in beta
regression with varying dispersion. \emph{Communications in Statistics -
Simulation and Computation}, \bold{46}(1),
729--746. \doi{10.1080/03610918.2014.977918}

Ferrari, S. L. P. and Cribari-Neto, F. (2004). Beta regression for
modelling rates and proportions. \emph{Journal of Applied Statistics},
\bold{31}(7), 799--815. \doi{10.1080/0266476042000214501}

Hu, B. and Shao, J. (2008). Generalized linear model selection using R2.
\emph{Journal of Statistical Planning and Inference}, \bold{138}(12),
3705--3712. \doi{10.1016/j.jspi.2007.12.009}

Mittlböck, M. and Schemper, M. (2002). Explained variation for logistic
regression -- small sample adjustments, confidence intervals and other
issues. \emph{Statistics in Medicine}, \bold{21}(23), 3547--3562.
\doi{10.1002/1521-4036(200204)44:3<263::AID-BIMJ263>3.0.CO;2-7}

Nagelkerke, N. J. D. (1991). A note on a general definition of the
coefficient of determination. \emph{Biometrika}, \bold{78}(3), 691--692.
\doi{10.1093/biomet/78.3.691}
}
\seealso{
\code{\link{simplexreg}}, \code{\link{press}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_influence.R
\name{local.influence}
\alias{local.influence}
\title{Local Influence for Simplex Regression Models}
\usage{
local.influence(
  model,
  scheme = c("case.weight", "response"),
  parameter = c("theta", "beta", "gamma"),
  type = c("Ci", "dmax"),
  plot = FALSE,
  threshold = NULL,
  label.pos = 3,
  plot.type = NULL,
  ...
)
}
\arguments{
\item{model}{An object of class \code{simplexregression}.}

\item{scheme}{Character string specifying the perturbation scheme:
\code{"case.weight"} or \code{"response"}.}

\item{parameter}{Character string indicating the parameter block:
\code{"theta"} (default, all parameters), \code{"beta"} (mean submodel),
or \code{"gamma"} (dispersion submodel).}

\item{type}{Character string specifying the influence measure to compute:
\code{"Ci"} (default, total local influence / normal curvature) or
\code{"dmax"} (maximum influence direction).}

\item{plot}{Logical; if \code{TRUE}, produces an index plot of the selected
measure. Default is \code{FALSE}.}

\item{threshold}{Numeric threshold for identifying influential observations in.
If \code{NULL} (default), no observations are highlighted.}

\item{label.pos}{Position(s) for outlier labels in plot. Can be a single value
(applied to all labels) or a vector. Values: 1 = below, 2 = left, 3 = above,
4 = right.}

\item{plot.type}{Character string controlling the plot style when
\code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} (vertical
lines). Passed to the \code{type} argument of \code{plot()}.}

\item{...}{Additional graphical parameters passed to \code{plot()}.}
}
\value{
If \code{plot = FALSE} (default), a list containing:
\itemize{
  \item \code{dmax.beta}: Maximum influence direction for mean parameters;
  \item \code{dmax.gamma}: Maximum influence direction for dispersion parameters;
  \item \code{dmax.theta}: Maximum influence direction for all parameters;
  \item \code{Ci.beta}: Total local influence for mean parameters;
  \item \code{Ci.gamma}: Total local influence for dispersion parameters;
  \item \code{Ci.theta}: Total local influence for all parameters.
}
If \code{plot = TRUE}, the same list is returned invisibly.
}
\description{
Computes local influence measures under case-weight and
response perturbation schemes for simplex regression models with
parametric or fixed mean link function.
}
\details{
Measures local influence based on the curvature of the log-likelihood surface
under small perturbations. Two perturbation schemes are implemented:
\itemize{
  \item \strong{Case-weight}: Perturbs observation weights;
  \item \strong{Response perturbation}: Perturbs response values.
}

The index plot of \code{dmax} can be used to detect observations that are
jointly influential for parameters. The index plot of the normal curvature
\code{Ci} can be used to detect observations that are individually influential
for parameters.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Local influence under case-weight perturbation — return results
infl_cw <- local.influence(fit, scheme = "case.weight")

# Plot Ci for beta directly from the function call
local.influence(fit, scheme = "case.weight",
                parameter = "beta", type = "Ci", plot = TRUE)

# Plot dmax for all parameters under response perturbation
local.influence(fit, scheme = "response",
                parameter = "theta", type = "dmax", plot = TRUE)

}
\references{
Espinheira, P. L., Silva, A. O. (2020). Residual and influence analysis to a
general class of simplex regression. \emph{TEST}, \bold{29}, 523–-552.
\doi{10.1007/s11749-019-00665-3}
}
\seealso{
\code{\link{hatvalues.simplexregression}},
\code{\link{cooks.distance.simplexregression}},
\code{\link{gleverage.simplexregression}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readingskills.R
\docType{data}
\name{ReadingSkills}
\alias{ReadingSkills}
\title{Reading Accuracy in Dyslexic and Non-Dyslexic Children}
\format{
A data frame with 44 observations and 4 variables:
\describe{
  \item{accuracy}{Numeric. Reading score transformed to the open (0, 1) interval.
        Values originally equal to 1 were replaced with 0.99. Suitable for
        standard beta regression.}
  \item{dyslexia}{Factor. Indicates whether the child has dyslexia
        (levels: \code{"no"}, \code{"yes"}). Note that sum contrasts are typically used
        instead of treatment contrasts in beta regression analyses of this data.}
  \item{iq}{Numeric. Non-verbal intelligence quotient, transformed to
        \eqn{z}-scores (mean = 0, standard deviation = 1).}
  \item{accuracy1}{Numeric. Unrestricted reading score in the [0, 1] interval.
        This version preserves the original maximum value of 1 and can be used
        with extended--support beta mixture regression models.}
}
}
\source{
Pammer, K. and Kevan, A. (2007). The contribution of visual sensitivity,
phonological processing and non-verbal IQ to children's reading.
\emph{Scientific Studies of Reading}, \bold{11}(1), 33--53.
\doi{10.1080/10888430709336633}

Smithson, M. and Verkuilen, J. (2006). A Better lemon squeezer?
Maximum-likelihood regression with beta-distributed dependent variables.
\emph{Psychological Methods}, \bold{11}(1), 54--71. \doi{10.1037/1082-989X.11.1.54}
}
\usage{
data(ReadingSkills)
}
\description{
This dataset examines the relationship between non-verbal IQ and reading
accuracy in children diagnosed with dyslexia and in typical readers.
The reading scores are proportions bounded in the open interval (0, 1), making
them suitable for beta regression and simplex regression analysis.
}
\details{
The data were originally collected by Pammer and Kevan (2004) and later
analyzed by Smithson and Verkuilen (2006) to demonstrate beta regression.

The transformation procedure for \code{accuracy} was as follows:
\enumerate{
  \item The original test scores were scaled to the [0, 1] interval using
        the minimum and maximum possible scores in the reading test,
        resulting in \code{accuracy1}.
  \item To avoid boundary values (0 and 1) that are problematic for standard
        beta regression, all observations with value 1 were replaced with 0.99,
        creating the \code{accuracy} variable.
}

The unrestricted \code{accuracy1} variable can be analyzed using extended--support
beta regression methods (Kosmidis & Zeileis, 2025), which naturally accommodate
boundary observations.
}
\examples{
# Load the data
data(ReadingSkills)

# Quick overview
head(ReadingSkills)
str(ReadingSkills)

# Summary statistics by dyslexia status
aggregate(accuracy ~ dyslexia, data = ReadingSkills, summary)
aggregate(iq ~ dyslexia, data = ReadingSkills, summary)

# Visualize the relationship between IQ and reading accuracy
plot(accuracy ~ iq, data = ReadingSkills,
     col = c(4, 2)[dyslexia], pch = 19,
     main = "Reading Accuracy vs. IQ by Dyslexia Status",
     xlab = "IQ (z-scored)", ylab = "Reading Accuracy")
legend("topleft", legend = c("Non-dyslexic", "Dyslexic"),
       col = c(4, 2), pch = 19, bty = "n")

# Check for boundary values
table(ReadingSkills$accuracy == 0.99)  # Values replaced from 1
table(ReadingSkills$accuracy1 == 1)    # Original boundary values
}
\references{
Cribari-Neto, F. and Zeileis, A. (2010). Beta regression in R.
\emph{Journal of Statistical Software}, \bold{34}(2), 1--24.
\doi{10.18637/jss.v034.i02}

Kosmidis, I. and Zeileis, A. (2025). Extended-support beta regression for
[0, 1] responses. \emph{Journal of the Royal Statistical Society C},
\bold{75}(1), 139--157. \doi{10.1093/jrsssc/qlaf039}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_press.R
\name{press}
\alias{press}
\title{PRESS-Based \eqn{P2} Statistics for Simplex Regression}
\usage{
press(..., type = c("standardized", "biasvariance"))
}
\arguments{
\item{...}{One or more objects of class \code{simplexregression}.}

\item{type}{Character string specifying the type of residual to use.
Options are \code{"standardized"} (default) or \code{"biasvariance"}
(see \code{\link{residuals.simplexregression}}).}
}
\value{
When a single model is provided, a named numeric vector with components
\code{P2}, \code{P2_c}, and \code{PRESS}. When multiple models are provided,
a data frame with one row per model and columns \code{P2}, \code{P2_c}, and
\code{PRESS}.
}
\description{
Computes the PRESS (Predicted Residual Error Sum of Squares)
statistic and the associated \eqn{P^2} and adjusted \eqn{P^2} measures for simplex
regression models with parametric or fixed mean link function, as proposed by
Espinheira and Silva (2026).
}
\details{
The PRESS statistic for the simplex regression model is given by:
\deqn{\text{PRESS} = \sum_{i=1}^{n} \left(\frac{r_i}{1 - h_{ii}}\right)^2,}
where \eqn{r_i} denotes the residual for observation \eqn{i} and
\eqn{h_{ii}} is the \eqn{i}-th diagonal element of the hat matrix.

The \eqn{P^2} statistic is a cross-validation analog of \eqn{R^2}, defined
for the simplex regression mode as:
\deqn{P^2 = 1 - \frac{\text{PRESS}}{\left(\frac{n}{n-r}\right)^2 \text{SST}},}
where \eqn{\text{SST} = \sum_{i=1}^{n}(\hat{y}_i - \bar{\hat{y}})^2} and
\eqn{r} is the number of estimated parameters.

The adjusted \eqn{P^2} is given by:
\deqn{P^2_c = 1 - (1 - P^2)\frac{n-1}{n-r}.}

The \code{type} argument controls which residuals \eqn{r_i} are used in
the PRESS computation.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills, link.mu = "loglog")
# Single model
press(fit)

# Comparing multiple models
press(fit, fit1)

# Using bias-variance residuals
press(fit, fit1, type = "biasvariance")

}
\references{
Espinheira, P. L., Silva, A. O. (2026). Prediction in the nonlinear simplex model.
\emph{International Journal of Data Science and Analytics}, \bold{22}, 161.
\doi{10.1007/s41060-026-01114-9}

Espinheira, P. L., Silva, A. O. (2020). Residual and influence analysis to a
general class of simplex regression. \emph{TEST}, \bold{29}, 523–-552.
\doi{10.1007/s11749-019-00665-3}
}
\seealso{
\code{\link{simplexreg}}, \code{\link{residuals.simplexregression}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_links.R
\name{dispersion_links}
\alias{dispersion_links}
\alias{dispersion_link}
\alias{dispersion_link_inv}
\alias{dispersion_link_deriv1}
\alias{dispersion_link_inv_deriv1}
\title{Dispersion Link Functions and Their Derivatives}
\usage{
dispersion_link(sigma2, type = c("log", "sqrt", "identity"))

dispersion_link_inv(eta, type = c("log", "sqrt", "identity"))

dispersion_link_deriv1(sigma2, type = c("log", "sqrt", "identity"))

dispersion_link_inv_deriv1(eta, type = c("log", "sqrt", "identity"))
}
\arguments{
\item{sigma2}{Dispersion parameter (numeric vector, \eqn{\sigma^2 > 0}).}

\item{type}{Type of link: \code{"log"}, \code{"sqrt"} or \code{"identity"}.}

\item{eta}{Linear predictor of dispersion (numeric vector).}
}
\value{
Numeric vector with transformed values.
}
\description{
Provides the link function, its inverse, and derivative for the dispersion
submodel in the simplex regression.
Supported link types are: \code{"log"}, \code{"sqrt"} and \code{"identity"}.
}
\details{
Available link functions:
\itemize{
  \item Log (\code{"log"}): \eqn{h(\sigma^2) = \log(\sigma^2)} (ensures positivity);
  \item Sqrt (\code{"sqrt"}): \eqn{h(\sigma^2) = \sqrt{\sigma^2}};
  \item Identity (\code{"identity"}): \eqn{h(\sigma^2) = \sigma^2} (no transformation).
}
}
\examples{
dispersion_link(1.5, type = "log")
dispersion_link(c(0.5, 1, 2), type = "sqrt")
dispersion_link_inv(0, type = "log")
dispersion_link_deriv1(1, type = "log")
dispersion_link_inv_deriv1(0, type = "log")

}
\seealso{
\code{\link{simplexreg}}, \code{\link{fixed_mean_links}},
\code{\link{parametric_mean_links}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_var.R
\name{variance.simplex}
\alias{variance.simplex}
\title{Variance Function of the Simplex Distribution}
\usage{
variance.simplex(mu, sigma2)
}
\arguments{
\item{mu}{Numeric scalar or vector of mean parameters (\eqn{0 < \mu < 1}).
If a vector, must be the same length as \code{sigma2} or recyclable.}

\item{sigma2}{Numeric scalar or vector of dispersion parameters
(\eqn{\sigma^2 > 0}).
If a vector, must be the same length as \code{mu} or recyclable.}
}
\value{
A numeric scalar or vector of variance values.
}
\description{
Computes the variance of the simplex distribution as a function
of the mean parameter \eqn{\mu} and dispersion parameter \eqn{\sigma^2}.
}
\details{
The variance function for the simplex distribution is given by:
\deqn{Var(Y) = \mu(1-\mu) - \frac{1}{\sqrt{2\sigma^2}} \exp(a) \Gamma(0.5, a),}
where \eqn{a = \frac{1}{2\sigma^2[\mu(1-\mu)]^2}} and \eqn{\Gamma(0.5, a)}
is the upper incomplete gamma function.

For large values of \eqn{a} (> 700), an asymptotic approximation is used
to avoid numerical overflow:
\deqn{Var(Y) \approx \mu(1-\mu) - \frac{1}{\sqrt{2\sigma^2}} \sqrt{\frac{1}{a}}.}
}
\examples{
# Single value
variance.simplex(mu = 0.5, sigma2 = 0.1)

# Vector of values
mu_vec <- c(0.3, 0.5, 0.7)
sigma2_vec <- c(0.1, 0.15, 0.2)
variance.simplex(mu = mu_vec, sigma2 = sigma2_vec)

}
\references{
Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
Chapman and Hall, London.

Song, P. X.-K. and Tan, M. (2000).
Marginal models for longitudinal continuous proportional data.
\emph{Biometrics}, \bold{56}(2), 496--502.
\doi{10.1111/j.0006-341X.2000.00496.x}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_links.R
\name{parametric_mean_links}
\alias{parametric_mean_links}
\alias{parametric_mean_link}
\alias{parametric_mean_link_inv}
\alias{parametric_mean_link_deriv1}
\alias{parametric_mean_link_inv_deriv1}
\alias{parametric_mean_link_deriv2}
\title{Parametric Mean Link Functions and Derivatives}
\usage{
parametric_mean_link(mu, lambda, type = c("plogit1", "plogit2"))

parametric_mean_link_inv(eta, lambda, type = c("plogit1", "plogit2"))

parametric_mean_link_deriv1(mu, lambda, type = c("plogit1", "plogit2"))

parametric_mean_link_inv_deriv1(eta, lambda, type = c("plogit1", "plogit2"))

parametric_mean_link_deriv2(mu, lambda, type = c("plogit1", "plogit2"))
}
\arguments{
\item{mu}{Mean parameter (numeric vector, \eqn{0 < \mu < 1}).}

\item{lambda}{Power parameter (numeric scalar, \eqn{\lambda > 0}).}

\item{type}{Type of link function: \code{"plogit1"} or \code{"plogit2"}.}

\item{eta}{Linear predictor of mean (numeric vector).}
}
\value{
Numeric vector with transformed values.
}
\description{
Provides the parametric mean link functions, their inverses, and derivatives
for the simplex regression models. Two parametric link types are supported:
\code{"plogit1"} and \code{"plogit2"}.
}
\details{
Two parametric mean link functions are available, as proposed by
Justino and Cribari-Neto (2026):
\itemize{
  \item Parametric logit type 1 (\code{"plogit1"}): \eqn{g(\mu; \lambda) = \log((1-\mu)^{-\lambda} - 1)};
  \item Parametric logit type 2 (\code{"plogit2"}): \eqn{g(\mu; \lambda) = \log(\mu^\lambda / (1 - \mu^\lambda))}.
}
Their inverses and derivatives with respect to \eqn{\mu} are also implemented.
}
\examples{
parametric_mean_link(0.2, lambda = 1.2, type = "plogit2")
parametric_mean_link(c(0.2, 0.5, 0.8), lambda = 1.5, type = "plogit1")
parametric_mean_link_inv(0, lambda = 1, type = "plogit2")
parametric_mean_link_deriv1(0.5, lambda = 1, type = "plogit2")
parametric_mean_link_inv_deriv1(0, lambda = 1, type = "plogit2")
parametric_mean_link_deriv2(0.5, lambda = 1, type = "plogit2")

}
\references{
Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}
}
\seealso{
\code{\link{simplexreg}}, \code{\link{fixed_mean_links}},
\code{\link{dispersion_links}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_penalized_ic.R
\name{penalized.ic}
\alias{penalized.ic}
\title{Penalized Information Criteria for Simplex Regression Model Selection}
\usage{
penalized.ic(
  ...,
  kappa = 0.1,
  verbose = TRUE,
  digits = max(3, getOption("digits") - 3)
)
}
\arguments{
\item{...}{One or more objects of class \code{simplexregression} fitted with a
parametric mean link functions ("plogit1" or "plogit2").}

\item{kappa}{A numeric value controlling the additional penalty for the link
mean parameter. Default is 0.1.}

\item{verbose}{Logical. If \code{TRUE} (default), prints the criteria values.
If \code{FALSE}, returns results silently.}

\item{digits}{Integer specifying the number of decimal places for output.
Default is \code{max(3, getOption("digits") - 3)}.}
}
\value{
A data frame with rows named after the candidate models and four columns:
\describe{
  \item{\code{df}}{Number of estimated parameters.}
  \item{\code{AICc}}{Penalized AIC value.}
  \item{\code{BICc}}{Penalized BIC value.}
  \item{\code{HQICc}}{Penalized HQIC value.}
}
When \code{verbose = TRUE}, the results are also printed to the console and
the data frame is returned invisibly. When \code{verbose = FALSE}, the data
frame is returned visibly without printing.
}
\description{
Implements the Akaike, Schwarz, and Hannan–Quinn information
criteria with a penalty term for selecting among competing simplex regression
models with parametric mean link functions.
}
\details{
The penalized information criteria, as proposed by Justino and
Cribari-Neto (2026), extend the classical Akaike, Schwarz and
Hannan--Quinn criteria with an additional penalty term for the link
parameter \eqn{\lambda}:
\deqn{AIC^{(\lambda)} = -2 \ell + (2 + \kappa \, |\log(\lambda)|)r}
\deqn{BIC^{(\lambda)} = -2 \ell + (\log(n) + \kappa \, |\log(\lambda)|)r}
\deqn{HQIC^{(\lambda)}  = -2 \ell + (2 \log(\log(n)) + \kappa \, |\log(\lambda)|)r}
where:
\itemize{
  \item \eqn{\ell} denotes the maximized log-likelihood function;
  \item \eqn{\kappa \geq 0} controls the additional penalty associated with
  the link parameter;
  \item \eqn{\lambda} is the parameter of the parametric mean link function;
  \item \eqn{r} indicate the dimension of their parameter vector;
  \item \eqn{n} is the number of observations.
}

\strong{Important}: These penalized versions of the criteria should only be used
when the candidate model employ a parametric link function in the mean submodel
(use \code{kappa = 0.1}). When candidate models includes specifications
with fixed link functions, the standard unpenalized versions of these criteria
should be applied instead (use \code{kappa = 0}).
}
\examples{
# Simulate data
set.seed(2026)
n <- 100
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)
z1 <- runif(n, 0, 1)
mu <- parametric_mean_link_inv(0.6 - 2*x1 - 1.5*x2, 0.5, "plogit1")
sigma2 <- dispersion_link_inv(-2 - 2.5*z1, "log")
y <- rsimplex(n, mu, sigma2)
data <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1)

# Fit two models with parametric mean link functions
fit1 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit1")
fit2 <- simplexreg(y ~ x1 + x2 | z1, data = data, link.mu = "plogit2")

# Compute penalized criteria
penalized.ic(fit1, fit2)

}
\references{
Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Akaike, H. (1973). Information theory and an extension of the maximum
likelihood principle. \emph{Akadémiai Kiadó}, 267--281.

Schwarz, G. E. (1978). Estimating the dimension of a model. \emph{Annals of
Statistics}, \bold{6}(2), 461-–464. \doi{10.1214/aos/1176344136}

Hannan, E. J. and Quinn, B. G. (1979). The Determination of the Order of an
Autoregression. \emph{Journal of the Royal Statistical Society Series B:
Statistical Methodology}, \bold{41}(2), 190--195.
\doi{10.1111/j.2517-6161.1979.tb01072.x}
}
\seealso{
\code{\link{penalized.ss}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_residuals.R
\name{residuals.simplexregression}
\alias{residuals.simplexregression}
\title{Residuals for Simplex Regression Models}
\usage{
\method{residuals}{simplexregression}(
  object,
  type = c("quantile", "pearson", "deviance", "standardized", "weighted", "variance",
    "biasvariance", "score", "dualscore", "response"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{"simplexregression"}.}

\item{type}{Character string specifying the type of residual to extract.
Options:
\code{"quantile"} (default), \code{"pearson"}, \code{"deviance"},
\code{"standardized"}, \code{"weighted"}, \code{"variance"},
\code{"biasvariance"}, \code{"score"}, \code{"dualscore"}, \code{"response"}.}

\item{...}{Additional arguments (currently not used).}
}
\value{
A numeric vector of residuals.
}
\description{
Extracts various types of residuals for diagnostic analysis in
simplex regression models with parametric or fixed mean link functions.
}
\details{
Several types of residuals are available for model diagnostics:

\strong{Quantile residuals} (\code{"quantile"}): Proposed by Dunn and Smyth
(1996) as
\deqn{r_i^Q = \Phi^{-1}(F(y_i; \hat{\mu}_i, \hat{\sigma}^2_i)),}
where \eqn{\Phi(\cdot)} is the standard normal CDF and
\eqn{F(\cdot; \cdot)} is the simplex CDF (see \code{\link{psimplex}}).
Under correct model specification, these residuals are approximately standard
normal and are therefore recommended for general diagnostic use.

\strong{Pearson residuals} (\code{"pearson"}): Defined in McCullagh and
Nelder (1989) as
\deqn{r_i^P = \dfrac{y_i - \hat{\mu}_i}{\sqrt{\widehat{\text{Var}}(y_i)}},}
where \eqn{\widehat{\text{Var}}(y_i)} is the estimated variance of the response
(see \code{\link{variance.simplex}}).

\strong{Deviance residuals} (\code{"deviance"}): Defined in Jørgensen (1997,
p. 115) as
\deqn{r_i^D = \dfrac{y_i - \hat{\mu}_i}{\hat{\mu}_i(1-\hat{\mu}_i)\sqrt{y_i(1-y_i)}}.}

\strong{Standardized residuals} (\code{"standardized"}): Proposed by Espinheira
and Silva (2020, Eq. 15) as
\deqn{r_i^\beta = \dfrac{\hat{u}_i(y_i - \hat{\mu}_i)}{\sqrt{\hat{\sigma}^2_i \hat{w}_i}},}
where
\eqn{\hat{w}_i = 3\hat{\sigma}^2_i[\hat{\mu}_i(1-\hat{\mu}_i)]^{-1} + [\hat{\mu}_i(1-\hat{\mu}_i)]^{-3}}
and \eqn{\hat{u}_i = \hat{d}_i[\hat{\mu}_i(1-\hat{\mu}_i)]^{-1} + [\hat{\mu}_i(1-\hat{\mu}_i)]^{-3}},
with \eqn{\hat{d}_i} is the estimated unit deviance (see \code{\link{dev.unit.simplex}}).

\strong{Weighted residuals} (\code{"weighted"}): Proposed by Espinheira and
Silva (2020, Eq. 16) as
\deqn{r_i^{\beta*} = \dfrac{\hat{u}_i(y_i - \hat{\mu}_i)}{\sqrt{\hat{\sigma}^2_i \hat{w}_i(1-\hat{h}_{ii})}},}
where \eqn{\hat{h}_{ii}} are the diagonal elements of the hat matrix
(see \code{\link{hatvalues.simplexregression}}).
These residuals are recommended for simulated envelope plots.

\strong{Variance residuals} (\code{"variance"}): Proposed by Espinheira et al.
(2021, Eq. 7) as
\deqn{r_i^\gamma = \dfrac{\hat{d}_i - \hat{\sigma}^2_i}{\hat{\sigma}^2_i\sqrt{2}}.}

\strong{Bias–variance residuals} (\code{"biasvariance"}): Proposed by Espinheira
et al. (2021, Eq. 8) as
\deqn{r_i^{\beta \gamma} = \dfrac{\hat{u_i}(y_i - \hat{\mu}_i) + \hat{a}_i}{\sqrt{\hat{\sigma}^2_i
\hat{w}_i + 1/(2\hat{\sigma}^4_i)}},} where
\eqn{\hat{a}_i = -(2\hat{\sigma}^2_i)^{-1} + \hat{d}_i(2\hat{\sigma}^4_i)^{-1}}.

\strong{Score residuals} (\code{"score"}): Defined in Jørgensen (1997, p. 115) as
\deqn{r_i^{S} = \dfrac{(y_i - \hat{\mu}_i)(\hat{\mu}_i^2 + y_i - 2y_i\hat{\mu}_i)}
{y_i(1-y_i)\hat{\mu}_i^{1.5}(1-\hat{\mu}_i)^{1.5}}.}

\strong{Dual score residuals} (\code{"dualscore"}): Defined in Jørgensen
(1997, p. 115) as
\deqn{r_i^{DS} = \dfrac{(y_i - \hat{\mu}_i)(y_i + \hat{\mu}_i - 2y_i\hat{\mu}_i)}
{2\sqrt{y_i(1-y_i)}\hat{\mu}_i^2(1-\hat{\mu}_i)^2}.}

\strong{Response residuals} (\code{"response"}): Simple difference between
observed and fitted values, \deqn{r_i^R = y_i - \hat{\mu}_i.}

\strong{Recommendations}: Quantile residuals are recommended for general model
diagnostics due to their theoretical properties. Weighted residuals are
particularly useful for constructing simulated envelope plots, as they account
for both the variance structure and leverage effects.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Compute different types of residuals
res_quantile <- residuals(fit, type = "quantile")
res_pearson <- residuals(fit, type = "pearson")
res_weighted <- residuals(fit, type = "weighted")

}
\references{
Dunn, P. K. and Smyth, G. K. (1996). Randomized quantile residuals.
\emph{Journal of Computational and Graphical Statistics}, \bold{5}(3),
236--244. \doi{10.2307/1390802}

Espinheira, P. L. and Silva, A. O. (2020). Residual and influence analysis to a
general class of simplex regression. \emph{TEST}, \bold{29}, 523--552.
\doi{10.1007/s11749-019-00665-3}

Espinheira, P. L.; Silva, L. C. M. and Cribari-Neto, F. (2021). Bias and
variance residuals for machine learning nonlinear simplex regressions.
\emph{Expert Systems With Applications}, \bold{185}, 115656.
\doi{10.1016/j.eswa.2021.115656}

Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
Chapman and Hall, London.

McCullagh, P. and Nelder, J. A. (1989). \emph{Generalized Linear Models}.
2nd ed.  Chapman and Hall, London.
}
\seealso{
\code{\link{plot.simplexregression}}, \code{\link{halfnormal.plot}},
\code{\link{hatvalues.simplexregression}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_links.R
\name{fixed_mean_links}
\alias{fixed_mean_links}
\alias{fixed_mean_link}
\alias{fixed_mean_link_inv}
\alias{fixed_mean_link_deriv1}
\alias{fixed_mean_link_deriv2}
\alias{fixed_mean_link_inv_deriv1}
\title{Fixed Mean Link Functions and Derivatives}
\usage{
fixed_mean_link(
  mu,
  type = c("logit", "probit", "loglog", "cloglog", "cauchit")
)

fixed_mean_link_inv(
  eta,
  type = c("logit", "probit", "loglog", "cloglog", "cauchit")
)

fixed_mean_link_deriv1(
  mu,
  type = c("logit", "probit", "loglog", "cloglog", "cauchit")
)

fixed_mean_link_deriv2(
  mu,
  type = c("logit", "probit", "loglog", "cloglog", "cauchit")
)

fixed_mean_link_inv_deriv1(
  eta,
  type = c("logit", "probit", "loglog", "cloglog", "cauchit")
)
}
\arguments{
\item{mu}{Mean parameter (numeric vector, \eqn{0 < \mu < 1}).}

\item{type}{Type of link function: \code{"logit"}, \code{"probit"},
\code{"loglog"}, \code{"cloglog"}, or \code{"cauchit"}.}

\item{eta}{Linear predictor of mean (numeric vector).}
}
\value{
A numeric vector corresponding to the evaluated link,
its inverse, or derivative depending on the function.
}
\description{
Provides the fixed mean link functions, their inverses, and derivatives
for the simplex regression model. Supported link types are:
\code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"}, and \code{"cauchit"}.
}
\details{
Available link functions:
\itemize{
  \item Logit (\code{"logit"}): \eqn{g(\mu) = \log(\mu/(1-\mu))};
  \item Probit (\code{"probit"}): \eqn{g(\mu) = \Phi^{-1}(\mu)};
  \item Log-log (\code{"loglog"}): \eqn{g(\mu) = -\log(-\log(\mu))};
  \item Complementary log-log (\code{"cloglog"}): \eqn{g(\mu) = \log(-\log(1-\mu))};
  \item Cauchit (\code{"cauchit"}): \eqn{g(\mu) = \tan(\pi(\mu - 0.5))}.
}
}
\examples{
fixed_mean_link(0.5, type = "logit")
fixed_mean_link(c(0.2, 0.5, 0.8), type = "probit")
fixed_mean_link_inv(eta = 0.2, type = "logit")
fixed_mean_link_deriv1(mu = 0.5, type = "logit")

}
\seealso{
\code{\link{simplexreg}}, \code{\link{parametric_mean_links}},
\code{\link{dispersion_links}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_influence.R
\name{diag.im}
\alias{diag.im}
\title{Sample Influence Measures for Simplex Regression Models}
\usage{
diag.im(
  model,
  data,
  type = c("s3", "s5"),
  interval = c("I1", "I2"),
  parameter = c("theta", "beta", "gamma"),
  plot = FALSE,
  verbose = TRUE,
  ncores = 1,
  label.pos = 3,
  plot.type = NULL,
  ...
)
}
\arguments{
\item{model}{An object of class \code{simplexregression}.}

\item{data}{The data frame used to fit \code{model}.}

\item{type}{Character vector specifying measure(s): \code{"s3"}, \code{"s5"},
or both (default).}

\item{interval}{Character string specifying the outlier detection threshold:
\code{"I1"} (default, moderate) or \code{"I2"} (strict).}

\item{parameter}{Character string indicating the parameter block:
\code{"theta"} (default, all parameters), \code{"beta"} (mean submodel),
or \code{"gamma"} (dispersion submodel).}

\item{plot}{Logical; if \code{TRUE}, produces index plots of \eqn{s_{3,i}}
and \eqn{s_{5,i}} with threshold lines and flagged-observation labels.
Default is \code{FALSE}.}

\item{verbose}{Logical; if \code{TRUE} (default), prints progress during
leave-one-out refitting. Ignored (set to \code{FALSE}) when
\code{ncores > 1}, since output from parallel workers does not reach
the main console.}

\item{ncores}{Positive integer specifying the number of CPU cores to use
for the leave-one-out loop. Default is \code{1} (sequential). Values
greater than \code{1} activate parallel computation via
\code{\link[parallel]{parLapply}}. If \code{ncores} exceeds
\code{parallel::detectCores() - 1}, it is silently clamped to that
value to avoid overloading the system. A safe explicit choice is
\code{parallel::detectCores() - 1}.}

\item{label.pos}{Position(s) for outlier labels in plot. Can be a single
value (applied to all labels) or a vector. Values: 1 = below, 2 = left,
3 = above, 4 = right.}

\item{plot.type}{Character string controlling the plot style when
\code{plot = TRUE}. If \code{NULL} (default), uses \code{"h"} for
\eqn{n \le 150} and \code{"p"} for \eqn{n > 150} (automatic). Passed
to the \code{type} argument of \code{plot()}.}

\item{...}{Additional graphical parameters passed to \code{plot()}.}
}
\value{
If \code{plot = FALSE} (default), a list containing only the
requested measures:
\describe{
  \item{\code{s3_i}}{(if requested) Numeric vector of \eqn{s_{3,i}} values.}
  \item{\code{s5_i}}{(if requested) Numeric vector of \eqn{s_{5,i}} values.}
  \item{\code{outliers_s3}}{(if requested) Data frame of flagged observations
    for \eqn{s_{3,i}}.}
  \item{\code{outliers_s5}}{(if requested) Data frame of flagged observations
    for \eqn{s_{5,i}}.}
  \item{\code{limits_s3}}{(if requested) Named vector with lower and upper
    thresholds for \eqn{s_{3,i}}.}
  \item{\code{limits_s5}}{(if requested) Named vector with lower and upper
    thresholds for \eqn{s_{5,i}}.}
  \item{\code{interval}}{Interval type used.}
  \item{\code{parameter}}{Parameter block used.}
  \item{\code{n}}{Number of observations.}
}
If \code{plot = TRUE}, the same list is returned invisibly.
}
\description{
Computes leave-one-out sample influence measures \eqn{s_{3,i}}
and \eqn{s_{5,i}} for simplex regression models, based on the
information-matrix-based criteria measures proposed by Cribari-Neto,
Vasconcellos and Santana e Silva (2025).
}
\details{
For each observation \eqn{i}, the model is refit on the dataset with
observation \eqn{i} removed. Let \eqn{\hat\theta} and
\eqn{\hat\theta_{(i)}} denote the full and leave-one-out MLEs. Following
Cribari-Neto, Vasconcellos and Santana e Silva (2025), let \eqn{A_n}
denote the sample average of the second-order log-likelihood derivatives
(\eqn{n^{-1}\sum_i \partial^2\ell_i/\partial\theta\partial\theta'}) and
\eqn{B_n} denote the sample average of the outer products of the
individual score vectors
(\eqn{n^{-1}\sum_i (\partial\ell_i/\partial\theta)(\partial\ell_i/\partial\theta)'}),
the two matrices that comprise the information matrix equality. Let
\eqn{A^{(i)}_{n-1}} and \eqn{B^{(i)}_{n-1}} denote these same quantities
recomputed with observation \eqn{i} excluded (i.e. based on \eqn{n-1}
observations).

\strong{Measures computed:}
\deqn{s_{3,i} = m_{3,(i)} / m_3}
\deqn{s_{5,i} = D_i^{\mathrm{mod}} - D_i^{\mathrm{gen}}}

Here \eqn{m_3 = \|\mathrm{vech}(P_n^{-1} B_n P_n^{-\top} - I)\|_2}, where
\eqn{P_n} is obtained from the Cholesky decomposition of \eqn{-A_n}
(i.e. \eqn{-A_n = P_n P_n^\top}); \eqn{m_{3,(i)}} is the same quantity
recomputed with observation \eqn{i} excluded from the sample.

\deqn{D_i^{\mathrm{gen}} = (n-1) \,
  (\hat\theta - \hat\theta_{(i)})^\top (-A^{(i)}_{n-1})
  (\hat\theta - \hat\theta_{(i)})}
\deqn{D_i^{\mathrm{mod}} = (n-1) \,
  (\hat\theta - \hat\theta_{(i)})^\top
  0.5(-A^{(i)}_{n-1} + B^{(i)}_{n-1})
  (\hat\theta - \hat\theta_{(i)})}

If \eqn{-A^{(i)}_{n-1}} is not positive definite, \code{nearPD} is used to
find the nearest positive-definite matrix and a message is printed.

\strong{Threshold intervals} use two asymmetric IQR spreads, as proposed
by Cribari-Neto, Vasconcellos and Santana e Silva (2025). Let
\eqn{Q(p)} denote the empirical \eqn{p}-th quantile of the relevant
measure (\eqn{s_{3,i}} or \eqn{s_{5,i}}, computed separately for each):
\eqn{IQR_1 = Q(0.50) - Q(0.125)} (left) and
\eqn{IQR_2 = Q(0.875) - Q(0.50)} (right).
Limits are \eqn{v - z \cdot IQR_1} (lower) and \eqn{v + z \cdot IQR_2}
(upper), with reference value \eqn{v = 1} for \eqn{s_3} and \eqn{v = 0}
for \eqn{s_5}:
\itemize{
  \item \code{I1} (moderate): \eqn{z = 2.5} for \eqn{s_3};
    \eqn{z = 4.0} for \eqn{s_5}.
  \item \code{I2} (strict): \eqn{z = 5.0} for \eqn{s_3};
    \eqn{z = 8.0} for \eqn{s_5}.
}

\strong{Parallel computation:} when \code{ncores > 1}, the
leave-one-out loop is distributed across workers using
\code{\link[parallel]{parLapply}} from the \pkg{parallel} package
(included in base R). Each worker receives the necessary objects and
loads the \pkg{simplexregression} package. The random-number stream is
initialized with \code{\link[parallel]{clusterSetRNGStream}} to ensure
reproducibility across runs. Progress messages (\code{verbose}) are
suppressed in parallel mode because worker output does not reach the
main console.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

\donttest{
# Sequential (default)
im <- diag.im(fit, data = ReadingSkills, type = "s3", interval = "I1",
              parameter = "theta")

# Produce index plots directly
diag.im(fit, data = ReadingSkills, type = "s3", interval = "I1",
        parameter = "theta", plot = TRUE)
}

}
\references{
Cribari-Neto, F.; Vasconcellos, K. L. P.; Santana e Silva, J. J. (2025).
New strategies for detecting atypical observations based on the information
matrix equality. \emph{Journal of Applied Statistics}, \bold{52},
2873--2893. \doi{10.1080/02664763.2025.2487914}
}
\seealso{
\code{\link{diag.distances}}, \code{\link{local.influence}},
\code{\link{gleverage}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aisrowing.R
\docType{data}
\name{AISRowing}
\alias{AISRowing}
\title{Body Composition Data for Australian Rowers}
\format{
A data frame with 37 observations and 12 variables:
\describe{
  \item{sex}{Factor. Sex of the athlete (\code{female} or \code{male}).}
  \item{rcc}{Numeric. Red blood cell count (in \eqn{10^{12}} per litre).}
  \item{wcc}{Numeric. White blood cell count (in \eqn{10^{12}} per litre).}
  \item{hc}{Numeric. Hematocrit (in percent).}
  \item{hb}{Numeric. Hemoglobin concentration (in g per decilitre).}
  \item{ferr}{Numeric. Plasma ferritin concentration (in ng per millilitre).}
  \item{bmi}{Numeric. Body mass index (in kg per metre-squared).}
  \item{ssf}{Numeric. Sum of skin folds (in mm).}
  \item{bfat}{Numeric. Body fat proportion, originally measured
        in percent and rescaled to the unit interval (0, 1).}
  \item{lbm}{Numeric. Lean body mass (in kg).}
  \item{ht}{Numeric. Height (in cm).}
  \item{wt}{Numeric. Weight (in kg).}
}
}
\source{
Telford, R. D. and Cunningham, R. B. (1991). Sex, sport, and body-size
dependency of hematology in highly trained athletes. \emph{Medicine &
Science in Sports & Exercise}, \bold{23}(7), 788--794.
}
\usage{
data(AISRowing)
}
\description{
Hematological, body composition, and anthropometric measurements on
37 elite rowers (22 female, 15 male) at the Australian Institute of
Sport (AIS), a subset of a larger dataset collected across multiple
sports. The data are useful for investigating sex-based differences
in blood and body composition among highly trained athletes.
}
\details{
The original measurements were collected in the late 1980s by Richard
Telford and Ross Cunningham at the Australian Institute of Sport (AIS),
and were later compiled and popularized by Cook and Weisberg (1994).
\code{AISRowing} is the 37-athlete subset corresponding to rowing,
out of 202 athletes across all sports in the original dataset. The
full dataset (covering all sports, not just rowing) is also discussed
in Weisberg (2005, Section 6.4).
}
\examples{
# Load the data
data(AISRowing)

# Quick overview
head(AISRowing)
str(AISRowing)

# Summary statistics
summary(AISRowing)

# Sex-based comparison
aggregate(bfat ~ sex, data = AISRowing, summary)
boxplot(bfat ~ sex, data = AISRowing,
        main = "Body Fat Proportion by Sex",
        ylab = "Body fat (proportion)")
}
\references{
Cook, R. D. and Weisberg, S. (1994). \emph{An Introduction to Regression
Graphics}. John Wiley & Sons, New York.

Weisberg, S. (2005). \emph{Applied Linear Regression}, 3rd edition.
New York: Wiley, Section 6.4.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_deviance.R
\name{dev.unit.simplex}
\alias{dev.unit.simplex}
\title{Unit Deviance Function of the Simplex Distribution}
\usage{
dev.unit.simplex(y, mu)
}
\arguments{
\item{y}{Numeric scalar or vector of observed values (\eqn{0 < y < 1}).
If a vector, must be the same length as \code{mu} or recyclable.}

\item{mu}{Numeric scalar or vector of mean values (\eqn{0 < \mu < 1}).
If a vector, must be the same length as \code{y} or recyclable.}
}
\value{
A numeric scalar or vector of unit deviance values.
}
\description{
Computes the unit deviance (scaled deviance component) for the
simplex distribution.
}
\details{
The unit deviance for the simplex distribution is defined as:
\deqn{d(y, \mu) = \frac{(y - \mu)^2}{y(1-y)[\mu(1-\mu)]^2}.}

This function is used internally in maximum likelihood estimation and
model diagnostics for simplex regression.
}
\examples{
# Single value
dev.unit.simplex(y = 0.6, mu = 0.5)

# Vector of values
y_vec <- c(0.2, 0.5, 0.8)
mu_vec <- c(0.3, 0.5, 0.7)
dev.unit.simplex(y = y_vec, mu = mu_vec)

# Perfect fit returns zero deviance
dev.unit.simplex(y = 0.5, mu = 0.5)

}
\references{
Jørgensen, B. (1997).
\emph{The Theory of Dispersion Models}.
Chapman and Hall, London.

Song, P. X.-K. and Tan, M. (2000).
Marginal models for longitudinal continuous proportional data.
\emph{Biometrics}, \bold{56}(2), 496--502.
\doi{10.1111/j.0006-341X.2000.00496.x}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_plots.R
\name{halfnormal.plot}
\alias{halfnormal.plot}
\title{Half-Normal Plots with Simulated Envelopes for Simplex Regression}
\usage{
halfnormal.plot(
  model,
  type = c("weighted", "quantile", "pearson", "deviance", "standardized", "variance",
    "biasvariance", "score", "dualscore", "response"),
  nsim = 100,
  level = 0.95,
  seed = 1987,
  ...
)
}
\arguments{
\item{model}{An object of class \code{"simplexregression"}.}

\item{type}{Character string specifying the residual type (default: \code{"weighted"}).
\cr See \code{\link{residuals.simplexregression}} for available options.}

\item{nsim}{Number of simulations for envelope construction (default: \code{100}).}

\item{level}{Confidence level for envelope bounds (default: \code{0.95}).}

\item{seed}{Integer setting the random seed for reproducibility (default: \code{1987}).}

\item{...}{Additional graphical parameters.}
}
\value{
Called for its side effects (half-normal plot with simulated envelope).
Returns \code{NULL} invisibly.
}
\description{
Produces half-normal plots with simulated envelopes for simplex
regression model with parametric or fixed mean link function.
}
\details{
The envelope is based on the following steps:
\enumerate{
  \item Simulate \code{nsim} response vectors from the fitted model
        using its estimated mean and dispersion parameter;
  \item Refitting the model to each simulated dataset;
  \item Computing absolute residuals and their order statistics;
  \item Obtaining envelope bounds from empirical quantiles.
}

Simulated datasets whose refitted model fails to converge are discarded and
resampled, up to \code{5 * nsim} total attempts. If \code{nsim} converged
fits cannot be obtained within this limit, the function stops with an error.

Points outside the envelope may indicate model inadequacy.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

\donttest{
halfnormal.plot(fit, seed = 2008)
}

}
\seealso{
\code{\link{residuals.simplexregression}},
\code{\link{plot.simplexregression}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_ic.R
\name{ic}
\alias{ic}
\title{Information Criteria for Simplex Regression}
\usage{
ic(...)
}
\arguments{
\item{...}{One or more objects of class \code{"simplexregression"} to be
evaluated.}
}
\value{
When a single model is provided, a named numeric vector with
components \code{AIC}, \code{AICc}, \code{BIC}, \code{BICc}, \code{HQIC},
and \code{HQICc}. When multiple models are provided, a data frame with one
row per model.
}
\description{
Extracts the Akaike information criterion (AIC), the Schwarz
(Bayesian) information criterion (BIC), and the Hannan-Quinn criterion
(HQIC), along with their finite-sample corrections AICc, BICc, and HQICc follow
Bayer and Cribari-Neto (2017).
}
\details{
Let \eqn{\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})}
denote the maximized log-likelihood of the fitted model and \eqn{k} the
total number of estimated parameters. The AIC (Akaike, 1973) is
\deqn{AIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ 2k.}
Sugiura (1978) proposed an unbiased estimator of the Kullback-Leibler
distance in linear regressions, later generalized by Hurvich and Tsai
(1989) to nonlinear regressions and autoregressive models. The resulting
finite-sample-corrected AIC is
\deqn{AICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ \frac{2nk}{n-k-1}.}

The Schwarz (1978) information criterion, also known as the Bayesian
information criterion (BIC), is a consistent criterion given by
\deqn{BIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ k\log(n).}
McQuarrie (1999) derived a finite-sample-corrected version, which remains
consistent:
\deqn{BICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ \frac{nk\log(n)}{n-k-1}.}

The Hannan and Quinn (1979) criterion, proposed for autoregressive model
selection, is
\deqn{HQIC = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ 2k\log(\log(n)).}
McQuarrie and Tsai (1998) proposed a finite-sample-corrected variant:
\deqn{HQICc = -2\ell(\widehat{\boldsymbol\beta}, \widehat{\boldsymbol\gamma})
+ \frac{2nk\log(\log(n))}{n-k-1}.}

Smaller values indicate better model fit.
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit1 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                   data = ReadingSkills)

# Single model
ic(fit1)

# Comparing multiple models
fit2 <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                   data = ReadingSkills, link.mu = "loglog")
ic(fit1, fit2)

}
\references{
Akaike, H. (1973). Information theory and an extension of the maximum
likelihood principle. In \emph{Proceedings of the 2nd International
Symposium on Information Theory}, 267--281.

Akaike, H. (1978). A Bayesian analysis of the minimum AIC procedure.
\emph{Annals of the Institute of Statistical Mathematics}, \bold{30}(1),
9--14. \doi{10.1007/BF02480194}

Bayer, F. M. and Cribari-Neto, F. (2017). Model selection criteria in beta
regression with varying dispersion. \emph{Communications in Statistics -
Simulation and Computation}, \bold{46}(1), 729--746.
\doi{10.1080/03610918.2014.977918}

Hannan, E. J. and Quinn, B. G. (1979). The determination of the order of
an autoregression. \emph{Journal of the Royal Statistical Society, Series
B}, \bold{41}(2), 190--195. \doi{10.1111/j.2517-6161.1979.tb01072.x}

Hurvich, C. M. and Tsai, C.-L. (1989). Regression and time series model
selection in small samples. \emph{Biometrika}, \bold{76}(2), 297--307.
\doi{10.1093/biomet/76.2.297}

McQuarrie, A. D. R. (1999). A small-sample correction for the Schwarz SIC
model selection criterion. \emph{Statistics & Probability Letters},
\bold{44}(1), 79--86. \doi{10.1016/S0167-7152(98)00294-6}

McQuarrie, A. D. R. and Tsai, C.-L. (1998). \emph{Regression and Time
Series Model Selection}. Singapore: World Scientific.

Schwarz, G. (1978). Estimating the dimension of a model. \emph{The Annals
of Statistics}, \bold{6}(2), 461--464. \doi{10.1214/aos/1176344136}

Sugiura, N. (1978). Further analysis of the data by Akaike's information
criterion and the finite corrections. \emph{Communications in Statistics -
Theory and Methods}, \bold{7}(1), 13--26.
\doi{10.1080/03610927808827599}
}
\seealso{
\code{\link{r2}}, \code{\link{press}}, \code{\link{ic.penalized}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_influence.R
\name{cooks.distance.simplexregression}
\alias{cooks.distance.simplexregression}
\title{Cook's Distance for Simplex Regression Models}
\usage{
\method{cooks.distance}{simplexregression}(model, type = c("pearson", "weighted"), ...)
}
\arguments{
\item{model}{An object of class \code{"simplexregression"}.}

\item{type}{Character string indicating the type of residual used in the
influence measure: \code{"pearson"} (default) or \code{"weighted"}.}

\item{...}{Currently not used.}
}
\value{
A numeric vector of Cook's distance values.
}
\description{
Computes approximate Cook's distances for simplex regression models with
parametric or fixed mean link function.
}
\details{
Cook's distance measures the influence of each observation on the estimated
regression coefficients. It combines the leverage of an observation (see
\code{\link{hatvalues.simplexregression}}) with the magnitude of its residual.
Observations with high Cook's distance may have a disproportionate effect
on the fitted model.

Two approximate versions are available, depending on \code{type}:
\itemize{
  \item \code{"pearson"}: the conventional approximate Cook's distance, based
  on the Pearson residual, analogous to the measure used in beta regression
  (Ferrari and Cribari-Neto, 2004).
  \item \code{"weighted"}: the approximate Cook's distance based on the
  weighted residual, as proposed specifically for the simplex regression
  model by Espinheira and Silva (2026).
}
}
\references{
Cook, R. D. (1977). Detection of influential observation in linear
regression. \emph{Technometrics}, \bold{19}(1), 15--18.
\doi{10.2307/1268249}

Espinheira, P. L. and Silva, A. O. (2026). Prediction in the nonlinear
simplex model. \emph{International Journal of Data Science and Analytics},
\bold{22}, 161. \doi{10.1007/s41060-026-01114-9}

Ferrari, S. L. P. and Cribari-Neto, F. (2004). Beta regression for modeling
rates and proportions. \emph{Journal of Applied Statistics}, \bold{31}(7),
799--815. \doi{10.1080/0266476042000214501}
}
\seealso{
\code{\link{hatvalues.simplexregression}},
\code{\link{gleverage.simplexregression}}, \code{\link{residuals.simplexregression}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_plots.R
\name{plot.simplexregression}
\alias{plot.simplexregression}
\title{Diagnostic Plots for Simplex Regression Models}
\usage{
\method{plot}{simplexregression}(
  x,
  which = 1:7,
  type = c("quantile", "pearson", "deviance", "standardized", "weighted", "variance",
    "biasvariance", "score", "dualscore", "response"),
  ask = prod(par("mfcol")) < length(which) && interactive(),
  threshold = NULL,
  label.pos = 3,
  plot.type = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"simplexregression"}.}

\item{which}{Numeric vector indicating which plots to display (\code{1:7}).}

\item{type}{Character string specifying the residual type (default: \code{"quantile"}).
\cr See \code{\link{residuals.simplexregression}} for available options.}

\item{ask}{Logical; if \code{TRUE}, the user is asked before each plot.
Default is \code{TRUE} when multiple plots are requested.}

\item{threshold}{Numeric threshold for identifying influential or outlying
observations in plots 1--3 (residuals plots), 6 (Cook's distance), and
7 (generalized leverage). If \code{NULL} (default), no observations are
highlighted.}

\item{label.pos}{Position(s) for outlier labels in plots 1--3, 6, and 7.
Can be a single value (applied to all labels) or a vector. Values:
1 = below, 2 = left, 3 = above, 4 = right. Default is \code{3} (above).
See \code{\link[graphics]{text}} for details.}

\item{plot.type}{Controls the plot symbol/type for scatter plots and index plots.
If \code{NULL} (default), uses \code{pch = 1} (open circles) for residual plots
(which = 1--4) and \code{type = "h"} (vertical lines) for Cook's distance
(\code{which = 6}) and generalized leverage plots (\code{which = 7}).
Otherwise, the value is passed directly to \code{pch} (for scatter plots) or
\code{type} (for index plots).}

\item{...}{Additional graphical parameters.}
}
\value{
Called for its side effects (diagnostic plots). Returns \code{NULL}
invisibly.
}
\description{
Produces diagnostic plots for fitted simplex regression models with
parametric or fixed mean link function.
}
\details{
Seven diagnostic plots are available:
\itemize{
  \item Residuals vs observation index (\code{which = 1}): Identifies outliers
  and temporal patterns;

  \item Residuals vs fitted values (\code{which = 2}): Checks for heteroscedasticity
  and patterns;

  \item Residuals vs linear predictor (\code{which = 3}): Evaluates link function
  adequacy;

  \item Observed vs fitted values (\code{which = 4}): Assesses overall model fit;

  \item Normal Q–Q plot (\code{which = 5}): Evaluates residual normality (especially
  useful for quantile residuals);

  \item Cook’s distance vs indices of observations (\code{which = 6}): Identifies
  influential observations;

  \item Generalized leverage vs indices of observations (\code{which = 7}): Identifies
  influential observations.
}
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)

# Display all diagnostic plots
oldpar <- par(mfrow = c(3, 3))
plot(fit, which = 1:7)
par(oldpar)

}
\seealso{
\code{\link{residuals.simplexregression}},
\code{\link{cooks.distance.simplexregression}},
\code{\link{halfnormal.plot}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relativehumidity.R
\docType{data}
\name{RelativeHumidity}
\alias{RelativeHumidity}
\title{Monthly Relative Humidity Data}
\format{
A data frame with 312 observations and 11 variables:
\describe{
  \item{date}{Date. Last day of the reference month for the monthly
        measurements (YYYY-MM-DD).}
  \item{rh}{Numeric. Monthly mean relative humidity, originally measured
        in percent and rescaled to the unit interval (0, 1).}
  \item{ins}{Numeric. Monthly total insolation (in hours), contains
        2 missing values.}
  \item{ins2}{Numeric. Monthly total insolation (in hours) with missing
        values imputed via seasonal interpolation.}
  \item{pre}{Numeric. Monthly total precipitation (in mm), contains
        2 missing values.}
  \item{pre2}{Numeric. Monthly total precipitation (in mm) with missing
        values imputed via seasonal interpolation.}
  \item{cld}{Numeric. Monthly mean cloudiness (in tenths).}
  \item{ap}{Numeric. Monthly mean atmospheric pressure (in hPa).}
  \item{tmax}{Numeric. Monthly mean of daily maximum temperature
        (in degrees Celsius).}
  \item{ws}{Numeric. Monthly mean wind speed (in m/s).}
  \item{wd}{Numeric. Monthly predominant wind direction, coded by
        INMET/BDMEP in tens of degrees (e.g., 9 = 90 degrees, East;
        32 = 320 degrees, Northwest). A value of 0 indicates calm
        conditions (no wind or undefined direction). Multiply by 10
        to obtain the direction in degrees (0-360).}
}
}
\source{
Instituto Nacional de Meteorologia (INMET). Banco de Dados
Meteorológicos para Ensino e Pesquisa (BDMEP), Estação de Brasília
(https://bdmep.inmet.gov.br). Data accessed in 2026.
}
\usage{
data(RelativeHumidity)
}
\description{
Monthly meteorological data including relative humidity (rh),
temperature, insolation, precipitation, and wind speed for the
meteorological station of Brasília, Brazil (2000-2025), obtained from
the National Institute of Meteorology (INMET).

The dataset contains two missing observations in the variables \code{ins}
and \code{pre}, corresponding to September 2020 and November 2025. To address
these missing values, imputation was performed via seasonal interpolation using
the \pkg{imputeTS} package, with the imputed versions stored as \code{ins2}
and \code{pre2}.
}
\examples{
# Load the dataset
data(RelativeHumidity)

# View first rows
head(RelativeHumidity)

# Check structure
str(RelativeHumidity)

# Insolation with and without imputation
plot(RelativeHumidity$ins, type = "l", col = "red",
     ylab = "Insolation", main = "Missing values visible as gaps")
points(RelativeHumidity$ins2, type = "l", col = "blue")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomass.R
\docType{data}
\name{Biomass}
\alias{Biomass}
\title{Biomass Allocation in Two Grass Species Under Different Nitrate Supply}
\format{
A data frame with 500 observations and 13 variables:
\describe{
  \item{group}{Factor. Combined species-by-nitrate-treatment code
        (\code{DfH}, \code{DfL}, \code{HlH}, \code{HlL}), corresponding to the combination of
        \code{species} and \code{trt} where:
    \itemize{
      \item \code{DfH} = D. flexuosa (slow-growing), high nitrate
      \item \code{DfL} = D. flexuosa (slow-growing), low nitrate
      \item \code{HlH} = H. lanatus (fast-growing), high nitrate
      \item \code{HlL} = H. lanatus (fast-growing), low nitrate
    }
  }
  \item{species}{Factor. Species scientific name (\code{D. flexuosa} or
        \code{H. lanatus}).}
  \item{trt}{Factor. Nitrate treatment level (\code{high} or \code{low}).}
  \item{day}{Numeric. Experimental day (0 to 49).}
  \item{pl_num}{Numeric. Plant number (individual plant identifier,
        6-8 replicates per treatment).}
  \item{ldm_mg}{Numeric. Leaf dry mass (in mg).}
  \item{sdm_mg}{Numeric. Stem dry mass (in mg).}
  \item{rdm_mg}{Numeric. Root dry mass (in mg).}
  \item{tdm_mg}{Numeric. Total dry mass (in mg).}
  \item{lmf}{Numeric. Leaf mass fraction, proportion of biomass allocated
        to leaves, bounded in the open interval (0, 1).}
  \item{smf}{Numeric. Stem mass fraction, proportion of biomass allocated
        to stems, bounded in the open interval (0, 1).}
  \item{rmf}{Numeric. Root mass fraction, proportion of biomass allocated
        to roots, bounded in the open interval (0, 1).}
  \item{ln_tdm}{Numeric. Natural log of total dry mass (log-transformed
        for allometric analysis).}
}
}
\source{
bobdouma (2019). \emph{bobdouma/proportions_beta_Dirichlet: v.01}.
\doi{10.5281/zenodo.3234670}.
}
\usage{
data(Biomass)
}
\description{
This dataset examines biomass allocation patterns in plants, specifically
the proportional distribution of biomass to different plant organs (stems,
leaves, and roots). The data come from an experiment manipulating nitrate
supply in fast-growing and slow-growing grass species.

The response variables (stem, leaves, roots) are proportions bounded in the
(0, 1) interval, making them suitable for simplex regression analysis.
}
\examples{
# Load the data
data(Biomass)

# Quick overview
head(Biomass)
str(Biomass)

# Check that proportions sum to 1 (within rounding error)
summary(rowSums(Biomass[, c("lmf", "smf", "rmf")]))

# Simple plot of root mass fraction by treatment
boxplot(rmf ~ trt * species, data = Biomass,
        main = "Root Mass Fraction by Species and Nitrate Treatment",
        las = 2)
}
\references{
bobdouma (2019). \emph{bobdouma/proportions_beta_Dirichlet: v.01}.
\doi{10.5281/zenodo.3234670}.

Poorter, H.; van de Vijver, C. A. D. M.; Boot, R. G. A. and Lambers, H. (1995).
Growth and carbon economy of a fast-growing and a slow-growing grass
species as dependent on nitrate supply. \emph{Plant and Soil}, \bold{171},
217--227. \doi{10.1007/BF00010275}

Poorter, H. and Sack, L. (2012). Pitfalls and possibilities in the analysis
of biomass allocation patterns in plants. \emph{Frontiers in Plant
Science}, \bold{3}, 259. \doi{10.3389/fpls.2012.00259}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplexreg_fit.R
\name{simplexreg.fit}
\alias{simplexreg.fit}
\alias{simplexreg}
\title{Simplex Regression with Parametric or Fixed Mean Link}
\usage{
simplexreg(
  formula,
  data,
  subset,
  na.action,
  weights,
  offset,
  link.mu = c("logit", "probit", "loglog", "cloglog", "cauchit", "plogit1", "plogit2"),
  link.sigma2 = NULL,
  contrasts = NULL,
  control = simplexreg.control(...),
  model = TRUE,
  y = TRUE,
  x = FALSE,
  ...
)

simplexreg.fit(
  y,
  x,
  z,
  weights = NULL,
  offset = NULL,
  link.mu = c("logit", "probit", "loglog", "cloglog", "cauchit", "plogit1", "plogit2"),
  link.sigma2 = c("log", "sqrt", "identity"),
  x_names = NULL,
  z_names = NULL,
  control = simplexreg.control(...),
  ...
)
}
\arguments{
\item{formula}{A two-part formula: \code{y ~ x} or \code{y ~ x | 1}
(mean submodel, constant dispersion), or \code{y ~ x | z} (submodels for both
mean and dispersion).}

\item{data}{A data frame containing the variables in formula.}

\item{subset}{A specification of the rows/observations to be used: defaults to all.}

\item{na.action}{An optional (name of a) function for treating missing values (NAs).}

\item{weights}{An optional numeric vector of case weights.}

\item{offset}{Optional numeric vector specifying a known component to be
included in the linear predictor of the mean submodel during fitting.
One or more \code{\link{offset}} terms can be included in the formula instead
or as well, and if more than one is specified their sum is used. See
\code{\link{model.offset}}.}

\item{link.mu}{Character specification of the link function in the mean submodel
(parametric functions: \code{"plogit1"}, \code{"plogit2"}; or fixed functions:
\code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"}, \code{"cauchit"}).}

\item{link.sigma2}{Character specification of the link function in the dispersion
submodel (\code{"log"}, \code{"sqrt"}, \code{"identity"}).}

\item{contrasts}{An optional list. See the \code{contrasts.arg} argument of
\code{\link[stats]{model.matrix.default}}.}

\item{control}{A list of control arguments specified via \code{\link{simplexreg.control}}.}

\item{model, y, x}{Logicals. If \code{TRUE} the corresponding components of the fit
(model frame, response, model matrix) are returned. For \code{\link{simplexreg.fit}},
\code{x} should be a numeric regressor matrix and \code{y} should be the numeric response
vector (with values in (0, 1)).}

\item{...}{Additional arguments passed to \code{\link{simplexreg.control}}.}

\item{z}{Design matrix for dispersion model (without intercept).}

\item{x_names}{Column names for mean design matrix (includes intercept).}

\item{z_names}{Column names for dispersion design matrix (includes intercept).}
}
\value{
An object of class \code{"simplexregression"}, i.e.,
a list with components as follows:
\itemize{
  \item \code{coefficients}: A list with elements \code{mean} and \code{dispersion},
  containing the estimated regression coefficients \eqn{\hat{\boldsymbol{\beta}}} and
  \eqn{\hat{\boldsymbol{\gamma}}} of the mean and dispersion submodels, respectively.
  For parametric mean link functions, the list also includes an additional element,
  \code{lambda}, containing the estimated link parameter \eqn{\hat{\lambda}}.
  \item \code{fitted.values}: a vector of fitted mean values,
  \item \code{optim}: a list containing \code{start} (initial values),
  \code{convergence} (convergence code), \code{counts} (number of iterations) and
  \code{method} (optimization method) from the optimization procedure,
  \item \code{scoring}: number of iterations from the optimization procedure via
  Fisher scoring,
  \item \code{mu.fv}: a vector of fitted mean values,
  \item \code{mu.lp}: a vector of fitted mean linear predictor,
  \item \code{mu.x}: design matrix for the mean model (with intercept),
  \item \code{mu.link}: character string specifying the mean link function,
  \item \code{mu.df}: degrees of freedom for the mean model,
  \item \code{sigma2.fv}: a vector of fitted dispersion values,
  \item \code{sigma2.lp}: a vector of fitted dispersion linear predictor,
  \item \code{sigma2.x}: design matrix for the dispersion model (with intercept),
  \item \code{sigma2.link}: character string specifying the dispersion link function,
  \item \code{sigma2.df}: degrees of freedom for the dispersion model,
  \item \code{lambda.fv}: estimated value of the parametric link function parameter
  (\code{NA} for mean fixed links),
  \item \code{df.residual}: residual degrees of freedom,
  \item \code{nobs}: number of observations,
  \item \code{loglik}: maximized log-likelihood value,
  \item \code{vcov}: variance-covariance matrix of the parameter estimates,
  \item \code{residuals}: a vector of quantile residuals,
  \item \code{AIC}, \code{BIC}, \code{HQIC}: Akaike, Schwarz, and Hannan-Quinn
  information criteria,
  \item \code{R2_N}, \code{R2_FC}: Nagelkerke, and Ferrari and Cribari-Neto
  pseudo R-squared measures,
  \item \code{zstat}: \eqn{z}-statistics for the coefficient tests,
  \item \code{pvalues}: \eqn{p}-values for the coefficient tests,
  \item \code{y}: the response vector,
  \item \code{x_names}: column names of the mean design matrix,
  \item \code{z_names}: column names of the dispersion design matrix,
  \item \code{control}: the control arguments passed to the \code{optim} call,
  \item \code{converged}: logical indicating successful convergence of \code{optim},
  \item \code{call}: the original function call,
  \item \code{formula}: the original two-part formula,
  \item \code{formula_mean}: formula for the mean submodel,
  \item \code{formula_disp}: formula for the dispersion submodel,
  \item \code{terms}: a list with \code{mean} and \code{dispersion} terms
  objects,
  \item \code{weights}: the weights used in fitting (if any),
  \item \code{offset}: the offset used in fitting (if any),
  \item \code{na.action}: the na.action attribute from the model frame,
  \item \code{subset}: the subset used in fitting (if any),
  \item \code{model}: the full model frame.

}
}
\description{
Fit simplex regression models for rates and proportions via
maximum likelihood estimation, modeling both the mean (via parametric
or fixed link function) and the dispersion parameter.
}
\details{
Simplex regression, introduced by Song and Tan (2000) and extended by
Song, Qiu and Tan (2004), is useful for modeling continuous response variables
restricted to the unit interval (0, 1), such as rates and proportions. The model
assumes that the dependent variable follows the simplex distribution, originally
proposed by Barndorff-Nielsen and Jørgensen (1991), which is indexed by the mean
\eqn{\mu} and the dispersion parameter \eqn{\sigma^2}.

Similar to generalized linear models (GLMs), simplex regression relates
the mean of the response variable and the dispersion parameter to their respective
linear predictors through link functions. This package implements five fixed link
functions (\code{"logit"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"},
\code{"cauchit"}) and two parametric link functions (\code{"plogit1"},
\code{"plogit2"}) for the mean submodel. For the dispersion submodel, the
links \code{"log"}, \code{"sqrt"} and \code{"identity"} are supported.

The model is specified through a two-part formula separated by \code{|}. The
left side contains the predictors for the mean submodel and the right side contains
the predictors for the dispersion submodel:
\itemize{
  \item Mean submodel: \eqn{g(\mu_i, \lambda) = \boldsymbol{x}_i'\boldsymbol{\beta}}
  (parametric link) or \eqn{g(\mu_i) = \boldsymbol{x}_i'\boldsymbol{\beta}} (fixed link),
  where \eqn{g(\cdot)} is the mean link function, \eqn{\lambda} is the
  extra shape parameter of the parametric link,
  \eqn{\boldsymbol{x}_i} is the vector of covariates for the \eqn{i}-th observation in
  the mean submodel, and \eqn{\boldsymbol{\beta}} is the corresponding
  vector of regression coefficients.
  \item Dispersion submodel: \eqn{h(\sigma^2_i) = \boldsymbol{z}_i'\boldsymbol{\gamma}},
  where \eqn{h(\cdot)} is the dispersion link function, \eqn{\boldsymbol{z}_i} is the
  vector of covariates for the \eqn{i}-th observation in the dispersion
  submodel, and \eqn{\boldsymbol{\gamma}} is the corresponding vector of
  regression coefficients.
}

Formula examples: \code{y ~ x1 + x2 | z1 + z2} (variable dispersion) or
\code{y ~ x1 + x2} (constant dispersion). The link functions for both
submodels are specified using \code{link.mu} and \code{link.sigma2}.

The parametric mean link functions include a parameter \eqn{\lambda} that
is estimated along with other model parameters. Parameter estimation is performed
via maximum likelihood using the \code{optim} function with analytical gradient
and initial values obtained from an auxiliary linear regression of the transformed
response. Subsequently, the \code{optim} result may be enhanced by an additional Fisher
scoring iteration using analytical gradients and expected information. The Fisher
scoring is just a refinement to move the gradients even closer to zero and can be
disabled by setting \code{fsmaxit = 0} in the control arguments.

Methods for extracting and analyzing results are implemented for objects
of class \code{"simplexregression"}, allowing the use of generic functions such as
\code{\link{summary}}, \code{\link{print}}, \code{\link{fitted}}, \code{\link{coef}},
\code{\link{formula}}, \code{\link{logLik}}, \code{\link{vcov}}, \code{\link{predict}},
\code{\link{terms}}, \code{\link{model.frame}}, \code{\link{model.matrix}},
\code{\link{plot}}, \code{\link{residuals}}, \code{\link{cooks.distance}},
\code{\link{gleverage}}, \code{\link{hatvalues}}, \code{\link{update}},
\code{\link{simulate}}, \code{\link{AIC}}, \code{\link[lmtest]{coeftest}} (from the
lmtest package), \code{\link[sandwich]{bread}} and \code{\link[sandwich]{estfun}} (from the sandwich package).
}
\examples{
data(ReadingSkills, package = "SimplexRegression")
fit <- simplexreg(accuracy ~ dyslexia * iq | dyslexia + iq + I(iq^2),
                 data = ReadingSkills)
summary(fit)

}
\references{
Barndorff-Nielsen, O. E. and Jørgensen, B. (1991).
Some parametric models on the simplex.
\emph{Journal of Multivariate Analysis}, \bold{39}(1), 106--116.
\doi{10.1016/0047-259X(91)90008-P}

Justino, M. E. C. and Cribari-Neto, F. (2026).
Simplex regression with a flexible logit link: Inference and application
to cross-country impunity data.
\emph{Applied Mathematical Modelling}, \bold{154}, 116713. \doi{10.1016/j.apm.2025.116713}

Jørgensen, B. (1997). \emph{The Theory of Dispersion Models}.
Chapman and Hall, London.

Song, P. X.-K. and Tan, M. (2000).
Marginal models for longitudinal continuous proportional data.
\emph{Biometrics}, \bold{56}(2), 496--502.
\doi{10.1111/j.0006-341X.2000.00496.x}

Song, P. X.-K.; Qiu, Z. and Tan, M. (2004).
Modelling heterogeneous dispersion in marginal models for longitudinal
proportional data.
\emph{Biometrical Journal}, \bold{46}(5), 540--553.
\doi{10.1002/bimj.200110052}

Song, P. X.-K. (2009).
Dispersion models in regression analysis.
\emph{Pakistan Journal of Statistics}, \bold{25}(4), 529--551.

Zhang, P. and Qiu, Z. G. (2014).
Regression analysis of proportional data using simplex distribution.
\emph{SCIENTIA SINICA Mathematica}, \bold{44}(1), 89--104.
\doi{10.1360/012013-200}
}
\seealso{
\code{\link{summary.simplexregression}},
\code{\link{predict.simplexregression}}, \code{\link{residuals.simplexregression}},
\code{\link{ic.penalized}}, \code{\link{ss.penalized}},
\code{\link{scoretest}}
}
